/*
 $Id: CScriptObjects.cpp,v 1.29 2003/01/08 06:34:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/


#include "stdafx.h"
#include "area.h"
#include "FileReadProgress.h"
#include "RemoteConnection.h"
#include "CScriptObjects.h"
#include <direct.h>

int CScriptObjects::m_iScriptSeq = 0;

CScriptObjects::CScriptObjects()
{
	this->m_bDefsLoaded = false;
	this->m_bItemsLoaded = false;
	this->m_bNPCsLoaded = false;
	this->m_bTablesLoaded = false;
	this->m_bTemplatesLoaded = false;
	this->m_bMapLoaded = false;
	this->m_bTriggersLoaded = false;
	this->m_bLocationsLoaded = false;
	this->m_bNamesLoaded = false;
	this->m_bSpeechLoaded = false;

	this->m_bNPCsCategorized = false;
	this->m_bItemsCategorized = false;
	this->m_bLocationsCategorized = false;

	this->m_wNextItemID = 0x6000;
	this->m_wNextNPCID = 0x4000;
	this->m_dwNextTemplate = TEMPLATE2_INDEX_BASE;
	this->m_bNewFormat = false;
	m_iICatSeq = 0;
	m_iNCatSeq = 0;
	m_iMCatSeq = 0;

	m_pDlg = NULL;

	if ( Main->GetProfileType() == PROFILE_REMOTE )
		m_pRemote = new CRemoteConnection;
	else
		m_pRemote = NULL;
}

CScriptObjects::~CScriptObjects()
{
	this->Unload();
	if ( m_pRemote )
		delete m_pRemote;
}

void CScriptObjects::Unload()
{
	Main->m_log.Add("Unloading scripts");
	RemoveNPCCategories();
	RemoveItemCategories();
	RemoveLocationCategories();
	for ( int i = 0; i < m_aDefs.GetSize(); i++ )
	{
		CDef * pDef = (CDef *) m_aDefs.GetAt(i);
		if ( pDef )
			delete pDef;
	}
	m_aDefs.RemoveAll();
	UnloadItems();
	UnloadNPCs();
	UnloadTemplates();

	for ( i = 0; i < this->m_aTriggers.GetSize(); i++ )
	{
		CStringBlock * pTrigger = (CStringBlock *) m_aTriggers.GetAt(i);
		if ( pTrigger )
			delete pTrigger;
	}
	m_aTriggers.RemoveAll();
	
	for ( i = 0; i < this->m_aRegionTrigs.GetSize(); i++ )
	{
		CStringBlock * pTable = (CStringBlock *) m_aRegionTrigs.GetAt(i);
		if ( pTable )
			delete pTable;
	}
	m_aRegionTrigs.RemoveAll();

	for ( i = 0; i < this->m_aPlotTrigs.GetSize(); i++ )
	{
		CStringBlock * pTable = (CStringBlock *) m_aPlotTrigs.GetAt(i);
		if ( pTable )
			delete pTable;
	}
	m_aPlotTrigs.RemoveAll();

	POSITION pos = this->m_olAreas.GetHeadPosition();
	while ( pos )
	{
		CArea * pArea = (CArea *) this->m_olAreas.GetNext(pos);
		if ( pArea )
			delete pArea;
	}
	this->m_olAreas.RemoveAll();
	
	pos = this->m_olRooms.GetHeadPosition();
	while ( pos )
	{
		CRoom * pRoom = (CRoom *) this->m_olRooms.GetNext(pos);
		if ( pRoom )
			delete pRoom;
	}
	this->m_olRooms.RemoveAll();

	for ( i = 0; i < m_aLocations.GetSize(); i++ )
	{
		CLocation * pLoc = (CLocation *) m_aLocations.GetAt(i);
		if ( pLoc )
			delete pLoc;
	}
	m_aLocations.RemoveAll();

	UnloadNames();

	for ( i = 0; i < m_aSpeechBlocks.GetSize(); i++ )
	{
		CStringBlock * pBlock = (CStringBlock *) m_aSpeechBlocks.GetAt(i);
		if ( pBlock )
			delete pBlock;
	}
	m_aSpeechBlocks.RemoveAll();

	for ( i = 0; i < m_aWebpages.GetSize(); i++ )
	{
		CTObject * pBlock = (CTObject *) m_aWebpages.GetAt(i);
		if ( pBlock )
			delete pBlock;
	}
	m_aWebpages.RemoveAll();

	this->m_tables.Unload();

	this->m_bDefsLoaded = false;
	this->m_bTablesLoaded = false;
	this->m_bMapLoaded = false;
	this->m_bTriggersLoaded = false;
	this->m_bLocationsLoaded = false;
	this->m_bSpeechLoaded = false;
}

// DEFS
void CScriptObjects::LoadDefs()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bDefsLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading defs");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	// Grab the custom file from the remote server if necessary.
#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_DEFS_FILE, st) )
		{
			if (m_pRemote && m_pRemote->IsValid())
				m_pRemote->GetCustomFile("graydefs");
		}
	}
#endif

	CString csMessage;
	DWORD j = 0;

	for ( int i = 0; i < 2; i++ )
	{
		CStdioFile csfDefs;
		CString csFilename;
		switch (i)
		{
		case 0:
			// This is the regular file
			if ( Main->GetProfileType() == PROFILE_REMOTE || csScpPath == "" )
				csFilename = DEFS_FILE;
			else
				csFilename.Format("%s%s", csScpPath, DEFS_FILE);
			break;
		case 1:
			// This is the custom file
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_DEFS_FILE);
			else if ( csScpPath == "" )
				csFilename = USER_DEFS_FILE;
			else
				csFilename.Format("%s%s", csScpPath, USER_DEFS_FILE);
			break;
		}
		if ( !csfDefs.Open(csFilename, CFile::modeRead | CFile::shareDenyNone))
		{
			if ( csFilename.Find("\\") == -1 )
			{
				CString csTemp = csFilename;
				char szPath[MAX_PATH];
				memset(&szPath[0], 0x00, sizeof(szPath) );
				getcwd(&szPath[0], sizeof(szPath));
				csFilename.Format("%s\\%s", szPath, csFilename);
			}
			Main->m_log.Add("Unable to open file %s", csFilename);
			break;
		}

		csMessage.Format("Reading %s", csFilename);
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);

		BOOL bStatus = true;
		DWORD dwFileLength = csfDefs.GetLength();
		int scalefactor = dwFileLength / 0xefff;
		scalefactor += 1;
		m_pDlg->SetRange(0, (short)(dwFileLength / scalefactor));
		CString csGroup;

		while ( bStatus )
		{
			CString csLine;
			bStatus = csfDefs.ReadString(csLine);
			j++;
			if ( j % 100 )
				m_pDlg->SetPos(csfDefs.Seek(0, CFile::current) / scalefactor);
			csLine.TrimLeft();
			csLine.TrimRight();
			if ( csLine.Find("[") != -1 )
			{
				csGroup = csLine.Mid(csLine.Find("[") + 1);
				csGroup = csGroup.SpanExcluding("]");
				if ( csGroup.Find("DEFNAME") != -1 && csGroup.GetLength() > 7)
				{
					CString csTemp = csGroup.Mid(8);
					csGroup = csTemp;
					csGroup.TrimLeft();
				}
			}
			csLine = csLine.SpanExcluding("//");
			if (csLine == "")
				continue;
			CDef * pDef = new CDef;
			pDef->Read(csLine);
			if ( pDef->m_csValue == "" )
			{
				delete pDef;
				continue;
			}
			pDef->m_csGroup = csGroup;
			pDef->m_bCustom = i == 0 ? false : true;
			// Does this one already exist?
			int iIndex = m_aDefs.Find(pDef->m_csValue);
			if ( iIndex != -1 )
			{
				CDef * pOld = (CDef *) m_aDefs.GetAt(iIndex);
				delete pOld;
				m_aDefs.RemoveAt(iIndex);
			}
			m_aDefs.Insert(pDef);
		}
	}
	m_bDefsLoaded = true;
	DestroyProgressDialog();
}

bool CScriptObjects::SaveDefs()
{
	return true;
}

// ITEMS
void CScriptObjects::LoadItems()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bItemsLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading items");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CreateProgressDialog();

	if ( !m_bDefsLoaded )
		LoadDefs();

	// Grab the custom file from the remote server if necessary.
#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_ITEM_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("grayitem2");
		}
	}
#endif

	CString csMessage;
	DWORD j = 0;

	for ( int i = 0; i < 2; i++ )
	{
		CStdioFile csfItems;
		CString csFilename;
		switch (i)
		{
		case 0:
			// This is the regular file
			if ( Main->GetProfileType() == PROFILE_REMOTE || csScpPath == "" )
				csFilename = ITEM_FILE;
			else
				csFilename.Format("%s%s", csScpPath, ITEM_FILE);
			break;
		case 1:
			// This is the custom file
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_ITEM_FILE);
			else if ( csScpPath == "" )
				csFilename = USER_ITEM_FILE;
			else
				csFilename.Format("%s%s", csScpPath, USER_ITEM_FILE);
			break;
		}
		if ( !csfItems.Open(csFilename, CFile::modeRead | CFile::shareDenyNone))
		{
			if ( csFilename.Find("\\") == -1 )
			{
				CString csTemp = csFilename;
				char szPath[MAX_PATH];
				memset(&szPath[0], 0x00, sizeof(szPath) );
				getcwd(&szPath[0], sizeof(szPath));
				csFilename.Format("%s\\%s", szPath, csFilename);
			}
			Main->m_log.Add("Unable to open file %s", csFilename);
			break;
		}

		csMessage.Format("Reading %s", csFilename);
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
		BOOL bStatus = true;
		DWORD dwFileLength = csfItems.GetLength();
		int scalefactor = dwFileLength / 0xefff;
		scalefactor += 1;
		m_pDlg->SetRange(0, (short)(dwFileLength / scalefactor));
		m_pDlg->UpdateData(false);

		while ( bStatus )
		{
			CString csLine;
			DWORD dwOffset = csfItems.GetPosition();
			bStatus = csfItems.ReadString(csLine);
			if ( !bStatus )
				break;
			if ( csLine.Find("[") != 0 )
				continue;
			CString csID = csLine.Right(csLine.GetLength() - 1);
			csID = csID.SpanExcluding("]");
			if (j % 100 == 0)
				m_pDlg->SetPos(csfItems.Seek(0, CFile::current) / scalefactor);
			CSObject * pItem = new CSObject;
			pItem->m_bType = TYPE_ITEM;
			pItem->m_csValue = csID;
			bStatus = pItem->Read( csfItems );
			pItem->m_dwFileOffset = dwOffset;
			pItem->m_csFilename = csFilename;
			//pItem->m_bCustom = i == 0 ? false : true;
			j++;

			// Insert the item into our array
			int iIndex = m_aItems.Find(pItem->m_csValue);
			if ( iIndex != -1 )
			{
				CSObject * pOld = (CSObject *) m_aItems.GetAt(iIndex);
				m_aItems.RemoveAt(iIndex);
				delete pOld;
			}
			m_aItems.Insert(pItem);
		}
	}
	m_bItemsLoaded = true;
	DestroyProgressDialog();
}

void CScriptObjects::UnloadItems()
{
	Main->m_log.Add("Unloading items");
	for ( int i = 0; i < m_aItems.GetSize(); i++ )
	{
		CSObject * pItem = (CSObject *) m_aItems.GetAt(i);
		if ( pItem )
			delete pItem;
	}
	m_aItems.RemoveAll();
	m_bItemsLoaded = false;
}

void CScriptObjects::ReloadItems()
{
	UnloadItems();
	LoadItems();
}

bool CScriptObjects::SaveItems()
{
	Main->m_log.Add("Saving items");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_ITEM_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temp file while saving items");
		AfxMessageBox("Unable to open temporary file while saving items.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, USER_ITEM_FILE);

	for ( int i = 0; i < this->m_aItems.GetSize(); i++ )
	{
		CItem * pItem = (CItem *) m_aItems.GetAt(i);
		if ( pItem && pItem->m_bCustom )
			pItem->Write(csfOutput);
	}
	csfOutput.Close();
	return UpdateFiles( TMP_ITEM_FILE, USER_ITEM_FILE, RMT_ITEM_FILE, "grayitems" );
}

// NPCS
void CScriptObjects::LoadNPCs()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bNPCsLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading NPCs");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CreateProgressDialog();

	if ( !m_bDefsLoaded )
		LoadDefs();
	if ( !m_bTablesLoaded )
		LoadTables();

	// Grab the custom file from the remote server if necessary.
#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_NPC_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("graychar2");
		}
	}
#endif
	CString csMessage;
	DWORD j = 0;

	for ( int i = 0; i < 2; i++ )
	{
		CStdioFile csfChars;
		CString csFilename;
		switch (i)
		{
		case 0:
			// This is the regular file
			if ( Main->GetProfileType() == PROFILE_REMOTE || csScpPath == "" )
				csFilename = NPC_FILE;
			else
				csFilename.Format("%s%s", csScpPath, NPC_FILE);
			break;
		case 1:
			// This is the custom file
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_NPC_FILE);
			else if ( csScpPath == "" )
				csFilename = USER_NPC_FILE;
			else
				csFilename.Format("%s%s", csScpPath, USER_NPC_FILE);
			break;
		}
		if ( !csfChars.Open(csFilename, CFile::modeRead | CFile::shareDenyNone) )
		{
			if ( csFilename.Find("\\") == -1 )
			{
				CString csTemp = csFilename;
				char szPath[MAX_PATH];
				memset(&szPath[0], 0x00, sizeof(szPath) );
				getcwd(&szPath[0], sizeof(szPath));
				csFilename.Format("%s\\%s", szPath, csFilename);
			}
			Main->m_log.Add("Unable to open file %s", csFilename);
			break;
		}
		csMessage.Format("Reading %s", csFilename);
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
		BOOL bStatus = true;
		DWORD dwFileLength = csfChars.GetLength();
		int scalefactor = dwFileLength / 0xefff;
		scalefactor += 1;
		m_pDlg->SetRange(0, (short)(dwFileLength / scalefactor));

		while ( bStatus )
		{
			CString csLine;
			DWORD dwOffset = csfChars.GetPosition();
			bStatus = csfChars.ReadString(csLine);
			if ( csLine.Find("[") == 0 )
			{
				CString csID = csLine.Right(csLine.GetLength() - 1);
				csID = csID.SpanExcluding("]");
				if ( csID.CompareNoCase("EOF") == 0 )
					continue;
				if (j % 10 == 0)
					m_pDlg->SetPos(csfChars.Seek(0, CFile::current) / scalefactor);
				CSObject * pChar = new CSObject;
				pChar->m_bType = TYPE_CHAR;
				pChar->m_csValue = csID;
				bStatus = pChar->Read( csfChars );
				pChar->m_dwFileOffset = dwOffset;
				pChar->m_csFilename = csFilename;
				//pChar->m_bCustom = i == 0 ? false : true;
				j++;

				// Insert the item into our array
				int iIndex = m_aNPCs.Find(pChar->m_csValue);
				if ( iIndex != -1 )
				{
					CSObject * pOld = (CSObject *) m_aNPCs.GetAt(iIndex);
					m_aNPCs.RemoveAt(iIndex);
					delete pOld;
				}
				m_aNPCs.Insert(pChar);
			}
		}
	}
	m_bNPCsLoaded = true;
	DestroyProgressDialog();
}

bool CScriptObjects::SaveNPCs()
{
	Main->m_log.Add("Saving NPCs");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_NPC_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temporary file while saving NPCs");
		AfxMessageBox("Unable to open temporary file while saving NPCs.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, USER_NPC_FILE);

	for ( int i = 0; i < this->m_aNPCs.GetSize(); i++ )
	{
		CNPC * pNPC = (CNPC *) m_aNPCs.GetAt(i);
		if ( pNPC && pNPC->m_bCustom )
			pNPC->Write(csfOutput);
	}
	csfOutput.Close();
	return UpdateFiles( TMP_NPC_FILE, USER_NPC_FILE, RMT_NPC_FILE, "graynpcs" );
}

void CScriptObjects::UnloadNPCs()
{
	Main->m_log.Add("Unloading NPCs");
	for ( int i = 0; i < m_aNPCs.GetSize(); i++ )
	{
		CSObject * pNPC = (CSObject *) m_aNPCs.GetAt(i);
		if ( pNPC )
			delete pNPC;
	}
	m_aNPCs.RemoveAll();
	this->m_bNPCsLoaded = false;
}

void CScriptObjects::ReloadNPCs()
{
	UnloadNPCs();
	LoadNPCs();
}

// TEMPLATES
void CScriptObjects::LoadTemplates()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bTemplatesLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading templates");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	m_pDlg->m_csMessage.SetWindowText("Initializing storage");

	if ( !m_bDefsLoaded )
		LoadDefs();

	// Grab the custom file from the remote server if necessary.
#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_TEMPLATE_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("graytemplate");
		}
	}
#endif
	CString csMessage;
	DWORD j = 0;

	for ( int i = 0; i < 2; i++ )
	{
		CStdioFile csfTemplates;
		CString csFilename;
		switch (i)
		{
		case 0:
			// This is the regular file
			if ( Main->GetProfileType() == PROFILE_REMOTE || csScpPath == "" )
				csFilename = TEMPLATE_FILE;
			else
				csFilename.Format("%s%s", csScpPath, TEMPLATE_FILE);
			break;
		case 1:
			// This is the custom file
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_TEMPLATE_FILE);
			else if ( csScpPath == "" )
				csFilename = Main->GetProfileType() == PROFILE_REMOTE ? RMT_TEMPLATE_FILE : USER_TEMPLATE_FILE;
			else
				csFilename.Format("%s%s", csScpPath, USER_TEMPLATE_FILE);
			break;
		}
		if ( !csfTemplates.Open(csFilename, CFile::modeRead | CFile::shareDenyNone))
		{
			if ( csFilename.Find("\\") == -1 )
			{
				CString csTemp = csFilename;
				char szPath[MAX_PATH];
				memset(&szPath[0], 0x00, sizeof(szPath) );
				getcwd(&szPath[0], sizeof(szPath));
				csFilename.Format("%s\\%s", szPath, csFilename);
			}
			Main->m_log.Add("Unable to open file %s", csFilename);
			break;
		}
		csMessage.Format("Reading %s", csFilename);
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
		BOOL bStatus = true;
		DWORD dwFileLength = csfTemplates.GetLength();
		int scalefactor = dwFileLength / 0xefff;
		scalefactor += 1;
		m_pDlg->SetRange(0, (short)(dwFileLength / scalefactor));

		while ( bStatus )
		{
			CString csLine;
			DWORD dwOffset = csfTemplates.GetPosition();
			bStatus = csfTemplates.ReadString(csLine);
			if ( csLine.Find("[") != 0 )
				continue;
			CString csID = csLine.Right(csLine.GetLength() - 1);
			csID = csID.SpanExcluding("]");
			if (j % 10 == 0)
				m_pDlg->SetPos(csfTemplates.Seek(0, CFile::current) / scalefactor);
			CSObject * pTemplate = new CSObject;
			pTemplate->m_bType = TYPE_TEMPLATE;
			pTemplate->m_csValue = csID;
			//pTemplate->m_dwID = atoi(csID);
			// The first line should be a comment containing the description.
			bStatus = csfTemplates.ReadString(csLine);
			if ( csLine.Find("//") == 0 )
				pTemplate->m_csDescription = csLine.Mid(2);
			bStatus = pTemplate->Read( csfTemplates );
			pTemplate->m_dwFileOffset = dwOffset;
			pTemplate->m_csFilename = csFilename;
			//pTemplate->m_bCustom = i == 0 ? false : true;
			j++;

			// Insert the item into our array
			int iIndex = m_aTemplates.Find(pTemplate->m_csValue);
			if ( iIndex != -1 )
			{
				CSObject * pOld = (CSObject *) m_aTemplates.GetAt(iIndex);
				m_aTemplates.RemoveAt(iIndex);
				delete pOld;
			}
			m_aTemplates.Insert(pTemplate);
			// We need to add the category, etc stuff to this
			pTemplate->m_csCategory = "Item Templates";
			DWORD dwValue = (DWORD) atoi(pTemplate->m_csValue);
			if ( dwValue < 100000 )
				pTemplate->m_csSubsection = "Uncategorized Templates";
			else if ( dwValue >= 100000 && dwValue < 102000 )
				pTemplate->m_csSubsection = "Item Templates";
			else if ( dwValue >= 102000 && dwValue < 150000 )
				pTemplate->m_csSubsection = "Vendor Templates";
			else if ( dwValue >= 150000 && dwValue < 500000 )
				pTemplate->m_csSubsection = "Loot Templates";
			else
				pTemplate->m_csSubsection = "Custom Templates";
		}
	}
	m_bTemplatesLoaded = true;
	DestroyProgressDialog();
}

void CScriptObjects::UnloadTemplates()
{
	Main->m_log.Add("Unloading templates");
	for ( int i = 0; i < m_aTemplates.GetSize(); i++ )
	{
		CSObject * pTemplate = (CSObject *) m_aTemplates.GetAt(i);
		if ( pTemplate )
			delete pTemplate;
	}
	m_aTemplates.RemoveAll();
	m_bTemplatesLoaded = false;
}

void CScriptObjects::ReloadTemplates()
{
	UnloadTemplates();
	LoadTemplates();
}

bool CScriptObjects::SaveTemplates()
{
	Main->m_log.Add("Saving templates");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_TEMPLATE_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("unable to open temporary file while saving templates");
		AfxMessageBox("Unable to open temporary file while saving templates.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, USER_TEMPLATE_FILE);

	for ( int i = 0; i < this->m_aTemplates.GetSize(); i++ )
	{
		CTemplate * pTemplate = (CTemplate *) m_aTemplates.GetAt(i);
		if ( pTemplate && pTemplate->m_bCustom )
			pTemplate->Write(&csfOutput);
	}

	for ( i = 0; i < this->m_aTemplates2.GetSize(); i++ )
	{
		CTemplate * pTemplate = (CTemplate *) m_aTemplates2.GetAt(i);
		if ( pTemplate && pTemplate->m_bCustom )
			pTemplate->Write(&csfOutput);
	}
	csfOutput.Close();
	return UpdateFiles( TMP_TEMPLATE_FILE, USER_TEMPLATE_FILE, RMT_TEMPLATE_FILE, "graytemplate" );
}

// TABLES
void CScriptObjects::LoadTables()
{
	/* All of this is done automatically when the application starts
	if ( m_bTablesLoaded )
		return;
	if ( !this->m_bDefsLoaded )
		this->LoadDefs();
	CreateProgressDialog();
	this->LoadFile(Main->GetProfileType() == PROFILE_REMOTE ? RMT_TABLE_FILE : TABLE_FILE, &m_pDlg);
	*/
	m_bTablesLoaded = true;
}

bool CScriptObjects::SaveTables()
{
	return m_tables.Write();
}

// MAP
void CScriptObjects::LoadMap()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bMapLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading map");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	m_pDlg->m_csMessage.SetWindowText("Initializing storage");

	if (!m_bDefsLoaded)
		LoadDefs();
	if (!m_bTriggersLoaded)
		LoadTriggers();

#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_MAP_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("graymap");
		}
	}
#endif
	CString csMessage;
	CString csFilename;
	DWORD j = 0;

	// Read the graymap file
	CStdioFile csfMapfile;
	for ( int i = 0; i < 2; i++ )
	{
		switch ( i )
		{
		case 0:
			// This is the regular file
			if ( Main->GetProfileType() == PROFILE_REMOTE || csScpPath == "" )
				csFilename = MAP_FILE;
			else
				csFilename.Format("%s%s", csScpPath, MAP_FILE);
			break;
		case 1:
			// This is the custom file
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_MAP_FILE);
			else if ( csScpPath == "" )
				csFilename = USER_MAP_FILE;
			else
				csFilename.Format("%s%s", csScpPath, USER_MAP_FILE);
			break;
		}

		if (!csfMapfile.Open(csFilename, CFile::modeRead | CFile::shareDenyNone))
		{
			if ( csFilename.Find("\\") == -1 )
			{
				CString csTemp = csFilename;
				char szPath[MAX_PATH];
				memset(&szPath[0], 0x00, sizeof(szPath) );
				getcwd(&szPath[0], sizeof(szPath));
				csFilename.Format("%s\\%s", szPath, csFilename);
			}
			Main->m_log.Add("Unable to open file %s", csFilename);
			break;
		}
		csMessage.Format("Reading %s", csFilename);
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
		BOOL bStatus = true;
		DWORD dwFileLength = csfMapfile.GetLength();
		int scalefactor = dwFileLength / 0xefff;
		scalefactor += 1;
		m_pDlg->SetRange(0, (short)(dwFileLength / scalefactor));
		while (bStatus)
		{
			CString csLine;
			bStatus = csfMapfile.ReadString(csLine);
			if (bStatus)
			{
				if (csLine.Find("[AREA") == 0)
				{
					if (j++ % 10 == 0)
						m_pDlg->SetPos(csfMapfile.Seek(0, CFile::current) / scalefactor);
					// We've entered a new area
					CArea * pArea = new(CArea);
					pArea->m_bCustom = i ? true : false;
					pArea->m_csName = csLine.Mid(csLine.Find(" ") + 1);
					pArea->m_csName = pArea->m_csName.SpanExcluding("]");
					pArea->m_csDescription = pArea->m_csName;
					pArea->ReadFromFile(csfMapfile);
					if ( this->m_olAreas.IsEmpty() )
						this->m_olAreas.AddHead( pArea );
					else
						this->m_olAreas.AddTail( pArea );
				}
				if (csLine.Find("[ROOM") == 0)
				{
					if (j++ % 10 == 0)
						m_pDlg->SetPos(csfMapfile.Seek(0, CFile::current) / scalefactor);
					// We've entered a new room
					CRoom * pRoom = new(CRoom);
					pRoom->m_bCustom = i ? true : false;
					pRoom->m_csName = csLine.Mid(csLine.Find(" ") + 1);
					pRoom->m_csName = pRoom->m_csName.SpanExcluding("]");
					pRoom->m_csDescription = pRoom->m_csName;
					pRoom->ReadFromFile(csfMapfile);
					if ( this->m_olRooms.IsEmpty() )
						this->m_olRooms.AddHead( pRoom );
					else
						this->m_olRooms.AddTail( pRoom );
				}
				if (csLine.Find("[LOCATION") == 0 )
				{
					if (j++ % 10 == 0)
						m_pDlg->SetPos(csfMapfile.Seek(0, CFile::current) / scalefactor);
					CLocation * pLoc = new (CLocation);
					pLoc->m_bCustom = i ? true : false;
					pLoc->Read(csfMapfile);
					this->m_aLocations.Add(pLoc);
				}
			}
		}
		csfMapfile.Close();
	}
	m_bMapLoaded = true;
}

bool CScriptObjects::SaveMap()
{
	Main->m_log.Add("Saving map");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_MAP_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temporary file while saving map");
		AfxMessageBox("Unable to open temporary file while saving map changes.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, USER_MAP_FILE);
	
	POSITION pos = m_olAreas.GetHeadPosition();
	while ( pos )
	{
		CArea * pArea = (CArea *) m_olAreas.GetNext(pos);
		if ( pArea && pArea->m_bCustom )
			pArea->Write(csfOutput);
	}

	pos = m_olRooms.GetHeadPosition();
	while ( pos )
	{
		CArea * pRoom = (CRoom *) m_olRooms.GetNext(pos);
		if ( pRoom && pRoom->m_bCustom )
			pRoom->Write(csfOutput);
	}

	csfOutput.Close();
	return UpdateFiles( TMP_MAP_FILE, USER_MAP_FILE, RMT_MAP_FILE, "graymap" );
}

// TRIGGERS
void CScriptObjects::LoadTriggers()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bTriggersLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading Triggers");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	m_pDlg->m_csMessage.SetWindowText("Initializing storage");

#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_TRIGGER_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("graytrig");
		}
	}
#endif
	CString csMessage;
	CString csFilename;
	DWORD j = 0;

	// Read the trigger file
	CStdioFile csfTrigfile;
	for ( int i = 0; i < 2; i++ )
	{
		switch ( i )
		{
		case 0:
			// This is the regular file
			if ( Main->GetProfileType() == PROFILE_REMOTE || csScpPath == "" )
				csFilename = TRIGGER_FILE;
			else
				csFilename.Format("%s%s", csScpPath, TRIGGER_FILE);
			break;
		case 1:
			// This is the custom file
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_TRIGGER_FILE);
			else if ( csScpPath == "" )
				csFilename = Main->GetProfileType() == PROFILE_REMOTE ? RMT_TRIGGER_FILE : USER_TRIGGER_FILE;
			else
				csFilename.Format("%s%s", csScpPath, USER_TRIGGER_FILE);
			break;
		}

		if (!csfTrigfile.Open(csFilename, CFile::modeRead | CFile::shareDenyNone))
		{
			if ( csFilename.Find("\\") == -1 )
			{
				CString csTemp = csFilename;
				char szPath[MAX_PATH];
				memset(&szPath[0], 0x00, sizeof(szPath) );
				getcwd(&szPath[0], sizeof(szPath));
				csFilename.Format("%s\\%s", szPath, csFilename);
			}
			Main->m_log.Add("Unable to open file %s", csFilename);
			break;
		}
		csMessage.Format("Reading %s", csFilename);
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
		BOOL bStatus = true;
		DWORD dwFileLength = csfTrigfile.GetLength();
		int scalefactor = dwFileLength / 0xefff;
		scalefactor += 1;
		m_pDlg->SetRange(0, (short)(dwFileLength / scalefactor));
		while (bStatus)
		{
			CString csLine;
			DWORD dwOffset = csfTrigfile.GetPosition();
			bStatus = csfTrigfile.ReadString(csLine);
			if (!bStatus)
				break;
			if (j++ % 50 == 0)
				m_pDlg->SetPos(csfTrigfile.Seek(0, CFile::current) / scalefactor);
			if ( csLine.Find("[") == 0 )
			{
				CString csTable = csLine.Mid(1);
				csTable = csTable.SpanExcluding("]");
				CTObject * pTable = new CTObject;
				pTable->Read(csfTrigfile);
				pTable->m_csValue = csTable;
				pTable->m_dwFileOffset = dwOffset;
				pTable->m_csFilename = csFilename;
				CString csKey = csTable.SpanExcluding(" ");
				CString csValue = csTable.Mid(csTable.Find(" "));
				int index = atoi(csValue);
				switch (FindTable(csKey, TriggerKeys, 3))
				{
				case 0:
					{
						if ( m_aPlotTrigs.GetUpperBound() < index )
							m_aPlotTrigs.SetAtGrow(index, pTable);
						else
						{
							CStringBlock * pOldTable = (CStringBlock *) m_aPlotTrigs.GetAt(index);
							if ( pOldTable )
								delete pOldTable;
							m_aPlotTrigs.SetAt(index, pTable);
						}
					}
					break;
				case 1:
					{
						if ( m_aRegionTrigs.GetUpperBound() < index )
							m_aRegionTrigs.SetAtGrow(index, pTable);
						else
						{
							CStringBlock * pOldTable = (CStringBlock *) m_aRegionTrigs.GetAt(index);
							if ( pOldTable )
								delete pOldTable;
							m_aRegionTrigs.SetAt(index, pTable);
						}
					}
					break;
				case 2:
					{
						if ( m_aTriggers.GetUpperBound() < index )
							m_aTriggers.SetAtGrow(index, pTable);
						else
						{
							CStringBlock * pOldTable = (CStringBlock *) m_aTriggers.GetAt(index);
							if ( pOldTable )
								delete pOldTable;
							m_aTriggers.SetAt(index, pTable);
						}
					}
					break;
				default:
					delete pTable;
					break;
				}
			}
		}
		csfTrigfile.Close();
	}
	
	m_bTriggersLoaded = true;
}

bool CScriptObjects::SaveTriggers()
{
	return true;
}

// LOCATIONS
void CScriptObjects::LoadLocations()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bLocationsLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading Locations");
	CWaitCursor hourglass;

	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	m_pDlg->m_csMessage.SetWindowText("Initializing storage");

	CString csMessage;
	DWORD j = 0;

	for (int i = 0; i < 2; i++)
	{
		CStdioFile csfLocs;
		if ( !csfLocs.Open( i ? USER_LOC_FILE : LOC_FILE, CFile::modeRead | CFile::shareDenyNone) )
		{
			Main->m_log.Add("Unable to open file %s", i ? USER_LOC_FILE : LOC_FILE );
			break;
		}
		csMessage.Format("Reading %s", i ? USER_LOC_FILE : LOC_FILE );
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
		m_pDlg->SetRange32(0, csfLocs.GetLength());
		BOOL bStatus = TRUE;
		while ( bStatus )
		{
			CString csLine;
			bStatus = csfLocs.ReadString(csLine);
			if ( !bStatus )
				break;
			csLine = csLine.SpanExcluding("//");
			if ( csLine == "" )
				continue;
			m_pDlg->SetPos(csfLocs.GetPosition());
			if ( csLine.Find("[LOCATION") != -1 || csLine.Find("[CUSTOM LOC") != -1)
			{
				CLocation * pLoc = new (CLocation);
				if ( !pLoc )
					break;
				pLoc->m_bCustom = i ? true : false;
				pLoc->Read(csfLocs);
				this->m_aLocations.Add(pLoc);
			}
		}
		csfLocs.Close();
	}

	m_bLocationsLoaded = true;
}

bool CScriptObjects::SaveLocations()
{
#ifdef _AXIS
	Main->m_log.Add("Saving locations");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_USER_LOC_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temporary file while saving locations");
		AfxMessageBox("Unable to open temporary file while saving custom locations.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	CString csLine;
	csLine.Format("// %s created by %s\n", USER_LOC_FILE, Main->GetVersionTitle());
	csfOutput.WriteString(csLine);
	CTime time;
	time = CTime::GetCurrentTime();
	csLine.Format("// Last modified ");
	csfOutput.WriteString(csLine);
	csLine = time.Format("%B %d, %Y at %H:%M:%S\n\n");
	csfOutput.WriteString(csLine);

	for ( int i = 0; i < this->m_aLocations.GetSize(); i++ )
	{
		CLocation * pLoc = (CLocation *) this->m_aLocations.GetAt(i);
		if ( !pLoc || !pLoc->m_bCustom )
			continue;
		csfOutput.WriteString("[LOCATION]\n");
		csLine.Format("CATEGORY=%s\n", pLoc->m_csCategory);
		csfOutput.WriteString(csLine);
		csLine.Format("SUBSECTION=%s\n", pLoc->m_csSubsection);
		csfOutput.WriteString(csLine);
		csLine.Format("DESCRIPTION=%s\n", pLoc->m_csDescription);
		csfOutput.WriteString(csLine);
		csLine.Format("POINT=%s\n\n", pLoc->m_csPoint);
		csfOutput.WriteString(csLine);
	}
	csfOutput.Close();
	ReplaceFiles(TMP_USER_LOC_FILE, USER_LOC_FILE);
#endif
	return true;
}

// NAMES
void CScriptObjects::LoadNames()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bNamesLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading names");
	CWaitCursor hourglass;

	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	m_pDlg->m_csMessage.SetWindowText("Initializing storage");

#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_NAME_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("grayname");
		}
	}
#endif
	CString csMessage;
	CString csFilename;
	DWORD j = 0;

	CStdioFile csfNames;
	if ( Main->GetProfileType() == PROFILE_REMOTE )
		csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_NAME_FILE);
	else if ( csScpPath == "" )
		csFilename = NAME_FILE;
	else
		csFilename.Format("%s%s", csScpPath, NAME_FILE);
	if ( !csfNames.Open(csFilename, CFile::modeRead | CFile::shareDenyNone) )
	{
		if ( csFilename.Find("\\") == -1 )
		{
			CString csTemp = csFilename;
			char szPath[MAX_PATH];
			memset(&szPath[0], 0x00, sizeof(szPath) );
			getcwd(&szPath[0], sizeof(szPath));
			csFilename.Format("%s\\%s", szPath, csFilename);
		}
		Main->m_log.Add("Unable to open file %s", csFilename);
		return;
	}

	csMessage.Format("Reading %s", csFilename);
	m_pDlg->m_csMessage.SetWindowText(csMessage);
	m_pDlg->SetPos(0);
	m_pDlg->SetRange32(0, csfNames.GetLength());

	BOOL bStatus = TRUE;
	while (bStatus)
	{
		CString csLine;
		DWORD dwOffset = csfNames.GetPosition();
		bStatus = csfNames.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			CTObject * pBlock = new (CTObject);
			if ( pBlock )
			{
				//pBlock->m_bIgnoreComments = true;
				//pBlock->m_bStringsOnly = true;
				//pBlock->m_bIgnoreBlanks = true;
				//pBlock->m_csID = csLine.Mid(csLine.Find("[") + 1);
				//pBlock->m_csID = pBlock->m_csID.SpanExcluding("]");
				CString csID = csLine.Mid(csLine.Find("[") + 1);
				csID =csID.SpanExcluding("]");
				pBlock->m_csValue = csID;
				pBlock->m_dwFileOffset = dwOffset;
				pBlock->m_csFilename = csFilename;
				bStatus = pBlock->Read(csfNames);
				this->m_aNames.Add(pBlock);
				m_pDlg->SetPos(csfNames.GetPosition());
			}
		}
	}
	csfNames.Close();
	this->m_bNamesLoaded = true;
}

bool CScriptObjects::SaveNames()
{
	Main->m_log.Add("Saving names");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_NAME_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temporary file while saving names");
		AfxMessageBox("Unable to open temporary file while saving names.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, NAME_FILE);

	for ( int i = 0; i < this->m_aNames.GetSize(); i++ )
	{
		CStringBlock * pBlock = (CStringBlock *) this->m_aNames.GetAt(i);
		if ( pBlock )
		{
			pBlock->m_bWriteStringCount = true;
			pBlock->Write(csfOutput);
		}
	}
	csfOutput.Close();
	return UpdateFiles( TMP_NAME_FILE, NAME_FILE, RMT_NAME_FILE, "grayname" );
}

void CScriptObjects::UnloadNames()
{
	Main->m_log.Add("Unloading names");
	this->m_bNamesLoaded = false;
	for ( int i = 0; i < this->m_aNames.GetSize(); i++ )
	{
		CStringBlock * pBlock = (CStringBlock *) m_aNames.GetAt(i);
		if ( pBlock )
			delete pBlock;
	}
	m_aNames.RemoveAll();
	m_bNamesLoaded = false;
}

void CScriptObjects::ReloadNames()
{
	this->UnloadNames();
	this->LoadNames();
}

// SPEECH
void CScriptObjects::LoadSpeechBlocks()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	if ( m_bSpeechLoaded )
		return;
	if ( m_bNewFormat )
		return;
	Main->m_log.Add("Loading speech");
	CWaitCursor hourglass;

	CString csScpPath = GetScriptPath();
	CreateProgressDialog();
	m_pDlg->m_csMessage.SetWindowText("Initializing storage");

#ifdef _AXIS
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_SPEECH_FILE, st) )
		{
			if (m_pRemote->IsValid())
				m_pRemote->GetCustomFile("grayspee");
		}
	}
#endif
	CString csMessage;
	CString csFilename;
	DWORD j = 0;

	CStdioFile csfSpeech;
	if ( Main->GetProfileType() == PROFILE_REMOTE )
		csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_SPEECH_FILE);
	else if ( csScpPath == "" )
		csFilename = SPEECH_FILE;
	else
		csFilename.Format("%s%s", csScpPath, SPEECH_FILE);
	if ( !csfSpeech.Open(csFilename, CFile::modeRead | CFile::shareDenyNone) )
	{
		if ( csFilename.Find("\\") == -1 )
		{
			CString csTemp = csFilename;
			char szPath[MAX_PATH];
			memset(&szPath[0], 0x00, sizeof(szPath) );
			getcwd(&szPath[0], sizeof(szPath));
			csFilename.Format("%s\\%s", szPath, csFilename);
		}
		Main->m_log.Add("Unable to open file %s", csFilename);
		return;
	}

	csMessage.Format("Reading %s", csFilename);
	m_pDlg->m_csMessage.SetWindowText(csMessage);
	m_pDlg->SetPos(0);
	m_pDlg->SetRange32(0, csfSpeech.GetLength());
	
	BOOL bStatus = TRUE;
	while (bStatus)
	{
		CString csLine;
		DWORD dwOffset = csfSpeech.GetPosition();
		bStatus = csfSpeech.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			CTObject * pBlock = new (CTObject);
			if ( pBlock )
			{
				CString csID = csLine.Mid(csLine.Find("[") + 1);
				csID =csID.SpanExcluding("]");
				pBlock->m_csValue = csID;
				pBlock->m_dwFileOffset = dwOffset;
				pBlock->m_csFilename = csFilename;
				bStatus = pBlock->Read(csfSpeech);
				this->m_aSpeechBlocks.Add(pBlock);
				m_pDlg->SetPos(csfSpeech.GetPosition());
			}
		}
	}
	csfSpeech.Close();
	this->m_bSpeechLoaded = true;
}

bool CScriptObjects::SaveSpeechBlocks()
{
	Main->m_log.Add("Saving speech");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_SPEECH_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temporary file while saving speech");
		AfxMessageBox("Unable to open temporary file while saving speech.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, SPEECH_FILE);

	for ( int i = 0; i < this->m_aNames.GetSize(); i++ )
	{
		CStringBlock * pBlock = (CStringBlock *) this->m_aNames.GetAt(i);
		if ( pBlock )
			pBlock->Write(csfOutput);
	}
	csfOutput.Close();
	return UpdateFiles( TMP_NAME_FILE, NAME_FILE, RMT_NAME_FILE, "grayname" );
}

void CScriptObjects::UnloadTables()
{
	m_bTablesLoaded = false;
	this->m_tables.Unload();
}

void CScriptObjects::ReloadTables()
{
	if ( m_bNewFormat )
		return;
	this->UnloadTables();
	this->LoadTables();
}

void CScriptObjects::CategorizeItems()
{
	bool bDestroy = false;

	if ( m_bItemsCategorized )
		return;
	if ( !m_bItemsLoaded )
		LoadItems();
	if ( !m_bTemplatesLoaded )
		LoadTemplates();

	CreateProgressDialog();

	Main->m_log.Add("Categorizing Items");
	CWaitCursor hourglass;
	m_pDlg->SetRange(0, m_aItems.GetSize());
	m_pDlg->SetPos(0);
	m_pDlg->SetWindowText("Loading");
	m_pDlg->m_csMessage.SetWindowText("Categorizing items");

	m_wNextItemID = 0x6000;

	// Go through the item array and organize it into categories.
	for ( int i = 0; i < m_aItems.GetSize(); i++ )
	{
		if ( i % 0x100 )
			m_pDlg->SetPos(i);
		CSObject * pItem = (CSObject *) m_aItems.GetAt(i);
		if ( pItem && i >= m_wNextItemID )
			m_wNextItemID = i + 1;
		if ( pItem && !pItem->m_bNoIndex )
		{
			if ( pItem->m_csDupeItem != "" )
			{
				CString csDupe;
				if ( !this->m_bNewFormat )
					csDupe.Format("%04x", ahextoi(pItem->m_csDupeItem));
				else
					csDupe.Format("%s", pItem->m_csDupeItem);
				int iIndex = m_aItems.Find(csDupe);
				if ( iIndex == -1 )
				{
					// Look in the defs
					iIndex = m_aDefs.Find(csDupe);
					if ( iIndex != -1 )
					{
						CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(iIndex);
						if ( pDef )
						{
							csDupe.Format("0%x", pDef->Evaluate() );
							iIndex = m_aItems.Find(csDupe);
						}
					}
				}
				if ( iIndex != -1 )
				{
					CSObject * pDupe = (CSObject *) m_aItems.GetAt(iIndex);
					if ( pDupe->m_bNoIndex )
						continue;
					if ( pItem->m_csCategory == "<none>" )
						pItem->m_csCategory = pDupe->m_csCategory;
					if ( pItem->m_csSubsection == "<none>" )
						pItem->m_csSubsection = pDupe->m_csSubsection;
					if ( pItem->m_csDescription.Find("<unnamed>") != -1 )
						pItem->m_csDescription = pDupe->m_csDescription;
				}
			}
			if ( pItem->m_csDescription.Find("@") != -1 )
				pItem->m_csDescription.Replace("@", pItem->m_csSubsection);
			if ( pItem->m_csCategory == "<none>" )
				pItem->m_csCategory = "<uncategorized items>";
			if ( pItem->m_csSubsection == "<none>" )
				pItem->m_csSubsection = pItem->m_csFilename;
			if ( pItem->m_csDescription == "<unnamed>" )
				pItem->m_csDescription = pItem->m_csValue;
			CCategory * pCategory = FindCategory(&m_olItems, pItem->m_csCategory);
			CSubsection * pSubsection = FindSubsection(pCategory, pItem->m_csSubsection);
			if (pSubsection->m_ItemList.IsEmpty())
				pSubsection->m_ItemList.AddHead(pItem);
			else
				pSubsection->m_ItemList.AddTail(pItem);
		}
	}
	// Now go through the templates
	m_pDlg->SetRange(0, m_aTemplates.GetSize());
	m_pDlg->SetPos(0);
	m_pDlg->SetWindowText("Loading");
	m_pDlg->m_csMessage.SetWindowText("Categorizing templates");
	for ( i = 0; i < m_aTemplates.GetSize(); i++ )
	{
		if ( i % 0x100 )
			m_pDlg->SetPos(i);
		CSObject * pTemplate = (CSObject *) m_aTemplates.GetAt(i);
		if ( pTemplate->m_bNoIndex )
			continue;
		if ( pTemplate->m_csCategory == "<none>" )
			pTemplate->m_csCategory = "<uncategorized templates>";
		if ( pTemplate->m_csSubsection == "<none>" )
			pTemplate->m_csSubsection = pTemplate->m_csFilename;
		if ( pTemplate->m_csDescription == "<unnamed>" )
			pTemplate->m_csDescription = pTemplate->m_csValue;
		CCategory * pCategory = FindCategory(&m_olItems, pTemplate->m_csCategory);
		CSubsection * pSubsection = FindSubsection(pCategory, pTemplate->m_csSubsection);
		if ( pSubsection->m_ItemList.IsEmpty())
			pSubsection->m_ItemList.AddHead(pTemplate);
		else
			pSubsection->m_ItemList.AddTail(pTemplate);
	}
	m_bItemsCategorized = true;
	m_iICatSeq++;
	DestroyProgressDialog();
}

void CScriptObjects::CategorizeNPCs()
{
	bool bDestroy = false;

	if ( m_bNPCsCategorized )
		return;
	if ( !m_bNPCsLoaded )
		LoadNPCs();
	if ( !m_bTablesLoaded )
		LoadTables();

	CreateProgressDialog();
	RemoveNPCCategories();

	Main->m_log.Add("Categorizing NPCs");
	m_pDlg->SetRange32(0, m_aNPCs.GetSize());
	m_pDlg->SetPos(0);
	m_pDlg->SetWindowText("Loading");
	m_pDlg->m_csMessage.SetWindowText("Categorizing NPCs");

	m_wNextNPCID = 0x4000;

	for ( int i = 0; i < m_aNPCs.GetSize(); i++ )
	{
		if ( i % 100 )
			m_pDlg->SetPos(i);
		CSObject * pChar = (CSObject *) m_aNPCs.GetAt(i);
		if ( pChar && i >= m_wNextNPCID )
			m_wNextNPCID = i + 1;
		if ( pChar && !pChar->m_bNoIndex )
		{
			if ( pChar->m_csDescription.Find("@") != -1 )
				pChar->m_csDescription.Replace("@", pChar->m_csSubsection);
			if ( pChar->m_csCategory == "<none>" )
				pChar->m_csCategory = "<uncategorized chars>";
			if ( pChar->m_csSubsection == "<none>" )
				pChar->m_csSubsection = pChar->m_csFilename;
			if ( pChar->m_csDescription == "<unnamed>" )
				pChar->m_csDescription = pChar->m_csValue;
			CCategory * pCategory = FindCategory(&m_olNPCs, pChar->m_csCategory);
			CSubsection * pSubsection = FindSubsection(pCategory, pChar->m_csSubsection);
			if (pSubsection->m_ItemList.IsEmpty())
				pSubsection->m_ItemList.AddHead((CSObject *) pChar);
			else
				pSubsection->m_ItemList.AddTail((CSObject *) pChar);
		}
	}

	m_pDlg->SetRange32(0, m_tables.m_spawns.GetSize());
	m_pDlg->SetPos(0);
	m_pDlg->m_csMessage.SetWindowText("Categorizing Spawn Groups");

	for ( i = 0; i < m_tables.m_spawns.GetSize(); i++ )
	{
		m_pDlg->SetPos(i);
		CSObject * pSpawn = (CSObject *) m_tables.m_spawns.GetAt(i);
		if ( pSpawn )
		{
			if ( pSpawn->m_csDescription.Find("@") != -1 )
				pSpawn->m_csDescription.Replace("@", pSpawn->m_csSubsection);
			if ( pSpawn->m_csCategory == "<none>" )
				pSpawn->m_csCategory = "<uncategorized spawns>";
			if ( pSpawn->m_csSubsection == "<none>" )
				pSpawn->m_csSubsection = pSpawn->m_csFilename;
			if ( pSpawn->m_csDescription == "<unnamed>" )
				pSpawn->m_csDescription = pSpawn->m_csValue;
			CCategory * pCategory = FindCategory(&m_olNPCs, pSpawn->m_csCategory);
			CSubsection * pSubsection = FindSubsection(pCategory, pSpawn->m_csSubsection);
			if ( pSubsection->m_ItemList.IsEmpty())
				pSubsection->m_ItemList.AddHead(pSpawn);
			else
				pSubsection->m_ItemList.AddTail(pSpawn);
		}
	}
	m_iNCatSeq++;
	m_bNPCsCategorized = true;
	DestroyProgressDialog();
}

void CScriptObjects::CategorizeLocations()
{
	bool bDestroy = false;

	if ( m_bLocationsCategorized )
		return;
	if ( !m_bMapLoaded )
		LoadMap();
	if ( !m_bLocationsLoaded )
		LoadLocations();

	CreateProgressDialog();

	Main->m_log.Add("Categorizing Locations");
	POSITION pos = m_olAreas.GetHeadPosition();
	while (pos)
	{
		CArea * pArea = (CArea *) m_olAreas.GetNext(pos);
		if ( pArea && pArea->m_csPoint != "" && pArea->m_csGroup != "" && pArea->m_csPoint != "")
		{
			CCategory * pCategory = FindCategory(&m_olLocations, "Map Regions");
			CSubsection * pSubsection = FindSubsection(pCategory, pArea->m_csGroup);
			if (pSubsection->m_ItemList.IsEmpty())
				pSubsection->m_ItemList.AddHead(pArea);
			else
				pSubsection->m_ItemList.AddTail(pArea);
		}
	}
	pos = m_olRooms.GetHeadPosition();
	while (pos)
	{
		CRoom * pRoom = (CRoom *) m_olRooms.GetNext(pos);
		if ( pRoom && pRoom->m_csPoint != "" && pRoom->m_csGroup != "" && pRoom->m_csPoint != "" )
		{
			CCategory * pCategory = FindCategory(&m_olLocations, "Map Regions");
			CSubsection * pSubsection = FindSubsection(pCategory, pRoom->m_csGroup);
			if (pSubsection->m_ItemList.IsEmpty())
				pSubsection->m_ItemList.AddHead(pRoom);
			else
				pSubsection->m_ItemList.AddTail(pRoom);
		}
	}
	for ( int i = 0; i < m_aLocations.GetSize(); i++ )
	{
		CLocation * pLoc = (CLocation *) m_aLocations.GetAt(i);
		if ( pLoc )
		{
			CCategory * pCategory = FindCategory(&m_olLocations, pLoc->m_csCategory);
			CSubsection * pSubsection = FindSubsection(pCategory, pLoc->m_csSubsection);
			if (pSubsection->m_ItemList.IsEmpty())
				pSubsection->m_ItemList.AddHead(pLoc);
			else
				pSubsection->m_ItemList.AddTail(pLoc);
		}
	}
	m_iMCatSeq++;
	m_bLocationsCategorized = true;
	DestroyProgressDialog();

}

void CScriptObjects::RemoveItemCategories()
{
	Main->m_log.Add("Clearing item categories");
	if (!m_olItems.IsEmpty())
	{
		POSITION catPos = m_olItems.GetHeadPosition();
		while (catPos != NULL)
		{
			CCategory * pCat = (CCategory *) m_olItems.GetNext(catPos);
			if (! pCat->m_SubsectionList.IsEmpty())
			{
				POSITION subPos = pCat->m_SubsectionList.GetHeadPosition();
				while (subPos != NULL)
				{
					CSubsection *pSub = (CSubsection *) pCat->m_SubsectionList.GetNext(subPos);
					delete (pSub);
				}
			}
			delete (pCat);
		}
	}
	m_olItems.RemoveAll();
	m_bItemsCategorized = false;
}

void CScriptObjects::RemoveNPCCategories()
{
	Main->m_log.Add("Clearing NPC categories");
	if (!m_olNPCs.IsEmpty())
	{
		POSITION catPos = m_olNPCs.GetHeadPosition();
		while (catPos != NULL)
		{
			CCategory * pCat = (CCategory *) m_olNPCs.GetNext(catPos);
			if (! pCat->m_SubsectionList.IsEmpty())
			{
				POSITION subPos = pCat->m_SubsectionList.GetHeadPosition();
				while (subPos != NULL)
				{
					CSubsection *pSub = (CSubsection *) pCat->m_SubsectionList.GetNext(subPos);
					delete (pSub);
				}
			}
			delete (pCat);
		}
	}
	m_olNPCs.RemoveAll();
	m_bNPCsCategorized = false;
}

void CScriptObjects::RemoveLocationCategories()
{
	Main->m_log.Add("Clearing location categories");
	if (!m_olLocations.IsEmpty())
	{
		POSITION catPos = m_olLocations.GetHeadPosition();
		while (catPos != NULL)
		{
			CCategory * pCat = (CCategory *) m_olLocations.GetNext(catPos);
			if (! pCat->m_SubsectionList.IsEmpty())
			{
				POSITION subPos = pCat->m_SubsectionList.GetHeadPosition();
				while (subPos != NULL)
				{
					CSubsection *pSub = (CSubsection *) pCat->m_SubsectionList.GetNext(subPos);
					delete (pSub);
				}
			}
			delete (pCat);
		}
	}
	m_olLocations.RemoveAll();
	m_bLocationsCategorized = false;
}

void CScriptObjects::CleanCategories(CPtrList &olCategories)
{
	if ( !olCategories.IsEmpty() )
	{
		POSITION catPos = m_olLocations.GetHeadPosition();
		while ( catPos )
		{
			CCategory * pCat = (CCategory *) olCategories.GetNext(catPos);
			if ( !pCat )
				continue;
			POSITION subPos = pCat->m_SubsectionList.GetHeadPosition();
			while ( subPos )
			{
				CSubsection * pSub = (CSubsection *) pCat->m_SubsectionList.GetNext(subPos);
				if ( pSub && pSub->m_ItemList.IsEmpty() )
				{
					pCat->m_SubsectionList.RemoveAt(pCat->m_SubsectionList.Find(pSub));
					delete pSub;
				}
			}
			if ( pCat->m_SubsectionList.IsEmpty() )
			{
				olCategories.RemoveAt(olCategories.Find(pCat));
				delete pCat;
			}
		}
	}
}

void CScriptObjects::RecategorizeItems()
{
	if ( !m_bItemsCategorized )
		return;
	RemoveItemCategories();
	CreateProgressDialog();
	CategorizeItems();
}

void CScriptObjects::RecategorizeLocations()
{
	if ( !m_bLocationsCategorized )
		return;
	RemoveLocationCategories();
	CreateProgressDialog();
	CategorizeLocations();
}

void CScriptObjects::RecategorizeNPCs()
{
	if ( !m_bNPCsCategorized )
		return;
	RemoveNPCCategories();
	CreateProgressDialog();
	CategorizeNPCs();
}

enum RES_TYPE	// all the script resource blocks we know how to deal with !
{
	// NOTE: SPHERE.INI, SPHERETABLE.SCP are read at start.
	// All other files are indexed from the SPEECHFILES and SCPFILES directories.
	// (SI) = Single instance types.
	// (SL) = single line multiple definitions.
	// Alphabetical order.
	RES_UNKNOWN = 0,		// Not to be used.
	RES_ACCOUNT,		// Define an account instance.
	RES_ADVANCE,		// Define the advance rates for stats.
	RES_AREA,			// Complex region. (w/extra tags)
	RES_BLOCKEMAIL,		// (SL) A list of email addresses we will not accept.
	RES_BLOCKIP,		// (SL) A list of IP's to block.
	RES_BOOK,			// A book or a page from a book.
	RES_CHARDEF,		// Define a char type. (overlap with RES_SPAWN)
	RES_COMMENT,		// A commented out block type.
	RES_CRYSTALBALL,	// (SL) Random (short) tip type messages.
	RES_DEFNAME,		// (SL) Just add a bunch of new defs and equivs str/values.
	RES_DEFNAMES,		// (SL) Just add a bunch of new defs and equivs str/values.  -- Same as DEFNAME, but for 99+
	RES_DIALOG,			// A scriptable gump dialog, text or handler block.
	RES_ECONOMY,		// Define the prices of a bunch of stuff.
	RES_EMAILMSG,		// define an email msg that could be sent to an account.
	RES_EVENTS,			// An Event handler block with the trigger type in it. ON=@Death etc.
	RES_FUNCTION,		// Define a new command verb script that applies to a char.
	RES_GMPAGE,			// A GM page. (SAVED in World)
	RES_HELP,			// Command help blocks. (read in as needed)
	RES_ITEMDEF,		// Define an item type. (overlap with RES_TEMPLATE)
	RES_LOCATION,		// Ignore this AXIS data.
	RES_MAP,			// Define info about the mapx.mul files
	RES_MENU,			// General scriptable menus.
	RES_MOONGATES,		// (SL) Define where the moongates are.
	RES_NAMES,			// A block of possible names for a NPC type. (read as needed)
	RES_NEWBIE,			// Triggers to execute on Player creation (based on skills selected)
	RES_NOTOTITLES,		// (SI) Define the noto titles used.
	RES_OBSCENE,		// (SL) A list of obscene words.
	RES_PLEVEL,			// Define the list of commands that a PLEVEL can access. (or not access)
	RES_PLOTITEM,		// (Phase this out in favor of better @trigger TEVENT= and EVENT= support.)
	RES_PROFESSION,		// Define specifics for a char with this skill class. (ex. skill caps)  -- used to be SKILLCLASS
	RES_QUEST,
	RES_RACECLASS,		// General race params about creature types. regen rates, etc
	RES_REGIONRESOURCE,	// Define an Ore type.
	RES_REGIONTYPE,		// Triggers etc. that can be assinged to a RES_AREA
	RES_RESOURCES,		// (SL) list of all the resource files we should index !
	RES_ROOM,			// Non-complex region. (no extra tags)
	RES_RUNES,			// (SI) Define list of the magic runes.
	RES_SCHEDULE,		// A list of times things can happen.
	RES_SCROLL,			// SCROLL_GUEST=message scroll sent to player at guest login. SCROLL_MOTD, SCROLL_NEWBIE
	RES_SECTOR,			// Make changes to a sector. (SAVED in World)
	RES_SERVER,			// Define a peer sphere server we should link to. (SAVED in World)
	RES_SERVERS,		// List a number of servers in 3 line format. (Phase this out)
	RES_SKILL,			// Define attributes for a skill (how fast it raises etc)
	RES_SKILLCLASS,		// Define specifics for a char with this skill class. (ex. skill caps)
	RES_SKILLMENU,		// A menu that is attached to a skill. special arguments over other menus.
	RES_SPAWN,			// Define a list of NPC's and how often they may spawn.
	RES_SPEECH,			// A speech block with ON=*blah* in it.
	RES_SPELL,			// Define a magic spell. (0-64 are reserved)
	RES_SPHERE,			// Main Server INI block
	RES_STARTS,			// (SI) List of starting locations for newbies.
	RES_TELEPORTERS,	// (SL) Where are the teleporters in the world ? dungeon transports etc.
	RES_TEMPLATE,		// Define lists of items. (for filling loot etc)
	RES_TIP,			// Tips (similar to RES_SCROLL) that can come up at startup.
	RES_TYPEDEF,			// Define a trigger block for a RES_WORLDITEM m_type.
	RES_TYPEDEFS,
	RES_WEBPAGE,		// Define a web page template.
	RES_WORLDCHAR,		// Define instance of char in the world. (SAVED in World)
	RES_WORLDITEM,		// Define instance of item in the world. (SAVED in World)
	RES_EOF,
	RES_QTY,			// Don't care
}; 

const char * pResourceBlocks[RES_QTY] =	// static
{
	"AAAUNUSED",	// unused / unknown.
    "ACCOUNT",		// Define an account instance.
	"ADVANCE",		// Define the advance rates for stats.
	"AREA",			// Complex region. (w/ extra tags)
	"BLOCKEMAIL",		// (SL) A list of email addresses we will not accept.
	"BLOCKIP",		// (SL) A list of IP's to block.
	"BOOK",			// A book or a page from a book.
	"CHARDEF",		// Define a char type.
	"COMMENT",		// A commented out block type.
	"CRYSTALBALL",	// (SL) Random (short) tip type messages.
	"DEFNAME",		// (SL) Just add a bunch of new defs and equivs str/values.
	"DEFNAMES",		// (SL) Just add a bunch of new defs and equivs str/values. Same as DEFNAME, but for 99+
	"DIALOG",			// A scriptable gump dialog", text or handler block.
	"ECONOMY",		// define the prices of a bunch of stuff.
	"EMAILMSG",		// define an email msg that could be sent to an account.
	"EVENTS",			// (SL) Preload these Event files.
	"FUNCTION",		// Define a new command verb script that applies to a char.
	"GMPAGE",			// A GM page. (SAVED in World)
	"HELP",			// Command help blocks. (read in as needed)
	"ITEMDEF",		// Define an item type
	"LOCATION",		// Ignore this AXIS data.
	"MAP",			// Define info about mapx.mul files.
	"MENU",			// General scriptable menus.
	"MOONGATES",		// (SL) Define where the moongates are.
	"NAMES",		// A block of possible names for a NPC type. (read as needed)
	"NEWBIE",			// Triggers to execute on Player creation (based on skills selected)
	"NOTOTITLES",		// (SI) Define the noto titles used.
	"OBSCENE",		// (SL) A list of obscene words.
	"PLEVEL",			// Define the list of commands that a PLEVEL can access. (or not access)
	"PLOTITEM",		// (Phase this out in favor of better @trigger TEVENT= and EVENT= support.)
	"PROFESSION",		// Define class specifics for a char with this skill class. -- Used to be SKILLCLASS
	"QUEST",			// ability to get accomplishment titles.
	"RACECLASS",		// General race params about creature types. regen rates, etc
	"REGIONRESOURCE",	// Define Ore types.
	"REGIONTYPE",			// Triggers etc. that can be assinged to a "AREA
	"RESOURCES",		// (SL) list of all the resource files we should index !
	"ROOM",			// Non-complex region. (no extra tags)
	"RUNES",			// (SI) Define list of the magic runes.
	"SCHEDULE",			// A list of times things can happen.
	"SCROLL",			// SCROLL_GUEST=message scroll sent to player at guest login. SCROLL_MOTD", SCROLL_NEWBIE
	"SECTOR",			// Make changes to a sector. (SAVED in World)
	"SERVER",			// Define a peer sphere server we should link to. (SAVED in World)
	"SERVERS",		// List a number of servers in 3 line format.
	"SKILL",			// Define attributes for a skill (how fast it raises etc)
	"SKILLCLASS",		// Define class specifics for a char with this skill class.
	"SKILLMENU",		// A menu that is attached to a skill. special arguments over other menus.
	"SPAWN",			// Define a list of NPC's and how often they may spawn.
	"SPEECH",			// (SL) Preload these speech files.
	"SPELL",			// Define a magic spell. (0-64 are reserved)
	"SPHERE",			// Main Server INI block
	"STARTS",			// (SI) List of starting locations for newbies.
	"TELEPORTERS",	// (SL) Where are the teleporteres in the world ?
	"TEMPLATE",		// Define a list of items. (for filling loot etc)
	"TIP",			// Tips that can come up at startup.
	"TYPEDEF",			// Define a trigger block for a "WORLDITEM m_type.
	"TYPEDEFS",
	"WEBPAGE",		// Define a web page template.
	"WORLDCHAR",		// Define instance of char in the world. (SAVED in World)
	"WORLDITEM",		// Define instance of item in the world. (SAVED in World)
	"EOF",
};

bool CScriptObjects::LoadFile(CString csFile)
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return true;
	csFile.Replace('/', '\\');
	CStdioFile csfInput;
	try
	{
		Main->m_log.Add("Attempting to load file %s", csFile);
		if ( !csfInput.Open(csFile, CFile::modeRead | CFile::shareDenyNone) )
		{
			Main->m_log.Add("Unable to open file %s", csFile);
			return false;
		}
		CWaitCursor hourglass;
		
		CString csMessage;
		csMessage.Format("Reading %s", csFile);
		if ( m_pDlg )
		{
			m_pDlg->m_csMessage.SetWindowText(csMessage);
			m_pDlg->SetPos(0);
			m_pDlg->SetRange32(0, csfInput.GetLength());
			// m_pDlg->UpdateData(false);
		}

		int j = 0;
		BOOL bStatus = TRUE;
		while ( bStatus )
		{
			CString csLine;
			DWORD dwOffset = csfInput.GetPosition();
			bStatus = csfInput.ReadString(csLine);
			if ( !bStatus )
				break;
			if ( csLine.Find("[") == 0 )
			{
				j++;
				if ( j % 10 == 0 && m_pDlg)
				{
					m_pDlg->SetPos(csfInput.GetPosition());
					//m_pDlg->UpdateData();
				}
				CString csKey = csLine.Mid(1);
				csKey = csKey.SpanExcluding("]");
				CString csIndex;
				if ( csKey.Find(" ") != -1 )
					csIndex = csKey.Mid(csKey.Find(" ") + 1);
				int iIndex = atoi(csIndex);
				CString csType = csKey.SpanExcluding(" ");
				int resource = FindTable(csType, pResourceBlocks, RES_QTY);
				switch (resource)
				{
				case RES_UNKNOWN:		// Not to be used.
					break;
				case RES_ACCOUNT:		// Define an account instance.
					break;
				case RES_ADVANCE:		// Define the advance rates for stats.
					{
						for ( int i = 0; i < 3; i++ )
						{
							bStatus = csfInput.ReadString(csLine);
							CString csKey = csLine.SpanExcluding("=");
							CString csValue = csLine.Mid(csLine.Find("=") + 1);
							csKey.MakeUpper();
							int index = -1;
							if ( csKey == "STR" )
								index = 0;
							if ( csKey == "INT" )
								index = 1;
							if ( csKey == "DEX" )
								index = 2;
							if ( index == -1 )
								continue;
							this->m_tables.m_statAdvance[index].Load(csValue);
						}
					}
					break;
				case RES_AREA:			// Complex region. (w/extra tags)
					{
						CArea * pArea = new CArea;
						pArea->m_csDescription = csIndex;
						pArea->m_csName = csIndex;
						pArea->m_dwFileOffset = dwOffset;
						pArea->ReadFromFile(csfInput);
						pArea->m_csFilename = csFile;
						if ( this->m_olAreas.IsEmpty() )
							this->m_olAreas.AddHead(pArea);
						else
							this->m_olAreas.AddTail(pArea);
					}
					break;
				case RES_BLOCKEMAIL:		// (SL) A list of email addresses we will not accept.
					this->m_tables.m_blockEmail.m_dwFileOffset = dwOffset;
					this->m_tables.m_blockEmail.Read(csfInput);
					this->m_tables.m_blockEmail.m_csFilename = csFile;
					break;
				case RES_BLOCKIP:		// (SL) A list of IP's to block.
					this->m_tables.m_BlockIP.Read(csfInput);
					this->m_tables.m_BlockIP.m_dwFileOffset = dwOffset;
					this->m_tables.m_BlockIP.m_csFilename = csFile;
					this->m_tables.m_BlockIP.m_bType = TYPE_BLOCKIP;
					break;
				case RES_BOOK:			// A book or a page from a book.
					break;
				case RES_CHARDEF:		// Define a char type. (overlap with RES_SPAWN)
					{
						CSObject * pNPC = new CSObject;
						pNPC->m_bType = TYPE_CHAR;
						pNPC->m_csValue = csIndex;
						pNPC->Read(csfInput);
						pNPC->m_dwFileOffset = dwOffset;
						pNPC->m_csFilename = csFile;
						int iOld = m_aNPCs.Find(pNPC->m_csValue);
						if ( iOld != -1 )
						{
							CSObject * pOld = (CSObject *) m_aNPCs.GetAt(iOld);
							m_aNPCs.RemoveAt(iOld);
							delete pOld;
						}
						m_aNPCs.Insert(pNPC);
					}
					break;
				case RES_COMMENT:		// A commented out block type.
					break;
				case RES_CRYSTALBALL:	// (SL) Random (short) tip type messages.
					break;
				case RES_DEFNAME:		// (SL) Just add a bunch of new defs and equivs str/values.
				case RES_DEFNAMES:		// (SL) Just add a bunch of new defs and equivs str/values.
				case RES_TYPEDEFS:
					{
						while ( bStatus )
						{
							DWORD dwCur = csfInput.GetPosition();
							bStatus = csfInput.ReadString(csLine);
							if ( !bStatus )
								break;
							if ( csLine.Find("[") == 0 )
							{
								csfInput.Seek((LONG)(dwCur - csfInput.GetPosition()), CFile::current);
								break;
							}
							csLine = csLine.SpanExcluding("//");
							csLine.TrimLeft(); csLine.TrimRight();
							if ( csLine != "" )
							{
								CDef * pDef = new CDef;
								pDef->Read(csLine);
								if ( pDef->m_csValue == "" )
								{
									delete pDef;
									continue;
								}
								pDef->m_csGroup = csIndex;
								pDef->m_csFilename = csFile;
								// Does this one already exist?
								int iOld = m_aDefs.Find(pDef->m_csValue);
								if ( iOld != -1 )
								{
									CDef * pOld = (CDef *) m_aDefs.GetAt(iOld);
									m_aDefs.RemoveAt(iOld);
									delete pOld;
								}
								m_aDefs.Insert(pDef);
							}
						}
					}
					break;
				case RES_DIALOG:
					break;
				case RES_ECONOMY:
					break;
				case RES_EMAILMSG:		// define an email msg that could be sent to an account.
					break;
				case RES_EVENTS:			// An Event handler block with the trigger type in it. ON=@Death etc.
					this->m_tables.m_events.Read(csfInput);
					this->m_tables.m_events.m_dwFileOffset = dwOffset;
					this->m_tables.m_events.m_csFilename = csFile;
					break;
				case RES_FUNCTION:
					break;
				case RES_GMPAGE:			// A GM page. (SAVED in World)
					break;
				case RES_HELP:			// Command help blocks. (read in as needed)
					break;
				case RES_ITEMDEF:		// Define an item type. (overlap with RES_TEMPLATE)
					{
						CSObject * pItem = new CSObject;
						pItem->m_bType = TYPE_ITEM;
						pItem->m_csValue = csIndex;
						pItem->Read(csfInput);
						pItem->m_dwFileOffset = dwOffset;
						pItem->m_csFilename = csFile;
						int iOld = m_aItems.Find(pItem->m_csValue);
						if ( iOld != -1 )
						{
							CSObject * pOld = (CSObject *) m_aItems.GetAt(iOld);
							m_aItems.RemoveAt(iOld);
							delete pOld;
						}
						m_aItems.Insert(pItem);
					}
					break;
				case RES_LOCATION:		// Ignore this AXIS data.
					{
						CLocation * pLoc = new CLocation;
						pLoc->Read(csfInput);
						pLoc->m_dwFileOffset = dwOffset;
						pLoc->m_csFilename = csFile;
						this->m_aLocations.Add(pLoc);
						if ( pLoc->m_csFilename.Find("custloc.scp") != -1 )
							pLoc->m_bCustom = true;
					}
					break;
				case RES_MAP:
					break;
				case RES_MENU:			// General scriptable menus. 
					break;
				case RES_MOONGATES:		// (SL) Define where the moongates are.
					break;
				case RES_NAMES:			// A block of possible names for a NPC type. (read as needed)
					{
						CTObject * pBlock = new (CTObject);
						pBlock->m_bType = TYPE_NAME;
						pBlock->m_csValue = csIndex;
						pBlock->m_dwFileOffset = dwOffset;
						pBlock->m_csFilename = csFile;
						this->m_aNames.Add(pBlock);
					}
					break;
				case RES_NEWBIE:			// Triggers to execute on Player creation (based on skills selected)
					break;
				case RES_NOTOTITLES:		// (SI) Define the noto titles used.
					this->m_tables.m_notoTitles.Read(csfInput);
					this->m_tables.m_notoTitles.m_dwFileOffset = dwOffset;
					this->m_tables.m_notoTitles.m_csFilename = csFile;
					break;
				case RES_OBSCENE:		// (SL) A list of obscene words.
					this->m_tables.m_obscene.Read(csfInput);
					this->m_tables.m_obscene.m_dwFileOffset = dwOffset;
					this->m_tables.m_obscene.m_csFilename = csFile;
					break;
				case RES_PLEVEL:			// Define the list of commands that a PLEVEL can access. (or not access)
					this->m_tables.m_pLevel[iIndex - 1].m_csTableName.Format("%s %s", csKey, csIndex);
					this->m_tables.m_pLevel[iIndex - 1].Read(csfInput);
					this->m_tables.m_pLevel[iIndex - 1].m_dwFileOffset = dwOffset;
					this->m_tables.m_pLevel[iIndex - 1].m_csFilename = csFile;
					break;
				case RES_PLOTITEM:		// (Phase this out in favor of better @trigger TEVENT= and EVENT= support.)
					{
						CTObject * pPlotItem = new CTObject;
						pPlotItem->m_csFilename = csFile;
						pPlotItem->m_dwFileOffset = dwOffset;
						pPlotItem->m_csValue = csIndex;
						pPlotItem->m_bType = TYPE_PLOT;
						int iOld = this->m_aPlotTrigs.Find(csIndex);
						if ( iOld != -1 )
						{
							CTObject * pOld = (CTObject *) this->m_aPlotTrigs.GetAt(iOld);
							m_aPlotTrigs.RemoveAt(iOld);
							delete pOld;
						}
						this->m_aPlotTrigs.Insert(pPlotItem);
					}
					break;
				case RES_QUEST:
					break;
				case RES_RACECLASS:
					break;
				case RES_REGIONRESOURCE:
					break;
				case RES_REGIONTYPE:			// Triggers etc. that can be assinged to a RES_AREA
					{
						CTObject * pRegion = new CTObject;
						pRegion->m_dwFileOffset = dwOffset;
						pRegion->m_csFilename = csFile;
						pRegion->m_csValue = csIndex;
						pRegion->m_bType = TYPE_REGION;
						int iOld = this->m_aRegionTrigs.Find(csIndex);
						if ( iOld != -1 )
						{
							CTObject * pOld = (CTObject *) this->m_aRegionTrigs.GetAt(iOld);
							m_aRegionTrigs.RemoveAt(iOld);
							delete pOld;
						}
						this->m_aRegionTrigs.Insert(pRegion);
					}
					break;
				case RES_RESOURCES:		// (SL) list of all the resource files we should index !
					this->m_tables.m_resources.Read(csfInput);
					this->m_tables.m_resources.m_dwFileOffset = dwOffset;
					this->m_tables.m_resources.m_csFilename = csFile;
					break;
				case RES_ROOM:			// Non-complex region. (no extra tags)
					{
						CRoom * pRoom = new CRoom;
						pRoom->m_csDescription = csIndex;
						pRoom->m_csName = csIndex;
						pRoom->m_dwFileOffset = dwOffset;
						pRoom->m_csFilename = csFile;
						pRoom->ReadFromFile(csfInput);
						if ( this->m_olRooms.IsEmpty() )
							this->m_olRooms.AddHead( pRoom );
						else
							this->m_olRooms.AddTail( pRoom );
					}
					break;
				case RES_RUNES:			// (SI) Define list of the magic runes.
					this->m_tables.m_runes.Read(csfInput);
					this->m_tables.m_runes.m_dwFileOffset = dwOffset;
					this->m_tables.m_runes.m_csFilename = csFile;
					break;
				case RES_SCHEDULE:
					break;
				case RES_SCROLL:			// SCROLL_GUEST=message scroll sent to player at guest login. SCROLL_MOTD: SCROLL_NEWBIE
					break;
				case RES_SECTOR:			// Make changes to a sector. (SAVED in World)
					break;
				case RES_SERVER:			// Define a peer sphere server we should link to. (SAVED in World)
					break;
				case RES_SERVERS:		// List a number of servers in 3 line format. (Phase this out)
					this->m_tables.m_Servers.Read(csfInput);
					this->m_tables.m_Servers.m_dwFileOffset = dwOffset;
					this->m_tables.m_Servers.m_csFilename = csFile;
					this->m_tables.m_Servers.m_bType = TYPE_SERVERS;
					break;
				case RES_SKILL:			// Define attributes for a skill (how fast it raises etc)
					{
						CTObject * pSkill = new (CTObject);
						pSkill->m_bType = TYPE_SKILL;
						pSkill->m_dwFileOffset = dwOffset;
						pSkill->m_csFilename = csFile;
						pSkill->m_csValue = csIndex;
						int iOld = this->m_tables.m_skills.Find(csIndex);
						if ( iOld != -1 )
						{
							CTObject * pOld = (CTObject *) m_tables.m_skills.GetAt(iOld);
							m_tables.m_skills.RemoveAt(iOld);
							delete pOld;
						}
						this->m_tables.m_skills.Insert(pSkill);
					}
					break;
				case RES_PROFESSION:
				case RES_SKILLCLASS:		// Define specifics for a char with this skill class. (ex. skill caps)
					{
						CTObject * pSkillClass = new (CTObject);
						pSkillClass->m_bType = TYPE_SKILLCLASS;
						pSkillClass->m_dwFileOffset = dwOffset;
						pSkillClass->m_csFilename = csFile;
						pSkillClass->m_csValue = csIndex;
						int iOld = this->m_tables.m_skillClasses.Find(csIndex);
						if ( iOld != -1 )
						{
							CTObject * pOld = (CTObject *) m_tables.m_skillClasses.GetAt(iOld);
							m_tables.m_skillClasses.RemoveAt(iOld);
							delete pOld;
						}
						this->m_tables.m_skillClasses.Insert(pSkillClass);
					}
					break;
				case RES_SKILLMENU:		// A menu that is attached to a skill. special arguments over other menus.
					break;
				case RES_SPAWN:			// Define a list of NPC's and how often they may spawn. 
					{
						CSObject * pSpawn = new CSObject;
						pSpawn->m_bType = TYPE_SPAWN;
						pSpawn->m_csValue = csIndex;
						pSpawn->Read(csfInput);
						pSpawn->m_dwFileOffset = dwOffset;
						pSpawn->m_csFilename = csFile;
						int iOld = this->m_tables.m_spawns.Find(csIndex);
						if ( iOld != -1 )
						{
							CSObject * pOld = (CSObject *) this->m_tables.m_spawns.GetAt(iOld);
							this->m_tables.m_spawns.RemoveAt(iOld);
							delete pOld;
						}
						this->m_tables.m_spawns.Insert(pSpawn);
					}
					break;
				case RES_SPEECH:			// A speech block with ON=*blah* in it.
					this->m_tables.m_Speech.Read(csfInput);
					this->m_tables.m_Servers.m_dwFileOffset = dwOffset;
					this->m_tables.m_Speech.m_csFilename = csFile;
					break;
				case RES_SPELL:			// Define a magic spell. (0-64 are reserved)
					{
						CTObject * pSpell = new (CTObject);
						pSpell->m_bType = TYPE_SPELL;
						pSpell->m_csValue = csIndex;
						pSpell->m_dwFileOffset = dwOffset;
						pSpell->m_csFilename = csFile;
						int iOld = this->m_tables.m_spells.Find(csIndex);
						if ( iOld != -1 )
						{
							CTObject * pOld = (CTObject *) this->m_tables.m_spells.GetAt(iOld);
							this->m_tables.m_spells.RemoveAt(iOld);
							delete pOld;
						}
						this->m_tables.m_spells.Insert(pSpell);
					}
					break;
				case RES_SPHERE:			// Main Server INI block
					this->m_tables.m_Sphere.Read(csfInput);
					this->m_tables.m_Sphere.m_dwFileOffset = dwOffset;
					this->m_tables.m_Sphere.m_csFilename = csFile;
					this->m_tables.m_Sphere.m_bType = TYPE_SPHERE;
					break;
				case RES_STARTS:			// (SI) List of starting locations for newbies.
					{
						bool bContinue = true;
						while ( bContinue )
						{
							CTStartLocation * pStart = new (CTStartLocation);
							bContinue = pStart->Read(csfInput);
							if ( bContinue )
								this->m_tables.m_starts.Add(pStart);
							else
								delete pStart;
						}
						this->m_tables.m_starts.m_dwFileOffset = dwOffset;
						this->m_tables.m_starts.m_csFilename = csFile;
					}
					break;
				case RES_TELEPORTERS:	// (SL) Where are the teleporters in the world ? dungeon transports etc.
					break;
				case RES_TEMPLATE:		// Define lists of items. (for filling loot etc)
					{
						CSObject * pTemplate = new CSObject;
						pTemplate->m_bType = TYPE_TEMPLATE;
						pTemplate->m_dwFileOffset = dwOffset;
						pTemplate->m_csFilename = csFile;
						pTemplate->m_csValue = csIndex;
						pTemplate->Read(csfInput);
						int iOld = this->m_aTemplates.Find(csIndex);
						if ( iOld != -1 )
						{
							CSObject * pOld = (CSObject *) this->m_aTemplates.GetAt(iOld);
							this->m_aTemplates.RemoveAt(iOld);
							delete pOld;
						}
						this->m_aTemplates.Insert(pTemplate);
					}
					break;
				case RES_TIP:			// Tips (similar to RES_SCROLL) that can come up at startup.
					break;
				case RES_TYPEDEF:			// Define a trigger block for a RES_WORLDITEM m_type.
					{
						CTObject * pTrigger = new CTObject;
						pTrigger->m_bType = TYPE_TRIGGER;
						pTrigger->m_dwFileOffset = dwOffset;
						pTrigger->m_csFilename = csFile;
						pTrigger->m_csValue = csIndex;
						int iOld = this->m_aTriggers.Find(csIndex);
						if ( iOld != -1 )
						{
							CTObject * pOld = (CTObject *) this->m_aTriggers.GetAt(iOld);
							this->m_aTriggers.RemoveAt(iOld);
							delete pOld;
						}
						this->m_aTriggers.Insert(pTrigger);
					}
					break;
				case RES_WEBPAGE:		// Define a web page template.
					{
						CTObject * pBlock = new (CTObject);
						pBlock->m_bType = TYPE_WEBPAGE;
						pBlock->m_csValue = csIndex;
						pBlock->m_dwFileOffset = dwOffset;
						pBlock->m_csFilename = csFile;
						this->m_aWebpages.Add(pBlock);
					}
					break;
				case RES_WORLDCHAR:		// Define instance of char in the world. (SAVED in World)
					break;
				case RES_WORLDITEM:		// Define instance of item in the world. (SAVED in World)
					break;
				case RES_EOF:
					break;
				case RES_QTY:			// Don't care
					break;
				default:
					Main->m_log.Add("Unknown section %s in file %s", csLine, csFile);
					break;
				}
			}
			else
			{
				j++;
				if ( j % 100 == 0 && m_pDlg)
				{
					m_pDlg->SetPos(csfInput.GetPosition());
					//m_pDlg->UpdateData();
				}
			}
		}
	}
	catch (CFileException *e)
	{
		Main->m_log.Add("Caught an exception while reading the file %s.  Cause code = %ld", e->m_strFileName, e->m_cause);
		e->Delete();
	}
	if ( m_pDlg )
		m_pDlg->SetPos(csfInput.GetLength());
	csfInput.Close();
	DestroyProgressDialog();
	return true;
}

void CScriptObjects::LoadAll()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	CString csLoadFile;
	bool bFoundScripts = false;
	char cwd[MAX_PATH];
	memset(&cwd[0], 0x00, MAX_PATH);
	if ( Main->GetProfileType() == PROFILE_LOCAL )
		sprintf(cwd, "%s", Main->m_csRootDirectory);
	if ( cwd[0] == 0x00 )
	{
		int cwdSize = sizeof(cwd);
		getcwd(&cwd[0], cwdSize);
	}
	Main->m_log.Add("Current working directory is %s", cwd);

	CreateProgressDialog();
	m_bDeleteDialog = false;
	if ( Main->GetProfileType() == PROFILE_REMOTE )
	{
		CStringArray csaScripts;
		CString csKey = _T("SOFTWARE\\Menasoft\\GM Tools\\Remote Profiles\\") + Main->GetProfileString();
		Main->GetRegistryMultiSz("Selected Scripts", &csaScripts, HKEY_LOCAL_MACHINE, csKey);
		if ( csaScripts.GetSize() > 0 )
		{
			// Load the specified scripts
			this->m_bNewFormat = true;
			m_pRemote->SetDialog(m_pDlg);
			for ( int i = 0; i <= csaScripts.GetUpperBound(); i++ )
			{
				m_pRemote->UpdateFile(csaScripts.GetAt(i));
				CString csLocalFile;
				csLocalFile.Format("%s\\%s", Main->GetProfileString(), csaScripts.GetAt(i));
				LoadFile(csLocalFile);
			}
			// Categorize everything
			CategorizeLocations();
			CategorizeItems();
			CategorizeNPCs();
			this->m_bDeleteDialog = true;
			DestroyProgressDialog();
			// We're done
			return;
		}
#ifdef _AXIS
		m_pRemote->SetDialog(m_pDlg);
		Main->m_log.Add("Attempting to get updates from remote server");
		m_pRemote->GetUpdates();
		bFoundScripts = true;
#endif
	}
	else
	{
		// Local profile
		// Has the user selected particular scripts to load?
		CStringArray csaScripts;
		CString csKey = _T("SOFTWARE\\Menasoft\\GM Tools\\Local Profiles\\") + Main->GetProfileString();
		Main->GetRegistryMultiSz("Selected Scripts", &csaScripts, HKEY_LOCAL_MACHINE, csKey);
		if ( csaScripts.GetSize() > 0 )
		{
			// Load the specified scripts
			this->m_bNewFormat = true;
			for ( int i = 0; i <= csaScripts.GetUpperBound(); i++ )
				LoadFile(csaScripts.GetAt(i));
			// Categorize everything
			CategorizeLocations();
			CategorizeItems();
			CategorizeNPCs();
			this->m_bDeleteDialog = true;
			DestroyProgressDialog();
			// We're done
			return;
		}
		// Load the INI file first
		Main->m_log.Add("Reading server INI file from working directory");
		csLoadFile.Format("%s\\%s", cwd, INI_FILE);
		Main->m_pScripts->m_tables.m_Sphere.m_csFilename = csLoadFile;
		Main->m_pScripts->m_config.m_sSCPBaseDir = cwd;
		if ( !this->LoadFile(csLoadFile) && !bFoundScripts )
			bFoundScripts = false;
		else
			bFoundScripts = true;
		Main->m_pScripts->m_config.Read();
		Main->m_log.Add("Loading table file from working directory");
		csLoadFile.Format("%s\\%s", cwd, TABLE_FILE);
		if ( !this->LoadFile(csLoadFile) )
		{
			Main->m_log.Add("Unable to load table file from working directory...trying the SCPFILES directory");
			CString csScpPath = Main->m_pScripts->m_config.m_sSCPBaseDir;
			if ( csScpPath.GetLength() > 0 && ( csScpPath.GetAt(csScpPath.GetLength() - 1 ) == '/' || csScpPath.GetAt(csScpPath.GetLength() - 1 ) == '\\' ) )
				csLoadFile.Format("%s%s", csScpPath, TABLE_FILE);
			else
				csLoadFile.Format("%s\\%s", csScpPath, TABLE_FILE);
			if ( !this->LoadFile(csLoadFile) )
				bFoundScripts = false;
			else
				bFoundScripts = true;
		}
		else
			bFoundScripts = true;
	}
	// What does the resource table look like?
	if ( !bFoundScripts )
	{
		Main->m_log.Add("Unable to find spheretables.scp or sphere.ini.  Trying to load axiscats.scp instead");
		if ( this->LoadFile("axiscats.scp") )
		{
			this->LoadFile("custloc.scp");
			this->m_bNewFormat = true;
			CategorizeLocations();
			CategorizeItems();
			CategorizeNPCs();
			this->m_bDeleteDialog = true;
			DestroyProgressDialog();
			return;
		}
	}
	// Some of the resource entries may be directories
	if ( Main->GetProfileType() != PROFILE_REMOTE )
	{
		int i = 0;
		while ( i < this->m_tables.m_resources.GetUpperBound() )
		{
			CString csFilename = m_tables.m_resources.GetAt(i);
			if ( csFilename != "" && ( csFilename.ReverseFind('\\') == csFilename.GetLength() - 1 ) )
			{
				Main->m_log.Add("Enumerating scripts in the %s directory", csFilename);
				int count = 0;
				// This is a directory.
				m_tables.m_resources.RemoveAt(i);
				WIN32_FIND_DATA findData;
				memset(&findData, 0x00, sizeof(findData));
				CString csScpPath = Main->m_pScripts->m_config.m_sSCPBaseDir;
				char szPath[MAX_PATH];
				if ( csFilename.Find(":") != -1 )
					sprintf(szPath, "%s*.scp", csFilename);
				else
				{
					if ( Main->m_csRootDirectory.GetLength() > 0 )
						sprintf(szPath, "%s\\%s*.scp",  Main->m_csRootDirectory, csFilename);
					else
					{
						// Check if the script path is part of this pathname.
						if ( ( csFilename.Find(csScpPath, 0) == 0))
						{
							// the directory is already set to be sphere.ini relative -- most likely
						}
						else
						{
							// the directory set to be SCPFILES relative
							CString csTemp;
							csTemp.Format("%s\\%s", csScpPath, csFilename);
							csFilename = csTemp;
							csFilename.Replace("\\.\\", "\\");
							csFilename.Replace("\\\\", "\\");
						}
						sprintf(szPath, "%s*.scp", csFilename);
					}
				}
				HANDLE hSearch = FindFirstFile(szPath, &findData);
				if ( hSearch != INVALID_HANDLE_VALUE )
				{
					BOOL bStatus = TRUE;
					while ( bStatus )
					{
						if ( findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
						{
						}
						else
						{
							// This is a file
							CString csNewFile;
							if ( csFilename.Find(":") != -1 && csFilename.Find("\\") != -1)
								csNewFile.Format("%s%s", csFilename, findData.cFileName);
							else
							{
								if ( Main->m_csRootDirectory.GetLength() > 0 )
									csNewFile.Format("%s\\%s%s", Main->m_csRootDirectory, csFilename, findData.cFileName);
								else
									csNewFile.Format("%s%s", csFilename, findData.cFileName);
							}
							CString csTest = csNewFile;
							csTest.MakeLower();
							if ( csTest.Find("hoglocs.scp") == -1 && csTest.Find("custloc.scp") == -1 && csTest.Find("axiscats.scp") == -1 && csTest.Find("spheretables.scp") == -1 )
							{
								m_tables.m_resources.Add(csNewFile);
								count++;
							}
						}
						bStatus = FindNextFile(hSearch, &findData);
					}
					Main->m_log.Add("Found %ld scripts in the %s directory", count, csFilename);
				}
				else
				{
					Main->m_log.Add("Couldn't find any script files matching %s", szPath);
				}
			}
			else
				i++;
		}
	}
	if ( this->m_tables.m_resources.GetSize() > 0 )
	{
		Main->m_log.Add("Found %ld scripts in the resources table.", m_tables.m_resources.GetSize());
		Main->m_log.Add("Assuming scripts are in the 0.53+ format");
		m_bNewFormat = true;
		// We've got resources to deal with here
		for ( int index = 0; index < this->m_tables.m_resources.GetSize(); index++ )
		{
			CString csFilename = this->m_tables.m_resources.GetAt(index);
			if ( csFilename.Find("//") != -1 )
				csFilename = csFilename.Left(csFilename.Find("//"));
			if ( csFilename == "" )
				continue;
			if ( Main->GetProfileType() == PROFILE_REMOTE )
			{
				CString csFile;

				if ( csFilename.GetAt(0) == '\\' || csFilename.GetAt(0) == '/')
					csFile = csFilename.Mid(1);
				else
					csFile = csFilename;
				csFilename = csFile;
			}
			/*
			CString csCwd = _T(cwd);
			csCwd.MakeLower();
			CString csFilenameLower = csFilename;
			csFilenameLower.MakeLower();
			if ( csFilenameLower.Find(csCwd) != -1 )
				csFilename = csFilename.Mid(csFilenameLower.Find(csCwd) + strlen(cwd) + 1);
			*/
			// See if there was no path specified here...if not, try using the SCPFILES path
			if ( csFilename.Find('\\') == -1 && csFilename.Find('/') == -1 )
			{
				CString csScpPath = Main->m_pScripts->m_config.m_sSCPBaseDir;
				if ( csScpPath.GetLength() > 0 && ( csScpPath.GetAt(csScpPath.GetLength() - 1) == '\\' || csScpPath.GetAt(csScpPath.GetLength() -1) == '/' ) )
					csScpPath.SetAt(csScpPath.GetLength() - 1, 0x00);
				if ( csScpPath.GetLength() > 0 )
					csLoadFile.Format("%s\\%s", csScpPath, csFilename);
				else
					csLoadFile = csFilename;
				if ( Main->GetProfileType() == PROFILE_REMOTE )
					csFilename = csLoadFile;
			}
			else
			{
				if ( csFilename.Find(":") == -1 )
				{
					if ( Main->m_csRootDirectory.GetLength() > 0 )
						csLoadFile.Format("%s\\%s", Main->m_csRootDirectory, csFilename);
					else
						csLoadFile = csFilename;
				}
				else
					csLoadFile = csFilename;
			}
			if ( Main->GetProfileType() == PROFILE_REMOTE )
			{
				csFilename.Replace(':', '!');
				csLoadFile.Format("%s\\%s\\%s", cwd, Main->GetProfileString(), csFilename);
			}
			CString csTest = csFilename;
			csTest.MakeLower();
			if ( csTest.Find(TABLE_FILE) == -1 )
				LoadFile(csLoadFile);
		}
		// Now find all of the rest of the files in this directory and load the ones that
		// aren't in the resources table.
		if ( !Main->m_dwResourcesOnly )
		{
			Main->m_log.Add("Checking for non-resource scripts in the current working directory");
			m_pDlg->m_csMessage.SetWindowText("Checking for non-resource scripts");
			char szScriptPath[MAX_PATH];
			memset(&szScriptPath[0], 0x00, sizeof(szScriptPath));
			GetPrivateProfileString("SPHERE", "SCPFILES", "", szScriptPath, sizeof(szScriptPath), INI_FILE);
			CString csTestPath;
			if ( Main->GetProfileType() == PROFILE_REMOTE )
				csTestPath.Format("%s\\", Main->GetProfileString());
			else
			{
				if ( szScriptPath[0] != 0x00 && szScriptPath[strlen(szScriptPath) - 1] == '\\' )
					szScriptPath[strlen(szScriptPath) - 1] = 0x00;
				if ( szScriptPath[0] == 0x00 )
					csTestPath.Format(".\\");
				else
					csTestPath.Format("%s\\", szScriptPath);
			}
			WIN32_FIND_DATA findData;
			memset(&findData, 0x00, sizeof(findData));
			CString csTestFile;
			csTestFile.Format("%s*.scp", csTestPath);
			HANDLE hSearch = FindFirstFile(csTestFile, &findData);
			if ( hSearch != INVALID_HANDLE_VALUE )
			{
				BOOL bStatus = TRUE;
				while ( bStatus )
				{
					if ( ! (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) )
					{
						CString csTest;
						if ( szScriptPath[0] != 0x00 )
							csTest.Format("%s\\%s", szScriptPath, findData.cFileName);
						else
							csTest = _T(findData.cFileName);
						bool bReadIt = true;
						for ( int i = 0; i < m_tables.m_resources.GetSize(); i++ )
						{
							csTest.MakeLower();
							CString csResource = m_tables.m_resources.GetAt(i);
							if ( csResource == "" )
								continue;
							if ( csTest.Find(m_tables.m_resources.GetAt(i)) != -1 )
							{
								bReadIt = false;
								break;
							}
							if ( csTest.Find("hoglocs.scp") != -1 )
							{
								bReadIt = false;
								break;
							}
							if ( csTest.Find("custloc.scp") != -1 )
							{
								bReadIt = false;
								break;
							}
							if ( csTest.Find(TABLE_FILE) != -1 )
							{
								bReadIt = false;
								break;
							}
							if ( csTest.Find("axiscats.scp") != -1 )
							{
								bReadIt = false;
								break;
							}
						}
						if ( bReadIt )
						{
							CString csFileRead;
							csFileRead.Format("%s%s", csTestPath, findData.cFileName);
							LoadFile(csTest);
						}
						bStatus = FindNextFile(hSearch, &findData);
					}
				}
			}
		}
		this->LoadFile("hoglocs.scp");
		this->LoadFile("custloc.scp");
	}
	else
	{
		// This must be the old script format
		Main->m_log.Add("No entries in the resource table.  Assuming scripts are in 0.52 format");
		CreateProgressDialog();
		this->LoadDefs();
		this->LoadItems();
		this->LoadLocations();
		this->LoadMap();
		this->LoadNames();
		this->LoadNPCs();
		this->LoadSpeechBlocks();
		this->LoadTables();
		this->LoadTemplates();
		this->LoadTriggers();
	}
	this->CategorizeItems();
	this->CategorizeLocations();
	this->CategorizeNPCs();
	Main->m_log.Add("Finished loading scripts");
	if ( m_pRemote )
		m_pRemote->SetDialog(NULL);
	m_bDeleteDialog = true;
	DestroyProgressDialog();
	m_iScriptSeq++;
}

void CScriptObjects::CreateProgressDialog()
{
	if ( m_pDlg )
		return;
	m_pDlg = new CFileReadProgress;
	m_pDlg->Create(IDD_FILEREAD_PROGRESS);
	m_bDeleteDialog = true;
}

void CScriptObjects::DestroyProgressDialog()
{
	if ( !m_bDeleteDialog )
		return;
	if ( !m_pDlg )
		return;
	m_pDlg->DestroyWindow();
	delete m_pDlg;
	m_pDlg = NULL;
}	