/*
 $Id: ClientPatch.cpp,v 1.10 2002/05/01 03:33:09 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// ClientPatch.cpp: implementation of the CClientPatch class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "drewsky.h"
#include "ClientPatch.h"
#include "FileReadProgress.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CClientPatch::CClientPatch()
{
	m_pData = NULL;
	m_pHeader = NULL;
	m_dwLoadVersion = 0xFFFFFFFF;
	m_dwHeaderSize = 0;
	m_dwDataSize = 0;
	m_dwError = 0;
#ifdef _AXIS
	m_csClientPath = csUOPath;
#endif
}

CClientPatch::~CClientPatch()
{
	Clean();
}

bool CClientPatch::Read(CString csFilename)
{
	m_dwError = 0;
	CFile input;
	if ( !input.Open(csFilename, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
	{
		m_dwError = ERROR_FILE_NOT_FOUND;
		return false;
	}
	input.Read(&m_dwLoadVersion, sizeof(DWORD));
	if ( m_dwLoadVersion < CURRENT_PATCH_VERSION )
	{
		// What do we do with this?
	}
	else
	{
		// This is easy.
		input.Read(&m_dwHeaderSize, sizeof(DWORD));
		// Allocate the header buffer.
		m_pHeader = new BYTE [m_dwHeaderSize];
		// Read the block descriptors
		input.Read(m_pHeader, m_dwHeaderSize);
		// Read the rest of the file
		m_dwDataSize = input.GetLength() - m_dwHeaderSize - 8;
		m_pData = new BYTE [m_dwDataSize];
		input.Read(m_pData, m_dwDataSize);
	}
	input.Close();
	return ConvertData();
}

bool CClientPatch::Write(CString csFilename)
{
	m_dwError = 0;
	CFile output;
	if ( ! output.Open(csFilename, CFile::modeWrite | CFile::modeCreate | CFile::typeBinary | CFile::shareDenyWrite) )
	{
		m_dwError = ERROR_FILE_NOT_FOUND;
		return false;
	}
	m_dwLoadVersion = CURRENT_PATCH_VERSION;
	output.Write(&m_dwLoadVersion, sizeof(DWORD));
	output.Write(&m_dwHeaderSize, sizeof(DWORD));
	output.Write(m_pHeader, m_dwHeaderSize);
	output.Write(m_pData, m_dwDataSize);

	output.Close();
	return true;
}

bool CClientPatch::Apply()
{
	m_dwError = 0;
	// Need to sort the data within each section.
	if ( m_pHeader == NULL )
		return true;	// Nothing to do!
	BYTE * pHeaderPtr = m_pHeader;
	DWORD dwHeaderIncrement = 0;
	DWORD dwDataIncrement = 0;
	DWORD dwHeaderOffset = 0;
	//CFileReadProgress progress;
	while ( dwHeaderOffset < m_dwHeaderSize )
	{
		struct UOPatchBlockDescriptor * pb;
		pb = (UOPatchBlockDescriptor *) pHeaderPtr;
		if ( pb == NULL )
			break;
		switch ( pb->bFileID )
		{
		case MUL_Statics0:
			// This one we actually know something about
			// Get the data, and then sort it.
			if ( !SortStaticData(m_pData + pb->dwOffset, pb->dwLength) )
			{
				m_dwError = ERROR_STATIC_SORT_FAILED;
				return false;
			}
			if ( !PatchStaticData(m_pData + pb->dwOffset, pb->dwLength) )
			{
				m_dwError = ERROR_STATIC_PATCH_FAILED;
				return false;
			}
			break;
		default:
			m_dwError = ERROR_UNKNOWN_CLIENT_FILE;
			return false;
		}
		dwHeaderOffset += sizeof(struct UOPatchBlockDescriptor);
		pHeaderPtr += sizeof(struct UOPatchBlockDescriptor);
	}
	return true;
}

bool CClientPatch::ConvertData()
{
	m_dwError = 0;
	switch (m_dwLoadVersion)
	{
	case CURRENT_PATCH_VERSION:
		// No conversion necessary
		break;
	default:
		// Unknown version
		m_dwError = ERROR_UNKNOWN_DATA_VERSION;
		return false;
	}
	return true;
}

bool CClientPatch::Insert(BYTE bID, DWORD dwLength, BYTE * pData)
{
	m_dwError = 0;
	// Is the fileID valid?
	DWORD dwObjectSize = 0;
	switch ( bID )
	{
	case MUL_Statics0:
		dwObjectSize = sizeof(UOStaticPatchObject);
		break;
	default:
		m_dwError = ERROR_UNKNOWN_CLIENT_FILE;
		return false;
	}
	// Is the legth valid?
	if ( dwLength != dwObjectSize )
	{
		m_dwError = ERROR_BAD_DATA_LENGTH;
		return false;
	}
	// Go through each item in the data block to see if we already have it in our patch information
	// First check if we even have any of this type of data
	BYTE * pHeaderPtr = m_pHeader;
	DWORD dwHeaderIncrement = 0;
	DWORD dwDataIncrement = 0;
	DWORD dwHeaderOffset = 0;
	while ( dwHeaderOffset < m_dwHeaderSize )
	{
		struct UOPatchBlockDescriptor * pb;
		pb = (UOPatchBlockDescriptor *) pHeaderPtr;
		if ( dwDataIncrement )
		{
			// We've inserted some data....now we just have to increment the headers.
			pb->dwOffset += dwDataIncrement;
		}
		else
		{
			// We haven't inserted anything yet
			if ( pb->bFileID == bID )
			{
				// Check to see if this object is already here
				switch ( bID )
				{
				case MUL_Statics0:
					{
						DWORD dwDataPtr = 0;
						struct UOStaticPatchObject * pObject = (struct UOStaticPatchObject *) pData;
						while ( dwDataPtr < pb->dwLength )
						{
							struct UOStaticPatchObject * po;
							po = (UOStaticPatchObject *) (m_pData + pb->dwOffset + dwDataPtr);
							dwDataPtr += sizeof(UOStaticPatchObject);
							if ( CompareObjects((BYTE *)po, (BYTE *)pObject, sizeof(UOStaticPatchObject) - sizeof(BYTE) ) )
							{
								// They are the same
								// Overwrite the existing guy with the new data
								memcpy(po, pObject, sizeof(UOStaticPatchObject));
								return true;
							}
						}
					}
					break;
				default:
					break;
				}
				// If we are here, the data type exists, but the data block doesn't.  We'll have to insert it.
				switch ( bID )
				{
				case MUL_Statics0:
					// Set the increment size so that the other headers can get updated.
					dwDataIncrement = sizeof(UOStaticPatchObject);
					break;
				default:
					break;
				}
				pb->dwLength += dwDataIncrement;
				// Insert the new data into the data block.
				BYTE * pNewData = new BYTE [m_dwDataSize + dwDataIncrement];
				BYTE * pNewDataPtr = pNewData;
				memcpy(pNewDataPtr, m_pData, pb->dwOffset);
				pNewDataPtr += pb->dwOffset;
				memcpy(pNewDataPtr, pData, dwDataIncrement);
				pNewDataPtr += dwDataIncrement;
				memcpy(pNewDataPtr, m_pData + pb->dwOffset, m_dwDataSize - pb->dwOffset);
				m_dwDataSize += dwDataIncrement;
				delete [] m_pData;
				m_pData = pNewData;
			}
			if ( pb->bFileID > bID )
			{
				// If we are here, we didn't have any of this data type, so we'll have to insert a new header block
				// as well as a new data segment.
				// First, insert the new header block
				switch ( bID )
				{
				case MUL_Statics0:
					dwDataIncrement = sizeof(UOStaticPatchObject);
					break;
				default:
					break;
				}
				BYTE * pNewHeader = new BYTE [m_dwHeaderSize + sizeof(UOPatchBlockDescriptor)];
				BYTE * pNewHeaderPtr = pNewHeader;
				memcpy(pNewHeaderPtr, m_pHeader, dwHeaderOffset);
				pNewHeaderPtr += dwHeaderOffset;
				struct UOPatchBlockDescriptor block;
				block.bFileID = bID;
				block.dwLength = dwDataIncrement;
				block.dwOffset = pb->dwOffset;
				memcpy(pNewHeaderPtr, &block, sizeof(UOPatchBlockDescriptor));
				pNewHeaderPtr += sizeof(UOPatchBlockDescriptor);
				memcpy(pNewHeaderPtr, pHeaderPtr, m_dwHeaderSize - dwHeaderOffset);
				pHeaderPtr = pNewHeader + dwHeaderOffset + sizeof(UOPatchBlockDescriptor);
				m_dwHeaderSize += sizeof(UOPatchBlockDescriptor);
				delete [] m_pHeader;
				m_pHeader = pNewHeader;
				// Insert the new data block
				BYTE * pNewData = new BYTE [m_dwDataSize + dwDataIncrement];
				BYTE * pNewDataPtr = pNewData;
				memcpy(pNewDataPtr, m_pData, pb->dwOffset);
				pNewDataPtr += pb->dwOffset;
				memcpy(pNewDataPtr, pData, dwDataIncrement);
				pNewDataPtr += dwDataIncrement;
				memcpy(pNewDataPtr, m_pData + pb->dwOffset, m_dwDataSize - pb->dwOffset);
				m_dwDataSize += dwDataIncrement;
				delete [] m_pData;
				m_pData = pNewData;
			}
		}
		pHeaderPtr+= sizeof(UOPatchBlockDescriptor);
		dwHeaderOffset = pHeaderPtr - m_pHeader;
	}
	if ( dwDataIncrement == 0 )
	{
		// We never inserted anything!
		// Append this to the end of the data.
		BYTE * pNewHeader = new BYTE [m_dwHeaderSize + sizeof(UOPatchBlockDescriptor)];
		BYTE * pNewHeaderPtr = pNewHeader;
		memcpy(pNewHeaderPtr, m_pHeader, dwHeaderOffset);
		pNewHeaderPtr += dwHeaderOffset;
		struct UOPatchBlockDescriptor block;
		block.bFileID = bID;
		block.dwLength = dwLength;
		block.dwOffset = m_dwDataSize;
		memcpy(pNewHeaderPtr, &block, sizeof(UOPatchBlockDescriptor));
		pNewHeaderPtr += sizeof(UOPatchBlockDescriptor);
		memcpy(pNewHeaderPtr, pHeaderPtr, m_dwHeaderSize - dwHeaderOffset);
		pHeaderPtr = pNewHeader + dwHeaderOffset + sizeof(UOPatchBlockDescriptor);
		m_dwHeaderSize += sizeof(UOPatchBlockDescriptor);
		delete [] m_pHeader;
		m_pHeader = pNewHeader;
		// Insert the new data block
		BYTE * pNewData = new BYTE [m_dwDataSize + dwLength];
		BYTE * pNewDataPtr = pNewData;
		memcpy(pNewDataPtr, m_pData, m_dwDataSize);
		pNewDataPtr += m_dwDataSize;
		memcpy(pNewDataPtr, pData, dwLength);
		pNewDataPtr += dwLength;
		m_dwDataSize += dwLength;
		delete [] m_pData;
		m_pData = pNewData;
	}
	return true;
}

CString CClientPatch::GetErrorString()
{
	m_dwError = 0;
	CString csMessage;
	switch ( m_dwError )
	{
	case ERROR_FILE_NOT_FOUND:
		csMessage = "file not found"; break;
	case ERROR_UNKNOWN_DATA_VERSION:
		csMessage = "unknown patch version"; break;
	case ERROR_UNKNOWN_CLIENT_FILE:
		csMessage = "unknown client file"; break;
	case ERROR_BAD_DATA_LENGTH:
		csMessage = "bad data length"; break;
	case ERROR_BUFFER_TOO_SMALL:
		csMessage = "buffer too small"; break;
	case ERROR_STATIC_SORT_FAILED:
		csMessage = "static data sort failed"; break;
	case ERROR_STATIC_PATCH_FAILED:
		csMessage = "static patch failed"; break;
	case 0:
		csMessage = "no error"; break;
	default:
		csMessage = "unknown error"; break;
	}
	return csMessage;
}

bool CClientPatch::CompareObjects(BYTE * pOb1, BYTE *pOb2, DWORD dwSize)
{
	m_dwError = 0;
	int iResult = memcmp((void *) pOb1, (void *) pOb2, dwSize);
	if ( iResult == 0 )
		return true;
	else
		return false;
}

struct UOPatchBlockDescriptor * CClientPatch::GetHeaderBlock(BYTE bID)
{
	m_dwError = 0;
	BYTE * pPtr = m_pHeader;
	while ( pPtr < m_pHeader + m_dwHeaderSize )
	{
		struct UOPatchBlockDescriptor * test = (UOPatchBlockDescriptor *) pPtr;
		if ( test->bFileID == bID )
			return test;
		pPtr += sizeof(struct UOPatchBlockDescriptor);
	}
	return NULL;
}

DWORD CClientPatch::GetDataBlockSize(BYTE bID)
{
	m_dwError = 0;
	struct UOPatchBlockDescriptor * hdr = GetHeaderBlock(bID);
	if ( hdr != NULL )
		return hdr->dwLength;
	else
		return 0;
}

DWORD CClientPatch::CopyDataBlock(BYTE bID, BYTE * pCopy, DWORD dwSize)
{
	m_dwError = 0;
	struct UOPatchBlockDescriptor * hdr = GetHeaderBlock(bID);
	if ( hdr == NULL )
		return 0;
	if ( hdr->dwLength > dwSize || pCopy == NULL )
	{
		m_dwError = ERROR_BUFFER_TOO_SMALL;
		return 0;
	}
	memset(pCopy, 0x00, dwSize);
	memcpy(pCopy, m_pData + hdr->dwOffset, hdr->dwLength);
	return hdr->dwLength;
}

bool CClientPatch::SortStaticData(BYTE * pBuffer, DWORD dwSize)
{
	BYTE * pDestBuffer = new BYTE [dwSize];
	if ( ! pDestBuffer )
		return false;
	memset(pDestBuffer, 0x00, dwSize);
	BYTE * pDataPtr = pBuffer;
	CFileReadProgress progress;
	progress.Create(IDD_FILEREAD_PROGRESS);
	progress.SetWindowText("Preparing patch data");
	progress.SetPos(0);
	progress.SetRange32(0, dwSize);
	progress.m_csMessage.SetWindowText("Sorting patch data");
	while ( (DWORD) (pDataPtr - pBuffer) < dwSize )
	{
		struct UOStaticPatchObject * pPatch = (struct UOStaticPatchObject *) pDataPtr;
		if ( !InsertStaticPatchObject(pPatch, pDestBuffer, dwSize) )
		{
			delete [] pDestBuffer;
			return false;
		}
		pDataPtr += sizeof(struct UOStaticPatchObject);
		progress.SetPos(pDataPtr - pBuffer);
	}
	memcpy(pBuffer, pDestBuffer, dwSize);
	delete [] pDestBuffer;
	return true;
}

bool CClientPatch::InsertStaticPatchObject(struct UOStaticPatchObject * pObject, BYTE * pDestBuffer, DWORD dwBufferSize)
{
	if ( pObject == NULL )
		return true;
	if ( pObject->blockData.wArtID == 0 )
		return true;
	BYTE * pDataPtr = pDestBuffer;
	while ( (DWORD) (pDataPtr - pDestBuffer) < dwBufferSize )
	{
		struct UOStaticPatchObject * pTest = (struct UOStaticPatchObject *) pDataPtr;
		if ( pTest->blockData.wArtID == 0 )
		{
			// We've reached the end of existing data.  Copy the new stuff here
			 memcpy(pDataPtr, pObject, sizeof(struct UOStaticPatchObject));
			 return true;
		}
		if ( pTest->dwBlockIndex >= pObject->dwBlockIndex )
		{
			// Insert the new data here
			BYTE * pNewBuffer = new BYTE [dwBufferSize];
			DWORD dwOffset, dwRemainder;
			dwOffset = pDataPtr - pDestBuffer;
			dwRemainder = dwBufferSize - dwOffset - sizeof(struct UOStaticPatchObject);
			// Copy the preceeding stuff
			memcpy(pNewBuffer, pDestBuffer, dwOffset);
			// Copy the new object
			memcpy(pNewBuffer + dwOffset, pObject, sizeof(struct UOStaticPatchObject));
			// Copy the rest
			memcpy(pNewBuffer + dwOffset + sizeof(struct UOStaticPatchObject), pDestBuffer + dwOffset, dwRemainder);
			// Copy this back to the original buffer
			memcpy(pDestBuffer, pNewBuffer, dwBufferSize);
			delete [] pNewBuffer;
			return true;
		}
		// else, keep going
		pDataPtr += sizeof(struct UOStaticPatchObject);
	}
	return true;
}

bool CClientPatch::PatchStaticData(BYTE * pBuffer, DWORD dwSize, bool bVerdata)
{
	CFile idxOld, idxNew, mulOld, mulNew;
	CString csIdxOld, csIdxNew, csMulOld, csMulNew;
	csIdxOld.Format("%sstaidx0.mul", m_csClientPath);
	csIdxNew.Format("%sstaidx0.new", m_csClientPath);
	csMulOld.Format("%sstatics0.mul", m_csClientPath);
	csMulNew.Format("%sstatics0.new", m_csClientPath);
	try
	{
		// Need to open the existing idx and data files
		idxOld.Open(csIdxOld, CFile::typeBinary | CFile::modeRead | CFile::shareDenyNone);
		mulOld.Open(csMulOld, CFile::typeBinary | CFile::modeRead | CFile::shareDenyNone);
		// Open a new idx and a new data file
		idxNew.Open(csIdxNew, CFile::typeBinary | CFile::modeWrite | CFile::modeCreate | CFile::shareDenyWrite);
		mulNew.Open(csMulNew, CFile::typeBinary | CFile::modeWrite | CFile::modeCreate | CFile::shareDenyWrite);
	}
	catch ( CFileException * e )
	{
		e->Delete();
		return false;
	}
	BYTE * pPatchPtr = pBuffer;
	BOOL bStatus = TRUE;
	DWORD dwBlockIndex = 0;
	CFileReadProgress progress;
	progress.Create(IDD_FILEREAD_PROGRESS);
	progress.SetPos(0);
	progress.m_csMessage.SetWindowText("Applying patch data");
	progress.SetRange32(0, 393216);
	while ( bStatus )
	{
		// Read a header block from the index file
		struct UOStaticIdx idx;
		bStatus = idxOld.Read(&idx, sizeof(struct UOStaticIdx));
		if ( ! bStatus )
			break;
		if ( (DWORD) ( pPatchPtr - pBuffer ) < dwSize )
		{
			struct UOStaticPatchObject * pPatch = (struct UOStaticPatchObject *) pPatchPtr;
			// Check to see if we have any patches for this block
			if ( pPatch->dwBlockIndex == dwBlockIndex )
			{
				// Patch this block
				int iAdd, iRemove;
				iAdd = iRemove = 0;
				BYTE * pData = NULL;
				DWORD dwDataSize = 0;
				if ( idx.dwStart != 0xFFFFFFFF )
				{
					pData = new BYTE [idx.dwLength];
					mulOld.Read(pData, idx.dwLength);
					dwDataSize = idx.dwLength;
				}
				while ( (DWORD)(pPatchPtr - pBuffer) < dwSize)
				{
					pPatch = (struct UOStaticPatchObject *) pPatchPtr;
					if ( pPatch->dwBlockIndex != dwBlockIndex )
						break;
					pPatchPtr += sizeof(struct UOStaticPatchObject);
					if ( pPatch->bFlags & 0x03 )
					{
						pData = PatchStaticRemove(pData, dwDataSize, pPatch);
						iRemove++;
					}
					else
					{
						pData = PatchStaticAdd(pData, dwDataSize, pPatch);
						iAdd++;
					}
				}
#ifdef _AXIS
				Main->m_log.Add("Patched static block %ld.  (%ld added, %ld removed)", dwBlockIndex, iAdd, iRemove);
#endif
				if ( dwDataSize == 0 )
				{
					idx.dwStart = 0xFFFFFFFF;
					idx.dwLength = 0xFFFFFFFF;
					delete [] pData;
					goto nopatch;
				}
				idx.dwStart = mulNew.GetPosition();
				idx.dwLength = dwDataSize;
				mulNew.Write(pData, idx.dwLength);
				delete [] pData;
				idxNew.Write(&idx, sizeof(struct UOStaticIdx));
			}
			else
				goto nopatch;
		}
		else
		{
			// Just write the idx and data to the new files
nopatch:
			if ( idx.dwStart != 0xFFFFFFFF )
			{
				BYTE * pData = new BYTE [idx.dwLength];
				mulOld.Read(pData, idx.dwLength);
				idx.dwStart = mulNew.GetPosition();
				mulNew.Write(pData, idx.dwLength);
				delete [] pData;
			}
			idxNew.Write(&idx, sizeof(struct UOStaticIdx));
		}
		dwBlockIndex++;
		if ( ( dwBlockIndex % 1000 ) == 0 )
			progress.SetPos(dwBlockIndex);
	}

	// Close the files
	idxOld.Close();
	idxNew.Close();
	mulOld.Close();
	mulNew.Close();
	// back up the existing idx and data files

	// rename the new idx and data files
	return true;
}

BYTE * CClientPatch::PatchStaticAdd(BYTE * pBuffer, DWORD &dwSize, UOStaticPatchObject *pObject)
{
	// Make sure we don't already have this one
	BYTE * pDataPtr = pBuffer;
	while ( (DWORD)( pDataPtr - pBuffer ) < dwSize )
	{
		struct UOStaticRec * pStatic = ( struct UOStaticRec *) pDataPtr;
		pDataPtr += sizeof(struct UOStaticRec);
		if ( memcmp(pStatic, &(pObject->blockData), sizeof(struct UOStaticRec)) == 0 )
			return pBuffer;
	}
	// Not here...add it
	BYTE * pNewBuffer = new BYTE [dwSize + sizeof(struct UOStaticRec)];
	memcpy(pNewBuffer, &(pObject->blockData), sizeof(struct UOStaticRec));
	memcpy(pNewBuffer + sizeof(struct UOStaticRec), pBuffer, dwSize);
	dwSize += sizeof(struct UOStaticRec);
	delete [] pBuffer;
	return pNewBuffer;
}

BYTE * CClientPatch::PatchStaticRemove(BYTE * pBuffer, DWORD &dwSize, UOStaticPatchObject *pObject)
{
	// Delete this guy if it is there
	BYTE * pDataPtr = pBuffer;
	while ( (DWORD) (pDataPtr - pBuffer) < dwSize )
	{
		struct UOStaticRec * pStatic = (struct UOStaticRec *) pDataPtr;
		if ( memcmp(pStatic, &(pObject->blockData), sizeof(struct UOStaticRec)) == 0 )
		{
			// Remove it
			BYTE * pNewBuffer = new BYTE [dwSize - sizeof(struct UOStaticRec)];
			memcpy(pNewBuffer, pBuffer, (pDataPtr - pBuffer));
			memcpy(pNewBuffer + (pDataPtr - pBuffer), pBuffer + (pDataPtr - pBuffer + sizeof(struct UOStaticRec)), (dwSize - sizeof(struct UOStaticRec) - (pDataPtr - pBuffer)));
			dwSize -= sizeof(struct UOStaticRec);
			delete [] pBuffer;
			return pNewBuffer;
		}
		pDataPtr += sizeof(struct UOStaticRec);
	}
	// Not here
	return pBuffer;
}

bool CClientPatch::IsEmpty()
{
	if ( m_dwHeaderSize != 0 )
		return true;
	else
		return false;
}

void CClientPatch::Clean()
{
	m_dwError = 0;
	if ( m_pData )
		delete [] m_pData;
	if ( m_pHeader )
		delete [] m_pHeader;
}
