/*
 $Id: ConfigFile.cpp,v 1.7 2002/11/18 20:16:23 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// ConfigFile.cpp: implementation of the CConfigFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef _AXIS
#include "drewsky.h"
#else
#include "SphereScript.h"
#endif
#include "ConfigFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
const TCHAR * sm_LoadKeys[SC_QTY] =
{
	"ACCTFILES",			// m_sAcctBaseDir
	"ARRIVEDEPARTMSG", 
	"AUTONEWBIEKEYS",		// m_fAutoNewbieKeys
	"BACKUPLEVELS",				// m_iSaveBackupLevels
	"BANKMAXITEMS",
	"BANKMAXWEIGHT",
	"CHANGEFILES",			// m_sChangedBaseDir
	"CHARTAGS",					// m_fCharTags
	"CLIENTLINGER",
	"CLIENTMAX",			// m_nClientsMax
	"CLIENTS",
	"CLIENTVERSION",
	"CORPSENPCDECAY",
	"CORPSEPLAYERDECAY",
	"CRIMINALTIMER",  // m_iCriminalTimer
	"DEADSOCKETTIME",
	"DEBUGFLAGS",
	"DECAYTIMER",
	"DUNGEONLIGHT",
	"EQUIPPEDCAST",				// m_fEquippedCast
	"FILES",	//
	"FLIPDROPPEDITEMS",			// m_fFlipDroppedItems
	"FORCEGARBAGECOLLECT",		// m_fSaveGarbageCollect
	"FREEZERESTARTTIME",		// m_iFreezeRestartTime
	"GAMEMINUTELENGTH",			// m_iGameMinuteLength
	"GUARDLINGER",			// m_iGuardLingerTime
	"GUARDSINSTANTKILL",
	"GUESTS",
	"GUILDS",
	"HEARALL",
	"HELPINGCRIMINALSISACRIME",	// m_fHelpingCriminalsIsACrime
	"HITPOINTPERCENTONREZ", // m_iHitpointPercentOnRez
	"LIGHTDAY",			// m_iLightDay
	"LIGHTNIGHT",			// m_iLightNight
	"LOG",
	"LOGMASK",				// GetLogMask
	"LOOTINGISACRIME",		// m_fLootingIsACrime
	"MAGICUNLOCKDOOR",		// m_iMagicUnlockDoor
	"MAINLOGSERVER",		// m_fMainLogServer
	"MAPCACHETIME",
	"MAXBASESKILL",			// m_iMaxBaseSkill
	"MAXCHARSPERACCOUNT",	// m_iMaxCharsPerAccount
	"MAXCOMPLEXITY",			// m_iMaxComplexity
	"MINCHARDELETETIME",
	"MONSTERFEAR",			// m_fMonsterFear
	"MONSTERFIGHT",
	"MULFILE",
	"MULFILES",
	"MURDERDECAYTIME",		// m_iMurderDecayTime;
	"MURDERMINCOUNT",		// m_iMurderMinCount;		// amount of murders before we get title.
	"NOWEATHER",				// m_fNoWeather
	"NPCTRAINMAX",			// m_iTrainSkillMax
	"NPCTRAINPERCENT",			// m_iTrainSkillPercent
	"NTSERVICE",				// m_fUseNTService
	"PLAYERGHOSTSOUNDS",	// m_fPlayerGhostSounds
	"PLAYERNEUTRAL",		// m_iPlayerKarmaNeutral
	"POLLSERVERS",				// m_iPollServers
	"PROFILE",
	"RCLOCK",
	"REAGENTLOSSFAIL",			// m_fReagentLossFail
	"REAGENTSREQUIRED",
	"REGEN",
	"REGISTERSERVER",			// m_sRegisterServer
	"REQUIREEMAIL",			// m_fRequireEmail
	"RTIME",
	"RTIMETEXT",
	"RUNNINGPENALTY",		// m_iStamRunningPenalty
	"SAVEBACKGROUND",			// m_iSaveBackgroundTime
	"SAVEPERIOD",
	"SCPFILES",
	"SECTORSLEEP",				// m_iSectorSleepMask
	"SECURE",
	"SNOOPCRIMINAL",
	"SPEECHFILES",
	"STAMINALOSSATWEIGHT",	// m_iStaminaLossAtWeight
	"STATAVG",					// m_iAvgSumOfStats
	"STATCAP",					// m_iMaxSumOfStats
	"STATGUILDS",
	"TIME",
	"TIMEUP",
	"TOTALPOLLEDACCOUNTS",
	"TOTALPOLLEDCLIENTS",
	"TOTALPOLLEDSERVERS",
	"VENDORMAXSELL",			// m_iVendorMaxSell
	"VERBOSE",
	"VERSION",
	"WOPPLAYER",
	"WOPSTAFF",
	"WORLDSAVE",
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CConfigFile::CConfigFile()
{
	m_fUseNTService = false;
	m_iPollServers = 0;
	m_sRegisterServer = GRAY_MAIN_SERVER;
	m_fMainLogServer = false;
	m_iMapCacheTime = 2 * 60 * TICK_PER_SEC;
	m_iSectorSleepMask = 0x3ff;

	m_ClientVersion = GRAY_CLIENT_VER;
	m_wDebugFlags = 0; //DEBUGF_NPC_EMOTE
	m_fSecure = true;
	m_iFreezeRestartTime = 10*TICK_PER_SEC;

	//Magic
	m_fReagentsRequired = true;
	m_fReagentLossFail = true;
	m_fWordsOfPowerPlayer = true;
	m_fWordsOfPowerStaff = true;
	m_fEquippedCast = true;
	m_iMagicUnlockDoor = 1000;

	//m_iSpell_Teleport_Effect_Staff = 0x3709;	// drama
	//m_iSpell_Teleport_Sound_Staff = 0x1f3;
	//m_iSpell_Teleport_Effect_Player = ;
	//m_iSpell_Teleport_Sound_Player;

	// Decay
	m_iDecay_Item = 30*60*TICK_PER_SEC;
	m_iDecay_CorpsePlayer = 45*60*TICK_PER_SEC;
	m_iDecay_CorpseNPC = 15*60*TICK_PER_SEC;

	// Accounts
	m_nClientsMax = FD_SETSIZE-1;
	m_fRequireEmail = false;
	m_nGuestsMax = 0;
	m_fArriveDepartMsg = true;
	m_iClientLingerTime = 60 * TICK_PER_SEC;
	m_iDeadSocketTimeMin = 10*60*TICK_PER_SEC;
	m_iMinCharDeleteTime = 3*24*60*60*TICK_PER_SEC;
	m_iMaxCharsPerAccount = 5;

	// Save
	m_iSaveBackupLevels = 3;
	m_iSaveBackgroundTime = 5* 60 * TICK_PER_SEC;	// Use the new background save.
	m_fSaveGarbageCollect = false;	// Always force a full garbage collection.
	m_iSavePeriod = 15*60*TICK_PER_SEC;

	// In game effects.
	m_fMonsterFight = true;
	m_fMonsterFear = true;
	m_iLightDungeon = 17;
	m_iLightNight = 17;	// dark before t2a.
	m_iLightDay = 0;
	m_iBankIMax = 1000;
	m_iBankWMax = 400;
	m_fGuardsInstantKill = false;
	m_iSnoopCriminal = 500;
	m_iTrainSkillPercent = 50;
	m_iTrainSkillMax = 500;
	m_fCharTags = true;
	m_iVendorMaxSell = 30;
	m_iGameMinuteLength = 8 * TICK_PER_SEC;
	m_sSpeechBaseDir = "speech";
	m_fNoWeather = false;
	m_fFlipDroppedItems = true;
	m_iAvgSumOfStats = 300;
	m_iMaxSumOfStats = 350;
	m_iMurderMinCount = 5;
	m_iMurderDecayTime = 8*60*60* TICK_PER_SEC;
	m_iMaxComplexity = 16;
	m_iPlayerKarmaNeutral = -2000; // How much bad karma makes a player neutral?
	m_iPlayerKarmaEvil = -8000;
	m_iGuardLingerTime = 1*60*TICK_PER_SEC; // "GUARDLINGER",
	m_iCriminalTimer = 3*60*TICK_PER_SEC;
	m_iHitpointPercentOnRez = 10;
	m_fLootingIsACrime = true;
	m_fHelpingCriminalsIsACrime = true;
	m_fPlayerGhostSounds = true;
	m_fAutoNewbieKeys = true;
	m_iMaxBaseSkill = 250;
	m_iStamRunningPenalty = 50;
	m_iStaminaLossAtWeight = 100;
}

CConfigFile::~CConfigFile()
{
	this->m_blockIP.RemoveAll();
	for ( int i = 0; i < this->m_servers.GetSize(); i++ )
	{
		CServerDef * pServer = (CServerDef *) this->m_servers.GetAt(i);
		if ( pServer )
			delete pServer;
	}
	this->m_servers.RemoveAll();
}

bool CConfigFile::Read()
{
	BOOL bStatus = TRUE;
	CStdioFile csfIni;
	if ( Main->m_pScripts->m_tables.m_Sphere.m_csFilename != "" )
		bStatus = csfIni.Open(Main->m_pScripts->m_tables.m_Sphere.m_csFilename, CFile::modeRead | CFile::shareDenyNone);
	else
		bStatus = csfIni.Open(INI_FILE, CFile::modeRead | CFile::shareDenyNone );
	if ( !bStatus )
		return FALSE;
	CString csSection;
	while ( bStatus )
	{
		CString csLine;
		bStatus = csfIni.ReadString(csLine);
		if ( ! bStatus )
			break;
		CString csKey, csValue;
		if ( csLine.Find("[") == -1 && csSection == "" )
			continue;
		else if ( csLine.Find("[") != -1 )
		{
			csSection = csLine.Mid(1);
			csSection = csSection.SpanExcluding("]");
			continue;
		}
		else
		{
			if ( csSection.CompareNoCase("sphere") == 0 )
			{
				csKey = csLine.SpanExcluding("=");
				if ( csKey.GetLength() < csLine.GetLength() )
					csValue = csLine.Mid(csKey.GetLength() + 1);
				else
					csValue = "";
				csValue.Remove(0x0d);
				switch ( FindTable(csKey, sm_LoadKeys, SC_QTY) )
				{
					case SC_ACCTFILES:			// m_sAcctBaseDir
						m_sAcctBaseDir = csValue; break;
					case SC_ARRIVEDEPARTMSG: 
						m_fArriveDepartMsg = atoi(csValue) ? true : false; break;
					case SC_AUTONEWBIEKEYS:		// m_fAutoNewbieKeys
						m_fAutoNewbieKeys = atoi(csValue) ? true : false; break;
					case SC_BACKUPLEVELS:				// m_iSaveBackupLevels
						m_iSaveBackupLevels = atoi(csValue); break;
					case SC_BANKMAXITEMS:
						m_iBankIMax = atoi(csValue); break;
					case SC_BANKMAXWEIGHT:
						m_iBankWMax = atoi(csValue); break;
					case SC_CHANGEFILES:			// m_sChangedBaseDir
						m_sChangedBaseDir = csValue; break;
					case SC_CHARTAGS:					// m_fCharTags
						m_fCharTags = atoi(csValue) ? true : false; break;
					case SC_CLIENTLINGER:
						m_iClientLingerTime = atoi(csValue); break;
					case SC_CLIENTMAX:			// m_nClientsMax
						m_nClientsMax = atoi(csValue); break;
					case SC_CLIENTS:
						break;
					case SC_CLIENTVERSION:
						m_ClientVersion = atoi(csValue); break;
					case SC_CORPSENPCDECAY:
						m_iDecay_CorpseNPC = atoi(csValue); break;
					case SC_CORPSEPLAYERDECAY:
						m_iDecay_CorpsePlayer = atoi(csValue); break;
					case SC_CRIMINALTIMER:		// m_iCriminalTimer
						m_iCriminalTimer = atoi(csValue); break;
					case SC_DEADSOCKETTIME:
						m_iDeadSocketTimeMin = atoi(csValue); break;
					case SC_DEBUGFLAGS:
						m_wDebugFlags = (WORD) ahextoi(csValue); break;
					case SC_DECAYTIMER:
						m_iDecay_Item = atoi(csValue); break;
					case SC_DUNGEONLIGHT:
						m_iLightDungeon = atoi(csValue); break;
					case SC_EQUIPPEDCAST:				// m_fEquippedCast
						m_fEquippedCast = atoi(csValue) ? true : false; break;
					case SC_FILES:	//
						break;
					case SC_FLIPDROPPEDITEMS:			// m_fFlipDroppedItems
						m_fFlipDroppedItems = atoi(csValue) ? true : false; break;
					case SC_FORCEGARBAGECOLLECT:		// m_fSaveGarbageCollect
						m_fSaveGarbageCollect = atoi(csValue) ? true : false; break;
					case SC_FREEZERESTARTTIME:		// m_iFreezeRestartTime
						m_iFreezeRestartTime = atoi(csValue); break;
					case SC_GAMEMINUTELENGTH:			// m_iGameMinuteLength
						m_iGameMinuteLength = atoi(csValue); break;
					case SC_GUARDLINGER:			// m_iGuardLingerTime
						m_iGuardLingerTime = atoi(csValue); break;
					case SC_GUARDSINSTANTKILL:
						m_fGuardsInstantKill = atoi(csValue) ? true : false; break;
					case SC_GUESTS:
						m_nGuestsMax = atoi(csValue); break;
					case SC_GUILDS:
						break;
					case SC_HEARALL:
						break;
					case SC_HELPINGCRIMINALSISACRIME:	// m_fHelpingCriminalsIsACrime
						m_fHelpingCriminalsIsACrime = atoi(csValue) ? true : false; break;
					case SC_HITPOINTPERCENTONREZ: // m_iHitpointPercentOnRez
						m_iHitpointPercentOnRez = atoi(csValue); break;
					case SC_LIGHTDAY:			// m_iLightDay
						m_iLightDay = atoi(csValue); break;
					case SC_LIGHTNIGHT:			// m_iLightNight
						m_iLightNight = atoi(csValue); break;
					case SC_LOG:
						break;
					case SC_LOGMASK:				// GetLogMask
						break;
					case SC_LOOTINGISACRIME:		// m_fLootingIsACrime
						m_fLootingIsACrime = atoi(csValue) ? true : false; break;
					case SC_MAGICUNLOCKDOOR:		// m_iMagicUnlockDoor
						m_iMagicUnlockDoor = atoi(csValue); break;
					case SC_MAINLOGSERVER:		// m_fMainLogServer
						m_fMainLogServer = atoi(csValue) ? true : false; break;
					case SC_MAPCACHETIME:
						m_iMapCacheTime = atoi(csValue); break;
					case SC_MAXBASESKILL:			// m_iMaxBaseSkill
						m_iMaxBaseSkill = atoi(csValue); break;
					case SC_MAXCHARSPERACCOUNT:	// m_iMaxCharsPerAccount
						m_iMaxCharsPerAccount = atoi(csValue); break;
					case SC_MAXCOMPLEXITY:			// m_iMaxComplexity
						m_iMaxComplexity = atoi(csValue); break;
					case SC_MINCHARDELETETIME:
						m_iMinCharDeleteTime = atoi(csValue); break;
					case SC_MONSTERFEAR:			// m_fMonsterFear
						m_fMonsterFear = atoi(csValue) ? true : false; break;
					case SC_MONSTERFIGHT:
						m_fMonsterFight = atoi(csValue) ? true : false; break;
					case SC_MULFILE:
						break;
					case SC_MULFILES:
						break;
					case SC_MURDERDECAYTIME:		// m_iMurderDecayTime;
						m_iMurderDecayTime = atoi(csValue); break;
					case SC_MURDERMINCOUNT:		// m_iMurderMinCount;		// amount of murders before we get title.
						m_iMurderMinCount = atoi(csValue); break;
					case SC_NOWEATHER:				// m_fNoWeather
						m_fNoWeather = atoi(csValue) ? true : false; break;
					case SC_NPCTRAINMAX:			// m_iTrainSkillMax
						m_iTrainSkillMax = atoi(csValue); break;
					case SC_NPCTRAINPERCENT:			// m_iTrainSkillPercent
						m_iTrainSkillPercent = atoi(csValue); break;
					case SC_NTSERVICE:				// m_fUseNTService
						m_fUseNTService = atoi(csValue) ? true : false; break;
					case SC_PLAYERGHOSTSOUNDS:	// m_fPlayerGhostSounds
						m_fPlayerGhostSounds = atoi(csValue) ? true : false; break;
					case SC_PLAYERNEUTRAL:		// m_iPlayerKarmaNeutral
						m_iPlayerKarmaNeutral = atoi(csValue); break;
					case SC_POLLSERVERS:				// m_iPollServers
						m_iPollServers = atoi(csValue); break;
					case SC_PROFILE:
						break;
					case SC_RCLOCK:
						break;
					case SC_REAGENTLOSSFAIL:			// m_fReagentLossFail
						m_fReagentLossFail = atoi(csValue) ? true : false; break;
					case SC_REAGENTSREQUIRED:
						m_fReagentsRequired = atoi(csValue) ? true : false; break;
					case SC_REGEN:
						break;
					case SC_REGISTERSERVER:			// m_sRegisterServer
						m_sRegisterServer = csValue;
					case SC_REQUIREEMAIL:		// m_fRequireEmail
						m_fRequireEmail = atoi(csValue) ? true : false; break;
					case SC_RTIME:
						break;
					case SC_RTIMETEXT:
						break;
					case SC_RUNNINGPENALTY:		// m_iStamRunningPenalty
						m_iStamRunningPenalty = atoi(csValue); break;
					case SC_SAVEBACKGROUND:			// m_iSaveBackgroundTime
						m_iSaveBackgroundTime = atoi(csValue); break;
					case SC_SAVEPERIOD:
						m_iSavePeriod = atoi(csValue); break;
					case SC_SCPFILES:
						m_sSCPBaseDir = csValue; break;
					case SC_SECTORSLEEP:				// m_iSectorSleepMask
						m_iSectorSleepMask = atoi(csValue); break;
					case SC_SECURE:
						m_fSecure = atoi(csValue) ? true : false; break;
					case SC_SNOOPCRIMINAL:
						m_iSnoopCriminal = atoi(csValue); break;
					case SC_SPEECHFILES:
						m_sSpeechBaseDir = csValue; break;
					case SC_STAMINALOSSATWEIGHT:	// m_iStaminaLossAtWeight
						m_iStaminaLossAtWeight = atoi(csValue); break;
					case SC_STATAVG:					// m_iAvgSumOfStats
						m_iAvgSumOfStats = atoi(csValue); break;
					case SC_STATCAP:					// m_iMaxSumOfStats
						m_iMaxSumOfStats = atoi(csValue); break;
					case SC_STATGUILDS:
						break;
					case SC_TIME:
						break;
					case SC_TIMEUP:
						break;
					case SC_TOTALPOLLEDACCOUNTS:
						break;
					case SC_TOTALPOLLEDCLIENTS:
						break;
					case SC_TOTALPOLLEDSERVERS:
						break;
					case SC_VENDORMAXSELL:			// m_iVendorMaxSell
						m_iVendorMaxSell = atoi(csValue); break;
					case SC_VERBOSE:
						break;
					case SC_VERSION:
						break;
					case SC_WOPPLAYER:
						m_fWordsOfPowerPlayer = atoi(csValue) ? true : false; break;
					case SC_WOPSTAFF:
						m_fWordsOfPowerStaff = atoi(csValue) ? true : false; break;
					case SC_WORLDSAVE:
						break;
					case SC_QTY:
						break;
				}
			}
			else if ( csSection.CompareNoCase("servers") == 0 )
			{
				CServerDef * pServer = new CServerDef;
				if ( pServer->Load(&csfIni) )
					m_servers.Add((void *) pServer);
				else
					delete pServer;
			}
			else if ( csSection.CompareNoCase("blockip") == 0 )
				this->m_blockIP.Add(csLine);
			else
				continue;
		}
	}
	csfIni.Close();
	return bStatus ? true : false;
}

bool CConfigFile::Write()
{
	return true;
}

CServerDef::CServerDef()
{
	m_iPort = 0;
}

CServerDef::~CServerDef()
{
}

bool CServerDef::Load(CStdioFile * psfInput)
{
	BOOL bStatus = TRUE;
	CString csLine;
	for (int i = 0; i < 3; i++)
	{
		bStatus = psfInput->ReadString(csLine);
		if ( bStatus || csLine == "" )
			break;
		switch ( i )
		{
		case 0:
			m_csName = csLine; break;
		case 1:
			m_csIP = csLine; break;
		case 2:
			m_iPort = atoi(csLine); break;
		}
	}
	return bStatus ? true : false;
}

bool CServerDef::Write(CStdioFile * psfOutput)
{
	CString csLine;
	csLine.Format("%s\n%s\n%ld\n", m_csName, m_csIP, m_iPort);
	psfOutput->WriteString(csLine);
	return true;
}