/*
 $Id: ItemScriptEditDlg.cpp,v 1.6 2002/05/14 20:03:34 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// ItemScriptEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "ItemScriptEditDlg.h"
#include "ColorSelectionDlg.h"
#include "NewScriptObjectDlg.h"
#include "RemoteConnection.h"
#include "SkillDef.h"
#include "SpellDef.h"
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const char * ItemAttributes [] = 
{
	"Identified",
	"Decay",
	"Newbie",
	"Always Moveable",
	"Never Moveable",
	"Magic",
	"Owned by town",
	"Invisible",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"Static",
};

const char * ItemTriggers [] =
{
	"DAMAGE",		// I have been damaged in some way
	"DCLICK",		// I have been dclicked.
	"DROPON_CHAR",		// I have been dropped on this char
	"DROPON_GROUND",		// I have been dropped on the ground here
	"DROPON_ITEM",		// I have been dropped on this item
	"EQUIP",		// I have been unequipped
	"PICKUP_GROUND",
	"PICKUP_PACK",	// picked up from inside some container.
	"SPELL",		// cast some spell on me.
	"STEP",			// I have been walked on.
	"TARGON_CHAR",
	"TARGON_GROUND",
	"TARGON_ITEM",	// I am being combined with an item
	"TIMER",		// My timer has expired.
	"UNEQUIP",		// i have been unequipped (or try to unequip)
};

const char * CharFlags [] =
{
	"Invulnerable", //			0x00000001	// Invulnerability
	"Dead", //			0x00000002
	"Paralyzed", //		0x00000004	// Paralyzed. (spell)
	"Invisible", //		0x00000008	// Invisible (spell).
	"Sleeping", //		0x00000010	// You look like a corpse ?
	"War Mode", //			0x00000020	// War mode on ?
	"Reactive", //		0x00000040	// have reactive armor on.
	"Poisoned", //		0x00000080	// Poison level is in the poison object
	"NightSight", //	0x00000100	// All a light to you
	"Magic Reflection", //	0x00000200	// Magic reflect on.
	"Polymorph", //		0x00000400	// We have polymorphed to another form.
	"Incognito", //		0x00000800	// Dont show skill titles
	"Spirit Speak", //	0x00001000	// I can hear ghosts clearly.
	"Insubstantial", //	0x00002000	// Ghost has not manifest. or GM hidden
	"Emote Action", //	0x00004000	// The creature will emote its actions to it's owners.
	"", // 0x00008000
	"Has Shield", //		0x00010000	// Using a shield
	"Script Play", //	0x00020000	// Playing a Script. (book script)
	"Stoned", //			0x00040000	// turned to stone.
	"Script Rec", //	0x00080000	// Recording a script.
	"Fly", //			0x00100000	// Flying or running ? (anim)
	"Drunk", //			0x00200000	// Drinking booze.
	"Hallucinating", //	0x00400000	// eat 'shrooms or bad food.
	"Hidden", //		0x00800000	// Hidden (non-magical)
	"Indoors", //		0x01000000	// we are covered from the rain.
	"Criminal", //		0x02000000	// The guards will attack me. (someone has called guards)
	"Conjured", //		0x04000000	// This creature is conjured and will expire. (leave no corpse or loot)
	"Pet", //			0x08000000	// I am a pet/hirling. check for my owner memory.
	"", // "Spawned", //		0x10000000	// I am spawned by a spawn item.
	"", // "SaveParity", //	0x20000000	// Has this char been saved or not ?
	"Ridden", //		0x40000000	// This is the horse. (don't display me) I am being ridden
	"OnHorse", //		0x80000000	// Mounted on horseback.
};

const char * CanFlags [] =
{
	"Ghost", //			0x0001	// Moves thru doors etc.
	"Swim", //			0x0002	// dolphin, elemental or is water
	"Walk", //			0x0004	// Can walk on land, climbed on walked over else Frozen by nature(Corpser) or can just swim
	"Passwalls", //		0x0008	// Walk thru walls.
	"Fly", //			0x0010	// Mongbat etc.
	"Fire immune", //	0x0020	// Has some immunity to fire ? (will walk into it (lava))
	"Indoors", //		0x0040	// Can go under roof. Not really used except to mask.
	"Move items", //	0x0080	// We can move items.
	"Equip", //			0x0100	// Can equip stuff. (humanoid)
	"Usehands", //		0x0200	// Can wield weapons (INT dependant), open doors ?, pick up/manipulate things
	"Eatraw", //		0x0400	// Carnivourous ?
	"Female", //		0x0800	// It is female by nature.
	"Nonhumanoid", //	0x1000	// Body type for combat messages.
	"Run", //			0x2000	// Can run (not needed if they can fly)
	"",
	"",
};

const char * Brains [] = 
{
	"None", 	// 0 = This should never really happen.
	"Animal",	// 1 = can be tamed.
	"Human",		// 2 = generic human.
	"Healer",	// 3 = can res.
	"Guard",		// 4 = inside cities
	"Banker",	// 5 = can open your bank box for you
	"Vendor",	// 6 = will sell from vendor boxes.
	"Beggar",	// 7 = begs.
	"Stable",	// 8 = will store your animals for you.
	"Theif",		// 9 = should try to steal ?
	"Monster",	// 10 = not tamable. normally evil.
	"Beserk",	// 11 = attack closest (blades", vortex)
	"Undead",	// 12 = disapears in the light.
	"Dragon",	// 13 = we can breath fire. may be tamable ? hirable ?
	"Vendor_offduty",	// 14 = "Sorry i'm not working right now. come back when my shop is open.
	"Crier",		// 15 = will speak periodically.
	"Conjured",	// 16 = elemental or other conjured creature.
};

const char * SpellFlags [] =
{
	"Animated", //   0x0001	// Evoke type cast or directed. (animation)
	"Object Target", //0x0002	// Need to target an object or char ?
	"Character Target", // 0x0004	// Needs to target a living thing
	"Location Target", // 0x0008	// Can just target a location.
	"Harmful", // 		0x0010	// The spell is in some way harmfull.
	"Bolt Effect", // 	0x0020	// Effect is a bolt to the target.
	"Effect is at the target", //	0x0040	// Effect is at the target.
	"Field effect", // 	0x0080	// create a field of stuff. (fire,poison,wall)
	"Summon", // 	0x0100	// summon a creature.
	"Beneficial", // 		0x0200	// The spell is a good spell. u intend to help to receiver.
	"",
	"",
	"",
	"",
	"",
	"Disabled", // 	0x8000
};

const char * ItemBoilerPlate =
"[ITEMDEF $DEFNAME$]\nNAME=$NAME$\nID=$ID$\nCATEGORY=<none>\nSUBSECTION=<none>\nDESCRIPTION=$NAME$\n\n@Create\n\n";

const char * TemplateBoilerPlate =
"[TEMPLATE $DEFNAME$]\nCATEGORY=Item Templates\nSUBSECTION=<none>\nDESCRIPTION=$NAME$\n\n";

const char * CharBoilerPlate =
"[CHARDEF $DEFNAME$]\nNAME=$NAME$\nID=$ID$\nCATEGORY=<none>\nSUBSECTION=<none>\nDESCRIPTION=$NAME$\n\n@Create\n\n";

const char * SpawnBoilerPlate =
"[SPAWN $DEFNAME$]\nCATEGORY=<none>\nSUBSECTION=<none>\nDESCRIPTION=$NAME$\n\n";

const char * TriggerBoilerPlate =
"[TRIG $DEFNAME$]\n\n";

const char * PlotBoilerPlate =
"[PLOT $DEFNAME$]\n\n";

const char * RegionBoilerPlate =
"[REGION $DEFNAME$]\n\n";

const char * SkillClassBoilerPlate =
"[SKILLCLASS $DEFNAME$]\nNAME=$NAME$\n\n";

const char * NameBoilerPlate =
"[NAMES $DEFNAME$]\n\n";

const char * WebpageBoilerPlate =
"[WEBPAGE $DEFNAME$]$EOL$WEBPAGEUPDATE=1800$EOL$WEBPAGESRC=spherestatusbase.html$EOL$WEBPAGEDST=c:\\status.html$EOL$WEBCLIENTLISTFORM=<tr><td>%NAME%\\t</td><td>%REGION.NAME%\\t</td></tr>\\n$EOL$WEBSERVERLISTFORM=<tr><td>%URLLINK%</td><td>%STATUS%</td></tr>$EOL$$EOL$";


/////////////////////////////////////////////////////////////////////////////
// CItemScriptEditDlg dialog

CItemScriptEditDlg::CItemScriptEditDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CItemScriptEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CItemScriptEditDlg)
	//}}AFX_DATA_INIT
	m_iBufferSize = 0;
	m_iBytesTransferred = 0;
	m_pszBuffer = NULL;
}

void CItemScriptEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CItemScriptEditDlg)
	DDX_Control(pDX, IDC_SCRIPTFILE, m_ceFile);
	DDX_Control(pDX, IDC_ITEMTREE, m_ctcTree);
	DDX_Control(pDX, IDC_PREVIEWVALUE, m_csPreview);
	DDX_Control(pDX, IDC_TAGHINT, m_csHint);
	DDX_Control(pDX, IDC_SCRIPTINSERT, m_cbScriptInsert);
	DDX_Control(pDX, IDC_ITEMFLAGS, m_clbFlags);
	DDX_Control(pDX, IDC_SELECTLIST, m_clcSelectList);
	DDX_Control(pDX, IDC_SELECTTREE, m_ctcSelectTree);
	DDX_Control(pDX, IDC_ITEMLIST, m_clcList);
	DDX_Control(pDX, IDC_ITEMCOLORPALETTE, m_cbPalette);
	DDX_Control(pDX, IDC_ITEMCOLORDISPLAY, m_csColorDisplay);
	DDX_Control(pDX, IDOK, m_cbOK);
	DDX_Control(pDX, IDCANCEL, m_cbCancelDlg);
	DDX_Control(pDX, IDC_NEWITEM, m_cbNew);
	DDX_Control(pDX, IDC_EDITITEM, m_cbEdit);
	DDX_Control(pDX, IDC_CANCELITEM, m_cbCancel);
	DDX_Control(pDX, IDC_APPLYITEM, m_cbApply);
	DDX_Control(pDX, IDC_ITEMTEXT, m_creScript);
	DDX_Control(pDX, IDC_ITEMMINVALUE, m_ceMinValue);
	DDX_Control(pDX, IDC_ITEMMAXVALUE, m_ceMaxValue);
	DDX_Control(pDX, IDC_ITEMCOMBO, m_ccbCombo);
	DDX_Control(pDX, IDC_ITEMCHECK, m_cbCheck);
	DDX_Control(pDX, IDC_ITEM_TAGS, m_clbTags);
	DDX_Control(pDX, IDC_ITEM_EDIT_DISPLAY, m_csDisplay);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CItemScriptEditDlg, CDialog)
	//{{AFX_MSG_MAP(CItemScriptEditDlg)
	ON_BN_CLICKED(IDC_NEWITEM, OnNewitem)
	ON_BN_CLICKED(IDC_EDITITEM, OnEdititem)
	ON_BN_CLICKED(IDC_APPLYITEM, OnApplyitem)
	ON_BN_CLICKED(IDC_CANCELITEM, OnCancelitem)
	ON_BN_CLICKED(IDOK, OnOk)
	ON_NOTIFY(TVN_SELCHANGED, IDC_SELECTTREE, OnSelchangedSelecttree)
	ON_NOTIFY(NM_CLICK, IDC_SELECTLIST, OnClickSelectlist)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_SELECTLIST, OnItemchangedSelectlist)
	ON_EN_CHANGE(IDC_ITEMTEXT, OnChangeItemtext)
	ON_BN_CLICKED(IDC_SCRIPTINSERT, OnScriptinsert)
	ON_LBN_SELCHANGE(IDC_ITEM_TAGS, OnSelchangeItemTags)
	ON_BN_CLICKED(IDC_ITEMCHECK, OnItemcheck)
	ON_CBN_SELCHANGE(IDC_ITEMCOMBO, OnSelchangeItemcombo)
	ON_LBN_SELCHANGE(IDC_ITEMFLAGS, OnSelchangeItemflags)
	ON_EN_CHANGE(IDC_ITEMMAXVALUE, OnChangeItemmaxvalue)
	ON_EN_CHANGE(IDC_ITEMMINVALUE, OnChangeItemminvalue)
	ON_NOTIFY(NM_CLICK, IDC_ITEMLIST, OnClickItemlist)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_ITEMLIST, OnItemchangedItemlist)
	ON_BN_CLICKED(IDC_ITEMCOLORPALETTE, OnItemcolorpalette)
	ON_NOTIFY(TVN_SELCHANGED, IDC_ITEMTREE, OnSelchangedItemtree)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CItemScriptEditDlg message handlers

BOOL CItemScriptEditDlg::OnInitDialog() 
{
	CWaitCursor hourglass;
	CDialog::OnInitDialog();
	FillItems();
	FillTags();
	m_hPaletteIcon = AfxGetApp()->LoadIcon(IDI_PALETTE);
	this->m_cbPalette.SetIcon(m_hPaletteIcon);
	this->m_creScript.SetEventMask(m_creScript.GetEventMask() | ENM_CHANGE);
	EnableFields(0);
	EnableEditButtons(FALSE);
	this->m_creScript.LimitText(0x7FFFFFFF);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CItemScriptEditDlg::PreTranslateMessage(MSG* pMsg) 
{
	// Find out which window has the focus
	CWnd * pFocus = (CWnd *) this->GetFocus();
	if ( pFocus->GetSafeHwnd() == this->m_creScript.GetSafeHwnd() )
	{
		if ( pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_TAB)
		{
			this->m_creScript.ReplaceSel("\t", TRUE);
			return TRUE;
		}
	}
	return CDialog::PreTranslateMessage(pMsg);
}

void CItemScriptEditDlg::FillTags()
{
	this->m_clbTags.ResetContent();
	switch ( this->ScriptType() )
	{
	case TYPE_ITEM:
		FillItemTags();
		break;
	case TYPE_TEMPLATE:
		FillTemplateTags();
		break;
	case TYPE_CHAR:
		FillCharTags();
		break;
	case TYPE_SPAWN:
		FillSpawnTags();
		break;
	case TYPE_TRIGGER:
	case TYPE_PLOT:
	case TYPE_REGION:
		FillTriggerTags();
		break;
	case TYPE_SKILL:
		FillSkillTags();
		break;
	case TYPE_SPELL:
		FillSpellTags();
		break;
	default:
		break;
	}
	return;
}


void CItemScriptEditDlg::OnNewitem() 
{
	m_bMode = true;
	HTREEITEM hSel= this->m_ctcSelectTree.GetSelectedItem();
	if ( m_ctcSelectTree.GetItemText(hSel) == "Webpages" ) 
	{
		int i = Main->m_pScripts->m_aWebpages.GetSize() + 1;
		ClearBuffer();
		CString csScript = _T(WebpageBoilerPlate);
		CString csDef;
		csDef.Format("%ld", i);
		csScript.Replace("$DEFNAME$", csDef);
		CString csEOL;
		csEOL.Format("%c%c", 0x0d, 0x0a);
		csScript.Replace("$EOL$", csEOL);
		ClearBuffer();
		m_pszBuffer = new char[csScript.GetLength() + 1];
		memset(m_pszBuffer, 0x00, csScript.GetLength() + 1);
		memcpy(this->m_pszBuffer, LPCTSTR(csScript), csScript.GetLength());
		CHARRANGE cr;
		cr.cpMax = -1;
		cr.cpMin = 0;
		m_creScript.SetSel(cr);
		this->m_creScript.ReplaceSel(m_pszBuffer, FALSE);
		this->m_ceFile.SetWindowText("sphere.ini");
		EnableEditButtons(TRUE);
		return;
	}
	int iSel = this->m_clcSelectList.GetNextItem(-1, LVNI_SELECTED);
	if ( iSel == -1 )
	{
		AfxMessageBox("You need to select an item to base your new item on");
		return;
	}
	CTObject * pBase = (CTObject*) m_clcSelectList.GetItemData(iSel);
	if ( !pBase )
		return;
	if ( pBase->m_bType == TYPE_SKILL )
	{
		AfxMessageBox("You cannot create new skills");
		return;
	}
	if ( pBase->m_bType > TYPE_CONFIG )
	{
		AfxMessageBox("You cannot create new configuration blocks");
		return;
	}
	CNewScriptObjectDlg dlg;
	dlg.m_bType = pBase->m_bType;
	if (dlg.DoModal() == IDOK)
	{
		// The current script window should have all of the information about the base object in it
		DWORD dwID = GetScriptID();
		CString csID;
		if ( dwID > 0xfff )
			csID.Format("0%04x", dwID);
		else
			csID.Format("%04x", dwID);
		CString csScript;
		if ( !dlg.m_bMode )
		{
			switch ( pBase->m_bType )
			{
			case TYPE_ITEM:
				{
					// Grab the current buffer
					ClearBuffer();
					EDITSTREAM stream;
					stream.dwCookie = (DWORD) this;
					stream.dwError = 0;
					stream.pfnCallback = writeBuffer;
					this->m_creScript.StreamOut(SF_TEXT, stream);

					csScript = _T(this->m_pszBuffer);
					CString csValue, csTemp;
					csValue.Format("[ITEMDEF %s]", dlg.m_csDef);
					csTemp.Format("%s%s", csValue, csScript.Mid(csScript.Find("]") + 1));
					csScript = csTemp;
					ClearBuffer();
					m_pszBuffer = new char[csScript.GetLength() + 1];
					memset(m_pszBuffer, 0x00, csScript.GetLength() + 1);
					memcpy(this->m_pszBuffer, LPCTSTR(csScript), csScript.GetLength());
					CHARRANGE cr;
					cr.cpMax = -1;
					cr.cpMin = 0;
					m_creScript.SetSel(cr);
					this->m_creScript.ReplaceSel(m_pszBuffer, FALSE);
					this->SelectTag("DUPELIST");
					this->m_creScript.ReplaceSel("", FALSE);
					this->SelectTag("DEFNAME");
					this->m_creScript.ReplaceSel("", FALSE);
					this->SelectTag("ID");
					csTemp.Format("ID=%s%c%c", csID, 0x0d, 0x0a);
					this->m_creScript.ReplaceSel(csTemp, FALSE);
					this->SelectTag("NAME");
					csTemp.Format("NAME=%s%c%c", dlg.m_csName, 0x0d, 0x0a);
					this->m_creScript.ReplaceSel(csTemp, FALSE);
					this->SelectTag("DESCRIPTION");
					csTemp.Format("DESCRIPTION=%s%c%c", dlg.m_csName, 0x0d, 0x0a);
					this->m_creScript.ReplaceSel(csTemp, FALSE);
				}
				break;
			case TYPE_CHAR:
				{
					// Grab the current buffer
					ClearBuffer();
					EDITSTREAM stream;
					stream.dwCookie = (DWORD) this;
					stream.dwError = 0;
					stream.pfnCallback = writeBuffer;
					this->m_creScript.StreamOut(SF_TEXT, stream);

					csScript = _T(this->m_pszBuffer);
					CString csValue, csTemp;
					csValue.Format("[CHARDEF %s]", dlg.m_csDef);
					csTemp.Format("%s%s", csValue, csScript.Mid(csScript.Find("]") + 1));
					csScript = csTemp;
					ClearBuffer();
					m_pszBuffer = new char[csScript.GetLength() + 1];
					memset(m_pszBuffer, 0x00, csScript.GetLength() + 1);
					memcpy(this->m_pszBuffer, LPCTSTR(csScript), csScript.GetLength());
					CHARRANGE cr;
					cr.cpMax = -1;
					cr.cpMin = 0;
					m_creScript.SetSel(cr);
					this->m_creScript.ReplaceSel(m_pszBuffer, FALSE);
					this->SelectTag("DEFNAME");
					this->m_creScript.ReplaceSel("", FALSE);
					this->SelectTag("ID");
					csTemp.Format("ID=%s%c%c", csID, 0x0d, 0x0a);
					this->m_creScript.ReplaceSel(csTemp, FALSE);
					this->SelectTag("NAME");
					csTemp.Format("NAME=%s%c%c", dlg.m_csName, 0x0d, 0x0a);
					this->m_creScript.ReplaceSel(csTemp, FALSE);
					this->SelectTag("DESCRIPTION");
					csTemp.Format("DESCRIPTION=%s%c%c", dlg.m_csName, 0x0d, 0x0a);
					this->m_creScript.ReplaceSel(csTemp, FALSE);
				}
				break;
			case TYPE_TEMPLATE:
			case TYPE_SPAWN:
			case TYPE_TRIGGER:
			case TYPE_PLOT:
			case TYPE_REGION:
			case TYPE_SKILLCLASS:
			case TYPE_NAME:
				// These all use the boilerplate, really
				break;
			}
		}
		else
		{
			switch ( pBase->m_bType )
			{
			case TYPE_ITEM:
				csScript = _T(ItemBoilerPlate);
				break;
			case TYPE_CHAR:
				csScript = _T(CharBoilerPlate);
				break;
			case TYPE_TEMPLATE:
				csScript = _T(TemplateBoilerPlate);
				break;
			case TYPE_SPAWN:
				csScript = _T(SpawnBoilerPlate);
				break;
			case TYPE_TRIGGER:
				csScript = _T(TriggerBoilerPlate);
				break;
			case TYPE_PLOT:
				csScript = _T(PlotBoilerPlate);
				break;
			case TYPE_REGION:
				csScript = _T(RegionBoilerPlate);
				break;
			case TYPE_SKILLCLASS:
				csScript = _T(SkillClassBoilerPlate);
				break;
			case TYPE_NAME:
				csScript = _T(NameBoilerPlate);
				break;
			}
			csScript.Replace("$ID$", csID);
			csScript.Replace("$NAME$", dlg.m_csName);
			csScript.Replace("$DEFNAME$", dlg.m_csDef);
			CString csEOL;
			csEOL.Format("%c%c", 0x0d, 0x0a);
			csScript.Replace("\n", csEOL);
			ClearBuffer();
			m_pszBuffer = new char[csScript.GetLength() + 1];
			memset(m_pszBuffer, 0x00, csScript.GetLength() + 1);
			memcpy(this->m_pszBuffer, LPCTSTR(csScript), csScript.GetLength());
			CHARRANGE cr;
			cr.cpMax = -1;
			cr.cpMin = 0;
			m_creScript.SetSel(cr);
			this->m_creScript.ReplaceSel(m_pszBuffer, FALSE);

			// Use the boilerplate
		}
	}
	else
		return;
	EnableEditButtons(TRUE);
}

void CItemScriptEditDlg::OnEdititem() 
{
	m_bMode = false;
	int iSel = this->m_clcSelectList.GetNextItem(-1, LVNI_SELECTED);
	if ( iSel == -1 )
	{
		AfxMessageBox("You need to select an item to edit");
		return;
	}
	EnableEditButtons(TRUE);
	if ( !Main->m_pScripts->m_bNewFormat )
	{
		switch ( ScriptType() )
		{
		case TYPE_ITEM:
			this->m_ceFile.SetWindowText("sphereitem2.scp"); break;
		case TYPE_TEMPLATE:
			this->m_ceFile.SetWindowText("spheretemplate2.scp"); break;
		case TYPE_CHAR:
			this->m_ceFile.SetWindowText("spherechar2.scp"); break;
		case TYPE_TRIGGER:
		case TYPE_PLOT:
		case TYPE_REGION:
			this->m_ceFile.SetWindowText("spheretrig2.scp"); break;
		}
	}
}

void CItemScriptEditDlg::OnApplyitem() 
{
	if ( this->m_creScript.GetModify() )
	{
		// Make sure we have a valid entry in the filename field.
		CString csFilename;
		this->m_ceFile.GetWindowText(csFilename);
		// Grab the ID
		CString csValue = GetScriptValue();
		if ( csValue != "" )
		{
			CScriptUpdate * pUpdate = new CScriptUpdate;
			pUpdate->m_csData = _T(this->m_pszBuffer);
			pUpdate->m_csFilename = csFilename;
			pUpdate->m_csValue = csValue;
			pUpdate->m_bType = ScriptType();
			switch ( pUpdate->m_bType )
			{
			case TYPE_ITEM:
				if ( Main->m_pScripts->m_bNewFormat )
					csValue = csValue.Mid(csValue.Find("ITEMDEF ") + 8);
				else
					csValue = csValue.Mid(csValue.Find("[") + 1);
				break;
			case TYPE_TEMPLATE:
				if ( Main->m_pScripts->m_bNewFormat )
					csValue = csValue.Mid(csValue.Find("TEMPLATE ") + 9);
				else
					csValue = csValue.Mid(csValue.Find("[") + 1);
				break;
			case TYPE_CHAR:
				if ( Main->m_pScripts->m_bNewFormat )
					csValue = csValue.Mid(csValue.Find("CHARDEF ") + 8);
				else
					csValue = csValue.Mid(csValue.Find("[") + 1);
				break;
			case TYPE_SPAWN:
				csValue = csValue.Mid(csValue.Find("SPAWN ") + 6); break;
			case TYPE_SKILL:
				csValue = csValue.Mid(csValue.Find("SKILL ") + 6); break;
			case TYPE_SKILLCLASS:
				csValue = csValue.Mid(csValue.Find("SKILLCLASS ") + 11); break;
			case TYPE_SPELL:
				csValue = csValue.Mid(csValue.Find("Spell ") + 6); break;
			case TYPE_TRIGGER:
				csValue = csValue.Mid(csValue.Find("TRIG ") + 5); break;
			case TYPE_PLOT:
				csValue = csValue.Mid(csValue.Find("PLOTITEM ") + 9); break;
			case TYPE_REGION:
				csValue = csValue.Mid(csValue.Find("REGION ") + 7); break;
			case TYPE_WEBPAGE:
				csValue = csValue.Mid(csValue.Find("WEBPAGE ") + 8); break;
			}
			csValue = csValue.SpanExcluding("]");
			// Did we already update this guy?
			for ( int i = 0; i < this->m_aMods.GetSize(); i++ )
			{
				CScriptUpdate * pOld = (CScriptUpdate *) m_aMods.GetAt(i);
				if ( pOld->m_csValue == pUpdate->m_csValue )
				{
					m_aMods.RemoveAt(i);
					delete pOld;
					break;
				}
			}
			m_aMods.Add(pUpdate);
			CString csCategory, csSubsection, csDescription;
			csCategory = this->GetValue("CATEGORY");
			csDescription = this->GetValue("DESCRIPTION");
			csSubsection = this->GetValue("SUBSECTION");

			if ( m_bMode )
			{
				// this is a new item.
				CSObject * pItem = new CSObject;
				pItem->m_csFilename = csFilename;
				pItem->m_csValue = csValue;
				pItem->m_csCategory = csCategory;
				pItem->m_csDescription = csDescription;
				pItem->m_csSubsection = csSubsection;
				// NoIndex ?
				pItem->m_bType = ScriptType();
				pItem->m_csDupeItem = this->GetValue("DUPEITEM");
				m_aNewItems.Add((void *) pItem);
				switch ( pItem->m_bType )
				{
				case TYPE_ITEM:
				case TYPE_TEMPLATE:
					{
						if ( pItem->m_bType == TYPE_ITEM )
							Main->m_pScripts->m_aItems.Add((void*) pItem);
						else
							Main->m_pScripts->m_aTemplates.Add((void*) pItem);
						CCategory * pCat = FindCategory(&(Main->m_pScripts->m_olItems), pItem->m_csCategory, true);
						CSubsection * pSub = FindSubsection(pCat, pItem->m_csSubsection, true);
						if ( pSub->m_ItemList.IsEmpty() )
							pSub->m_ItemList.AddHead(pItem);
						else
							pSub->m_ItemList.AddTail(pItem);
						// This this may have changed classifications
						this->OnSelchangedSelecttree(NULL, NULL);
						// Go through the list and select the one we just added
						for ( int i = 0; i < m_clcSelectList.GetItemCount(); i++ )
						{
							CTObject * pObject = (CTObject *) this->m_clcSelectList.GetItemData(i);
							if ( pObject == pItem )
							{
								this->m_clcSelectList.SetItemState( i, LVIS_SELECTED, LVIS_SELECTED );
								this->m_clcSelectList.EnsureVisible( i, FALSE );
								break;
							}
						}
					}
					break;
				case TYPE_CHAR:
				case TYPE_SPAWN:
					{
						if ( pItem->m_bType == TYPE_CHAR )
							Main->m_pScripts->m_aNPCs.Add((void *) pItem);
						else
							Main->m_pScripts->m_tables.m_spawns.Add((void*)pItem);
						CCategory * pCat = FindCategory(&(Main->m_pScripts->m_olNPCs), pItem->m_csCategory, true);
						CSubsection * pSub = FindSubsection(pCat, pItem->m_csSubsection, true);
						if ( pSub->m_ItemList.IsEmpty() )
							pSub->m_ItemList.AddHead(pItem);
						else
							pSub->m_ItemList.AddTail(pItem);
						// This this may have changed classifications
						this->OnSelchangedSelecttree(NULL, NULL);
						// Go through the list and select the one we just added
						for ( int i = 0; i < m_clcSelectList.GetItemCount(); i++ )
						{
							CTObject * pObject = (CTObject *) this->m_clcSelectList.GetItemData(i);
							if ( pObject == pItem )
							{
								this->m_clcSelectList.SetItemState( i, LVIS_SELECTED, LVIS_SELECTED );
								this->m_clcSelectList.EnsureVisible( i, FALSE );
								break;
							}
						}
					}
					break;
				case TYPE_TRIGGER:
					{
						Main->m_pScripts->m_aTriggers.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_PLOT:
					{
						Main->m_pScripts->m_aPlotTrigs.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_REGION:
					{
						Main->m_pScripts->m_aRegionTrigs.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_SKILL:
					{
						Main->m_pScripts->m_tables.m_skills.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_SKILLCLASS:
					{
						Main->m_pScripts->m_tables.m_skillClasses.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_SPELL:
					{
						Main->m_pScripts->m_tables.m_spells.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_NAME:
					{
						Main->m_pScripts->m_aNames.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				case TYPE_WEBPAGE:
					{
						Main->m_pScripts->m_aWebpages.Add((void *)pItem);
						int index = this->m_clcSelectList.InsertItem(m_clcSelectList.GetItemCount(), pItem->m_csValue, 0);
						this->m_clcSelectList.SetItemData(index, (DWORD) pItem);
						this->m_clcSelectList.SetItemState( index, LVIS_SELECTED, LVIS_SELECTED);
						this->m_clcSelectList.EnsureVisible(index, FALSE);
					}
					break;
				default:
					break;
				}
			}
		}
	}
	LoadScript();
	EnableFields(0);
	EnableEditButtons(FALSE);
	UpdatePreview();
}

void CItemScriptEditDlg::OnCancelitem() 
{
	LoadScript();
	EnableFields(0);
	EnableEditButtons(FALSE);
	UpdatePreview();
}

void CItemScriptEditDlg::OnOk() 
{
	// We need to save our modifications.
	// Find out which files we need to mess with here
	CWaitCursor hourglass;
	if ( m_aMods.GetSize() > 0 )
	{
		CStringArray aFiles;
		for ( int i = 0; i < m_aMods.GetSize(); i++ )
		{
			CScriptUpdate * pUpdate = (CScriptUpdate *) m_aMods.GetAt(i);
			if ( pUpdate )
			{
				bool bNew = true;
				for ( int j = 0; j < aFiles.GetSize(); j++ )
				{
					if ( pUpdate->m_csFilename.CompareNoCase(aFiles.GetAt(j)) == 0 )
					{
						bNew = false;
						break;
					}
				}
				if ( bNew )
					aFiles.Add(pUpdate->m_csFilename);
			}
		}
		// Back up the files
		time_t now = time(NULL);
		for ( i = 0; i < aFiles.GetSize(); i++ )
		{
			// Make sure the original file even exists.
			struct _stat fileStat;
			memset(&fileStat, 0x00, sizeof(fileStat));
			if ( _stat(aFiles.GetAt(i), &fileStat) != -1 )
			{
				CString csBackupName;
				csBackupName.Format("%s.bkp.%ld", aFiles.GetAt(i), now);
				int iCount = 0;
				while ( !CopyFile(aFiles.GetAt(i), csBackupName, TRUE) && iCount < 5 )
				{
					Sleep(1000);
					time_t retry = time(NULL);
					csBackupName.Format("%s.bkp.%ld", aFiles.GetAt(i), retry);
					iCount++;
				}
			}
		}
		bool bRedoItems = false;
		bool bRedoChars = false;
		// Now update the files
		for ( i = 0; i < m_aMods.GetSize(); i++ )
		{
			CScriptUpdate * pUpdate = (CScriptUpdate *) m_aMods.GetAt(i);
			switch ( pUpdate->m_bType )
			{
			case TYPE_ITEM:
			case TYPE_TEMPLATE:
				bRedoItems = true;
				break;
			case TYPE_CHAR:
			case TYPE_SPAWN:
				bRedoChars = true;
			}
			if ( !pUpdate->Write() )
			{
			}
		}
		if ( Main->GetProfileType() == PROFILE_REMOTE )
		{
			CRemoteConnection remote;
			for ( i = 0; i < aFiles.GetSize(); i++ )
				remote.SendCustomFile(aFiles.GetAt(i), aFiles.GetAt(i));
		}
		// Re-read the files we just updated
		for ( i = 0; i < aFiles.GetSize(); i++ )
			Main->m_pScripts->LoadFile(aFiles.GetAt(i));
		if ( bRedoItems )
			Main->m_pScripts->RecategorizeItems();
		if ( bRedoChars )
			Main->m_pScripts->RecategorizeNPCs();
	}

	CDialog::OnOK();
}

void CItemScriptEditDlg::OnCancel() 
{
	if ( m_aMods.GetSize() != 0 )
	{
		CString csMessage;
		csMessage.Format("You have modified %ld script objects.  If you cancel now, you will lose your changes.", m_aMods.GetSize());
		if (AfxMessageBox(csMessage, MB_OKCANCEL | MB_ICONEXCLAMATION) == IDOK )
		{
			if ( m_aNewItems.GetSize() > 0 )
			{
				// Delete any new items we created
				bool bRedoItems = false;
				bool bRedoChars = false;
				for ( int i = 0; i < m_aNewItems.GetSize(); i++ )
				{
					CTObject * pItem = (CTObject *) m_aNewItems.GetAt(i);
					if ( pItem )
					{
						// Remove it from the table it just came out of
						switch ( pItem->m_bType )
						{
						case TYPE_ITEM:
							bRedoItems = true;
							Main->m_pScripts->m_aItems.Remove(pItem);
							break;
						case TYPE_CHAR:
							bRedoChars = true;
							Main->m_pScripts->m_aNPCs.Remove(pItem);
							break;
						case TYPE_SPAWN:
							bRedoChars = true;
							Main->m_pScripts->m_tables.m_spawns.Remove(pItem);
							break;
						case TYPE_TEMPLATE:
							bRedoItems = true;
							Main->m_pScripts->m_aTemplates.Remove(pItem);
							break;
						case TYPE_TRIGGER:
							Main->m_pScripts->m_aTriggers.Remove(pItem);
							break;
						case TYPE_PLOT:
							Main->m_pScripts->m_aPlotTrigs.Remove(pItem);
							break;
						case TYPE_REGION:
							Main->m_pScripts->m_aRegionTrigs.Remove(pItem);
							break;
						case TYPE_SKILL:
							Main->m_pScripts->m_tables.m_skills.Remove(pItem);
							break;
						case TYPE_SKILLCLASS:
							Main->m_pScripts->m_tables.m_skillClasses.Remove(pItem);
							break;
						case TYPE_SPELL:
							Main->m_pScripts->m_tables.m_spells.Remove(pItem);
							break;
						case TYPE_NAME:
							Main->m_pScripts->m_aNames.Remove(pItem);
							break;
						case TYPE_WEBPAGE:
							Main->m_pScripts->m_aWebpages.Remove(pItem);
							break;
						default:
							delete pItem;
							break;
						}
					}
				}
				m_aNewItems.RemoveAll();
				// Recategorize the items.
				if ( bRedoItems )
					Main->m_pScripts->RecategorizeItems();
				if ( bRedoChars )
					Main->m_pScripts->RecategorizeNPCs();
			}
		}
		else
			return;
	}
	CDialog::OnCancel();
}

void CItemScriptEditDlg::FillItems()
{
	CDrewskyApp * pApp = (CDrewskyApp *) AfxGetApp();
	HTREEITEM ItemParent, CharParent, TriggerParent, TableParent, ConfigParent;
	HTREEITEM CategoryParent;
	HTREEITEM SubsectionParent;
	TV_INSERTSTRUCT InsertItem;
	
	m_ctcSelectTree.DeleteAllItems();
	CString csType;
	InsertItem.item.mask = TVIF_TEXT;
	csType = "Character Types";
	InsertItem.item.pszText = (char *)LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	CharParent = this->m_ctcSelectTree.InsertItem(&InsertItem);

	InsertItem.item.mask = TVIF_TEXT;
	csType = "Item Types";
	InsertItem.item.pszText = (char *)LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	ItemParent = this->m_ctcSelectTree.InsertItem(&InsertItem);

	csType = "Trigger Types";
	InsertItem.item.pszText = (char *) LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	TriggerParent = this->m_ctcSelectTree.InsertItem(&InsertItem);

	csType = "Table Entries";
	InsertItem.item.pszText = (char *) LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	TableParent = this->m_ctcSelectTree.InsertItem(&InsertItem);

	csType = "Configuration";
	InsertItem.item.pszText = (char *) LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	ConfigParent = this->m_ctcSelectTree.InsertItem(&InsertItem);

	if (!pApp->m_pScripts->m_olItems.IsEmpty())
	{
		POSITION pos = pApp->m_pScripts->m_olItems.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) pApp->m_pScripts->m_olItems.GetNext(pos);
			InsertItem.item.mask = TVIF_TEXT;
			InsertItem.item.pszText = (char *)LPCTSTR(pCategory->m_csName);
			InsertItem.item.cchTextMax = pCategory->m_csName.GetLength();
			InsertItem.hParent = ItemParent;
			InsertItem.hInsertAfter = TVI_SORT;
			CategoryParent = this->m_ctcSelectTree.InsertItem(&InsertItem);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION sPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (sPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(sPos);
					InsertItem.item.mask = TVIF_TEXT;
					InsertItem.item.pszText = (char *)LPCTSTR(pSubsection->m_csName);
					InsertItem.item.cchTextMax = pSubsection->m_csName.GetLength();
					InsertItem.hParent = CategoryParent;
					InsertItem.hInsertAfter = TVI_SORT;
					SubsectionParent = this->m_ctcSelectTree.InsertItem(&InsertItem);
				}
			}
		}
	}
	if (!pApp->m_pScripts->m_olNPCs.IsEmpty())
	{
		POSITION pos = pApp->m_pScripts->m_olNPCs.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) pApp->m_pScripts->m_olNPCs.GetNext(pos);
			InsertItem.item.mask = TVIF_TEXT;
			InsertItem.item.pszText = (char *)LPCTSTR(pCategory->m_csName);
			InsertItem.item.cchTextMax = pCategory->m_csName.GetLength();
			InsertItem.hParent = CharParent;
			InsertItem.hInsertAfter = TVI_SORT;
			CategoryParent = this->m_ctcSelectTree.InsertItem(&InsertItem);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION sPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (sPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(sPos);
					InsertItem.item.mask = TVIF_TEXT;
					InsertItem.item.pszText = (char *)LPCTSTR(pSubsection->m_csName);
					InsertItem.item.cchTextMax = pSubsection->m_csName.GetLength();
					InsertItem.hParent = CategoryParent;
					InsertItem.hInsertAfter = TVI_SORT;
					SubsectionParent = this->m_ctcSelectTree.InsertItem(&InsertItem);
				}
			}
		}
	}
	if (pApp->m_pScripts->m_aTriggers.GetSize() > 0)
	{
		csType = "Items";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TriggerParent;
		InsertItem.hInsertAfter = TVI_SORT;
		HTREEITEM hItems = this->m_ctcSelectTree.InsertItem(&InsertItem);
	}
	if ( pApp->m_pScripts->m_aPlotTrigs.GetSize() > 0 )
	{
		csType = "Plots";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TriggerParent;
		InsertItem.hInsertAfter = TVI_SORT;
		HTREEITEM hPlots = this->m_ctcSelectTree.InsertItem(&InsertItem);
	}
	if ( pApp->m_pScripts->m_aRegionTrigs.GetSize() > 0 )
	{
		csType = "Regions";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TriggerParent;
		InsertItem.hInsertAfter = TVI_SORT;
		HTREEITEM hRegions = this->m_ctcSelectTree.InsertItem(&InsertItem);
	}
	// Table Stuff
	{
		csType = "Skills";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TableParent;
		InsertItem.hInsertAfter = TVI_SORT;
		this->m_ctcSelectTree.InsertItem(&InsertItem);

		csType = "Spells";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TableParent;
		InsertItem.hInsertAfter = TVI_SORT;
		this->m_ctcSelectTree.InsertItem(&InsertItem);

		csType = "Skill Classes";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TableParent;
		InsertItem.hInsertAfter = TVI_SORT;
		this->m_ctcSelectTree.InsertItem(&InsertItem);

		csType = "Names";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = TableParent;
		InsertItem.hInsertAfter = TVI_SORT;
		this->m_ctcSelectTree.InsertItem(&InsertItem);
	}
	// Config Stuff
	{
		csType = "Server Configuration";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = ConfigParent;
		InsertItem.hInsertAfter = TVI_SORT;
		this->m_ctcSelectTree.InsertItem(&InsertItem);

		csType = "Webpages";
		InsertItem.item.mask = TVIF_TEXT;
		InsertItem.item.pszText = (char*)LPCTSTR(csType);
		InsertItem.item.cchTextMax = csType.GetLength();
		InsertItem.hParent = ConfigParent;
		InsertItem.hInsertAfter = TVI_SORT;
		this->m_ctcSelectTree.InsertItem(&InsertItem);
	}
	this->m_clcSelectList.InsertColumn(0, "Description", LVCFMT_LEFT, 150, 0);
	this->m_clcSelectList.InsertColumn(1, "ID", LVCFMT_LEFT, 50, 1);

	// Now do it in the other item tree
	m_ctcTree.DeleteAllItems();
	InsertItem.item.mask = TVIF_TEXT;
	csType = "Character Types";
	InsertItem.item.pszText = (char *)LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	CharParent = this->m_ctcTree.InsertItem(&InsertItem);

	InsertItem.item.mask = TVIF_TEXT;
	csType = "Item Types";
	InsertItem.item.pszText = (char *)LPCTSTR(csType);
	InsertItem.item.cchTextMax = csType.GetLength();
	InsertItem.hParent = NULL;
	InsertItem.hInsertAfter = TVI_SORT;
	ItemParent = this->m_ctcTree.InsertItem(&InsertItem);
	if (!pApp->m_pScripts->m_olItems.IsEmpty())
	{
		POSITION pos = pApp->m_pScripts->m_olItems.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) pApp->m_pScripts->m_olItems.GetNext(pos);
			InsertItem.item.mask = TVIF_TEXT;
			InsertItem.item.pszText = (char *)LPCTSTR(pCategory->m_csName);
			InsertItem.item.cchTextMax = pCategory->m_csName.GetLength();
			InsertItem.hParent = ItemParent;
			InsertItem.hInsertAfter = TVI_SORT;
			CategoryParent = this->m_ctcTree.InsertItem(&InsertItem);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION sPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (sPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(sPos);
					InsertItem.item.mask = TVIF_TEXT;
					InsertItem.item.pszText = (char *)LPCTSTR(pSubsection->m_csName);
					InsertItem.item.cchTextMax = pSubsection->m_csName.GetLength();
					InsertItem.hParent = CategoryParent;
					InsertItem.hInsertAfter = TVI_SORT;
					SubsectionParent = this->m_ctcTree.InsertItem(&InsertItem);
				}
			}
		}
	}
	if (!pApp->m_pScripts->m_olNPCs.IsEmpty())
	{
		POSITION pos = pApp->m_pScripts->m_olNPCs.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) pApp->m_pScripts->m_olNPCs.GetNext(pos);
			InsertItem.item.mask = TVIF_TEXT;
			InsertItem.item.pszText = (char *)LPCTSTR(pCategory->m_csName);
			InsertItem.item.cchTextMax = pCategory->m_csName.GetLength();
			InsertItem.hParent = CharParent;
			InsertItem.hInsertAfter = TVI_SORT;
			CategoryParent = this->m_ctcTree.InsertItem(&InsertItem);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION sPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (sPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(sPos);
					InsertItem.item.mask = TVIF_TEXT;
					InsertItem.item.pszText = (char *)LPCTSTR(pSubsection->m_csName);
					InsertItem.item.cchTextMax = pSubsection->m_csName.GetLength();
					InsertItem.hParent = CategoryParent;
					InsertItem.hInsertAfter = TVI_SORT;
					SubsectionParent = this->m_ctcTree.InsertItem(&InsertItem);
				}
			}
		}
	}
	this->m_clcList.InsertColumn(0, "Description", LVCFMT_LEFT, 150, 0);
	this->m_clcList.InsertColumn(1, "ID", LVCFMT_LEFT, 50, 1);
}

void CItemScriptEditDlg::LoadScript()
{
	ClearBuffer();
	ClearScript();
	FillTags();
	int iSel = this->m_clcSelectList.GetNextItem(-1, LVNI_SELECTED);
	if ( iSel == -1 )
	{
		ClearScript();
		return;
	}
	CTObject * pObject = (CTObject *) this->m_clcSelectList.GetItemData(iSel);
	if ( !pObject )
		return;
	CString csTest;
	switch ( ScriptType() )
	{
	case TYPE_ITEM:
		if ( Main->m_pScripts->m_bNewFormat )
			csTest.Format("[%s%s]", "ITEMDEF ", pObject->m_csValue);
		else
			csTest.Format("[%s]", pObject->m_csValue);
		break;
	case TYPE_TEMPLATE:
		if ( Main->m_pScripts->m_bNewFormat )
			csTest.Format("[%s%s]", "TEMPLATE ", pObject->m_csValue);
		else
			csTest.Format("[%s]", pObject->m_csValue);
		break;
	case TYPE_CHAR:
		if ( Main->m_pScripts->m_bNewFormat )
			csTest.Format("[%s%s]", "CHARDEF ", pObject->m_csValue);
		else
			csTest.Format("[%s]", pObject->m_csValue);
		break;
	case TYPE_SPAWN:
		csTest.Format("[%s%s]", "SPAWN ", pObject->m_csValue);
		break;
	case TYPE_TRIGGER:
		csTest.Format("[%s%s]", "TRIG ", pObject->m_csValue);
		break;
	case TYPE_PLOT:
		csTest.Format("[%s%s]", "PLOTITEM ", pObject->m_csValue);
		break;
	case TYPE_REGION:
		csTest.Format("[%s%s]", "REGION ", pObject->m_csValue);
		break;
	case TYPE_SKILL:
		csTest.Format("[%s%s]", "SKILL ", pObject->m_csValue);
		break;
	case TYPE_SPELL:
		csTest.Format("[%s%s]", "Spell ", pObject->m_csValue);
		break;
	case TYPE_SKILLCLASS:
		csTest.Format("[%s%s]", "SKILLCLASS ", pObject->m_csValue);
		break;
	case TYPE_NAME:
		csTest.Format("[%s%s]", "NAMES ", pObject->m_csValue);
		break;
	case TYPE_SPHERE:
		csTest.Format("[SPHERE]");
		break;
	case TYPE_SERVERS:
		csTest.Format("[SERVERS]");
		break;
	case TYPE_BLOCKIP:
		csTest.Format("[BLOCKIP]");
		break;
	case TYPE_WEBPAGE:
		csTest.Format("[%s%s]", "WEBPAGE ", pObject->m_csValue);
		break;
	default:
		return;
	}
	this->m_ceFile.SetWindowText(pObject->m_csFilename);
	CStdioFile csfFile;
	// Before we check the file, make sure it's not in our array of modifications
	for ( int i = 0; i < m_aMods.GetSize(); i++ )
	{
		CScriptUpdate * pUpdate = (CScriptUpdate *) m_aMods.GetAt(i);
		if ( pUpdate->m_csValue.CompareNoCase(csTest) == 0 )
		{
			// Fill our buffer with this guy instead.
			this->m_ceFile.SetWindowText(pUpdate->m_csFilename);
			m_iBufferSize = pUpdate->m_csData.GetLength();
			m_pszBuffer = new char [m_iBufferSize];
			memset(m_pszBuffer, 0x00, m_iBufferSize);
			memcpy((void *) m_pszBuffer, LPCTSTR(pUpdate->m_csData), pUpdate->m_csData.GetLength());
			m_iBytesTransferred = 0;
			EDITSTREAM stream;
			stream.dwCookie = (DWORD) this;
			stream.dwError = 0;
			stream.pfnCallback = readBuffer;
			this->m_creScript.StreamIn(SF_TEXT, stream);
			return;
		}
	}
	if ( !csfFile.Open(pObject->m_csFilename, CFile::modeRead | CFile::shareDenyNone | CFile::typeBinary ) )
	{
		AfxMessageBox("Unable to open file");
		return;
	}
	int iStatus = 4096;
	int iBufSize = 0;
	char * pszBuffer = NULL;
	// Read the whole file into memory
	while ( iStatus == 4096 )
	{
		int iOldSize = iBufSize;
		iBufSize += 4096;
		char * pszNewBuffer = new char [iBufSize];
		memset(pszNewBuffer, 0x00, iBufSize);
		memcpy(pszNewBuffer, pszBuffer, iOldSize);
		delete [] pszBuffer;
		pszBuffer = pszNewBuffer;
		char szBuffer[4096];
		memset(szBuffer, 0x00, sizeof(szBuffer));
		iStatus = csfFile.Read((void *) szBuffer, 4096);
		memcpy(pszBuffer + iOldSize, &szBuffer[0], iStatus);
	}
	// Find the script in our buffer
	m_iBufferSize = 0;
	if ( strstr(pszBuffer, LPCTSTR(csTest)) != NULL )
	{
		int position = (strstr(pszBuffer, LPCTSTR(csTest)) - pszBuffer);
		m_iBufferSize = iBufSize - position;
		m_pszBuffer = new char [m_iBufferSize];
		memset(m_pszBuffer, 0x00, sizeof(m_pszBuffer));
		memcpy(m_pszBuffer, pszBuffer + position, m_iBufferSize);
		// Check for the next script entry
		for ( int i = 1; i < m_iBufferSize; i++ )
		{
			if ( ( m_pszBuffer[i] == '[') || (m_pszBuffer[i] == 0x00 ) )
			{
				m_pszBuffer[i] = 0x00;
				m_iBufferSize = i;
				break;
			}
		}
	}
	delete [] pszBuffer;
	csfFile.Close();
	m_iBytesTransferred = 0;
	EDITSTREAM stream;
	stream.dwCookie = (DWORD) this;
	stream.dwError = 0;
	stream.pfnCallback = readBuffer;
	this->m_creScript.StreamIn(SF_TEXT, stream);
}


DWORD CALLBACK readBuffer(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	CItemScriptEditDlg * pDlg = (CItemScriptEditDlg *) dwCookie;

	memset((void*) pbBuff, 0x00, cb);
	int iCopySize = pDlg->m_iBufferSize - pDlg->m_iBytesTransferred;
	if ( iCopySize > cb )
		iCopySize = cb;
	memcpy(pbBuff, &(pDlg->m_pszBuffer[pDlg->m_iBytesTransferred]), iCopySize);
	pDlg->m_iBytesTransferred += iCopySize;	
	*pcb = iCopySize;
	if ( pDlg->m_iBytesTransferred > pDlg->m_iBufferSize )
		return 1;
	else
		return 0;
}

DWORD CALLBACK writeBuffer(DWORD dwCookie,  LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	CItemScriptEditDlg * pDlg = (CItemScriptEditDlg *) dwCookie;

	int iOldSize = pDlg->m_iBufferSize;
	pDlg->m_iBufferSize += cb;
	char * pszNewBuffer = new char [pDlg->m_iBufferSize + 1];
	memset(pszNewBuffer, 0x00, pDlg->m_iBufferSize + 1);
	if ( iOldSize )
		memcpy(pszNewBuffer, pDlg->m_pszBuffer, iOldSize);
	memcpy(pszNewBuffer + iOldSize, pbBuff, cb);
	*pcb = cb;
	delete [] pDlg->m_pszBuffer;
	pDlg->m_pszBuffer = pszNewBuffer;
	return 0;
}

void CItemScriptEditDlg::OnSelchangedSelecttree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CDrewskyApp * pApp = (CDrewskyApp *) AfxGetApp();
	this->m_clcSelectList.SetHotItem(-1);

	HTREEITEM hSelectedItem = NULL;
	HTREEITEM hParentItem = NULL;
	this->m_clcSelectList.DeleteAllItems();
	ClearScript();

	// Do the bold/nobold thing
	if (pNMTreeView != NULL)
	{
		HTREEITEM hOldItem = pNMTreeView->itemOld.hItem;
		if (hOldItem != NULL)
			this->m_ctcSelectTree.SetItemState(hOldItem, 0, TVIS_BOLD);
	}

	hSelectedItem = this->m_ctcSelectTree.GetSelectedItem();
	this->m_ctcSelectTree.SetItemState(hSelectedItem, TVIS_BOLD, TVIS_BOLD);
	hParentItem = this->m_ctcSelectTree.GetParentItem(hSelectedItem);
	HTREEITEM hRoot = NULL;
	if ( hParentItem )
			hRoot = m_ctcSelectTree.GetParentItem( hParentItem );
	if ( !hRoot )
		hRoot = hParentItem;
	CString csRoot = m_ctcSelectTree.GetItemText( hRoot );


	if (hParentItem != NULL)
	{
		CString csParent = this->m_ctcSelectTree.GetItemText(hParentItem);
		CString csSelected = this->m_ctcSelectTree.GetItemText(hSelectedItem);
		CCategory * pCategory = NULL;
		if ( csRoot == "Character Types" )
			pCategory = FindCategory(&pApp->m_pScripts->m_olNPCs, csParent, false);
		else if ( csRoot == "Item Types" )
			pCategory = FindCategory(&pApp->m_pScripts->m_olItems, csParent, false);
		else if ( csRoot == "Trigger Types" )
		{
			// Which type is it?
			if ( csSelected == "Items" )
			{
				int iCount = 0;
				for (int i = 0; i < Main->m_pScripts->m_aTriggers.GetSize(); i++ )
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_aTriggers.GetAt(i);
					if ( !pObject )
						continue;
					int index = this->m_clcSelectList.InsertItem(iCount, pObject->m_csValue, 0);
					this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
					iCount++;
				}
			}
			else if ( csSelected == "Plots" )
			{
				int iCount = 0;
				for (int i = 0; i < Main->m_pScripts->m_aPlotTrigs.GetSize(); i++ )
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_aPlotTrigs.GetAt(i);
					if ( !pObject )
						continue;
					int index = this->m_clcSelectList.InsertItem(iCount, pObject->m_csValue, 0);
					this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
					iCount++;
				}
			}
			else if ( csSelected == "Regions" )
			{
				int iCount = 0;
				for (int i = 0; i < Main->m_pScripts->m_aRegionTrigs.GetSize(); i++ )
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_aRegionTrigs.GetAt(i);
					if ( !pObject )
						continue;
					int index = this->m_clcSelectList.InsertItem(iCount, pObject->m_csValue, 0);
					this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
					iCount++;
				}
			}
			return;
		}
		else if ( csRoot == "Table Entries" )
		{
			if( csSelected == "Skills" )
			{
				int iCount = 0;
				for ( int i = 0; i < Main->m_pScripts->m_tables.m_skills.GetSize(); i++)
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_tables.m_skills.GetAt(i);
					if ( !pObject )
						continue;
					CSkillDef skill;
					skill.m_csFilename = pObject->m_csFilename;
					skill.m_csValue = pObject->m_csValue;
					skill.Load();
					int index = this->m_clcSelectList.InsertItem(iCount, skill.m_csKey, 0);
					this->m_clcSelectList.SetItemText(index, 1, pObject->m_csValue);
					this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
					iCount++;
				}
			}
			else if ( csSelected == "Spells" )
			{
				int iCount = 0;
				for ( int i = 0; i < Main->m_pScripts->m_tables.m_spells.GetSize(); i++)
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_tables.m_spells.GetAt(i);
					if ( !pObject )
						continue;
					CSpellDef spell;
					spell.m_csFilename = pObject->m_csFilename;
					spell.m_csValue = pObject->m_csValue;
					spell.Load();
					int index = this->m_clcSelectList.InsertItem(iCount, spell.m_csName, 0);
					this->m_clcSelectList.SetItemText(index, 1, pObject->m_csValue);
					this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
					iCount++;
				}
			}
			else if ( csSelected == "Skill Classes" )
			{
				int iCount = 0;
				for ( int i = 0; i < Main->m_pScripts->m_tables.m_skillClasses.GetSize(); i++)
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_tables.m_skillClasses.GetAt(i);
					if ( !pObject )
						continue;
					CTSkillClass skillclass;
					skillclass.m_csFilename = pObject->m_csFilename;
					skillclass.m_csValue = pObject->m_csValue;
					skillclass.Load();
					int index = this->m_clcSelectList.InsertItem(iCount, skillclass.m_csName, 0);
					this->m_clcSelectList.SetItemText(index, 1, pObject->m_csValue);
					this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
					iCount++;
				}
			}
			else if ( csSelected == "Names" )
			{
				int iCount = 0;
				for ( int i = 0; i < Main->m_pScripts->m_aNames.GetSize(); i++ )
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_aNames.GetAt(i);
					if ( !pObject )
						continue;
					int index = this->m_clcSelectList.InsertItem(iCount, pObject->m_csValue, 0);
					this->m_clcSelectList.SetItemData(index, (DWORD) pObject);
					iCount++;
				}
			}
		}
		else if ( csRoot == "Configuration" )
		{
			if ( csSelected == "Server Configuration" )
			{
				int index = this->m_clcSelectList.InsertItem(0, "Main Configuration", 0);
				this->m_clcSelectList.SetItemData(index, (DWORD) (CTObject *) &(Main->m_pScripts->m_tables.m_Sphere));
				index = this->m_clcSelectList.InsertItem(1, "Blocked IPs", 0);
				this->m_clcSelectList.SetItemData(index, (DWORD) (CTObject *) &(Main->m_pScripts->m_tables.m_BlockIP));
				index = this->m_clcSelectList.InsertItem(2, "Servers", 0);
				this->m_clcSelectList.SetItemData(index, (DWORD) (CTObject *) &(Main->m_pScripts->m_tables.m_Servers));
			}
			else if ( csSelected == "Webpages" )
			{
				int iCount = 0;
				for ( int i = 0; i < Main->m_pScripts->m_aWebpages.GetSize(); i++ )
				{
					CTObject * pObject = (CTObject *) Main->m_pScripts->m_aWebpages.GetAt(i);
					if ( !pObject )
						continue;
					int index = this->m_clcSelectList.InsertItem(iCount, pObject->m_csValue, 0);
					this->m_clcSelectList.SetItemData(index, (DWORD) pObject);
					iCount++;
				}
			}
		}
		else
			return;
		if (pCategory == NULL)
			return;
		CSubsection * pSubsection = FindSubsection(pCategory, csSelected);
		if (!pSubsection->m_ItemList.IsEmpty())
		{
			POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
			int iCount = 0;
			while (pos != NULL)
			{
				CSObject * pObject = (CSObject *) pSubsection->m_ItemList.GetNext(pos);
				int index = this->m_clcSelectList.InsertItem(iCount, pObject->m_csDescription, 0);
				this->m_clcSelectList.SetItemText(index, 1, pObject->m_csValue);
				this->m_clcSelectList.SetItemData(index, (DWORD)pObject);
				iCount++;
			}
		}
	}
}

void CItemScriptEditDlg::OnClickSelectlist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	//LoadScript();
}

void CItemScriptEditDlg::OnItemchangedSelectlist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	if ( !pNMListView )
		return;
	if (pNMListView->uNewState & LVNI_SELECTED)
	{
		// this is the one that has been selected
		LoadScript();
		return;
	}
}

void CItemScriptEditDlg::OnChangeItemtext() 
{
	// Grab the current buffer
	ClearBuffer();
	EDITSTREAM stream;
	stream.dwCookie = (DWORD) this;
	stream.dwError = 0;
	stream.pfnCallback = writeBuffer;
	this->m_creScript.StreamOut(SF_TEXT, stream);

	// Now that we have the current buffer, what the hell do we do with it?
	switch ( ScriptType() )
	{
	case TYPE_ITEM:
		DrawItem(GetScriptID(), &m_csDisplay, (WORD) DefLookup(GetValue("COLOR"))); break;
	case TYPE_CHAR:
		DrawChar(GetScriptID(), &m_csDisplay, (WORD) DefLookup(GetValue("COLOR"))); break;
		break;
	default:
		break;
	}
}

void CItemScriptEditDlg::ClearBuffer()
{
	if ( m_iBufferSize )
		delete [] m_pszBuffer;
	m_iBufferSize = 0;
	m_pszBuffer = NULL;
}

CItemScriptEditDlg::~CItemScriptEditDlg()
{
	ClearBuffer();
	for ( int i = 0; i < this->m_aMods.GetSize(); i++ )
	{
		CScriptUpdate * pUpdate = (CScriptUpdate *) m_aMods.GetAt(i);
		if ( pUpdate )
			delete pUpdate;
	}
}

DWORD CItemScriptEditDlg::GetScriptID()
{
	if ( !m_pszBuffer )
		return -1;
	CString csBuffer = _T(m_pszBuffer);
	CString csID;
	CString csEOL;
	csEOL.Format("%c", 0x0d);
	if ( csBuffer.Find("ONTRIGGER") != -1 )
		csBuffer.SetAt(csBuffer.Find("ONTRIGGER"), 0);
	DWORD dwMaxBase = 0xFFFFFFFF;
	switch ( ScriptType() )
	{
	case TYPE_ITEM:
		{
			if ( csBuffer.Find("ITEMDEF ") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("ITEMDEF ") + 8);
			else if ( csBuffer.Find("[") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("[") + 1);
		}
		dwMaxBase = 0x4000;
		break;
	case TYPE_TEMPLATE:
		{
			if ( csBuffer.Find("TEMPLATE ") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("TEMPLATE ") + 9);
			else if ( csBuffer.Find("[") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("[") + 1);
		}
		break;
	case TYPE_CHAR:
		{
			if ( csBuffer.Find("CHARDEF ") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("CHARDEF ") + 8);
			else if ( csBuffer.Find("[") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("[") + 1);
		}
		dwMaxBase = 0x1000;
		break;
	case TYPE_SPAWN:
		{
			if ( csBuffer.Find("SPAWN ") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("SPAWN ") + 6);
			else if ( csBuffer.Find("[") != -1 )
				csID = csBuffer.Mid(csBuffer.Find("[") + 1);
		}
		break;
	}
	csID = csID.SpanExcluding("]");
	if ( ScriptType() != TYPE_SPAWN )
	{
		if ( GetValue("ID") != "" )
			csID = GetValue("ID");
	}
	if ( ahextoi(csID) > dwMaxBase )
		return 0;
	if ( csID != "" )
		return DefLookup(csID);
	else
		return -1;
}

void CItemScriptEditDlg::EnableEditButtons(BOOL bMode)
{
	this->m_cbApply.EnableWindow(bMode);
	this->m_cbCancel.EnableWindow(bMode);
	this->m_cbNew.EnableWindow(!bMode);
	this->m_cbEdit.EnableWindow(!bMode);
	this->m_creScript.EnableWindow(bMode);
	this->m_clbTags.EnableWindow(bMode);
	this->m_cbCancelDlg.EnableWindow(!bMode);
	this->m_cbOK.EnableWindow(!bMode);
	this->m_cbScriptInsert.EnableWindow(FALSE);
	this->m_csPreview.SetWindowText("");
	this->m_clcSelectList.EnableWindow(!bMode);
	this->m_ctcSelectTree.EnableWindow(!bMode);
	
	// Show/Hide windows
	this->m_cbApply.ShowWindow(bMode ? SW_SHOW : SW_HIDE);
	this->m_cbCancel.ShowWindow(bMode ? SW_SHOW : SW_HIDE);
	this->m_cbNew.ShowWindow(bMode ? SW_HIDE : SW_SHOW);
	this->m_cbEdit.ShowWindow(bMode ? SW_HIDE : SW_SHOW);
	this->m_cbCancelDlg.ShowWindow(bMode ? SW_HIDE : SW_SHOW);
	this->m_cbOK.ShowWindow(bMode ? SW_HIDE : SW_SHOW);

	if ( Main->m_pScripts->m_bNewFormat )
		m_ceFile.EnableWindow(bMode);

}

void CItemScriptEditDlg::OnScriptinsert() 
{
	// Find the appropriate place in the script to place this entry.
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	int iType = (dwData >> 16) & 0xFFFF;
	bool bTagExists = false;
	switch ( iType )
	{
	case 0:
		switch ( ScriptType() )
		{
		case TYPE_ITEM:
			bTagExists = SelectTag(sm_szItemKeysDef[iTag]);
			break;
		case TYPE_TEMPLATE:
			bTagExists = SelectTag(sm_szTemplateKeysDef[iTag]);
			break;
		case TYPE_CHAR:
			bTagExists = SelectTag(sm_szCharKeysDef[iTag]);
			break;
		case TYPE_SPAWN:
			bTagExists = SelectTag(sm_szSpawnKeysDef[iTag]);
			break;
		case TYPE_TRIGGER:
		case TYPE_PLOT:
		case TYPE_REGION:
			break;
		case TYPE_SKILL:
			bTagExists = SelectTag(knownSkillTags[iTag]);
			break;
		case TYPE_SPELL:
		default:
			break;
		}
		break;
	case 1:
		switch ( ScriptType() )
		{
		case TYPE_ITEM:
			bTagExists = SelectTag(sm_szItemKeysCreate[iTag - ITAG_COLOR]);
			break;
		case TYPE_CHAR:
			bTagExists = SelectTag(sm_szCharKeysCreate[iTag - CTAG_STR]);
			break;
		case TYPE_TEMPLATE:
		case TYPE_SPAWN:
		case TYPE_TRIGGER:
		case TYPE_PLOT:
		case TYPE_REGION:
		case TYPE_SKILL:
		case TYPE_SPELL:
		default:
			break;
		}
		break;
	case 2:
		// Don't select this
		break;
	}
	if (!bTagExists)
		PositionCaret(iType);
	CString csText, csTemp;
	this->m_csPreview.GetWindowText(csTemp);
	csText.Format("%s%c%c", csTemp, 0x0d, 0x0a);
	m_creScript.ReplaceSel(csText, TRUE);
}

void CItemScriptEditDlg::OnSelchangeItemTags() 
{
	this->m_ceMaxValue.SetWindowText("");
	this->m_ceMinValue.SetWindowText("");
	this->m_ccbCombo.ResetContent();
	this->m_cbCheck.SetCheck(0);
	UpdatePreview();

	switch ( ScriptType() )
	{
	case TYPE_ITEM:
		ChangeItemTag();
		break;
	case TYPE_TEMPLATE:
		ChangeTemplateTag();
		break;
	case TYPE_CHAR:
		ChangeCharTag();
		break;
	case TYPE_SPAWN:
		ChangeSpawnTag();
		break;
	case TYPE_TRIGGER:
	case TYPE_PLOT:
	case TYPE_REGION:
		ChangeTriggerTag();
		break;
	case TYPE_SKILL:
		ChangeSkillTag();
		break;
	case TYPE_SPELL:
		ChangeSpellTag();
		break;
	default:
		break;
	}
	UpdatePreview();
}

void CItemScriptEditDlg::EnableFields(int iType)
{
	this->m_ceMaxValue.EnableWindow(FALSE);
	this->m_ceMinValue.EnableWindow(FALSE);
	this->m_ccbCombo.EnableWindow(FALSE);
	this->m_clbFlags.EnableWindow(FALSE);
	this->m_cbPalette.EnableWindow(FALSE);
	this->m_clcList.EnableWindow(FALSE);
	this->m_ctcTree.EnableWindow(FALSE);
	this->m_cbCheck.EnableWindow(FALSE);
	this->m_ccbCombo.ResetContent();
	this->m_clbFlags.ResetContent();
	this->m_ceMaxValue.SetWindowText("");
	this->m_ceMinValue.SetWindowText("");
	this->m_clbFlags.ResetContent();

	if ( iType & MODE_MAX )
		this->m_ceMaxValue.EnableWindow(TRUE);
	if ( iType & MODE_MIN )
		this->m_ceMinValue.EnableWindow(TRUE);
	if ( iType & MODE_SELECT )
		this->m_ccbCombo.EnableWindow(TRUE);
	if ( iType & MODE_FLAG )
		this->m_clbFlags.EnableWindow(TRUE);
	if ( iType & MODE_COLOR )
		this->m_cbPalette.EnableWindow(TRUE);
	if ( iType & MODE_ITEM )
	{
		this->m_clcList.EnableWindow(TRUE);
		this->m_ctcTree.EnableWindow(TRUE);
	}
	if ( iType & MODE_BOOL )
		this->m_cbCheck.EnableWindow(TRUE);
}

void CItemScriptEditDlg::OnItemcheck() 
{
	int iState = this->m_cbCheck.GetCheck();
	if ( iState )
		this->m_cbCheck.SetWindowText("True");
	else
		this->m_cbCheck.SetWindowText("False");
	UpdatePreview();
}

void CItemScriptEditDlg::UpdatePreview()
{
	// Find out which tag is selected.
	switch ( ScriptType() )
	{
	case TYPE_ITEM:
		PreviewItemTag();
		break;
	case TYPE_TEMPLATE:
		PreviewTemplateTag();
		break;
	case TYPE_CHAR:
		PreviewCharTag();
		break;
	case TYPE_SPAWN:
		PreviewSpawnTag();
		break;
	case TYPE_TRIGGER:
	case TYPE_PLOT:
	case TYPE_REGION:
		PreviewTriggerTag();
		break;
	case TYPE_SKILL:
		PreviewSkillTag();
		break;
	case TYPE_SPELL:
		PreviewSpellTag();
		break;
	default:
		break;
	}
	CString csLine;
	this->m_csPreview.GetWindowText(csLine);
	if ( csLine != "" )
		this->m_cbScriptInsert.EnableWindow(TRUE);
	else
		this->m_cbScriptInsert.EnableWindow(FALSE);
}

void CItemScriptEditDlg::OnSelchangeItemcombo() 
{
	UpdatePreview();
}

void CItemScriptEditDlg::OnSelchangeItemflags() 
{
	UpdatePreview();
}

void CItemScriptEditDlg::OnChangeItemmaxvalue() 
{
	UpdatePreview();
}

void CItemScriptEditDlg::OnChangeItemminvalue() 
{
	UpdatePreview();
}

void CItemScriptEditDlg::OnClickItemlist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdatePreview();
}

void CItemScriptEditDlg::OnItemchangedItemlist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	OnClickItemlist(pNMHDR, pResult);	
}

bool CItemScriptEditDlg::SelectTag(CString csEntry)
{
	int line = FindTagLine(csEntry);
	if ( line == -1 )
		return false;
	// Search the control for the specified tag, and select the entire line, if it is there.
	CHARRANGE range;
	range.cpMax = m_creScript.LineIndex(line);
	range.cpMin = m_creScript.LineIndex(line + 1);
	m_creScript.SetSel(range);
	return true;
}

void CItemScriptEditDlg::PositionCaret(int iType)
{
	// iType = 0 -> Base value
	// iType = 1 -> @Create
	// iType = 2 -> Triggers
	// iType = 3 -> Last Line
	// Find the index of the beginning of triggers
	int lastLine = m_creScript.GetLineCount();
	lastLine++;
	int triggerLine = FindTagLine("ONTRIGGER");
	if ( triggerLine == -1 )
		triggerLine = lastLine - 1;
	int createLine = FindTagLine("@Create");
	if ( createLine != -1 )
		createLine--;
	int line = -1;
	switch ( iType )
	{
	case 0:
		line = createLine; break;
	case 1:
		line = triggerLine; break;
	case 2:
	case 3:
		line = lastLine; break;
	}

	// Make sure we are at the beginning of a line
	if ( line == -1 )
	{
		CHARRANGE cr;
		m_creScript.GetSel(cr);
		line = this->m_creScript.LineFromChar(cr.cpMin);
	}
	if ( iType == 1 && createLine == -1 )
	{
		// We need to put the "@Create" in here.
		PositionCaret(0);
		CString csCreate;
		csCreate.Format("%c%c@Create%c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		m_creScript.ReplaceSel(csCreate, TRUE);
		PositionCaret(1);
		return;
	}
	if ( line == 0 )
		line = lastLine;
	this->m_creScript.SetSel(m_creScript.LineIndex(line), m_creScript.LineIndex(line));
	if ( iType == 2 )
	{
		CString csEOL;
		csEOL.Format("%c%c%c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		m_creScript.ReplaceSel(csEOL, TRUE);
	}
}

void CItemScriptEditDlg::FillTypes()
{
	this->m_ccbCombo.ResetContent();
	for (int i = 0; i < ITEM_QTY; i++)
	{
		int index = this->m_ccbCombo.AddString(ItemTypeName[i]);
		this->m_ccbCombo.SetItemData(index, i);
	}
}

void CItemScriptEditDlg::FillItemCategories()
{
	this->m_ccbCombo.ResetContent();
	CDrewskyApp * pApp = (CDrewskyApp *) AfxGetApp();
	POSITION pos = pApp->m_pScripts->m_olItems.GetHeadPosition();
	while ( pos )
	{
		CCategory * pCat = (CCategory *) pApp->m_pScripts->m_olItems.GetNext(pos);
		if ( pCat )
			this->m_ccbCombo.AddString(pCat->m_csName);
	}
}

void CItemScriptEditDlg::FillItemSubsections()
{
	this->m_ccbCombo.ResetContent();
	CDrewskyApp * pApp = (CDrewskyApp *) AfxGetApp();
	CString csValue = this->GetValue("CATEGORY");
	CCategory * pCategory = FindCategory(&pApp->m_pScripts->m_olItems, csValue, false);
	if ( !pCategory )
		return;
	POSITION pos = pCategory->m_SubsectionList.GetHeadPosition();
	while ( pos )
	{
		CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(pos);
		if ( pSubsection )
			this->m_ccbCombo.AddString(pSubsection->m_csName);
	}
}

void CItemScriptEditDlg::FillItemIDs()
{
	this->m_ccbCombo.ResetContent();
	switch ( ScriptType() )
	{
	case TYPE_ITEM:
		{
			for ( int i = 0; i < Main->m_pScripts->m_aItems.GetSize(); i++ )
			{
				CSObject * pObject = (CSObject *) Main->m_pScripts->m_aItems.GetAt(i);
				if ( pObject && ahextoi(pObject->m_csValue) < 0x4000 )
				{
					CString csEntry;
					csEntry.Format("%s [%s]", pObject->m_csDescription, pObject->m_csValue);
					m_ccbCombo.AddString(csEntry);
				}
			}
		}
		break;
	case TYPE_CHAR:
		{
			for ( int i = 0; i < Main->m_pScripts->m_aNPCs.GetSize(); i++ )
			{
				CSObject * pObject = (CSObject *) Main->m_pScripts->m_aNPCs.GetAt(i);
				if ( pObject && ahextoi(pObject->m_csValue) < 0x1000 )
				{
					CString csEntry;
					csEntry.Format("%s [%s]", pObject->m_csDescription, pObject->m_csValue);
					m_ccbCombo.AddString(csEntry);
				}
			}
		}
	}
}

void CItemScriptEditDlg::FillColors()
{
	this->m_ccbCombo.ResetContent();
	for ( int i = 0; i < Main->m_pScripts->m_aDefs.GetSize(); i++ )
	{
		CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(i);
		if ( pDef && pDef->m_csGroup.Find("COLOR") != -1 )
			m_ccbCombo.AddString(pDef->m_csValue);
	}
}

void CItemScriptEditDlg::FillDupes()
{
	this->m_ccbCombo.ResetContent();
	if ( ScriptType() != TYPE_ITEM )
		return;
	for ( int i = 0; i < Main->m_pScripts->m_aItems.GetSize(); i++ )
	{
		CSObject * pObject = (CSObject *) Main->m_pScripts->m_aItems.GetAt(i);
		if ( pObject && pObject->m_csDupeItem == "")
		{
			CString csEntry;
			csEntry.Format("%s [%s]", pObject->m_csDescription, pObject->m_csValue);
			m_ccbCombo.AddString(csEntry);
		}
	}
}

void CItemScriptEditDlg::FillItemTriggers()
{
	m_ccbCombo.ResetContent();
	if ( ScriptType() != TYPE_ITEM )
		return;
	for ( int i = 0; i < 15; i++ )
		this->m_ccbCombo.AddString(ItemTriggers[i]);
}

void CItemScriptEditDlg::OnItemcolorpalette() 
{
	CColorSelectionDlg dlg;
	if ( dlg.DoModal() == IDOK )
	{
		WORD wColor = dlg.GetColorIndex();
		CString csColor;
		csColor.Format("0%04x", wColor);
		this->m_ceMaxValue.SetWindowText(csColor);
		UpdatePreview();
	}
}

CString CItemScriptEditDlg::GetValue(CString csTag)
{
	CString csEOL;
	csEOL.Format("%c%c", 0x0d, 0x0a);
	int line = FindTagLine(csTag);
	char szBuffer[MAX_PATH];
	memset(szBuffer, 0x00, sizeof(szBuffer));
	m_creScript.GetLine(line, szBuffer, MAX_PATH - 1);
	CString csLine = _T(szBuffer);
	CString csValue;
	if ( csLine.Find("=") != -1 )
		csValue = csLine.Mid(csLine.Find("=") + 1);
	csValue = csValue.SpanExcluding(csEOL);
	return csValue;
}

int CItemScriptEditDlg::FindTagLine(CString csTag)
{
	// First, we need to find if there are any triggers in this item.
	int iPos = -1;
	const char * pszEntry;
	FINDTEXTEX search;
	if ( csTag != "ONTRIGGER" )
	{
		CString csTrigger = _T("ONTRIGGER");
		pszEntry = LPCTSTR(csTrigger);
		search.chrg.cpMin = 0;
		search.chrg.cpMax = iPos;
		search.lpstrText = (char *) pszEntry ;
		iPos = this->m_creScript.FindText(FR_WHOLEWORD, &search);
	}

	pszEntry = LPCTSTR(csTag);
	search.chrg.cpMin = 0;
	search.chrg.cpMax = iPos;
	search.lpstrText = (char *) pszEntry ;
	iPos = 0;
	int line = -1;
	while (iPos != -1 )
	{
		iPos = this->m_creScript.FindText(FR_WHOLEWORD, &search);
		if ( iPos == -1 )
			return line;
		line = this->m_creScript.LineFromChar(iPos);
		// Is this at the beginning of a line?  If not, skip it
		if ( iPos != m_creScript.LineIndex(line) )
		{
			search.chrg.cpMin = iPos + 1;
			continue;
		}
		else
			break;
	}
	return line;
}

void CItemScriptEditDlg::OnSelchangedItemtree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CDrewskyApp * pApp = (CDrewskyApp *) AfxGetApp();
	this->m_clcList.SetHotItem(-1);

	HTREEITEM hSelectedItem = NULL;
	HTREEITEM hParentItem = NULL;
	this->m_clcList.DeleteAllItems();

	// Do the bold/nobold thing
	if (pNMTreeView != NULL)
	{
		HTREEITEM hOldItem = pNMTreeView->itemOld.hItem;
		if (hOldItem != NULL)
			this->m_ctcTree.SetItemState(hOldItem, 0, TVIS_BOLD);
	}

	hSelectedItem = this->m_ctcTree.GetSelectedItem();
	this->m_ctcTree.SetItemState(hSelectedItem, TVIS_BOLD, TVIS_BOLD);
	hParentItem = this->m_ctcTree.GetParentItem(hSelectedItem);

	if (hParentItem != NULL)
	{
		CString csParent = this->m_ctcTree.GetItemText(hParentItem);
		CString csSelected = this->m_ctcTree.GetItemText(hSelectedItem);
		CCategory * pCategory = FindCategory(&pApp->m_pScripts->m_olItems, csParent, false);
		if (pCategory == NULL)
			pCategory = FindCategory(&pApp->m_pScripts->m_olNPCs, csParent, false);
		if (pCategory == NULL)
			return;
		CSubsection * pSubsection = FindSubsection(pCategory, csSelected);
		if (!pSubsection->m_ItemList.IsEmpty())
		{
			POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
			int iCount = 0;
			while (pos != NULL)
			{
				CSObject * pObject = (CSObject *) pSubsection->m_ItemList.GetNext(pos);
				int index = this->m_clcList.InsertItem(iCount, pObject->m_csDescription, 0);
				this->m_clcList.SetItemText(index, 1, pObject->m_csValue);
				this->m_clcList.SetItemData(index, (DWORD)pObject);
				iCount++;
			}
		}
	}
}

void CItemScriptEditDlg::FillFlags()
{
	this->m_clbFlags.ResetContent();
	for ( int i = 0; i < 16; i++ )
	{
		if ( strcmp(ItemAttributes[i], "") != 0 )
		{
			int index = this->m_clbFlags.AddString(ItemAttributes[i]);
			this->m_clbFlags.SetItemData(index, 1 << i );
		}
	}
}

void CItemScriptEditDlg::SetFlags(DWORD dwFlags)
{
	for ( int i = 0; i < this->m_clbFlags.GetCount(); i++ )
	{
		DWORD dwThis = m_clbFlags.GetItemData(i);
		m_clbFlags.SetSel(i, dwThis & dwFlags);
	}
}

DWORD CItemScriptEditDlg::GetFlags()
{
	int * array;
	int size = this->m_clbFlags.GetSelCount();
	if ( size <= 0 )
		return 0;
	DWORD dwFlags = 0;
	array = new int [size];
	this->m_clbFlags.GetSelItems(size, array);
	for ( int i = 0; i < size; i++ )
	{
		DWORD dwThis = m_clbFlags.GetItemData(array[i]);
		dwFlags |= dwThis;
	}
	delete [] array;
	return dwFlags;
}

void CItemScriptEditDlg::FillWeaponSkills()
{
	this->m_ccbCombo.ResetContent();
}

void CItemScriptEditDlg::FillSkills()
{
	this->m_ccbCombo.ResetContent();
	for ( int i = 0; i < Main->m_pScripts->m_tables.m_skills.GetSize(); i++ )
	{
		CTObject * pBase = (CTObject *) Main->m_pScripts->m_tables.m_skills.GetAt(i);
		if (pBase)
		{
			CTSkillDef skill;
			skill.m_csFilename = pBase->m_csFilename;
			skill.m_csValue = pBase->m_csValue;
			skill.Load();
			CString csKey = skill.m_csKey;
			csKey.MakeUpper();
			this->m_ccbCombo.AddString(csKey);
		}
	}
}

void CItemScriptEditDlg::FillLayers()
{
	this->m_ccbCombo.ResetContent();
}

void CItemScriptEditDlg::FillSpells()
{
	this->m_ccbCombo.ResetContent();
	for ( int i = 0; i < Main->m_pScripts->m_tables.m_spells.GetSize(); i++ )
	{
		CTObject * pBase = (CTObject *) Main->m_pScripts->m_tables.m_spells.GetAt(i);
		if ( pBase )
		{
			CTSpellDef spell;
			spell.m_csFilename = pBase->m_csFilename;
			spell.m_csValue = pBase->m_csValue;
			spell.Load();
			this->m_ccbCombo.AddString(spell.m_csName);
		}
	}
}

void CItemScriptEditDlg::FillItemEvents()
{
	this->m_ccbCombo.ResetContent();
	this->m_ccbCombo.AddString("@Create");
	this->m_ccbCombo.AddString("@Delete");
}

CString CItemScriptEditDlg::GetScriptValue()
{
	char szBuffer[MAX_PATH];
	memset(szBuffer, 0x00, sizeof(szBuffer));
	m_creScript.GetLine(0, szBuffer, MAX_PATH - 1);
	CString csOutput = _T(szBuffer);
	CString csEOL;
	csEOL.Format("%c%c", 0x0d, 0x0a);
	csOutput = csOutput.SpanExcluding(csEOL);
	if ( csOutput.Find("[") == -1 || csOutput.Find("]") == -1 )
		return "";
	return csOutput;
}

void CItemScriptEditDlg::ClearScript()
{
	CHARRANGE cr;
	cr.cpMax = -1;
	cr.cpMin = 0;
	m_creScript.SetSel(cr);
	this->m_creScript.ReplaceSel("", FALSE);
	DrawItem(GetScriptID(), &m_csDisplay, (WORD) DefLookup(GetValue("COLOR")));
	return;
}

int CItemScriptEditDlg::ScriptType()
{
	int iSel = this->m_clcSelectList.GetNextItem(-1, LVNI_SELECTED);
	if ( iSel == -1 )
	{
		HTREEITEM hSel = m_ctcSelectTree.GetSelectedItem();
		if ( m_ctcSelectTree.GetItemText(hSel) == "Webpages" )
			return TYPE_WEBPAGE;
		return -1;
	}
	CTObject * pObject = (CTObject *) this->m_clcSelectList.GetItemData(iSel);
	if ( !pObject )
		return -1;
	return pObject->m_bType;

}

void CItemScriptEditDlg::FillContainers()
{
	for ( int i = 0; i < Main->m_pScripts->m_aDefs.GetSize(); i++ )
	{
		CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(i);
		if ( !pDef )
			continue;
		if ( pDef->m_csGroup.CompareNoCase("ITEMS_CONTAINERS") == 0 )
			this->m_ccbCombo.AddString(pDef->m_csValue);
	}
}

void CItemScriptEditDlg::FillCharIDs()
{
	FillItemIDs();
}

void CItemScriptEditDlg::FillCharFlags()
{
	this->m_clbFlags.ResetContent();
	for ( int i = 0; i < 32; i++ )
	{
		if ( strcmp(CharFlags[i], "") != 0 )
		{
			int index = this->m_clbFlags.AddString(CharFlags[i]);
			this->m_clbFlags.SetItemData(index, 1 << i );
		}
	}
}

void CItemScriptEditDlg::FillCanFlags()
{
	this->m_clbFlags.ResetContent();
	for ( int i = 0; i < 16; i++ )
	{
		if ( strcmp(CanFlags[i], "") != 0 )
		{
			int index = this->m_clbFlags.AddString(CanFlags[i]);
			this->m_clbFlags.SetItemData(index, 1 << i );
		}
	}
}

void CItemScriptEditDlg::FillCharIcons()
{
	for ( int i = 0; i < Main->m_pScripts->m_aDefs.GetSize(); i++ )
	{
		CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(i);
		if ( !pDef )
			continue;
		if ( pDef->m_csGroup.CompareNoCase("ITEMS_CONTAINERS") == 0 )
			this->m_ccbCombo.AddString(pDef->m_csValue);
	}
}

void CItemScriptEditDlg::FillCharResources()
{
}

void CItemScriptEditDlg::FillFoodtypes()
{
}

void CItemScriptEditDlg::FillCharEvents()
{
	// List the speech files ??
}

void CItemScriptEditDlg::FillCharSpeech()
{
	// List the speech files
}

void CItemScriptEditDlg::FillBrains()
{
	this->m_ccbCombo.ResetContent();
	for (int i = 0; i < 17; i++)
	{
		int index = this->m_clbFlags.AddString(Brains[i]);
		this->m_ccbCombo.SetItemData(index, i);
	}
}

void CItemScriptEditDlg::FillCharTriggers()
{
	this->m_ccbCombo.ResetContent();
	this->m_ccbCombo.AddString("@Create");
}

void CItemScriptEditDlg::FillItemTags()
{
	for ( int i = 0; i < ITAG_QTY; i++ )
	{
		if ( i == ITAG_MORE1 )
			continue;
		int index = 0;
		int iType = 0;
		if ( i < ITAG_COLOR )
		{
			index = this->m_clbTags.AddString(sm_szItemKeysDef[i]);
			iType = 0;
		}
		else if ( i < ITAG_ON )
		{
			index = this->m_clbTags.AddString(sm_szItemKeysCreate[i - ITAG_COLOR]);
			iType = 1;
		}
		else
		{
			index = this->m_clbTags.AddString(sm_szItemKeysEnd[i - ITAG_ON]);
			iType = 2;
		}
		DWORD dwData = i | (iType << 16);
		this->m_clbTags.SetItemData(index, dwData);
	}
}

void CItemScriptEditDlg::FillTemplateTags()
{
	for ( int i = 0; i < TTAG_QTY; i++ )
	{
		int index = 0;
		int iType = 0;
		index = this->m_clbTags.AddString(sm_szTemplateKeysDef[i]);
		switch ( i )
		{
		case TTAG_BUY:
		case TTAG_SELL:
		case TTAG_ITEM:
		case TTAG_ITEMNEWBIE:
			iType = 3;
			break;
		default:
			iType = 0;
			break;
		}
		DWORD dwData = i | (iType << 16);
		this->m_clbTags.SetItemData(index, dwData);
	}
}

void CItemScriptEditDlg::FillCharTags()
{
	for ( int i = 0; i < CTAG_QTY; i++ )
	{
		if ( i == CTAG_ATT ||
			i == CTAG_CONTAINER ||
			i == CTAG_ONTRIGGER ||
			i == CTAG_ANIM ||
			i == CTAG_SOUND ||
			i == CTAG_BODY ||
			i == CTAG_EVENTS ||
			i == CTAG_SPEECH ||
			i == CTAG_TIMER)
			continue;
		int index = 0;
		int iType = 0;
		if ( i < CTAG_STR )
		{
			index = this->m_clbTags.AddString(sm_szCharKeysDef[i]);
			iType = 0;
		}
		else if ( i < CTAG_CONTAINER )
		{
			index = this->m_clbTags.AddString(sm_szCharKeysCreate[i - CTAG_STR]);
			iType = 1;
		}
		else
		{
			index = this->m_clbTags.AddString(sm_szCharKeysEnd[i - CTAG_CONTAINER]);
			iType = 2;
		}
		DWORD dwData = i | (iType << 16);
		this->m_clbTags.SetItemData(index, dwData);
	}
}

void CItemScriptEditDlg::ChangeItemTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csMessage;
	CString csTag;

	if ( iTag < ITAG_COLOR )
		csTag = sm_szItemKeysDef[iTag];
	else if ( iTag < ITAG_ON )
		csTag = sm_szItemKeysCreate[iTag - ITAG_COLOR];
	else
		csTag = sm_szItemKeysEnd[iTag - ITAG_ON];
	switch (iTag)
	{
	case ITAG_DEFNAME:
		EnableFields(MODE_MAX);
		csMessage.Format("String associated with this item\nCannot contain spaces\n\nUsage: %s=<text>", csTag);
		break;
	case ITAG_DEFNAME2:
		EnableFields(MODE_MAX);
		csMessage.Format("Alternate string associated with this item\nCannot contain spaces\n\nUsage: %s=<text>", csTag);
		break;
	case ITAG_NAME:
		EnableFields(MODE_MAX);
		csMessage.Format("Name of the item in game\n\nUsage: %s=<text>", csTag);
		break;
	case ITAG_ID:
		EnableFields(MODE_SELECT);
		FillItemIDs();
		csMessage.Format("Artwork ID to be used for the item\n\nUsage: %s=<hex>", csTag);
		break;
	case ITAG_TYPE:
		EnableFields(MODE_SELECT);
		FillTypes();
		csMessage.Format("Type of the item, or trigger to associate with the item\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_VALUE:
		EnableFields(MODE_MAX);
		csMessage.Format("Base Value of the item to vendors.  Can be modified by item attributes.\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_DYE:
		EnableFields(MODE_BOOL);
		csMessage.Format("Item is dyeable or not.  Default = 0 (False)\n\nUsage: %s=0|1", csTag);
		break;
	case ITAG_PILE:
		EnableFields(MODE_BOOL);
		csMessage.Format("Item is stackable or not.  Default = 0 (False)\n\nUsage: %s=0|1", csTag);
		break;
	case ITAG_REPAIR:
		EnableFields(MODE_BOOL);
		csMessage.Format("Item is repairable.  Default = 0 (False)\n\nUsage: %s=0|1", csTag);
		break;
	case ITAG_REPLICATE:
		EnableFields(MODE_BOOL);
		csMessage.Format("When crafting this item, make as many as we have raw materials to build.  Default = 0 (False)\n\nUsage: %s=0|1", csTag);
	case ITAG_REQSTR:
		EnableFields(MODE_MAX);
		csMessage.Format("Strength required to wield the item\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_SKILL:
		EnableFields(MODE_SELECT);
		FillWeaponSkills();
		csMessage.Format("Skill used when fighting with this item\n\nUsage: %s=<skill>", csTag);
		break;
	case ITAG_SKILLMAKE:
		EnableFields(MODE_MAX|MODE_SELECT);
		FillSkills();
		csMessage.Format("Skill required to make this item\n\nUsage: %s=<skill> <float>(, ...)", csTag);
		break;
	case ITAG_TWOHANDS:
		EnableFields(MODE_BOOL);
		csMessage.Format("Weapon requires two hands to wield.  Default is false\n\nUsage: %s=0|1", csTag);
		break;
	case ITAG_WEIGHT:
		EnableFields(MODE_MAX);
		csMessage.Format("Weight of the item\nFor weapons, lower weight = faster attack\n\nUsage: %s=<integer> or <float>", csTag);
		break;
	case ITAG_DUPELIST:
		EnableFields(MODE_MAX);
		csMessage.Format("Items which inherit values from this\nUsed with the .flip command\n\nUsage: %s=<itemid>...", csTag);
		break;
	case ITAG_DUPEITEM:
		EnableFields(MODE_SELECT);
		FillDupes();
		csMessage.Format("Item to inherit all ingame values from\n\nUsage: %s=<itemid>", csTag);
		break;
	case ITAG_RESOURCES:
		EnableFields(MODE_MAX|MODE_ITEM);
		csMessage.Format("Resources required to make this item\n\nUsage: %s=<amount> <itemdef>(, ...)", csTag);
		break;
	case ITAG_DAM:
		EnableFields(MODE_MAX|MODE_MIN);
		csMessage.Format("Damage rating for weapons\nMeaningless for other items\n\nUsage: %s={<integer> <integer>}", csTag);
		break;
	case ITAG_ARMOR:
		EnableFields(MODE_MAX);
		csMessage.Format("Armor rating for the item\nActual applied value depends on body coverage\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_MULTIREGION:
		EnableFields(MODE_MAX);
		csMessage.Format("Extent of the area of a multi object\n\nUsage: %s=<x1>,<x2>,<y1>,<y2>", csTag);
		break;
	case ITAG_COMPONENT:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Item used in a multi object\n\nUsage: %s=<itemid>,<dx>,<dy>,<dz>", csTag);
		break;
	case ITAG_CATEGORY:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemCategories();
		csMessage.Format("Axis category classification\n\nUsage: %s=<text>", csTag);
		break;
	case ITAG_DESCRIPTION:
		EnableFields(MODE_MAX);
		csMessage.Format("Axis description\n\nUsage: %s=<text>", csTag);
		break;
	case ITAG_SUBSECTION:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemSubsections();
		csMessage.Format("Axis sub-category classification\n\nUsage: %s=<text>", csTag);
		break;
	case ITAG_NOINDEX:
		EnableFields(MODE_BOOL);
		csMessage.Format("The presence of this tag indicates that it should not be listed in Axis.  Has no arguments.");
		break;
	// Create
	case ITAG_COLOR:
		EnableFields(MODE_COLOR|MODE_SELECT|MODE_MAX);
		FillColors();
		csMessage.Format("Color of the item\n\nUsage: %s=<hex>", csTag);
		break;
	case ITAG_AMOUNT:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Amount value for this item when created\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_ATTR:
		EnableFields(MODE_FLAG);
		FillFlags();
		csMessage.Format("Attribute flags set on the item\n\nUsage: %s=<flags>", csTag);
		break;
	case ITAG_FRUIT:
		EnableFields(MODE_ITEM);
		csMessage.Format("Fruit item that is harvested from this crop\n\nUsage: %s=<itemdef>", csTag);
		break;
	case ITAG_HITPOINTS:	// for weapons
		EnableFields(MODE_MAX|MODE_MIN);
		csMessage.Format("Amount of damage the item can take before being destroyed\n\nUsage: %s={<integer> <integer>}", csTag);
		break;
	case ITAG_LAYER:
		EnableFields(MODE_SELECT);
		FillLayers();
		csMessage.Format("Layer this item gets equipped to\n\nUsage: %s=<layer>");
		break;
	case ITAG_MORE:
		EnableFields(MODE_MAX);
		csMessage.Format("Meaning depends on item type\n\nUsage: %s=<hex>", csTag);
		break;
	case ITAG_MORE2:
		EnableFields(MODE_MAX);
		csMessage.Format("Meaning depends on item type\n\nUsage: %s=<hex>", csTag);
		break;
	case ITAG_MOREP:
		EnableFields(MODE_MAX);
		csMessage.Format("Meaning depends on item type\n\nUsage: %s=<DWORD DWORD BYTE>", csTag);
		break;
	case ITAG_MOREX:
		EnableFields(MODE_MAX|MODE_SELECT);
		FillSpells();
		csMessage.Format("First MOREP value\nWhen used in magic items, this is the spell used\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_MOREY:
		EnableFields(MODE_MAX);
		csMessage.Format("Second MOREP value\nWhen used in magic items, this is the magical bonus\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_MOREZ:
		EnableFields(MODE_MAX);
		csMessage.Format("Third MOREP value\nWhen used in magic items, this is the number of charges\n\nUsage: %s=<integer>", csTag);
		break;
	case ITAG_TIMER:
		EnableFields(MODE_MAX);
		csMessage.Format("Timer value to set on this item at creation.  Used only for non-default settings\n\nUsage: %s=<integer>", csTag);
		break;
	// End
	case ITAG_ON:
		EnableFields(MODE_SELECT);
		FillItemEvents();
		csMessage.Format("Beginning of an event block\n\nUsage: %s=<event>", csTag);
		break;
	case ITAG_ONTRIGGER:
		EnableFields(MODE_SELECT);
		FillItemTriggers();
		csMessage.Format("Beginning of a trigger block\n\nUsage: %s=<triggertype>", csTag);
		break;
	default:
		EnableFields(-1);
		break;
	}

	this->m_csHint.SetWindowText(csMessage);
	CString csCurrentValue = GetValue(csTag);
	switch (iTag)
	{
		// Boolean Values
	case ITAG_DYE:
	case ITAG_REPAIR:
	case ITAG_NOINDEX:
	case ITAG_TWOHANDS:
	case ITAG_PILE:
	case ITAG_REPLICATE:
		this->m_cbCheck.SetCheck(atoi(csCurrentValue));
		this->OnItemcheck();
		break;
		// Text Values
	case ITAG_NAME:
	case ITAG_REQSTR:
	case ITAG_MOREP:
	case ITAG_MOREX:
	case ITAG_MOREY:
	case ITAG_MOREZ:
	case ITAG_VALUE:
	case ITAG_WEIGHT:
	case ITAG_ARMOR:
	case ITAG_MORE:
	case ITAG_MORE2:
	case ITAG_DUPELIST:
	case ITAG_DEFNAME:
	case ITAG_DEFNAME2:
	case ITAG_TIMER:
	case ITAG_MULTIREGION:
	case ITAG_RESOURCES:
	case ITAG_SKILLMAKE:
		this->m_ceMaxValue.SetWindowText(csCurrentValue);
		break;
		// Selections
	case ITAG_TYPE:
		{
			for (int i = 0; i < this->m_ccbCombo.GetCount(); i++ )
			{
				if ( atoi(csCurrentValue) == (int) m_ccbCombo.GetItemData(i) )
				{
					m_ccbCombo.SetCurSel(i);
					break;
				}
			}
		}
		break;
	case ITAG_CATEGORY:
	case ITAG_SUBSECTION:
	case ITAG_DESCRIPTION:
	case ITAG_ID:
	case ITAG_COLOR:
	case ITAG_LAYER:
	case ITAG_DUPEITEM:
		{
			int index = this->m_ccbCombo.SelectString(-1, csCurrentValue);
			if ( index == -1 )
			{
				if ( this->m_ceMaxValue.IsWindowEnabled() )
					this->m_ceMaxValue.SetWindowText(csCurrentValue);
				else
				{
					index = m_ccbCombo.FindString(-1, csCurrentValue);
					if (index != -1 )
						this->m_ccbCombo.SetCurSel(index);
				}
			}
		}
		break;
		// Ranged values
	case ITAG_DAM:
	case ITAG_AMOUNT:
	case ITAG_HITPOINTS:
		{
			CIntRange ir;
			ir.ReadRange(csCurrentValue);
			if ( ir.min != ir.max )
			{
				CString csMin, csMax;
				csMin.Format("%ld", ir.min);
				csMax.Format("%ld", ir.max);
				this->m_ceMaxValue.SetWindowText(csMax);
				this->m_ceMinValue.SetWindowText(csMin);
			}
			else
				this->m_ceMaxValue.SetWindowText(csCurrentValue);
		}
		break;
		// Flag values
	case ITAG_ATTR:
		SetFlags(ahextoi(csCurrentValue));
		break;
	case ITAG_FRUIT:
		// Find this item
		break;
		// multi-instance values
	case ITAG_ONTRIGGER:
	case ITAG_COMPONENT:
	case ITAG_ON:
		break;
	}
}

void CItemScriptEditDlg::ChangeTemplateTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csMessage;
	CString csTag;

	csTag = sm_szTemplateKeysDef[iTag];
	switch ( iTag )
	{
	case TTAG_BUY:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Specifies an item for a vendor to buy.\n\nUsage: %s=<item>", csTag);
		break;
	case TTAG_SELL:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Specifies an item for a vendor to sell.\n\nUsage: %s=<item>", csTag);
		break;
	case TTAG_CONTAINER:
		EnableFields(MODE_ITEM|MODE_MAX|MODE_SELECT);
		FillContainers();
		csMessage.Format("Specifies a container to hold all items in this template.\n\nUsage: %s=<item>", csTag);
		break;
	case TTAG_ITEM:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Specifies an item to be included in this template.\n\nUsage: %s=<item>", csTag);
		break;
	case TTAG_ITEMNEWBIE:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Specifies an item to be included in this template, but won't appear in NPC loot.\n\nUsage: %s=<item>", csTag);
		break;
	case TTAG_CATEGORY:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemCategories();
		csMessage.Format("Axis category classification\n\nUsage: %s=<text>", csTag);
		break;
	case TTAG_DESCRIPTION:
		EnableFields(MODE_MAX);
		csMessage.Format("Axis description\n\nUsage: %s=<text>", csTag);
		break;
	case TTAG_SUBSECTION:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemSubsections();
		csMessage.Format("Axis sub-category classification\n\nUsage: %s=<text>", csTag);
		break;
	case TTAG_NOINDEX:
		EnableFields(MODE_BOOL);
		csMessage.Format("The presence of this tag indicates that it should not be listed in Axis.  Has no arguments.");
		break;
	}
	this->m_csHint.SetWindowText(csMessage);
	CString csCurrentValue = GetValue(csTag);
	switch ( iTag )
	{
	case TTAG_NOINDEX:
		this->m_cbCheck.SetCheck(atoi(csCurrentValue));
		this->OnItemcheck();
		break;
	case TTAG_DESCRIPTION:
	case TTAG_CONTAINER:
	case TTAG_CATEGORY:
	case TTAG_SUBSECTION:
		{
			// Display the current value also
			int index = this->m_ccbCombo.SelectString(-1, csCurrentValue);
			if ( index == -1 )
			{
				if ( this->m_ceMaxValue.IsWindowEnabled() )
					this->m_ceMaxValue.SetWindowText(csCurrentValue);
				else
				{
					index = m_ccbCombo.FindString(-1, csCurrentValue);
					if (index != -1 )
						this->m_ccbCombo.SetCurSel(index);
				}
			}
		}
	}
}

void CItemScriptEditDlg::ChangeCharTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csMessage;
	CString csTag;

	if ( iTag < CTAG_STR )
		csTag = sm_szCharKeysDef[iTag];
	else if ( iTag < CTAG_CONTAINER )
		csTag = sm_szCharKeysCreate[iTag - CTAG_STR];
	else
		csTag = sm_szCharKeysEnd[iTag - CTAG_CONTAINER];
	switch ( iTag )
	{
	// Set the hint text
	case CTAG_DEFNAME:
		EnableFields(MODE_MAX);
		csMessage.Format("String associated with this character\nCannot contain spaces\n\nUsage: %s=<text>", csTag);
		break;
	case CTAG_DEFNAME2:
		EnableFields(MODE_MAX);
		csMessage.Format("Alternate string associated with this character\nCannot contain spaces\n\nUsage: %s=<text>", csTag);
		break;
	case CTAG_NAME:
		EnableFields(MODE_MAX|MODE_SELECT);
		FillCharNames();
		csMessage.Format("Name of the character in game\n\nUsage: %s=<text>", csTag);
		break;
	case CTAG_ID:
		EnableFields(MODE_SELECT);
		FillCharIDs();
		csMessage.Format("Artwork ID to be used for the character\n\nUsage: %s=<hex>", csTag);
		break;
	case CTAG_CAN:
		EnableFields(MODE_FLAG);
		FillCanFlags();
		csMessage.Format("Movement/miscellaneous flags for this character\n\nUsage: %s=<flags>", csTag);
		break;
	case CTAG_ICON:
		EnableFields(MODE_ITEM);
		FillCharIcons();
		csMessage.Format("Icon to be used for the tracking skill for this character\n\nUsage: %s=<item>", csTag);
		break;
	case CTAG_ARMOR:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Armor rating for this character\n\nUsage: %s={<integer> <integer>}", csTag);
		break;
	case CTAG_RESOURCES:
		EnableFields(MODE_MAX|MODE_ITEM);
		csMessage.Format("Resources gained by carving the corpse of this creature\n\nUsage: %s=<resources>,...", csTag);
		break;
	case CTAG_DESIRES:
		EnableFields(MODE_ITEM);
		csMessage.Format("Items this creature desires\n\nUsage: %s=<item>,...", csTag);
		break;
	case CTAG_FOODTYPE:
		EnableFields(MODE_MAX);
		csMessage.Format("Foods this creature eats\n\nUsage: %s=<foodtype>,...", csTag);
		break;
	case CTAG_HIREDAYWAGE:
		EnableFields(MODE_MAX);
		csMessage.Format("Daily wage required to hire this character\n\nUsage: %s=<integer>", csTag);
		break;
	case CTAG_JOB:
		EnableFields(MODE_MAX);
		csMessage.Format("Character's occupation\n\nUsage: %s=<string>", csTag);
		break;
	case CTAG_MAXFOOD:
		EnableFields(MODE_MAX);
		csMessage.Format("Maximum food this character can eat\n\nUsage: %s=<integer>", csTag);
		break;
	case CTAG_TEVENTS:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillCharEvents();
		csMessage.Format("Event scripts associated with this character\n\nUsage: %s=<filename>", csTag);
		break;
	case CTAG_TSPEECH:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillCharSpeech();
		csMessage.Format("Speech scripts associated with this character\n\nUsage: %s=<filename>", csTag);
		break;
	case CTAG_DAM:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Damage this character hits with\n\nUsage: %s={<integer> <integer>}", csTag);
		break;

	case CTAG_CATEGORY:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemCategories();
		csMessage.Format("Axis category classification\n\nUsage: %s=<text>", csTag);
		break;
	case CTAG_DESCRIPTION:
		EnableFields(MODE_MAX);
		csMessage.Format("Axis description\n\nUsage: %s=<text>", csTag);
		break;
	case CTAG_SUBSECTION:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemSubsections();
		csMessage.Format("Axis sub-category classification\n\nUsage: %s=<text>", csTag);
		break;
	case CTAG_NOINDEX:
		EnableFields(MODE_BOOL);
		csMessage.Format("The presence of this tag indicates that it should not be listed in Axis.  Has no arguments.");
		break;

	case CTAG_STR:
	case CTAG_DEX:
	case CTAG_INT:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Specifies the range for the value of this skill.\n\nUsage: %s={<integer> <integer>}\n", csTag );
		break;
	case CTAG_COLOR:
		EnableFields(MODE_COLOR|MODE_MAX);
		FillColors();
		csMessage.Format("Specifies the color to use for this creature.\n\nUsage: %s=<color>", csTag);
		break;
	case CTAG_FLAGS:
		EnableFields(MODE_FLAG);
		FillCharFlags();
		csMessage.Format("Flags associated with this character\n\nUsage %s=<flags>", csTag);
		break;

	case CTAG_ALCHEMY:
	case CTAG_ANATOMY:
	case CTAG_ANIMALLORE:
	case CTAG_ITEMID:
	case CTAG_ARMSLORE:
	case CTAG_PARRYING:
	case CTAG_BEGGING:
	case CTAG_BLACKSMITHING:
	case CTAG_BOWCRAFT:
	case CTAG_PEACEMAKING:
	case CTAG_CAMPING:
	case CTAG_CARPENTRY:
	case CTAG_CARTOGRAPHY:
	case CTAG_COOKING:
	case CTAG_DETECTINGHIDDEN:
	case CTAG_ENTICEMENT:
	case CTAG_EVALUATINGINTEL:
	case CTAG_HEALING:
	case CTAG_FISHING:
	case CTAG_FORENSICS:
	case CTAG_HERDING:
	case CTAG_HIDING:
	case CTAG_PROVOCATION:
	case CTAG_INSCRIPTION:
	case CTAG_LOCKPICKING:
	case CTAG_MAGERY:
	case CTAG_MAGICRESISTANCE:
	case CTAG_TACTICS:
	case CTAG_SNOOPING:
	case CTAG_MUSICIANSHIP:
	case CTAG_POISONING:
	case CTAG_ARCHERY:
	case CTAG_SPIRITSPEAK:
	case CTAG_STEALING:
	case CTAG_TAILORING:
	case CTAG_TAMING:
	case CTAG_TASTEID:
	case CTAG_TINKERING:
	case CTAG_TRACKING:
	case CTAG_VETERINARY:
	case CTAG_SWORDSMANSHIP:
	case CTAG_MACEFIGHTING:
	case CTAG_FENCING:
	case CTAG_WRESTLING:
	case CTAG_LUMBERJACKING:
	case CTAG_MINING:
	case CTAG_MEDITATION:
	case CTAG_STEALTH:
	case CTAG_REMOVETRAP:
	case CTAG_NECROMANCY:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Specifies the range for the value of this skill.\n\nUsage: %s={<float> <float>}\n", csTag);
		break;

	case CTAG_NPC:
		EnableFields(MODE_SELECT);
		FillBrains();
		csMessage.Format("Brain type for this character\n\nUsage %s=<braintype>", csTag);
		break;
	case CTAG_FAME:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Specifies the range for the fame value for this character.  Range is 0 to 10000\n\nUsage %s={<integer> <integer>}", csTag);
		break;
	case CTAG_KARMA:
		EnableFields(MODE_MIN|MODE_MAX);
		csMessage.Format("Specifies the range for the karma value for this character. Range is -10000 to 10000\n\nUsage %s={<integer> <integer>}", csTag);
		break;
	case CTAG_NEED:
		EnableFields(MODE_ITEM);
		csMessage.Format("Items that the character has a need for\n\nUsage %s=<item>", csTag);
		break;

	case CTAG_SPEECH:
	case CTAG_EVENTS:
	case CTAG_TIMER:
	case CTAG_CONTAINER:
		break;

	case CTAG_ITEM:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Loot item for this character\n\nUsage: %s=<item>", csTag );
		break;
	case CTAG_ITEMNEWBIE:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Item for this character (non-loot)\n\nUsage: %s=<item>", csTag );
		break;
	case CTAG_ON:
		EnableFields(MODE_SELECT);
		FillCharTriggers();
		csMessage.Format("Beginning of an event block\n\nUsage: %s=<event>", csTag);
		break;
	}
	this->m_csHint.SetWindowText(csMessage);
	CString csCurrentValue = GetValue(csTag);
	// Show the current value
	switch ( iTag )
	{
		// Boolean Values
	case CTAG_NOINDEX:
		this->m_cbCheck.SetCheck(atoi(csCurrentValue));
		this->OnItemcheck();
		break;
		// Text Values
	case CTAG_DEFNAME:
	case CTAG_DEFNAME2:
	case CTAG_DESIRES:
	case CTAG_FOODTYPE:
	case CTAG_RESOURCES:
	case CTAG_HIREDAYWAGE:
	case CTAG_JOB:
	case CTAG_MAXFOOD:
	case CTAG_DESCRIPTION:
	case CTAG_COLOR:
		this->m_ceMaxValue.SetWindowText(csCurrentValue);
		break;
		// Selections
	case CTAG_NPC:
		{
			for (int i = 0; i < this->m_ccbCombo.GetCount(); i++ )
			{
				if ( atoi(csCurrentValue) == (int) m_ccbCombo.GetItemData(i) )
				{
					m_ccbCombo.SetCurSel(i);
					break;
				}
			}
		}
		break;
		// Name selections
	case CTAG_NAME:
		{
			CString csBlock;
			CString csName;
			if ( csCurrentValue.Find("#") != -1 )
			{
				csBlock = csCurrentValue.Mid(csCurrentValue.Find("#") + 1);
				csBlock = csBlock.SpanExcluding(" \t");
				csName = csCurrentValue.Mid(csCurrentValue.Find("#") + csBlock.GetLength() + 1);
				csBlock.TrimLeft(); csBlock.TrimRight();
				csName.TrimLeft(); csBlock.TrimLeft();
			}
			else
			{
				csName = csCurrentValue;
			}
			int index = this->m_ccbCombo.SelectString(-1, csBlock);
			this->m_ceMaxValue.SetWindowText(csName);
		}
		break;
		// Other selections
	case CTAG_ID:
	case CTAG_CATEGORY:
	case CTAG_SUBSECTION:
	case CTAG_ON:
		{
			int index = this->m_ccbCombo.SelectString(-1, csCurrentValue);
			if ( index == -1 )
			{
				if ( this->m_ceMaxValue.IsWindowEnabled() )
					this->m_ceMaxValue.SetWindowText(csCurrentValue);
				else
				{
					index = m_ccbCombo.FindString(-1, csCurrentValue);
					if (index != -1 )
						this->m_ccbCombo.SetCurSel(index);
				}
			}
		}
		break;
		// Ranged integer values
	case CTAG_DAM:
	case CTAG_ARMOR:
	case CTAG_STR:
	case CTAG_DEX:
	case CTAG_INT:
	case CTAG_FAME:
	case CTAG_KARMA:
		{
			CIntRange ir;
			ir.ReadRange(csCurrentValue);
			if ( ir.min != ir.max )
			{
				CString csMin, csMax;
				csMin.Format("%ld", ir.min);
				csMax.Format("%ld", ir.max);
				this->m_ceMaxValue.SetWindowText(csMax);
				this->m_ceMinValue.SetWindowText(csMin);
			}
			else
				this->m_ceMaxValue.SetWindowText(csCurrentValue);
		}
		break;
		// Ranged float values
	case CTAG_ALCHEMY:
	case CTAG_ANATOMY:
	case CTAG_ANIMALLORE:
	case CTAG_ITEMID:
	case CTAG_ARMSLORE:
	case CTAG_PARRYING:
	case CTAG_BEGGING:
	case CTAG_BLACKSMITHING:
	case CTAG_BOWCRAFT:
	case CTAG_PEACEMAKING:
	case CTAG_CAMPING:
	case CTAG_CARPENTRY:
	case CTAG_CARTOGRAPHY:
	case CTAG_COOKING:
	case CTAG_DETECTINGHIDDEN:
	case CTAG_ENTICEMENT:
	case CTAG_EVALUATINGINTEL:
	case CTAG_HEALING:
	case CTAG_FISHING:
	case CTAG_FORENSICS:
	case CTAG_HERDING:
	case CTAG_HIDING:
	case CTAG_PROVOCATION:
	case CTAG_INSCRIPTION:
	case CTAG_LOCKPICKING:
	case CTAG_MAGERY:
	case CTAG_MAGICRESISTANCE:
	case CTAG_TACTICS:
	case CTAG_SNOOPING:
	case CTAG_MUSICIANSHIP:
	case CTAG_POISONING:
	case CTAG_ARCHERY:
	case CTAG_SPIRITSPEAK:
	case CTAG_STEALING:
	case CTAG_TAILORING:
	case CTAG_TAMING:
	case CTAG_TASTEID:
	case CTAG_TINKERING:
	case CTAG_TRACKING:
	case CTAG_VETERINARY:
	case CTAG_SWORDSMANSHIP:
	case CTAG_MACEFIGHTING:
	case CTAG_FENCING:
	case CTAG_WRESTLING:
	case CTAG_LUMBERJACKING:
	case CTAG_MINING:
	case CTAG_MEDITATION:
	case CTAG_STEALTH:
	case CTAG_REMOVETRAP:
	case CTAG_NECROMANCY:
		{
			CFloatRange fr;
			fr.ReadRange(csCurrentValue);
			if ( fr.min != fr.max )
			{
				CString csMin, csMax;
				csMin.Format("%.1f", fr.min);
				csMax.Format("%.1f", fr.max);
				this->m_ceMaxValue.SetWindowText(csMax);
				this->m_ceMinValue.SetWindowText(csMin);
			}
			else
				this->m_ceMaxValue.SetWindowText(csCurrentValue);
		}
		break;
		// Flag values
	case CTAG_CAN:
	case CTAG_FLAGS:
		SetFlags(ahextoi(csCurrentValue));
		break;
		// Item values
	case CTAG_ICON:
	case CTAG_NEED:
	case CTAG_ITEM:
	case CTAG_ITEMNEWBIE:
		// multi-instance values
	case CTAG_TEVENTS:
	case CTAG_TSPEECH:
		break;
	}
}

void CItemScriptEditDlg::PreviewItemTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if (iSel == -1)
	{
		this->m_csPreview.SetWindowText("");
		return;
	}
	CString csValue, csLine;
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csTag;

	if ( iTag < ITAG_COLOR )
		csTag = sm_szItemKeysDef[iTag];
	else if ( iTag < ITAG_ON )
		csTag = sm_szItemKeysCreate[iTag - ITAG_COLOR];
	else
		csTag = sm_szItemKeysEnd[iTag - ITAG_ON];
	switch (iTag)
	{
		case ITAG_NAME:
		case ITAG_REQSTR:
		case ITAG_MOREP:
		case ITAG_MOREX:
		case ITAG_MOREY:
		case ITAG_MOREZ:
		case ITAG_VALUE:
		case ITAG_WEIGHT:
		case ITAG_ARMOR:
		case ITAG_MORE:
		case ITAG_MORE2:
		case ITAG_DUPELIST:
		case ITAG_DEFNAME:
		case ITAG_DEFNAME2:
		case ITAG_TIMER:
		case ITAG_MULTIREGION:
		case ITAG_RESOURCES:
		case ITAG_SKILLMAKE:
		case ITAG_DESCRIPTION:
		this->m_ceMaxValue.GetWindowText(csValue);
			break;
		case ITAG_DYE:
		case ITAG_REPAIR:
		case ITAG_NOINDEX:
		case ITAG_TWOHANDS:
		case ITAG_PILE:
		case ITAG_REPLICATE:
			csValue = this->m_cbCheck.GetCheck() ? "1" : "0";
			break;
		case ITAG_ONTRIGGER:
			this->m_ccbCombo.GetWindowText(csValue);
			break;
		case ITAG_TYPE:
			{
				int index = this->m_ccbCombo.GetCurSel();
				int value = 0;
				if ( index )
					value = this->m_ccbCombo.GetItemData(index);
				csValue.Format("%ld", value);
			}
			break;
		case ITAG_ID:
		case ITAG_DUPEITEM:
			{
				CString csTemp;
				this->m_ccbCombo.GetWindowText(csTemp);
				if ( csTemp.Find("[") != -1 )
				{
					csValue = csTemp.Mid(csTemp.Find("[") + 1);
					csValue = csValue.SpanExcluding("]");
				}
			}
			break;
		case ITAG_DAM:
		case ITAG_AMOUNT:
		case ITAG_HITPOINTS:
			{
				CString csMin, csMax;
				this->m_ceMaxValue.GetWindowText(csMax);
				this->m_ceMinValue.GetWindowText(csMin);
				if ( csMin == "" || csMax == "" )
				{
					if ( csMin == "" )
						csValue = csMax;
					else
						csValue = csMin;
				}
				else
					csValue.Format("{%ld %ld}", atoi(csMin), atoi(csMax));
			}
			break;
		case ITAG_CATEGORY:
		case ITAG_SUBSECTION:
			this->m_ceMaxValue.GetWindowText(csValue);
			if ( csValue == "" )
				this->m_ccbCombo.GetWindowText(csValue);
			break;
		case ITAG_COLOR:
			{
				this->m_ceMaxValue.GetWindowText(csValue);
				if ( csValue == "" )
					this->m_ccbCombo.GetWindowText(csValue);
				WORD wColor = (WORD) DefLookup(csValue);
				DrawSpectrum(&m_csColorDisplay, wColor);
			}
			break;
		case ITAG_ATTR:
			{
				int iSel = -1;
				DWORD dwFlags = GetFlags();
				csValue.Format("0%x", dwFlags);
			}
			break;
		case ITAG_COMPONENT:
			{
				// Get the item ID
				CString csText;
				this->m_ceMaxValue.GetWindowText(csText);
				int iSel = this->m_clcList.GetNextItem(-1, LVNI_SELECTED);
				if ( iSel != -1 )
				{
					CTObject * pObject = (CTObject *) this->m_clcList.GetItemData(iSel);
					if ( pObject )
					{
						CString csDef = pObject->m_csValue;
						if ( pObject->m_bType == TYPE_ITEM )
						{
							CItem item;
							item.m_csFilename = pObject->m_csFilename;
							item.m_csValue = pObject->m_csValue;
							item.Load();
							if ( item.m_csDef )
								csDef = item.m_csDef;
						}
						csValue.Format("%s,%s", csDef, csText);
					}
				}
			}
			break;
		case ITAG_FRUIT:
			{
				// Get the item ID
				CString csText;
				this->m_ceMaxValue.GetWindowText(csText);
				int iSel = this->m_clcList.GetNextItem(-1, LVNI_SELECTED);
				if ( iSel != -1 )
				{
					CTObject * pObject = (CTObject *) this->m_clcList.GetItemData(iSel);
					if ( pObject )
					{
						CString csDef = pObject->m_csValue;
						if ( pObject->m_bType == TYPE_ITEM )
						{
							CItem item;
							item.m_csFilename = pObject->m_csFilename;
							item.m_csValue = pObject->m_csValue;
							item.Load();
							if ( item.m_csDef )
								csDef = item.m_csDef;
						}
						csValue.Format("%s", csDef, csText);
					}
				}
			}
			break;
		default:
			csValue = "";
	}
	if ( iTag == ITAG_NOINDEX )
	{
		if ( csValue == "1" )
			csLine = "NOINDEX";
		else
			csLine = "";
	}
	else if ( csValue == "" )
		csLine = "";
	else
	{
		if ( iTag == ITAG_ONTRIGGER && csValue.Find("@") != -1 )
			csLine = csValue;
		else
			csLine.Format("%s=%s", csTag, csValue);
	}
	this->m_csPreview.SetWindowText(csLine);
}

void CItemScriptEditDlg::PreviewTemplateTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if (iSel == -1)
	{
		this->m_csPreview.SetWindowText("");
		return;
	}
	CString csValue, csLine;
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csTag;

	csTag = sm_szTemplateKeysDef[iTag];
	switch (iTag)
	{
	case TTAG_CATEGORY:
	case TTAG_SUBSECTION:
		this->m_ceMaxValue.GetWindowText(csValue);
		if ( csValue == "" )
			this->m_ccbCombo.GetWindowText(csValue);
		csLine.Format("%s=%s", csTag, csValue);
		break;
	case TTAG_DESCRIPTION:
		this->m_ceMaxValue.GetWindowText(csValue);
		csLine.Format("%s=%s", csTag, csValue);
		break;
	case TTAG_NOINDEX:
		{
			csValue = m_cbCancel.GetCheck() ? "1" : "0";
			if ( csValue == "1" )
				csLine = "NOINDEX";
			else
				csLine = "";
		}
		break;
	case TTAG_BUY:
	case TTAG_SELL:
	case TTAG_CONTAINER:
	case TTAG_ITEM:
	case TTAG_ITEMNEWBIE:
		{
			// Get the item ID
			CString csText;
			this->m_ceMaxValue.GetWindowText(csText);
			if ( csText == "" )
			{
				int iSel = this->m_clcList.GetNextItem(-1, LVNI_SELECTED);
				if ( iSel != -1 )
				{
					CTObject * pObject = (CTObject *) this->m_clcList.GetItemData(iSel);
					if ( pObject )
					{
						CString csDef = pObject->m_csValue;
						if ( pObject->m_bType == TYPE_ITEM )
						{
							CItem item;
							item.m_csFilename = pObject->m_csFilename;
							item.m_csValue = pObject->m_csValue;
							item.Load();
							if ( item.m_csDef != "")
								csDef = item.m_csDef;
						}
						else if ( pObject->m_bType == TYPE_TEMPLATE )
						{
							CTemplate item;
							item.m_csFilename = pObject->m_csFilename;
							item.m_csValue = pObject->m_csValue;
							item.Load();
							if ( item.m_csDef != "" )
								csDef = item.m_csDef;
						}
						csValue = csDef;
					}
				}
			}
			else
				csValue = csText;
			csLine.Format("%s=%s", csTag, csValue);
		}
	}
	this->m_csPreview.SetWindowText(csLine);
}

void CItemScriptEditDlg::PreviewCharTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if (iSel == -1)
	{
		this->m_csPreview.SetWindowText("");
		return;
	}
	CString csValue, csLine;
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csTag;

	if ( iTag < CTAG_STR )
		csTag = sm_szCharKeysDef[iTag];
	else if ( iTag < CTAG_CONTAINER )
		csTag = sm_szCharKeysCreate[iTag - CTAG_STR];
	else
		csTag = sm_szCharKeysEnd[iTag - CTAG_CONTAINER];
	switch (iTag)
	{
		// Text values
	case CTAG_DEFNAME:
	case CTAG_DEFNAME2:
	case CTAG_HIREDAYWAGE:
	case CTAG_MAXFOOD:
	case CTAG_TEVENTS:
	case CTAG_TSPEECH:
	case CTAG_JOB:
	case CTAG_DESCRIPTION:
	case CTAG_FOODTYPE:
		this->m_ceMaxValue.GetWindowText(csValue);
		break;
		// Name
	case CTAG_NAME:
		{
			CString csBlock, csName;
			if ( this->m_ccbCombo.GetCurSel() != -1 )
				m_ccbCombo.GetWindowText(csBlock);
			m_ceMaxValue.GetWindowText(csName);
			if ( csBlock != "" && csName != "")
				csValue.Format("#%s %s", csBlock, csName);
			else if ( csBlock == "" )
				csValue = csName;
			else
				csValue.Format("#%s", csBlock);
		}
		break;
		// boolean values
	case CTAG_NOINDEX:
		csValue = this->m_cbCheck.GetCheck() ? "1" : "0";
		break;
		// Simple selections
	case CTAG_NPC:
	case CTAG_ON:
		this->m_ccbCombo.GetWindowText(csValue);
		break;
		// Complex selections
	case CTAG_RESOURCES:
		{
			int index = this->m_ccbCombo.GetCurSel();
			int value = 0;
			if ( index )
				value = this->m_ccbCombo.GetItemData(index);
			csValue.Format("%ld", value);
		}
		break;
		// Other selections
	case CTAG_ID:
		{
			CString csTemp;
			this->m_ccbCombo.GetWindowText(csTemp);
			if ( csTemp.Find("[") != -1 )
			{
				csValue = csTemp.Mid(csTemp.Find("[") + 1);
				csValue = csValue.SpanExcluding("]");
			}
		}
		break;
		// Ranged values
	case CTAG_ARMOR:
	case CTAG_DAM:


	case CTAG_STR:
	case CTAG_DEX:
	case CTAG_INT:


	case CTAG_FAME:
	case CTAG_KARMA:
		{
			CString csMin, csMax;
			this->m_ceMaxValue.GetWindowText(csMax);
			this->m_ceMinValue.GetWindowText(csMin);
			if ( csMin == "" || csMax == "" )
			{
				if ( csMin == "" )
					csValue = csMax;
				else
					csValue = csMin;
			}
			else
				csValue.Format("{%ld %ld}", atoi(csMin), atoi(csMax));
		}
		break;
		// Ranged float values
	case CTAG_ALCHEMY:
	case CTAG_ANATOMY:
	case CTAG_ANIMALLORE:
	case CTAG_ITEMID:
	case CTAG_ARMSLORE:
	case CTAG_PARRYING:
	case CTAG_BEGGING:
	case CTAG_BLACKSMITHING:
	case CTAG_BOWCRAFT:
	case CTAG_PEACEMAKING:
	case CTAG_CAMPING:
	case CTAG_CARPENTRY:
	case CTAG_CARTOGRAPHY:
	case CTAG_COOKING:
	case CTAG_DETECTINGHIDDEN:
	case CTAG_ENTICEMENT:
	case CTAG_EVALUATINGINTEL:
	case CTAG_HEALING:
	case CTAG_FISHING:
	case CTAG_FORENSICS:
	case CTAG_HERDING:
	case CTAG_HIDING:
	case CTAG_PROVOCATION:
	case CTAG_INSCRIPTION:
	case CTAG_LOCKPICKING:
	case CTAG_MAGERY:
	case CTAG_MAGICRESISTANCE:
	case CTAG_TACTICS:
	case CTAG_SNOOPING:
	case CTAG_MUSICIANSHIP:
	case CTAG_POISONING:
	case CTAG_ARCHERY:
	case CTAG_SPIRITSPEAK:
	case CTAG_STEALING:
	case CTAG_TAILORING:
	case CTAG_TAMING:
	case CTAG_TASTEID:
	case CTAG_TINKERING:
	case CTAG_TRACKING:
	case CTAG_VETERINARY:
	case CTAG_SWORDSMANSHIP:
	case CTAG_MACEFIGHTING:
	case CTAG_FENCING:
	case CTAG_WRESTLING:
	case CTAG_LUMBERJACKING:
	case CTAG_MINING:
	case CTAG_MEDITATION:
	case CTAG_STEALTH:
	case CTAG_REMOVETRAP:
	case CTAG_NECROMANCY:
		{
			CString csMin, csMax;
			this->m_ceMaxValue.GetWindowText(csMax);
			this->m_ceMinValue.GetWindowText(csMin);
			if ( csMin == "" || csMax == "" )
			{
				if ( csMin == "" )
					csValue = csMax;
				else
					csValue = csMin;
			}
			else
				csValue.Format("{%.1f %.1f}", atof(csMin), atof(csMax));
		}
		break;
		// More selection types
	case CTAG_CATEGORY:
	case CTAG_SUBSECTION:
		this->m_ceMaxValue.GetWindowText(csValue);
		if ( csValue == "" )
			this->m_ccbCombo.GetWindowText(csValue);
		break;
		// Color
	case CTAG_COLOR:
		{
			this->m_ceMaxValue.GetWindowText(csValue);
			if ( csValue == "" )
				this->m_ccbCombo.GetWindowText(csValue);
			WORD wColor = (WORD) DefLookup(csValue);
			DrawSpectrum(&m_csColorDisplay, wColor);
		}
		break;
		// Flags
	case CTAG_CAN:
	case CTAG_FLAGS:
		{
			int iSel = -1;
			DWORD dwFlags = GetFlags();
			csValue.Format("0%x", dwFlags);
		}
		break;
		// Items
	case CTAG_ICON:
	case CTAG_ITEM:
	case CTAG_ITEMNEWBIE:
	case CTAG_DESIRES:
	case CTAG_NEED:
		{
			// Get the item ID
			CString csText;
			int iSel = this->m_clcList.GetNextItem(-1, LVNI_SELECTED);
			if ( iSel != -1 )
			{
				CTObject * pObject = (CTObject *) this->m_clcList.GetItemData(iSel);
				if ( pObject )
				{
					csValue = pObject->m_csValue;
					if ( pObject->m_bType == TYPE_ITEM )
					{
						CItem item;
						item.m_csFilename = pObject->m_csFilename;
						item.m_csValue = pObject->m_csValue;
						item.Load();
						if ( item.m_csDef )
							csValue = item.m_csDef;
					}
				}
			}
		}
		break;
	}
	if ( iTag == CTAG_NOINDEX )
	{
		if ( csValue == "1" )
			csLine = "NOINDEX";
		else
			csLine = "";
	}
	else if ( csValue == "" )
		csLine = "";
	else
	{
		if ( iTag == CTAG_ON && csValue.Find("@") != -1 )
			csLine = csValue;
		else
			csLine.Format("%s=%s", csTag, csValue);
	}
	this->m_csPreview.SetWindowText(csLine);
}

void CItemScriptEditDlg::FillCharNames()
{
	for ( int i = 0; i < Main->m_pScripts->m_aNames.GetSize(); i++ )
	{
		CTObject * pNameBlock = (CTObject *) Main->m_pScripts->m_aNames.GetAt(i);
		if ( pNameBlock )
			this->m_ccbCombo.AddString(pNameBlock->m_csValue);
	}
}

void CItemScriptEditDlg::FillSpawnTags()
{
	for ( int i = 0; i < SPAWN_TAG_QTY; i++ )
	{
		int index = 0;
		int iType = 0;
		index = this->m_clbTags.AddString(sm_szSpawnKeysDef[i]);
		switch ( i )
		{
		case SPAWN_TAG_ID:
		case SPAWN_TAG_WEIGHT:
			iType = 3;
			break;
		default:
			iType = 0;
			break;
		}
		DWORD dwData = i | (iType << 16);
		this->m_clbTags.SetItemData(index, dwData);
	}
}

void CItemScriptEditDlg::ChangeSpawnTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csMessage;
	CString csTag;

	csTag = sm_szSpawnKeysDef[iTag];
	switch ( iTag )
	{
	case SPAWN_TAG_DEFNAME:
		EnableFields(MODE_MAX);
		csMessage.Format("Specifies the DEFNAME associated with this spawn group.\n\nUsage: %s=<text>", csTag);
		break;
	case SPAWN_TAG_ID:
		EnableFields(MODE_ITEM|MODE_MAX);
		csMessage.Format("Specifies an NPC to be included in this Spawn Group.\n\nUsage: %s=<npc id>", csTag);
		break;
	case SPAWN_TAG_WEIGHT:
		EnableFields(MODE_MAX);
		csMessage.Format("Specifies the weighted chance of occurrence for the last NPC.\n\nUsage: %s=<integer>", csTag);
		break;
	case SPAWN_TAG_CATEGORY:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemCategories();
		csMessage.Format("Axis category classification\n\nUsage: %s=<text>", csTag);
		break;
	case SPAWN_TAG_DESCRIPTION:
		EnableFields(MODE_MAX);
		csMessage.Format("Axis description\n\nUsage: %s=<text>", csTag);
		break;
	case SPAWN_TAG_SUBSECTION:
		EnableFields(MODE_SELECT|MODE_MAX);
		FillItemSubsections();
		csMessage.Format("Axis sub-category classification\n\nUsage: %s=<text>", csTag);
		break;
	}
	this->m_csHint.SetWindowText(csMessage);
	CString csCurrentValue = GetValue(csTag);
	switch ( iTag )
	{
	case SPAWN_TAG_DESCRIPTION:
	case SPAWN_TAG_CATEGORY:
	case SPAWN_TAG_SUBSECTION:
	case SPAWN_TAG_DEFNAME:
		{
			// Display the current value also
			int index = this->m_ccbCombo.SelectString(-1, csCurrentValue);
			if ( index == -1 )
			{
				if ( this->m_ceMaxValue.IsWindowEnabled() )
					this->m_ceMaxValue.SetWindowText(csCurrentValue);
				else
				{
					index = m_ccbCombo.FindString(-1, csCurrentValue);
					if (index != -1 )
						this->m_ccbCombo.SetCurSel(index);
				}
			}
		}
	}
}

void CItemScriptEditDlg::PreviewSpawnTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if (iSel == -1)
	{
		this->m_csPreview.SetWindowText("");
		return;
	}
	CString csValue, csLine;
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csTag;

	csTag = sm_szSpawnKeysDef[iTag];
	switch (iTag)
	{
	case SPAWN_TAG_CATEGORY:
	case SPAWN_TAG_SUBSECTION:
		this->m_ceMaxValue.GetWindowText(csValue);
		if ( csValue == "" )
			this->m_ccbCombo.GetWindowText(csValue);
		csLine.Format("%s=%s", csTag, csValue);
		break;
	case SPAWN_TAG_WEIGHT:
	case SPAWN_TAG_DEFNAME:
	case SPAWN_TAG_DESCRIPTION:
		this->m_ceMaxValue.GetWindowText(csValue);
		csLine.Format("%s=%s", csTag, csValue);
		break;
	case SPAWN_TAG_ID:
		{
			// Get the item ID
			CString csText;
			this->m_ceMaxValue.GetWindowText(csText);
			if ( csText == "" )
			{
				int iSel = this->m_clcList.GetNextItem(-1, LVNI_SELECTED);
				if ( iSel != -1 )
				{
					CTObject * pObject = (CTObject *) this->m_clcList.GetItemData(iSel);
					if ( pObject )
					{
						CString csDef = pObject->m_csValue;
						if ( pObject->m_bType == TYPE_ITEM )
						{
							CItem item;
							item.m_csFilename = pObject->m_csFilename;
							item.m_csValue = pObject->m_csValue;
							item.Load();
							if ( item.m_csDef != "")
								csDef = item.m_csDef;
						}
						else if ( pObject->m_bType == TYPE_TEMPLATE )
						{
							CTemplate item;
							item.m_csFilename = pObject->m_csFilename;
							item.m_csValue = pObject->m_csValue;
							item.Load();
							if ( item.m_csDef != "" )
								csDef = item.m_csDef;
						}
						csValue = csDef;
					}
				}
			}
			else
				csValue = csText;
			csLine.Format("%s=%s", csTag, csValue);
		}
	}
	this->m_csPreview.SetWindowText(csLine);
}

void CItemScriptEditDlg::FillTriggerTags()
{
}

void CItemScriptEditDlg::FillSkillTags()
{
	for ( int i = 0; i < SKILLTAG_QTY; i++ )
	{
		int index = 0;
		int iType = 0;
		index = this->m_clbTags.AddString(knownSkillTags[i]);
		iType = 0;
		DWORD dwData = i | (iType << 16);
		this->m_clbTags.SetItemData(index, dwData);
	}
}

void CItemScriptEditDlg::FillSpellTags()
{
	for ( int i = 0; i < SPELLTAG_QTY; i++ )
	{
		int index = 0;
		int iType = 0;
		index = this->m_clbTags.AddString(knownSpellTags[i]);
		iType = 0;
		DWORD dwData = i | (iType << 16);
		this->m_clbTags.SetItemData(index, dwData);
	}
}

void CItemScriptEditDlg::ChangeTriggerTag()
{
}

void CItemScriptEditDlg::ChangeSkillTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csMessage;
	CString csTag = knownSkillTags[iTag];
	// Set the hint text
	switch ( iTag )
	{
	case SKILLTAG_KEY:
		EnableFields(MODE_MAX);
		csMessage.Format("String associated with this skill\nCannot contain spaces\n\nUsage: %s=<text>", csTag);
		break;
	case SKILLTAG_TITLE:
		EnableFields(MODE_MAX);
		csMessage.Format("Title bestowed on characters with this skill as their highest\n\nUsage: %s=<text>", csTag);
		break;
	case SKILLTAG_PROMPT_MSG:
		EnableFields(MODE_MAX);
		csMessage.Format("Message to be displayed when this skill requires a target\nNot applicable for all skills\n\nUsage: %s=<text>", csTag);
		break;
	case SKILLTAG_FAIL_MSG:
		EnableFields(MODE_MAX);
		csMessage.Format("Message to be displayed when this skill fails\nNot applicable for all skills\n\nUsage: %s=<text>", csTag);
		break;
	case SKILLTAG_DELAY:
		EnableFields(MODE_MAX);
		csMessage.Format("Tenths of a second between skill start and completion\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_STAT_STR:
		EnableFields(MODE_MAX);
		csMessage.Format("Target Strength value for a character who uses this skill exclusively\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_STAT_INT:
		EnableFields(MODE_MAX);
		csMessage.Format("Target Intelligence value for a character who uses this skill exclusively\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_STAT_DEX:
		EnableFields(MODE_MAX);
		csMessage.Format("Target Dexterity value for a character who uses this skill exclusively\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_BONUS_STR:
		EnableFields(MODE_MAX);
		csMessage.Format("Percent of stat bonus for the skill that comes from the character's strength\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_BONUS_DEX:
		EnableFields(MODE_MAX);
		csMessage.Format("Percent of stat bonus for the skill that comes from the character's dexterity\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_BONUS_INT:
		EnableFields(MODE_MAX);
		csMessage.Format("Percent of stat bonus for the skill that comes from the character's intelligence\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_SKILL_STAT:
		EnableFields(MODE_MAX);
		csMessage.Format("Percentage of total success chance which is derived from the skill value alone\n\nUsage: %s=<integer>", csTag);
		break;
	case SKILLTAG_ADV_RATE:
		EnableFields(MODE_MAX);
		csMessage.Format("Skill advancement rates ( from 67-100, 34-66, and 0-33 )\nHigher number indicates slower gain rate\n\nUsage: %s=<integer>,<integer>,<integer>", csTag);
		break;
	}
	this->m_csHint.SetWindowText(csMessage);
	CString csCurrentValue = GetValue(csTag);
	// Show the current value
	this->m_ceMaxValue.SetWindowText(csCurrentValue);
}

void CItemScriptEditDlg::ChangeSpellTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if ( iSel == -1 )
	{
		this->m_csHint.SetWindowText("");
		EnableFields(-1);
		return;
	}
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csMessage;
	CString csTag = knownSpellTags[iTag];
	// Set the hint text
	switch ( iTag )
	{
	case SPELLTAG_NAME:
		EnableFields(MODE_MAX);
		csMessage.Format("Name of the spell\n\nUsage: %s=<text>", csTag);
		break;
	case SPELLTAG_SOUND:
		EnableFields(MODE_SELECT);
		FillSounds();
		csMessage.Format("Sound heard on success\n\nUsage: %s=<sound_id>", csTag);
		break;
	case SPELLTAG_RUNES:
		EnableFields(MODE_MAX);
		csMessage.Format("Words of Power uttered when spell is cast\n\nUsage: %s=<string>", csTag);
		break;
	case SPELLTAG_REAGENTS:
		EnableFields(MODE_MAX);
		csMessage.Format("Reagents required for this spell\n\nUsage: %s=<string>", csTag);
		break;
	case SPELLTAG_RUNE_ITEM:
		EnableFields(MODE_MAX);
		csMessage.Format("Icon associated with this spell\n\nUsage: %s=<item_id>", csTag);
		break;
	case SPELLTAG_SCROLL_ITEM:
		EnableFields(MODE_MAX);
		csMessage.Format("Scroll associated with this spell\n\nUsage: %s=<item_id>", csTag);
		break;
	case SPELLTAG_FLAGS:
		EnableFields(MODE_FLAG);
		FillSpellFlags();
		csMessage.Format("Flags associated with this spell\n\nUsage: %s=<flags>", csTag);
		break;
	case SPELLTAG_EFFECT_ID:
		EnableFields(MODE_MAX);
		csMessage.Format("Animation ID for the spell effect\n\nUsage: %s=<item_id>", csTag);
		break;
	case SPELLTAG_EFFECT_LO:
		EnableFields(MODE_MAX);
		csMessage.Format("Low value for the effect.  Meaning depends on the spell\n\nUsage: %s=<integer>", csTag);
		break;
	case SPELLTAG_EFFECT_HI:
		EnableFields(MODE_MAX);
		csMessage.Format("High value for the effect.  Meaning depends on the spell\n\nUsage: %s=<integer>", csTag);
		break;
	}
	this->m_csHint.SetWindowText(csMessage);
	CString csCurrentValue = GetValue(csTag);
	// Show the current value
	switch ( iTag )
	{
		// Simple values
	case SPELLTAG_NAME:
	case SPELLTAG_EFFECT_LO:
	case SPELLTAG_EFFECT_HI:
	case SPELLTAG_RUNE_ITEM:
	case SPELLTAG_SCROLL_ITEM:
	case SPELLTAG_EFFECT_ID:
	case SPELLTAG_REAGENTS:
	case SPELLTAG_RUNES:
		this->m_ceMaxValue.SetWindowText(csCurrentValue);
		break;
		// Item values
		// Selection
	case SPELLTAG_SOUND:
		{
			DWORD dwSound = DefLookup(csCurrentValue);
			if (dwSound != -1 )
				m_ccbCombo.SelectString(-1, Sounds[dwSound]);
		}
		break;
		// Others
	case SPELLTAG_FLAGS:
		SetFlags(CSpellDef::Flags(csCurrentValue));
		break;
	}
}

void CItemScriptEditDlg::PreviewTriggerTag()
{
}

void CItemScriptEditDlg::PreviewSkillTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if (iSel == -1)
	{
		this->m_csPreview.SetWindowText("");
		return;
	}
	CString csValue, csLine;
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csTag = knownSkillTags[iTag];
	this->m_ceMaxValue.GetWindowText(csValue);
	if ( csValue != "" )
		csLine.Format("%s=%s", csTag, csValue);
	this->m_csPreview.SetWindowText(csLine);
}

void CItemScriptEditDlg::PreviewSpellTag()
{
	int iSel = this->m_clbTags.GetCurSel();
	if (iSel == -1)
	{
		this->m_csPreview.SetWindowText("");
		return;
	}
	CString csValue, csLine;
	DWORD dwData = this->m_clbTags.GetItemData(iSel);
	int iTag = dwData & 0xFFFF;
	CString csTag = knownSpellTags[iTag];
	switch ( iTag )
	{
		// Simple values
	case SPELLTAG_NAME:
	case SPELLTAG_EFFECT_LO:
	case SPELLTAG_EFFECT_HI:
	case SPELLTAG_RUNE_ITEM:
	case SPELLTAG_SCROLL_ITEM:
	case SPELLTAG_EFFECT_ID:
	case SPELLTAG_REAGENTS:
	case SPELLTAG_RUNES:
		this->m_ceMaxValue.GetWindowText(csValue);
		break;
		// Item values
		// Selection
	case SPELLTAG_SOUND:
		// Others
	case SPELLTAG_FLAGS:
		csValue = CSpellDef::FlagString(GetFlags());
		break;
	}
	if ( csValue != "" )
		csLine.Format("%s=%s", csTag, csValue);
	this->m_csPreview.SetWindowText(csLine);
}

void CItemScriptEditDlg::FillSpellFlags()
{
	this->m_clbFlags.ResetContent();
	for ( int i = 0; i < 16; i++ )
	{
		if ( strcmp(SpellFlags[i], "") != 0 )
		{
			int index = this->m_clbFlags.AddString(SpellFlags[i]);
			this->m_clbFlags.SetItemData(index, 1 << i );
		}
	}
}

void CItemScriptEditDlg::FillSounds()
{
	m_ccbCombo.ResetContent();
	for ( int i = 0; i < 589; i++ )
	{
		int index = this->m_ccbCombo.AddString(Sounds[i]);
		this->m_ccbCombo.SetItemData(index, i);
	}
}
