/*
 $Id: PotionEdit.cpp,v 1.5 2002/05/01 03:33:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// PotionEdit.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "PotionEdit.h"
#include "ColorSelectionDlg.h"
#include "RemoteConnection.h"
#include "SpawnMessageDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPotionEdit dialog

CPotionEdit::CPotionEdit(CWnd* pParent /*=NULL*/)
	: CDialog(CPotionEdit::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPotionEdit)
	//}}AFX_DATA_INIT
	m_pCurrentPotion = NULL;
}


void CPotionEdit::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPotionEdit)
	DDX_Control(pDX, IDC_POTIONSKILLREQUIRED, m_ceSkillRequired);
	DDX_Control(pDX, IDC_POTIONREAGENTAMOUNT, m_ceReagentAmount);
	DDX_Control(pDX, IDC_POTIONREAGENT, m_ccbReagent);
	DDX_Control(pDX, IDC_POTIONITEM, m_ccbItem);
	DDX_Control(pDX, IDC_POTIONENTRY, m_ccbEntry);
	DDX_Control(pDX, IDC_POTIONCOLOURPALETTE, m_csPalette);
	DDX_Control(pDX, IDC_POTIONCOLOUR, m_ceColour);
	DDX_Control(pDX, IDC_POTIONNAME, m_ceName);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPotionEdit, CDialog)
	//{{AFX_MSG_MAP(CPotionEdit)
	ON_EN_CHANGE(IDC_POTIONCOLOUR, OnChangePotioncolour)
	ON_BN_CLICKED(IDC_POTIONCOLOURPALETTE, OnPotioncolourpalette)
	ON_CBN_SELCHANGE(IDC_POTIONENTRY, OnSelchangePotionentry)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPotionEdit message handlers

BOOL CPotionEdit::OnInitDialog() 
{
	CWaitCursor hourglass;
	CDialog::OnInitDialog();
	m_pCurrentPotion = NULL;
	
	Main->m_pScripts->LoadDefs();
	Main->m_pScripts->LoadTables();
	for ( int i = 0; i < Main->m_pScripts->m_tables.m_potions.GetSize(); i++ )
	{
		CTPotionDef * pPotion = (CTPotionDef *) Main->m_pScripts->m_tables.m_potions.GetAt(i);
		if ( pPotion )
		{
			int iEntry = this->m_ccbEntry.AddString(pPotion->m_csName);
			this->m_ccbEntry.SetItemDataPtr(iEntry, (void *) pPotion);
		}
	}
	FillData();
	if ( this->m_ccbEntry.GetCount() > 0 )
	{
		this->m_ccbEntry.SetCurSel(0);
		this->OnSelchangePotionentry();
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CPotionEdit::OnChangePotioncolour() 
{
	CString csValue;
	this->m_ceColour.GetWindowText(csValue);
	m_wColorIndex = (WORD) DefLookup(csValue);
	UpdateSpectrum();
}

void CPotionEdit::OnPotioncolourpalette() 
{
	CColorSelectionDlg dlg;
	if ( dlg.DoModal() == IDOK )
	{
		m_wColorIndex = dlg.GetColorIndex();
		CString csValue;
		csValue.Format("%05x", m_wColorIndex);
		this->m_ceColour.SetWindowText(csValue);		
	}
	return;		
}

void CPotionEdit::OnSelchangePotionentry() 
{
	// Get the current values
	if ( this->m_pCurrentPotion != NULL)
	{
		this->m_ceName.GetWindowText(this->m_pCurrentPotion->m_csName);
		this->m_ccbItem.GetWindowText(this->m_pCurrentPotion->m_csPotionID);
		this->m_ccbReagent.GetWindowText(this->m_pCurrentPotion->m_csRegID);
		CString csVal;
		this->m_ceReagentAmount.GetWindowText(csVal);
		this->m_pCurrentPotion->m_iRegReq = atoi(csVal);
		this->m_ceSkillRequired.GetWindowText(csVal);
		if ( csVal.Find(".") == -1 )
			this->m_pCurrentPotion->m_iSkillReq = atoi(csVal) * 10;
		else
		{
			csVal.Replace(".", "");
			this->m_pCurrentPotion->m_iSkillReq = atoi(csVal);
		}
		this->m_ceColour.GetWindowText(this->m_pCurrentPotion->m_csColor);
	}
	// Fill in the values of the new selection
	int iSel = this->m_ccbEntry.GetCurSel();
	if ( iSel == -1 )
		return;
	this->m_pCurrentPotion = (CTPotionDef *) this->m_ccbEntry.GetItemDataPtr(iSel);
	if ( this->m_pCurrentPotion != NULL )
	{
		this->m_ceName.SetWindowText(m_pCurrentPotion->m_csName);
		this->m_ccbItem.SetWindowText(m_pCurrentPotion->m_csPotionID);
		this->m_ccbReagent.SetWindowText(m_pCurrentPotion->m_csRegID);
		CString csVal;
		csVal.Format("%ld", m_pCurrentPotion->m_iRegReq);
		this->m_ceReagentAmount.SetWindowText(csVal);
		csVal.Format("%.1f", (float) (m_pCurrentPotion->m_iSkillReq / 10.0));
		this->m_ceSkillRequired.SetWindowText(csVal);
		this->m_ceColour.SetWindowText(m_pCurrentPotion->m_csColor);
		csVal = "";
		CString csTmp = "";
	}
	else
	{
		// This should theoretically never happen.
		AfxMessageBox("Invalid Entry!", MB_OK | MB_ICONEXCLAMATION);
		this->m_ceName.SetWindowText("");
		this->m_ceColour.SetWindowText("");
		this->m_ccbItem.SetWindowText("");
		this->m_ccbReagent.SetWindowText("");
		this->m_ceSkillRequired.SetWindowText("");
		this->m_ceReagentAmount.SetWindowText("");
	}
}

CPotionEdit::~CPotionEdit()
{
}

void CPotionEdit::FillData()
{
	for ( int i = 0; i < Main->m_pScripts->m_aDefs.GetSize(); i++ )
	{
		CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(i);
		if ( !pDef )
			continue;
		if ( pDef->m_csGroup == "REAGENTS" )
			this->m_ccbReagent.AddString(pDef->m_csValue);
		if ( pDef->m_csGroup == "POTIONS" && pDef->m_csValue.Find("BOTTLE_") == 0 )
			this->m_ccbItem.AddString(pDef->m_csValue);
	}
}

void CPotionEdit::UpdateSpectrum()
{
	CDC * pDC = this->m_csPalette.GetDC();
	CRgn rgn;
	CRect rc;
	CBrush * pNewBrush = new (CBrush);
	pNewBrush->CreateSolidBrush(GetSysColor(COLOR_3DFACE));
	CBrush *pOldBrush = pDC->SelectObject(pNewBrush);
	m_csPalette.GetClientRect(&rc);
	rgn.CreateRectRgnIndirect(rc);
	pDC->SelectClipRgn(&rgn);
	int iWidth = rc.Width();
	int iHeight = rc.Height();
	pDC->FillRect(rc, pNewBrush);
	pDC->SelectObject(pOldBrush);
	delete (pNewBrush);

	if ( m_wColorIndex < 1 || m_wColorIndex > 3000 )
	{
		this->m_csPalette.ReleaseDC(pDC);
		return;
	}
	int groupIndex = (m_wColorIndex - 1) / 8;
	int hueIndex = (m_wColorIndex - 1) % 8;
	CHueGroup * pGroup = (CHueGroup *) Main->m_aHueGroups.GetAt(groupIndex);
	if ( pGroup == NULL )
	{
		this->m_csPalette.ReleaseDC(pDC);
		return;
	}
	CHueEntry Hue = pGroup->Hues[hueIndex];

	int cw = iWidth / 32; // Cell width
	int left = (iWidth / 2) - (16 * cw);
	int right = (iWidth / 2) + (16 * cw);

	for ( int x = left; x < right; x++ )
	{
		for ( int y = 0; y < iHeight; y++ )
		{
			// Which cell are we in?
			int shadeIndex = x / cw;
			if ( shadeIndex > 0x1f ) 
				continue;
			pDC->SetPixel(x, y, ScaleColor(Hue.wColorTable[shadeIndex]));
		}
	}

	this->m_csPalette.ReleaseDC(pDC);
}

void CPotionEdit::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	UpdateSpectrum();	
}

void CPotionEdit::OnOK() 
{
	CWaitCursor hourglass;
	if ( Main->m_pScripts->SaveTables() )
		CDialog::OnOK();
}

void CPotionEdit::OnCancel() 
{
	Main->m_pScripts->ReloadTables();
	CDialog::OnCancel();
}
