// RoofingDlg.cpp : implementation file
//
/*
 **********************************************************************
 * Copyright (C) Adron 2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************
*/
#include "stdafx.h"
#include "drewsky.h"
#include "RoofingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRoofingDlg dialog


CRoofingDlg::CRoofingDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRoofingDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRoofingDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	FILE *f = fopen("rooftiles.cfg", "rt");
	if(!f)
		return;
	char line[128];
	int flags, id;
	TileSet *ts = 0;
	while(fgets(line, 128, f)) {
		if(line[0] == '[') {
			ts = new TileSet;
			ts->name = strtok(line, "[]\r\n");
			m_tilesets.AddTail(ts);
		} else if(ts && sscanf(line, "%x %x", &flags, &id) == 2) {
			ts->tiles.AddTail(TileMask(flags, (short)id));
		}
	}
	fclose(f);
}


void CRoofingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRoofingDlg)
	DDX_Control(pDX, IDC_TESTGEN, m_testgen);
	DDX_Control(pDX, IDC_NOSCRIPT, m_noscript);
	DDX_Control(pDX, IDC_TENT, m_tent);
	DDX_Control(pDX, IDC_ROOFTYPE, m_rooftype);
	DDX_Control(pDX, IDC_CLEAR, m_clear);
	DDX_Control(pDX, IDC_REMOVELAST, m_removelast);
	DDX_Control(pDX, IDC_HEIGHT, m_height);
	DDX_Control(pDX, IDC_GENERATE, m_generate);
	DDX_Control(pDX, IDC_ADD, m_add);
	DDX_Control(pDX, IDC_UP, m_up);
	DDX_Control(pDX, IDC_ROOFIMAGE, m_roofimage);
	DDX_Control(pDX, IDC_Y2, m_y2);
	DDX_Control(pDX, IDC_Y1, m_y1);
	DDX_Control(pDX, IDC_X2, m_x2);
	DDX_Control(pDX, IDC_X1, m_x1);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRoofingDlg, CDialog)
	//{{AFX_MSG_MAP(CRoofingDlg)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_BN_CLICKED(IDC_GENERATE, OnGenerate)
	ON_BN_CLICKED(IDC_REMOVELAST, OnRemovelast)
	ON_EN_CHANGE(IDC_X1, OnChangeX1)
	ON_EN_CHANGE(IDC_X2, OnChangeX2)
	ON_EN_CHANGE(IDC_Y1, OnChangeY1)
	ON_EN_CHANGE(IDC_Y2, OnChangeY2)
	ON_EN_CHANGE(IDC_HEIGHT, OnChangeHeight)
	ON_BN_CLICKED(IDC_TESTGEN, OnTestgen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRoofingDlg message handlers

void ShowRoofing()
{
	CRoofingDlg dlg;
	dlg.DoModal();
}

void CRoofingDlg::OnAdd() 
{
	Edge e;
	e.rc.left = GetDlgItemInt(IDC_X1);
	e.rc.top = GetDlgItemInt(IDC_Y1);
	e.rc.right = GetDlgItemInt(IDC_X2);
	e.rc.bottom = GetDlgItemInt(IDC_Y2);
	e.up = !!m_up.GetCheck();
	e.tent = !!m_tent.GetCheck();
	e.rc.NormalizeRect();
	if(e.rc.Height() && e.rc.Width()) {
		if(!e.up && (e.rc.bottom - e.rc.top) % 2 || e.up && (e.rc.right - e.rc.left) % 2) {
			if(MessageBox("This roof isn't odd wide - it'll look weird. Do it anyway?", 0, MB_YESNO) == IDNO)
				return;
		}
	} else if(!e.rc.Height() && !e.rc.Width()) {
		MessageBox("Can't add an empty rectangle");
		return;
	} else if(e.rc.Height() && (e.rc.bottom - e.rc.top) % 2 || e.rc.Width() && (e.rc.right - e.rc.left) % 2) {
		if(MessageBox("This roof isn't odd wide - it'll look weird. Do it anyway?", 0, MB_YESNO) == IDNO)
			return;
	}
	POSITION pos;
	pos = m_edges.GetHeadPosition();
	while(pos) {
		Edge& eo = m_edges.GetNext(pos);
		if(eo.rc == e.rc && eo.up == e.up) {
			MessageBox("You have already added these coordinates");
			return;
		}
	}

	m_edges.AddTail(e);
	if(!Calculate()) {
		MessageBox("Sorry, can't roof those coordinates");
		m_edges.RemoveTail();
		Calculate();
	}
	CheckRemove();
	CheckGenerate();
	m_roofimage.RedrawWindow();
}

void CRoofingDlg::OnClear() 
{
	m_edges.RemoveAll();
	Calculate();
	CheckRemove();
	CheckGenerate();
	m_roofimage.RedrawWindow();
}

// Generate the roof height map in m_roofimage.m_data
bool CRoofingDlg::Calculate()
{
	int i;
	POSITION pos;
	CRect bounds;
	pos = m_edges.GetHeadPosition();
	if(m_roofimage.m_data) 
		delete [] m_roofimage.m_data;
	m_roofimage.m_data = 0;
	if(!pos) {
		return true;
	}
	bounds = m_edges.GetNext(pos).rc;
	while(pos) {
		Edge& e = m_edges.GetNext(pos);
		if(bounds.left > e.rc.left)
			bounds.left = e.rc.left;
		if(bounds.top > e.rc.top)
			bounds.top = e.rc.top;
		if(bounds.right < e.rc.right)
			bounds.right = e.rc.right;
		if(bounds.bottom < e.rc.bottom)
			bounds.bottom = e.rc.bottom;
	}
	if(bounds.Width() > 150 || bounds.Height() > 150)
		return false;

	m_roofimage.m_w = bounds.Width()+3;
	m_roofimage.m_h = bounds.Height()+3;
	m_roofimage.m_data = new char[m_roofimage.m_w * m_roofimage.m_h];
	memset(m_roofimage.m_data, 0, m_roofimage.m_w * m_roofimage.m_h);
	
	m_base = bounds.TopLeft() + CPoint(-1, -1);
	pos = m_edges.GetHeadPosition();
	bounds -= m_base;

	while(pos) {
		Edge e = m_edges.GetNext(pos);
		e.rc -= m_base;
		if(e.rc.Width() && e.rc.Height()) {
			if(e.up) {
				for(i = 0; i < e.rc.Height()+1; i++) {
					AddVX(e.rc.left, e.rc.right, e.rc.top + i, e.tent ? min(i+1, e.rc.Height() + 1 - i) : 1000);
				}
			} else {
				for(i = 0; i < e.rc.Width()+1; i++) {
					AddVY(e.rc.top, e.rc.bottom, e.rc.left + i, e.tent ? min(i+1, e.rc.Width() + 1 - i) : 1000);
				}
			}
		} else if(e.rc.Width()) {
			int maxh = e.tent ? 1 : 1000;
			if(e.up) {
				while(1) {
					if(!AddVX(e.rc.left, e.rc.right, e.rc.top, maxh))
						break;
					maxh++;
					if(++e.rc.top >= bounds.bottom)
						return false;
				}
			} else {
				while(1) {
					if(!AddVX(e.rc.left, e.rc.right, e.rc.top, maxh))
						break;
					maxh++;
					if(--e.rc.top < 0)
						return false;
				}
			}
		} else {
			int maxh = e.tent ? 1 : 1000;
			if(e.up) {
				while(1) {
					if(!AddVY(e.rc.top, e.rc.bottom, e.rc.left, maxh))
						break;
					maxh++;
					if(++e.rc.left >= bounds.right)
						return false;
				}
			} else {
				while(1) {
					if(!AddVY(e.rc.top, e.rc.bottom, e.rc.left, maxh))
						break;
					maxh++;
					if(--e.rc.left < 0)
						return false;
				}
			}
		}
	}

	return true;
}



// Add V formed roof, but only if higher than existing roof
// return whether anything was added
bool CRoofingDlg::AddVX(int x1, int x2, int y, int hlimit)
{
	bool added = false;
	int h = 1;
	while(x2 >= x1) {
		if(m_roofimage.m_data[x2 + y*m_roofimage.m_w] < h) {
			m_roofimage.m_data[x2 + y*m_roofimage.m_w] = h;
			added = true;
		}
		if(m_roofimage.m_data[x1 + y*m_roofimage.m_w] < h) {
			m_roofimage.m_data[x1 + y*m_roofimage.m_w] = h;
			added = true;
		}
		x1++, x2--;
		if(h < hlimit) h++;
	}
	return added;
}



// Add V formed roof, but only if higher than existing roof
// return whether anything was added
bool CRoofingDlg::AddVY(int y1, int y2, int x, int hlimit)
{
	bool added = false;
	int h = 1;
	while(y2 >= y1) {
		if(m_roofimage.m_data[x + y2*m_roofimage.m_w] < h) {
			m_roofimage.m_data[x + y2*m_roofimage.m_w] = h;
			added = true;
		}
		if(m_roofimage.m_data[x + y1*m_roofimage.m_w] < h) {
			m_roofimage.m_data[x + y1*m_roofimage.m_w] = h;
			added = true;
		}
		y1++, y2--;
		if(h < hlimit) h++;
	}
	return added;
}

void CRoofingDlg::OnGenerate() 
{
	Generate(NOTEST);
}

void CRoofingDlg::OnRemovelast() 
{
	m_edges.RemoveTail();
	Calculate();
	CheckRemove();
	m_roofimage.RedrawWindow();
}

void CRoofingDlg::OnChangeX1() 
{
	CheckAdd();	
}

void CRoofingDlg::OnChangeX2() 
{
	CheckAdd();	
}

void CRoofingDlg::OnChangeY1() 
{
	CheckAdd();	
}

void CRoofingDlg::OnChangeY2() 
{
	CheckAdd();	
}

void CRoofingDlg::CheckAdd()
{
	if(m_x1.GetWindowTextLength() && m_x2.GetWindowTextLength() && m_y1.GetWindowTextLength() && m_y2.GetWindowTextLength()) {
		m_add.EnableWindow(true);
	} else {
		m_add.EnableWindow(false);
	}
}

void CRoofingDlg::CheckGenerate()
{
	if(m_roofimage.m_w && m_roofimage.m_h && m_roofimage.m_data && m_height.GetWindowTextLength()) {
		m_generate.EnableWindow(true);
		m_testgen.EnableWindow(true);
	} else {
		m_generate.EnableWindow(false);
		m_testgen.EnableWindow(false);
	}
}

void CRoofingDlg::CheckRemove()
{
	if(m_edges.IsEmpty()) {
		m_removelast.EnableWindow(false);
		m_clear.EnableWindow(false);
	} else {
		m_removelast.EnableWindow(true);
		m_clear.EnableWindow(true);
	}
}

short CRoofingDlg::LookupID(unsigned flags)
{
	POSITION pos;
	pos = m_tileset->tiles.GetHeadPosition();
	while(pos) {
		TileMask& tm = m_tileset->tiles.GetNext(pos);
		if((flags & ~tm.flags) == 0)
			return tm.id;
	}
	return 0;
}

unsigned CRoofingDlg::Compare(char middle, char relative)
{
	if(relative == 0)
		return EMPTY;
	if(relative < 0)
		relative = -relative;
	if(relative == middle)
		return EVEN;
	if(relative < middle)
		return LOWER;
	if(relative > middle)
		return HIGHER;
	return 0;
}

void CRoofingDlg::OnChangeHeight() 
{
	CheckGenerate();
}

CRoofingDlg::~CRoofingDlg()
{
	while(!m_tilesets.IsEmpty()) {
		delete m_tilesets.RemoveHead();
	}
}

BOOL CRoofingDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	POSITION pos;
	TileSet *ts = 0;
	pos = m_tilesets.GetHeadPosition();
	while(pos) {
		ts = m_tilesets.GetNext(pos);
		m_rooftype.AddString(ts->name);
	}
	m_rooftype.SetCurSel(0);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CRoofingDlg::OnTestgen() 
{
	if(Generate(TEST)) {
		if(MessageBox("Would you like to generate the remaining pieces?", 0, MB_YESNO) == IDYES) {
			Generate(REST);
		}
	}
}

bool CRoofingDlg::Generate(testmodes testmode)
{
	bool usescript = !m_noscript.GetCheck();

	int seltype = m_rooftype.GetCurSel();
	if(seltype == CB_ERR) {
		MessageBox("You must select a tile set");
		return false;
	}
	
	CString seltypename;
	m_rooftype.GetLBText(seltype, seltypename);
	POSITION pos;
	pos = m_tilesets.GetHeadPosition();
	while(pos) {
		TileSet* ts = m_tilesets.GetNext(pos);
		if(seltypename == ts->name) {
			m_tileset = ts;
			break;
		}
	}
	
	short *roofids = new short[m_roofimage.m_w * m_roofimage.m_h];
	int i;
	bool fail = false;
	for(i = 0; i < m_roofimage.m_w * m_roofimage.m_h; i++) {
		if(m_roofimage.m_data[i] < 0)
			m_roofimage.m_data[i] = -m_roofimage.m_data[i];
	}
	for(i = 0; i < m_roofimage.m_w * m_roofimage.m_h; i++) {
		if(m_roofimage.m_data[i] == 0) {
			roofids[i] = 0;
		} else {
			unsigned flags;
			flags = GetFlags(m_roofimage.m_data + i - m_roofimage.m_w,
				m_roofimage.m_data + i,
				m_roofimage.m_data + i + m_roofimage.m_w);
			roofids[i] = LookupID(flags);
			if(roofids[i] == 0) {
				m_roofimage.m_data[i] = -m_roofimage.m_data[i];
				fail = true;
			}
			if(testmode) {
				bool corner = !(flags & ~0x88878778) ||
					!(flags & ~0x88887877) ||
					!(flags & ~0x77878888) ||
					!(flags & ~0x87787888) ||
					!(flags & ~0x87777777) ||
					!(flags & ~0x77877777) ||
					!(flags & ~0x77777877) ||
					!(flags & ~0x77777778);

				if(testmode == TEST && !corner)
					roofids[i] = 0;
				if(testmode == REST && corner)
					roofids[i] = 0;
			}
		}
	}
	if(fail) {
		m_roofimage.RedrawWindow();
		if(MessageBox("Generation failed, missing roof pieces marked red - generate anyway?", 0, MB_YESNO) == IDNO) {
			delete [] roofids;
			return false;
		}
	}

	int baseheight = GetDlgItemInt(IDC_HEIGHT);
	int j, p = 0;
	int dx, dy;
	CString cmd;
	for(j = 0; j < m_roofimage.m_h; j++) {
		for(i = 0; i < m_roofimage.m_w; i++, p++) {
			if(!roofids[p])
				continue;
			for(dx = 1; dx + i < m_roofimage.m_w; dx++)
				if(roofids[p + dx] != roofids[p] || 
						m_roofimage.m_data[p] != m_roofimage.m_data[p + dx])
					break;
			for(dy = 1; dy + j < m_roofimage.m_h; dy++)
				if(roofids[p + m_roofimage.m_w * dy] != roofids[p] || 
						m_roofimage.m_data[p] != m_roofimage.m_data[p + m_roofimage.m_w * dy])
					break;
			dx--, dy--;
			if(dx > 0 || dy > 0) {
				cmd.Format(".tile %d 0%x", baseheight + 3 * m_roofimage.m_data[p] - 3, (unsigned short)roofids[p]);
				SendToUO(cmd);
				if(usescript) {
					if ( Main->m_dwCommandVersion )
						cmd.Format(".f_roof_target(%d,%d)", m_base.x + i, m_base.y + j);
					else
						cmd.Format(".f_roof_target %d,%d", m_base.x + i, m_base.y + j);
					SendToUO(cmd);
				} else {
					cmd.Format(".go %d,%d", m_base.x + i, m_base.y + j);
					SendToUO(cmd);
					SendToUO(".self");
				}
				if(dy > dx) {
					if(usescript) {
						if ( Main->m_dwCommandVersion )
							cmd.Format(".f_roof_target(%d,%d)", m_base.x + i, m_base.y + j + dy);
						else
							cmd.Format(".f_roof_target %d,%d", m_base.x + i, m_base.y + j + dy);
						SendToUO(cmd);
					} else {
						cmd.Format(".go %d,%d", m_base.x + i, m_base.y + j + dy);
						SendToUO(cmd);
						SendToUO(".self");
					}
					while(dy >= 0) {
						roofids[p + m_roofimage.m_w * dy--] = 0;
					}
				} else {
					if(usescript) {
						if ( Main->m_dwCommandVersion )
							cmd.Format(".f_roof_target(%d,%d)", m_base.x + i + dx, m_base.y + j);
						else
							cmd.Format(".f_roof_target %d,%d", m_base.x + i + dx, m_base.y + j);
						SendToUO(cmd);
					} else {
						cmd.Format(".go %d,%d", m_base.x + i + dx, m_base.y + j);
						SendToUO(cmd);
						SendToUO(".self");
					}
					while(dx >= 0) {
						roofids[p + dx--] = 0;
					}
					i += dx;
					p += dx;
				}
			} else {
				cmd.Format(".static 0%x", (unsigned short)roofids[p]);
				SendToUO(cmd);
				if(usescript) {
					if ( Main->m_dwCommandVersion )
						cmd.Format(".f_roof_target(%d,%d,%d)", m_base.x + i, m_base.y + j, baseheight + 3 * m_roofimage.m_data[p] - 3 - 1);
					else
						cmd.Format(".f_roof_target %d,%d,%d", m_base.x + i, m_base.y + j, baseheight + 3 * m_roofimage.m_data[p] - 3 - 1);
					SendToUO(cmd);
				} else {
					cmd.Format(".go %d,%d,%d", m_base.x + i, m_base.y + j, baseheight + 3 * m_roofimage.m_data[p] - 3 - 1);
					SendToUO(cmd);
					SendToUO(".self");
				}
				roofids[p] = 0;
			}
		}
	}
	delete [] roofids;
	return true;
}

unsigned CRoofingDlg::GetFlags(char *pline, char *line, char *nline)
{
	unsigned flags = 0;
	flags |= Compare(line[0], pline[-1]); flags <<= 4;
	flags |= Compare(line[0], pline[0]); flags <<= 4;
	flags |= Compare(line[0], pline[1]); flags <<= 4;
	flags |= Compare(line[0], line[-1]); flags <<= 4;
	flags |= Compare(line[0], line[1]); flags <<= 4;
	flags |= Compare(line[0], nline[-1]); flags <<= 4;
	flags |= Compare(line[0], nline[0]); flags <<= 4;
	flags |= Compare(line[0], nline[1]);
	return flags;
}
