/*
 $Id: ScriptEditDlg.cpp,v 1.6 2002/05/14 20:03:35 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// ScriptEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "ScriptEditDlg.h"
#include "RemoteConsole.h"
#include "RemoteFileDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScriptEditDlg dialog


CScriptEditDlg::CScriptEditDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CScriptEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CScriptEditDlg)
	//}}AFX_DATA_INIT
}


void CScriptEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScriptEditDlg)
	DDX_Control(pDX, IDC_SCRIPTEDIT, m_creScript);
	DDX_Control(pDX, IDOK, m_cbOK);
	DDX_Control(pDX, IDCANCEL, m_cbCancel);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CScriptEditDlg, CDialog)
	//{{AFX_MSG_MAP(CScriptEditDlg)
	ON_WM_SIZE()
	ON_EN_CHANGE(IDC_SCRIPTEDIT, OnChangeScriptedit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScriptEditDlg message handlers


void CScriptEditDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	if ( this->IsWindowVisible() )
	{
		CRect cr, wr;
		this->GetWindowRect(&wr);
		this->m_creScript.GetWindowRect(&cr);
		int border = (cr.left - wr.left);
		cr.right = wr.right - border;
		cr.bottom = wr.bottom - border;
		this->m_creScript.SetWindowPos( NULL, 0, 0, cr.Width(), cr.Height(), SWP_NOMOVE | SWP_NOZORDER);
		this->m_cbCancel.GetWindowRect(&cr);
		cr.left = wr.right - border - cr.Width();
		this->ScreenToClient(&cr);
		this->m_cbCancel.SetWindowPos( NULL, cr.left, cr.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		this->m_cbOK.GetWindowRect(&cr);
		cr.left = wr.right - border - cr.Width();
		this->ScreenToClient(&cr);
		this->m_cbOK.SetWindowPos( NULL, cr.left, cr.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
	}
}

BOOL CScriptEditDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	if ( Main->GetProfileType() == PROFILE_REMOTE )
	{
		CRemoteFileDlg dlg;
		if ( dlg.DoModal() == IDOK )
		{
			this->m_csFilename.Format("%s", dlg.m_csFilename);
			this->m_csRemoteName = dlg.m_csRemoteFile;
		}
		else
		{
			this->m_csFilename = "";
			this->m_creScript.EnableWindow(FALSE);
			this->m_cbOK.EnableWindow(FALSE);
			this->OnCancel();
		}
	}
	else
	{
		char szCwd[MAX_PATH];
		memset(&szCwd[0], 0x00, sizeof(szCwd));
		getcwd(szCwd, sizeof(szCwd));
		CFileDialog dlg(TRUE, NULL, NULL, 0, "Sphere Files (*.scp;*.ini)|*.scp; *.ini|Log Files (*.log)|*.log||");
		if ( dlg.DoModal() == IDOK)
		{
			chdir(szCwd);
			this->m_csFilename = dlg.GetPathName();
		}
		else
		{
			chdir(szCwd);
			this->m_csFilename = "";
			this->m_creScript.EnableWindow(FALSE);
			this->m_cbOK.EnableWindow(FALSE);
			this->OnCancel();
		}
	}

	CWaitCursor hourglass;

	CRect wr, cr;
	this->m_cbCancel.GetWindowRect(&cr);
	this->GetWindowRect(&wr);
	this->m_iRightBorder = wr.right - cr.right - GetSystemMetrics(SM_CXBORDER);
	this->m_creScript.LimitText(0x7FFFFFFF);
	if ( this->m_csFilename != "" )
	{
		CString csCaption;
		csCaption.Format("Script Editor - %s", this->m_csFilename);
		this->SetWindowText(csCaption);
		if ( !m_cfFile.Open(m_csFilename, CFile::modeRead | CFile::shareDenyNone | CFile::typeBinary ) )
		{
			Main->m_log.Add("Unable to open file %s in raw text editor", m_csFilename);
			AfxMessageBox("Unable to open file");
		}
		else
		{
			Main->m_log.Add("Successfully opened file %s in raw text editor", m_csFilename);
			Main->m_log.Add("File size is %ld bytes", m_cfFile.GetLength());
			EDITSTREAM stream;
			stream.dwCookie = (DWORD) this;
			stream.dwError = 0;
			stream.pfnCallback = streamIn;
			this->m_creScript.StreamIn(SF_TEXT, stream);
			m_cfFile.Close();
		}
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CScriptEditDlg::OnOK() 
{
	CWaitCursor hourglass;
	bool bDeleteConsole = true;
	CRemoteConsole * pConsole = Main->m_pRConsole;
	if ( pConsole != NULL )
		bDeleteConsole = false;
	else
		pConsole = new (CRemoteConsole);
	if ( Main->GetProfileType() != PROFILE_REMOTE )
		pConsole->Pause();

	if (m_cfFile.Open(this->m_csFilename, CFile::modeWrite | CFile::modeCreate | CFile::typeBinary | CFile::shareDenyWrite))
	{
		EDITSTREAM stream;
		stream.dwCookie = (DWORD) this;
		stream.dwError = 0;
		stream.pfnCallback = streamOut;
		this->m_creScript.StreamOut(SF_TEXT, stream);
		m_cfFile.Close();
	}
	else
	{
		CString csMessage;
		csMessage.Format("Unable to open %s for writing.", this->m_csFilename);
		if ( AfxMessageBox(csMessage, MB_OKCANCEL | MB_ICONEXCLAMATION) == IDCANCEL )
		{
			pConsole->Resync();
			if ( bDeleteConsole )
				delete pConsole;
			return;
		}
	}
	if ( Main->GetProfileType() == PROFILE_REMOTE )
	{
		CRemoteConnection remote;
		remote.SendCustomFile(m_csRemoteName, m_csFilename);
		CFile::Remove(m_csFilename);
	}
	if ( Main->GetProfileType() != PROFILE_REMOTE )
		pConsole->Resync();
	if ( bDeleteConsole )
		delete pConsole;
	CDialog::OnOK();
}

void CScriptEditDlg::OnCancel() 
{
	if ( Main->GetProfileType() == PROFILE_REMOTE && m_csFilename != "" )
	{
		try
		{
			CFile::Remove(m_csFilename);
		}
		catch (CException * e)
		{
			e->Delete();
		}
	}
	CDialog::OnCancel();
}

void CScriptEditDlg::OnChangeScriptedit() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
}

DWORD CALLBACK streamIn(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	CScriptEditDlg * pDlg = (CScriptEditDlg *) dwCookie;

	memset((void*) pbBuff, 0x00, cb);
	long lBytesRead = pDlg->m_cfFile.Read((void *)pbBuff, cb);
	*pcb = lBytesRead;
	if ( lBytesRead )
		return 0;
	else
		return 1;
}

DWORD CALLBACK streamOut(DWORD dwCookie,  LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	CScriptEditDlg * pDlg = (CScriptEditDlg *) dwCookie;

	pDlg->m_cfFile.Write(pbBuff, cb);
	*pcb = cb;
	return 0;
}

BOOL CScriptEditDlg::PreTranslateMessage(MSG* pMsg) 
{
	// Find out which window has the focus
	CWnd * pFocus = (CWnd *) this->GetFocus();
	if ( pFocus->GetSafeHwnd() == this->m_creScript.GetSafeHwnd() )
	{
		if ( pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_TAB)
		{
			this->m_creScript.ReplaceSel("\t", TRUE);
			return TRUE;
		}
	}
	return CDialog::PreTranslateMessage(pMsg);
}
