/*
 $Id: SkillDef.cpp,v 1.3 2002/05/01 03:33:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// SkillDef.cpp

#include "stdafx.h"
#include "SkillDef.h"

const char * knownSkillTags [] =
{
	"KEY",
	"TITLE",
	"PROMPT_MSG",
	"FAIL_MSG",
	"DELAY",
	"STAT_STR",
	"STAT_INT",
	"STAT_DEX",
	"BONUS_STR",
	"BONUS_DEX",
	"BONUS_INT",
	"SKILL_STAT",
	"ADV_RATE",
};

CSkillDef::CSkillDef()
{
	this->m_iBonusDex = 0;
	this->m_iBonusInt = 0;
	this->m_iBonusStr = 0;
	this->m_iRate1 = 0;
	this->m_iRate2 =0;
	this->m_iRate3 = 0;
	this->m_iSkillDelay = 0;
	this->m_iSkillStat = 0;
	this->m_iStatDex = 0;
	this->m_iStatInt = 0;
	this->m_iStatStr = 0;
	this->m_bType = TYPE_SKILL;
}

bool CSkillDef::Read(CStdioFile &csfInput)
{
	CString csLine;
	BOOL bStatus = TRUE;
	CString csVal;
	while (bStatus)
	{
		bStatus = csfInput.ReadString(csLine);
		if (!bStatus)
			break;
		if (csLine != "")
		{
			if (csLine.Find("[") == 0)
			{
				csfInput.Seek(-(csLine.GetLength() + 2), CFile::current);
				break;
			}
			else
			{
				csLine = csLine.SpanExcluding("//");
				csLine.TrimLeft();
				csLine.TrimRight();
				if (csLine != "")
				{
					if (csLine.Find("=") != -1)
					{
						CString csTag;
						csTag = csLine.SpanExcluding("=");
						csVal = csLine.Mid(csLine.Find("=") + 1);
						ReadTag(csTag, csVal);
					}
				}
			}
		}
	}
	return bStatus ? true : false;
}

void CSkillDef::ReadTag(CString csTag, CString csValue)
{
	switch(FindTable(csTag, knownSkillTags, KNOWN_SKILL_TAGS, sizeof(char *)))
	{
	case 0:
		this->m_csKey = csValue; break;
	case 1:
		this->m_csTitle = csValue; break;
	case 2:
		this->m_csPromptMessage = csValue; break;
	case 3:
		this->m_csFailureMessage = csValue; break;
	case 4:
		this->m_iSkillDelay = atoi(csValue); break;
	case 5:
		this->m_iStatStr = atoi(csValue); break;
	case 6:
		this->m_iStatInt = atoi(csValue); break;
	case 7:
		this->m_iStatDex = atoi(csValue); break;
	case 8:
		this->m_iBonusStr = atoi(csValue); break;
	case 9:
		this->m_iBonusDex = atoi(csValue); break;
	case 10:
		this->m_iBonusInt = atoi(csValue); break;
	case 11:
		this->m_iSkillStat = atoi(csValue); break;
	case 12:
		{
			CString csVal1, csVal2, csVal3;
			csVal1 = csValue.SpanExcluding(",");
			csVal2 = csValue.Mid(csValue.Find(",") + 1);
			csVal3 = csVal2.Mid(csVal2.Find(",") + 1);
			csVal2 = csVal2.SpanExcluding(",");
			this->m_iRate1 = atoi(csVal1);
			this->m_iRate2 = atoi(csVal2);
			this->m_iRate3 = atoi(csVal3);
			break;
		}
	default:
		break;
	}
	return;
}

bool CSkillDef::Write(CStdioFile &csfOutput)
{
	CString csOut;
	csOut.Format("[SKILL %ld]\n", this->m_iIndex);
	csfOutput.WriteString(csOut);
	for (int i = 0; i < KNOWN_SKILL_TAGS; i++)
		WriteTag(knownSkillTags[i], csfOutput);
	return true;
}

void CSkillDef::WriteTag(CString csTag, CStdioFile &csfOutput)
{
	CString csOutput = "";
	switch(FindTable(csTag, knownSkillTags, KNOWN_SKILL_TAGS, sizeof(char *)))
	{
	case 0:
		csOutput.Format("KEY=%s\n", this->m_csKey); break;
	case 1:
		csOutput.Format("TITLE=%s\n", this->m_csTitle); break;
	case 2:
		csOutput.Format("PROMPT_MSG=%s\n", this->m_csPromptMessage); break;
	case 3:
		csOutput.Format("FAIL_MSG=%s\n", this->m_csFailureMessage); break;
	case 4:
		if (this->m_iSkillDelay != 0) csOutput.Format("DELAY=%ld\n", this->m_iSkillDelay); break;
	case 5:
		csOutput.Format("STAT_STR=%ld\n", this->m_iStatStr); break;
	case 6:
		csOutput.Format("STAT_INT=%ld\n", this->m_iStatInt); break;
	case 7:
		csOutput.Format("STAT_DEX=%ld\n", this->m_iStatDex); break;
	case 8:
		csOutput.Format("BONUS_STR=%ld\n", this->m_iBonusStr); break;
	case 9:
		csOutput.Format("BONUS_DEX=%ld\n", this->m_iBonusDex); break;
	case 10:
		csOutput.Format("BONUS_INT=%ld\n", this->m_iBonusInt); break;
	case 11:
		csOutput.Format("SKILL_STAT=%ld\n", this->m_iSkillStat); break;
	case 12:
		csOutput.Format("ADV_RATE=%ld,%ld,%ld\n", this->m_iRate1, this->m_iRate2, this->m_iRate3); break;
	default:
		break;
	}
	if (csOutput != "")
		csfOutput.WriteString(csOutput);
	return;
}