/*
 $Id: SpellDef.cpp,v 1.4 2002/05/05 01:47:44 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// SpellDef.cpp

#include "stdafx.h"
#include "SpellDef.h"

const char * knownSpellTags [] =
{
	"NAME",
	"SOUND",
	"RUNES",
	"REAGENTS",
	"RUNE_ITEM",
	"SCROLL_ITEM",
	"FLAGS",
	"EFFECT_ID",
	"EFFECT_LO",
	"EFFECT_HI",
};

const char * sm_SpellFlagDefs [] =
{
	"SPELLFLAG_DIR_ANIM",
	"SPELLFLAG_TARG_OBJ",
	"SPELLFLAG_TARG_CHAR",
	"SPELLFLAG_TARG_XYZ",
	"SPELLFLAG_HARM",
	"SPELLFLAG_FX_BOLT",
	"SPELLFLAG_FX_TARG",
	"SPELLFLAG_FIELD",
	"SPELLFLAG_SUMMON",
	"SPELLFLAG_GOOD",
	"",
	"",
	"",
	"",
	"",
	"SPELLFLAG_DISABLED",
};

enum SPELL_FLAGS
{
	SPELLFLAG_DIR_ANIM = 0x0001,	// Evoke type cast or directed. (animation)
	SPELLFLAG_TARG_OBJ = 0x0002,	// Need to target an object or char ?
	SPELLFLAG_TARG_CHAR = 0x0004,	// Needs to target a living thing
	SPELLFLAG_TARG_XYZ = 0x0008,	// Can just target a location.
	SPELLFLAG_HARM = 0x0010,	// The spell is in some way harmfull.
	SPELLFLAG_FX_BOLT = 0x0020,	// Effect is a bolt to the target.
	SPELLFLAG_FX_TARG = 0x0040,	// Effect is at the target.
	SPELLFLAG_FIELD = 0x0080,	// create a field of stuff. (fire,poison,wall)
	SPELLFLAG_SUMMON = 0x0100,	// summon a creature.
	SPELLFLAG_GOOD = 0x0200,	// The spell is a good spell. u intend to help to receiver.
	SPELLFLAG_DISABLED = 0x8000,
};

CSpellDef::CSpellDef()
{
	this->m_csName = "";
	this->m_csReagents = "";
	this->m_csWOP = "";
	this->m_dwEffect = 0;
	this->m_dwFlags = 0;
	this->m_dwID = 0;
	this->m_dwRune = 0;
	this->m_dwScroll = 0;
	this->m_dwSound = 0;
	this->m_iEffectHigh = 0;
	this->m_iEffectLo = 0;
	this->m_bType = TYPE_SPELL;
}

CSpellDef::~CSpellDef()
{
}

bool CSpellDef::Read(CStdioFile & csfInput)
{
	BOOL bStatus = true;
	while (bStatus)
	{
		CString csLine;
		bStatus = csfInput.ReadString(csLine);
		if (!bStatus)
			break;
		if (csLine != "")
		{
			if (csLine.Find("[") == 0)
			{
				bStatus = false;
				csfInput.Seek(-(csLine.GetLength() + 2), CFile::current);
				break;
			}
			csLine.TrimLeft();
			csLine.TrimRight();
			if (csLine != "")
			{
				CString csTag = csLine.SpanExcluding("=");
				CString csValue = csLine.Mid(csLine.Find("=") + 1);
				this->ReadTag(csTag, csValue);
			}
		}
	}
	return bStatus ? true : false;
}

void CSpellDef::ReadTag(CString csTag, CString csValue)
{
	switch(FindTable(csTag, &knownSpellTags[0], SPELLTAGCOUNT, sizeof(char *)))
	{
	case 0:
		this->m_csName = csValue; break;
	case 1:
		if (csValue.SpanIncluding("1234567890abcdefABCDEF") != csValue)
			this->m_dwSound = DefLookup(csValue);
		else
			this->m_dwSound = ahextoi(csValue);
		break;
	case 2:
		this->m_csWOP = csValue; break;
	case 3:
		this->m_csReagents = csValue; break;
	case 4:
		if (csValue.SpanIncluding("1234567890abcdefABCDEF") != csValue)
			this->m_dwRune = DefLookup(csValue);
		else
			this->m_dwRune = ahextoi(csValue);
		break;
	case 5:
		if (csValue.SpanIncluding("1234567890abcdefABCDEF") != csValue)
			this->m_dwScroll = DefLookup(csValue);
		else
			this->m_dwScroll = ahextoi(csValue);
		break;
	case 6:
		if (csValue.SpanIncluding("01234567890abcdefABCDEF") != csValue)
		{
			while (csValue != "")
			{
				CString csLeft = csValue.SpanExcluding("|");
				csLeft.TrimLeft();
				csLeft.TrimRight();
				this->m_dwFlags |= DefLookup(csLeft);
				if (csValue.Find("|") != -1)
					csValue = csValue.Mid(csValue.Find("|") + 1);
				else
					csValue = "";
			}
		}
		else
			this->m_dwFlags = ahextoi(csValue);
		break;
	case 7:
		if (csValue.SpanIncluding("1234567890abcdefABCDEF") != csValue)
			this->m_dwEffect = DefLookup(csValue);
		else
			this->m_dwEffect = ahextoi(csValue);
		break;
	case 8:
		this->m_iEffectLo = atoi(csValue); break;
	case 9:
		this->m_iEffectHigh = atoi(csValue); break;
	default:
		break;
	}
}

bool CSpellDef::Write(CStdioFile &csfOutput)
{
	CString csOutput;
	csOutput.Format("[Spell %ld]\n", this->m_dwID);
	csfOutput.WriteString(csOutput);
	for (int i = 0; i < SPELLTAGCOUNT; i++)
		WriteTag(csfOutput, knownSpellTags[i]);
	csfOutput.WriteString("\n");
	return true;
}

void CSpellDef::WriteTag(CStdioFile &csfOutput, CString csTag)
{
	CString csOutput;
	switch(FindTable(csTag, &knownSpellTags[0], SPELLTAGCOUNT, sizeof(char *)))
	{
	case 0:
		csOutput.Format("NAME=%s\n", this->m_csName); break;
	case 1:
		csOutput.Format("SOUND=%05x\n", this->m_dwSound); break;
	case 2:
		csOutput.Format("RUNES=%s\n", this->m_csWOP); break;
	case 3:
		csOutput.Format("REAGENTS=%s\n", this->m_csReagents); break;
	case 4:
		csOutput.Format("RUNE_ITEM=%05x\n", this->m_dwRune); break;
	case 5:
		csOutput.Format("SCROLL_ITEM=%05x\n", this->m_dwScroll); break;
	case 6:
		csOutput.Format("FLAGS=%05x\n", this->m_dwFlags); break;
	case 7:
		csOutput.Format("EFFECT_ID=%05x\n", this->m_dwEffect); break;
	case 8:
		csOutput.Format("EFFECT_LO=%ld\n", this->m_iEffectLo); break;
	case 9:
		csOutput.Format("EFFECT_HI=%ld\n", this->m_iEffectHigh); break;
	default:
		break;
	}
	csfOutput.WriteString(csOutput);
	return;
}

DWORD CSpellDef::Flags(CString csFlagString)
{
	DWORD dwFlags = 0;
	CString csString = csFlagString;
	while ( csString != "" )
	{
		CString csFlag = csString.SpanExcluding("|");
		csFlag.TrimLeft(); csFlag.TrimRight();
		if ( csString.Find("|") != -1 )
		{
			CString csTemp;
			csTemp = csString.Mid( csString.Find("|") + 1);
			csString = csTemp;
		}
		else
			csString = "";
		for ( int i = 0; i < 16; i ++ )
		{
			if ( csFlag.CompareNoCase(sm_SpellFlagDefs[i]) == 0 )
			{
				dwFlags |= ( 1 << i );
				break;
			}
		}
	}
	return dwFlags;
}

CString CSpellDef::FlagString(DWORD dwFlags)
{
	CString csString;
	for (int i = 0; i < 16; i++)
	{
		if ( dwFlags & (1 << i ) )
		{
			CString csTemp, csFlag;
			csFlag = _T(sm_SpellFlagDefs[i]);
			if (csFlag == "" )
				continue;
			if ( csString == "" )
				csTemp = csFlag;
			else
				csTemp.Format("%s|%s", csString, csFlag);
			csString = csTemp;
		}
	}
	return csString;
}