/*
 $Id: SpellEditDlg.cpp,v 1.4 2002/05/01 03:33:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// SpellEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "SpellEditDlg.h"
#include "RemoteConnection.h"
#include "SpawnMessageDlg.h"
#include "SpellDef.h"
#include "RemoteConsole.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

	const char * knownReagents [] = // preserve the old defs.
	{
		"Bw",	// ITEMID_REAG_1,	//"Batwing"
		"Bl", 	//"Blackmoor"
		"Bp", 	//"Black Pearl"
		"Bm", 	//"Blood Moss"
		"Bs", 	//"Bloodspawn"
		"Vb", 	//"Vial of Blood"
		"Bo", 	//"Bone"
		"Br", 	//"Brimstone"
		"Da",	//"Daemon Bone"
		"Fd", 	//"Fertile Dirt"
		"Db", 	//"Dragon"s Blood"
		"Ec", 	//"Executioner"s Cap"
		"Ga", 	//"Garlic"
		"Gi", 	//"Ginseng"
		"Mr", 	//"Mandrake Root"
		"En", 	//"Eye of Newt"
		"Ns", 	//"Nightshade"
		"Ob", 	//"Obsidian"
		"Pi", 	//"Pig Iron"
		"Pu", 	//"Pumice"
		"Sa",	//"Sulfurous Ash"
		"Ss",	//"Spider"s Silk"
		"Se",	//"Serpent"s Scales"
		"Va",	//"Volcanic Ash"
		"Dw",	//"Dead Wood "
		"Wh",	//"Worm"s Heart"
	};

	const char * expandedReagents [] = 
	{
		"Batwing",
		"Blackmoor",
		"Black Pearl",
		"Blood Moss",
		"Bloodspawn",
		"Vial of Blood",
		"Bone",
		"Brimstone",
		"Daemon Bone",
		"Fertile Dirt",
		"Dragon's Blood",
		"Executioner's Cap",
		"Garlic",
		"Ginseng",
		"Mandrake Root",
		"Eye of Newt",
		"Nightshade",
		"Obsidian",
		"Pig Iron",
		"Pumice",
		"Sulfurous Ash",
		"Spider Silk",
		"Serpent's Scales",
		"Volcanic Ash",
		"Dead Wood",
		"Wyrm's Heart"
	};

#define KNOWN_REAGENTS 25

/////////////////////////////////////////////////////////////////////////////
// CSpellEditDlg dialog


CSpellEditDlg::CSpellEditDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSpellEditDlg::IDD, pParent)
{
	this->m_csWOP = "";
	this->m_csaWOP.SetSize(0);
	this->m_pSpell = NULL;
	//{{AFX_DATA_INIT(CSpellEditDlg)
	//}}AFX_DATA_INIT
}


void CSpellEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSpellEditDlg)
	DDX_Control(pDX, IDC_SEREAGENTS, m_clbReagents);
	DDX_Control(pDX, IDC_SESPELLEFFECTHIGH, m_ceEffectHi);
	DDX_Control(pDX, IDC_SESPELLEFFECTLO, m_ceEffectLo);
	DDX_Control(pDX, IDC_SESPELLFLAGS, m_clbFlags);
	DDX_Control(pDX, IDC_SESPELLEFFECT, m_ccbEffect);
	DDX_Control(pDX, IDC_SESPELLSCROLL, m_ccbScroll);
	DDX_Control(pDX, IDC_SESPELLRUNE, m_ccbRunes);
	DDX_Control(pDX, IDC_SESPELLWOP, m_ceWOP);
	DDX_Control(pDX, IDC_SESPELLSOUND, m_ccbSounds);
	DDX_Control(pDX, IDC_SESPELLNAME, m_ceName);
	DDX_Control(pDX, IDC_SESPELLLIST, m_ccbSpellList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSpellEditDlg, CDialog)
	//{{AFX_MSG_MAP(CSpellEditDlg)
	ON_CBN_SELCHANGE(IDC_SESPELLLIST, OnSelchangeSespelllist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSpellEditDlg message handlers

BOOL CSpellEditDlg::OnInitDialog() 
{
	CWaitCursor hourglass;
	CDialog::OnInitDialog();
	Main->m_pScripts->LoadDefs();
	Main->m_pScripts->LoadTables();

	FillData();
	FillReagents();
	FillWordsOfPower();
	FillSpells();
	
	this->m_ccbSpellList.SetCurSel(0);
	this->OnSelchangeSespelllist();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSpellEditDlg::FillData()
{
	for ( int i = 0; i < Main->m_pScripts->m_aDefs.GetSize(); i++ )
	{
		CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(i);
		if ( !pDef )
			continue;
		if ( pDef->m_csValue.Find("SPELLFLAG") != -1 )
		{
			int index = this->m_clbFlags.AddString(pDef->m_csValue);
			this->m_clbFlags.SetItemData(index, pDef->m_dwValueMin);
		}
		if ( pDef->m_csValue.Find("SPELL_") != -1 )
		{
			int index = this->m_ccbSounds.AddString(pDef->m_csValue);
			this->m_ccbSounds.SetItemData(index, pDef->m_dwValueMin);
		}
		if ( pDef->m_csValue.Find("FX_") != -1 )
		{
			int index = this->m_ccbEffect.AddString(pDef->m_csValue);
			this->m_ccbEffect.SetItemData(index, pDef->m_dwValueMin);
		}
		if ( pDef->m_csValue.Find("RUNE_") != -1 )
		{
			int index = this->m_ccbRunes.AddString(pDef->m_csValue);
			this->m_ccbRunes.SetItemData(index, pDef->m_dwValueMin);
		}
		if ( pDef->m_csValue.Find("SCROLL_") != -1 )
		{
			int index = this->m_ccbScroll.AddString(pDef->m_csValue);
			this->m_ccbScroll.SetItemData(index, pDef->m_dwValueMin);
		}
	}
}

void CSpellEditDlg::FillReagents()
{
	for (int i = 0; i < KNOWN_REAGENTS; i++)
	{
		int iIndex = this->m_clbReagents.AddString(expandedReagents[i]);
		this->m_clbReagents.SetItemData(iIndex, i);
	}
}

void CSpellEditDlg::FillWordsOfPower()
{
	for ( int i = 0; i < Main->m_pScripts->m_tables.m_runes.GetSize(); i++ )
	{
		CString csRune = Main->m_pScripts->m_tables.m_runes.GetAt(i);
		this->m_csaWOP.Add(csRune);
	}
}

void CSpellEditDlg::FillSpells()
{
	for ( int i = 0; i < Main->m_pScripts->m_tables.m_spells.GetSize(); i++)
	{
		CTObject * pBase = (CTObject *) Main->m_pScripts->m_tables.m_spells.GetAt(i);
		if ( !pBase )
			continue;
		CTSpellDef spell;
		spell.m_csValue = pBase->m_csValue;
		spell.m_csFilename = pBase->m_csFilename;
		spell.Load();
		int index = this->m_ccbSpellList.AddString(spell.m_csName);
		this->m_ccbSpellList.SetItemData(index, i);
		this->m_ccbSpellList.SetItemDataPtr(index, (void *) pBase);
	}
}

CString CSpellEditDlg::ExpandWOP()
{
	CString csOutput;
	for (int i = 0; i < this->m_csWOP.GetLength(); i++)
	{
		CString csTemp;
		char cThisOne = this->m_csWOP.GetAt(i);
		if (cThisOne != 0x00)
		{
			int wopindex = cThisOne - 'A';
			if (wopindex >= 0 && wopindex < 26)
			{
				if (csOutput != "")
					csTemp.Format("%s %s", csOutput, this->m_csaWOP[wopindex]);
				else
					csTemp.Format("%s", this->m_csaWOP[wopindex]);
			}
			csOutput = csTemp;
		}
	}
	return csOutput;
}

BOOL CSpellEditDlg::PreTranslateMessage(MSG* pMsg) 
{
	if (pMsg->message == WM_KEYDOWN)
	{
		if (this->GetFocus()->GetSafeHwnd() == this->m_ceWOP.GetSafeHwnd())
		{
			char nChar = (char) pMsg->wParam;
			if (nChar == VK_DELETE || nChar == VK_BACK)
			{
				if (this->m_csWOP.GetLength() != 0)
					this->m_csWOP = this->m_csWOP.Left(this->m_csWOP.GetLength() - 1);
			}
			else
			{
				if (nChar >= 'A' && nChar <= 'z')
				{
					CString csTemp;
					csTemp.Format("%s%c", this->m_csWOP, toupper(nChar));
					this->m_csWOP = csTemp;
				}
			}
			this->m_ceWOP.SetWindowText(ExpandWOP());
			return TRUE;
		}
	}
	return CDialog::PreTranslateMessage(pMsg);
}

CSpellEditDlg::~CSpellEditDlg()
{
		if ( m_pSpell )
			delete m_pSpell;
}

void CSpellEditDlg::OnSelchangeSespelllist() 
{
	if (m_pSpell != NULL)
	{
		// Save the current stuff to the previously selected spell.
		CString csTemp;
		this->m_ceEffectHi.GetWindowText(csTemp);
		this->m_pSpell->m_wEffectHi = (WORD) atoi(csTemp);
		this->m_ceEffectLo.GetWindowText(csTemp);
		this->m_pSpell->m_wEffectLo = (WORD) atoi(csTemp);
		this->m_ccbEffect.GetWindowText(csTemp);
		this->m_pSpell->m_csEffectID = csTemp;
		this->m_ccbSounds.GetWindowText(csTemp);
		this->m_pSpell->m_csSound = csTemp;
		this->m_ccbRunes.GetWindowText(csTemp);
		this->m_pSpell->m_csRuneID = csTemp;
		this->m_ccbScroll.GetWindowText(csTemp);
		this->m_pSpell->m_csScrollID = csTemp;
		this->m_pSpell->m_csRunes = this->m_csWOP;
		this->m_ceName.GetWindowText(this->m_pSpell->m_csName);
		csTemp = "";
		for (int i = 0; i < this->m_clbReagents.GetCount(); i++)
		{
			if (this->m_clbReagents.GetSel(i))
			{
				CString csTemp2 = csTemp;
				if (csTemp != "")
					csTemp2.Format("%s,%s", csTemp, knownReagents[this->m_clbReagents.GetItemData(i)]);
				else
					csTemp2.Format("%s", knownReagents[this->m_clbReagents.GetItemData(i)]);
				csTemp = csTemp2;
			}
		}
		this->m_pSpell->m_csReags = csTemp;
		m_pSpell->m_wFlags = 0;
		for (i = 0; i < this->m_clbFlags.GetCount(); i++)
		{
			if (this->m_clbFlags.GetSel(i))
				m_pSpell->m_wFlags |= this->m_clbFlags.GetItemData(i);
		}
	}
	// Fill in the data for the newly selected spell.
	int iIndex = this->m_ccbSpellList.GetCurSel();
	if (iIndex != -1)
	{
		if ( m_pSpell )
			delete m_pSpell;
		CTObject * pBase = (CTObject *) m_ccbSpellList.GetItemData(iIndex);
		this->m_pSpell = new CTSpellDef;
		m_pSpell->m_csFilename = pBase->m_csFilename;
		m_pSpell->m_csValue = pBase->m_csValue;
		m_pSpell->Load();
		// Fill this stuff in
		if (this->m_pSpell != NULL)
		{
			this->m_ceName.SetWindowText(this->m_pSpell->m_csName);
			this->m_csWOP = m_pSpell->m_csRunes;
			this->m_ceWOP.SetWindowText(ExpandWOP());
			CString csTemp;
			csTemp.Format("%ld", m_pSpell->m_wEffectHi);
			this->m_ceEffectHi.SetWindowText(csTemp);
			csTemp.Format("%ld", m_pSpell->m_wEffectLo);
			this->m_ceEffectLo.SetWindowText(csTemp);
			this->m_ccbEffect.SetWindowText(m_pSpell->m_csEffectID);
			this->m_ccbRunes.SetWindowText(m_pSpell->m_csRuneID);
			this->m_ccbScroll.SetWindowText(m_pSpell->m_csScrollID);
			this->m_ccbSounds.SetWindowText(m_pSpell->m_csSound);
			// Check the reagents
			for (int i = 0; i < this->m_clbReagents.GetCount(); i++)
			{
				int iIndex = this->m_clbReagents.GetItemData(i);
				CString csTemp = m_pSpell->m_csReags;
				csTemp.MakeUpper();
				if (csTemp.Find(knownReagents[iIndex]) != -1)
					this->m_clbReagents.SetSel(i, TRUE);
				else
					this->m_clbReagents.SetSel(i, FALSE);
			}
			// Check the flags
			for (i = 0; i < this->m_clbFlags.GetCount(); i++)
			{
				int iFlag = this->m_clbFlags.GetItemData(i);
				if (m_pSpell->m_wFlags & iFlag)
					this->m_clbFlags.SetSel(i, TRUE);
				else
					this->m_clbFlags.SetSel(i, FALSE);
			}
		}
	}
}

void CSpellEditDlg::OnOK() 
{
	CWaitCursor hourglass;
	// Save all of the skills
	// First save any changes to the current skill
	this->OnSelchangeSespelllist();
	if ( Main->m_pScripts->m_tables.Write() )
		CDialog::OnOK();
}

void CSpellEditDlg::OnCancel() 
{
	Main->m_pScripts->ReloadTables();
	CDialog::OnCancel();
}
