/*
 $Id: TABLES.CPP,v 1.12 2002/11/17 04:25:27 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// tables.cpp
//
// Contains the implementation of the table classes
//

#include "stdafx.h"
#ifdef _AXIS
#include "drewsky.h"
#else
#include "SphereScript.h"
#endif#include "FileReadProgress.h"
#include "RemoteConnection.h"
#include "tables.h"
#include <direct.h>

CTStringTable::CTStringTable()
{
}

CTStringTable::~CTStringTable()
{
	this->RemoveAll();
}

bool CTStringTable::Read(CStdioFile &csfInput)
{
	CString csLine;
	BOOL bStatus = TRUE;
	while (bStatus)
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( csLine.Find("//") != -1 )
			csLine = csLine.Left(csLine.Find("//"));
		csLine.Remove(0x0d);
		if ( csLine.Find("[") == -1 )
			this->Add(csLine);
		else
		{
			csfInput.Seek((LONG)(dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
	}
	return bStatus ? true : false;
}

bool CTStringTable::Write(CStdioFile &csfInput)
{
	CString csLine;
	if ( this->m_csTableName == "" )
		return false;
	csLine.Format("[%s]\n", this->m_csTableName);
	csfInput.WriteString(csLine);
	for ( int i = 0; i < this->GetSize(); i++ )
	{
		csLine.Format("%s\n", this->GetAt(i));
		csfInput.WriteString(csLine);
	}
	return true;
}

CTObArray::CTObArray()
{
	m_bHexIndex = false;
	m_dwIndexOffset = 0;
}

CTObArray::~CTObArray()
{
	this->RemoveAll();
}

bool CTObArray::Write(CStdioFile &csfOutput)
{
	CString csLine;
	for ( int i = 0; i < this->GetSize(); i++ )
	{
		CTObject * pObject = (CTObject *) this->GetAt(i);
		if ( pObject )
		{
			CString csLine;
			if ( m_bHexIndex )
				csLine.Format("[%s %4x]\n", m_csBaseName, i + m_dwIndexOffset);
			else
				csLine.Format("[%s %ld]\n", m_csBaseName, i + m_dwIndexOffset);
			csfOutput.WriteString(csLine);
			pObject->Write(csfOutput);
			csfOutput.WriteString("\n");
		}
	}
	return true;
}

CTObject::CTObject()
{
	m_bType = 0;
	m_dwFileOffset = 0;
}

CTObject::~CTObject()
{
}

void CTObject::CopyBase(CTObject * pObject)
{
	this->m_bType = pObject->m_bType;
	this->m_csFilename = pObject->m_csFilename;
	this->m_csValue = pObject->m_csValue;
	this->m_dwFileOffset = pObject->m_dwFileOffset;
}

bool CTObject::Write(CStdioFile &csfOutput)
{
	// Write the basic stuff
	CString csType;
	switch ( this->m_bType )
	{
	case TYPE_ITEM:
		csType = "ITEMDEF";
		break;
	case TYPE_CHAR:
		csType = "CHARDEF";
		break;
	case TYPE_SKILL:
		csType = "SKILL";
		break;
	case TYPE_SKILLCLASS:
		csType = "SKILLCLASS";
		break;
	case TYPE_SPELL:
		csType = "Spell";
		break;
	case TYPE_SPAWN:
		csType = "SPAWN";
		break;
	case TYPE_POTION:
		csType = "POTION";
		break;
	case TYPE_NAME:
		csType = "NAMES";
		break;
	case TYPE_TEMPLATE:
		csType = "TEMPLATE";
		break;
	case TYPE_NONE:
	default:
		return false;
	}
	CString csHeader;
	csHeader.Format("[%s %s]\n", csType, this->m_csValue);
	csfOutput.WriteString("\n");
	csfOutput.WriteString(csHeader);
	return true;
}

bool CTObject::Read(CStdioFile &csfInput)
{
	return true;
}

bool CTObject::Load()
{
	bool bFound = false;
	if ( this->m_csValue == "" )
		return false;
	if ( this->m_csFilename == "" )
		return false;
	CStdioFile csfInput;
	if ( !csfInput.Open(m_csFilename, CFile::modeRead | CFile::shareDenyNone ) )
		return false;
	csfInput.Seek(m_dwFileOffset, CFile::begin);
	BOOL bStatus = TRUE;
	CString csKey, csFormat;
	switch ( m_bType )
	{
	case TYPE_ITEM:
		if ( Main->m_pScripts->m_bNewFormat )
			csFormat = "[ITEMDEF %s]";
		else
			csFormat = "[%s]";
		break;
	case TYPE_CHAR:
		if ( Main->m_pScripts->m_bNewFormat )
			csFormat = "[CHARDEF %s]";
		else
			csFormat = "[%s]";
		break;
	case TYPE_SKILL:
		csFormat = "[SKILL %s]";
		break;
	case TYPE_SKILLCLASS:
		csFormat = "[SKILLCLASS %s]";
		break;
	case TYPE_SPELL:
		csFormat = "[Spell %s]";
		break;
	case TYPE_SPAWN:
		csFormat = "[SPAWN %s]";
		break;
	case TYPE_POTION:
		csFormat = "[POTION %s]";
		break;
	case TYPE_NAME:
		csFormat = "[NAMES %s]";
		break;
	case TYPE_TEMPLATE:
		csFormat = "[TEMPLATE %s]";
		break;
	case TYPE_NONE:
		return false;
	default:
		csFormat = "[%s]";
		break;
	}
	csKey.Format(csFormat, m_csValue);
	csKey.MakeUpper();
	while ( bStatus )
	{
		CString csLine;
		bStatus = csfInput.ReadString(csLine);
		csLine.MakeUpper();
		if ( !bStatus )
			break;
		if ( csLine.Find(csKey) != -1 )
		{
			this->Read(csfInput);
			bFound = true;
			break;
		}
	}
	csfInput.Close();
	return bFound;
}

CAdvanceDef::CAdvanceDef()
{
	this->m_Rate[0] = 0;
	this->m_Rate[1] = 0;
	this->m_Rate[2] = 0;
}

CAdvanceDef::~CAdvanceDef()
{
}

void CAdvanceDef::Load(CString csLine)
{
	// The values are either comma separated or space separated
	char cDel = ',';
	if ( csLine.Find(",") == -1 )
		cDel = ' ';
	CString csTemp;
	CString csOrig = csLine;
	for (int i = 0; i < 3; i++)
	{
		if ( csLine.Find(cDel) != -1 )
		{
			csTemp = csLine.Left(csLine.Find(cDel));
			this->m_Rate[i] = atoi(csTemp);
			csTemp = csLine.Mid(csLine.Find(cDel) + 1);
			csLine = csTemp;
		}
		else
		{
			this->m_Rate[i] = atoi(csLine);
			break;
		}
	}
}

void CAdvanceDef::Write(CStdioFile &csfOutput)
{
	CString csLine;
	csLine.Format("%ld,%ld,%ld\n", m_Rate[0], m_Rate[1], m_Rate[2]);
	csfOutput.WriteString(csLine);
}

void CAdvanceDef::Format(CString &csOutput)
{
	csOutput.Format("%ld,%ld,%ld\n", m_Rate[0], m_Rate[1], m_Rate[2]);
	return;
}

CTStartLocation::CTStartLocation()
{
}

CTStartLocation::~CTStartLocation()
{
}

bool CTStartLocation::Read(CStdioFile &csfInput)
{
	CString csLine;
	BOOL bStatus = TRUE;
	DWORD dwPosition = csfInput.GetPosition();
	bStatus = csfInput.ReadString(csLine);
	if ( !bStatus )
		return false;
	csLine.TrimLeft(); csLine.TrimRight();
	csLine.Remove(0x0d);
	if ( csLine.Find("[") != -1 || csLine == "")
	{
		csfInput.Seek((LONG) (dwPosition - csfInput.GetPosition()), CFile::current);
		return false;
	}
	csLine = csLine.SpanExcluding("//");
	this->m_csTown = csLine;
	bStatus = csfInput.ReadString(csLine);
	if ( !bStatus )
		return false;
	csLine = csLine.SpanExcluding("//");
	this->m_csLocation = csLine;
	bStatus = csfInput.ReadString(csLine);
	if ( !bStatus )
		return false;
	csLine = csLine.SpanExcluding("//");
	this->m_csCoords = csLine;
	return bStatus ? true : false;
}

bool CTStartLocation::Write(CStdioFile &csfOutput)
{
	CString csLine;
	csLine.Format("%s\n%s\n%s\n", this->m_csTown, this->m_csLocation, this->m_csCoords);
	csfOutput.WriteString(csLine);
	return false;
}

CTOreDef::CTOreDef()
{
	this->m_csName = "";
	this->m_iMaxAmount = 0;
	this->m_iMinAmount = 0;
	this->m_iMinSkill = 0;
	this->m_iVeinChance = 0;
	this->m_csColor = "";
	this->m_csIngot = "";
}

CTOreDef::~CTOreDef()
{
}

void CTOreDef::Load(CString csLine)
{
	// The format of the ore entry is
	// Iron,0,0.0,50.0,9-30,IRON_INGOT
	CString csTemp;
	int i = 0;
	while ( csLine.Find(",") != -1 )
	{
		csTemp = csLine.Left(csLine.Find(","));
		switch(i++)
		{
		case 0:
			this->m_csName = csTemp;
			break;
		case 1:
			this->m_csColor = csTemp;
			break;
		case 2:
			if ( csTemp.Find(".") != -1)
			{
				csTemp.Replace(".", "");
					this->m_iMinSkill = atoi(csTemp);
			}
			else
				this->m_iMinSkill = atoi(csTemp) * 10;
			break;
		case 3:
			if ( csTemp.Find(".") != -1)
			{
				csTemp.Replace(".", "");
					this->m_iVeinChance = atoi(csTemp);
			}
			else
				this->m_iVeinChance = atoi(csTemp) * 10;
			break;
		case 4:
			{
				CString csTemp2;
				if (csTemp.Find("-") != -1)
				{
					csTemp2 = csTemp.Left(csTemp.Find("-"));
					this->m_iMinAmount = atoi(csTemp2);
					csTemp2 = csTemp.Mid(csTemp.Find("-") + 1);
					this->m_iMaxAmount = atoi(csTemp2);
				}
				else
				{
					this->m_iMaxAmount = atoi(csTemp);
					this->m_iMinAmount = atoi(csTemp);
				}
			}
			break;
		}
		csTemp = csLine.Mid(csLine.Find(",") + 1);
		csLine = csTemp;
	}
	if ( i == 5 )
		this->m_csIngot = csLine;
	return;
}

bool CTOreDef::Write(CStdioFile &csfOutput)
{
	// The format of the ore entry is
	// Iron,0,0.0,50.0,9-30,IRON_INGOT
	CString csLine;
	csLine.Format("%s,%s,%.1f,%.1f,%ld-%ld,%s\n", this->m_csName, this->m_csColor, (float) (this->m_iMinSkill / 10.0), (float) (this->m_iVeinChance / 10.0),
		this->m_iMinAmount, this->m_iMaxAmount, this->m_csIngot);
	csfOutput.WriteString(csLine);
	return true;
}

const char * pSkillClassKeys [] = 
{
	"NAME",
	"STR",
	"INT",
	"DEX",
	"PLOTFLAGS",
};

const char * pSkillNames [] =
{
	"Anatomy",
	"ItemID",
	"ArmsLore",
	"Parrying",
	"Begging",
	"Blacksmithing",
	"Bowcraft",
	"Peacemaking",
	"Camping",
	"Carpentry",
	"Cartography",
	"Cooking",
	"DetectingHidden",
	"Enticement",
	"EvaluatingIntel",
	"Healing",
	"Fishing",
	"Forensics",
	"Herding",
	"Hiding",
	"Provocation",
	"Inscription",
	"LockPicking",
	"Magery",
	"MagicResistance",
	"Tactics",
	"Snooping",
	"Musicianship",
	"Poisoning",
	"Archery",
	"SpiritSpeak",
	"Stealing",
	"Tailoring",
	"Taming",
	"TasteID",
	"Tinkering",
	"Tracking",
	"Veterinary",
	"Swordsmanship",
	"Macefighting",
	"Fencing",
	"Wrestling",
	"Lumberjacking",
	"Mining",
	"Meditation",
	"Stealth",
	"RemoveTrap",
	"Necromancy",
};

CTSkillClass::CTSkillClass()
{
	m_bType = TYPE_SKILLCLASS;
	this->m_csName = "";
	for ( int i = 0; i < 3; i++ )
		this->m_StatMax[i] = -1;
	for ( i = 0; i < 50; i++ )
		this->m_SkillLevelMax[i] = -1;
	this->m_dwPlotFlags = 0;
}

CTSkillClass::~CTSkillClass()
{
	this->m_Events.RemoveAll();
}

bool CTSkillClass::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	CString csLine;
	while ( bStatus )
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		csLine = csLine.SpanExcluding("//");
		if ( csLine == "" )
			continue;
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		if ( csLine.Find("=") == -1 )	// If there's no key, what is this value for?
			continue;
		CString csKey = csLine.Left(csLine.Find("="));
		CString csValue = csLine.Mid(csLine.Find("=") + 1);
		csValue.Remove(0x0d);
		switch (FindTable(csKey, pSkillClassKeys, 5))
		{
		case 0:
			this->m_csName = csValue; break;
		case 1:
			this->m_StatMax[0] = atoi(csValue); break;
		case 2:
			this->m_StatMax[1] = atoi(csValue); break;
		case 3:
			this->m_StatMax[2] = atoi(csValue); break;
		case 4:
			if (csValue.GetAt(0) == '0')
				this->m_dwPlotFlags = ahextoi(csValue);
			else
				this->m_dwPlotFlags = atoi(csValue);
			break;
		default:
			{
				// It must be a skill
				int index = FindTable(csKey, pSkillNames, 50);
				if ( index != -1 )
				{
					if ( csValue.Find(".") != -1 )
					{
						csValue.Replace(".", "");
						m_SkillLevelMax[index] = atoi(csValue);
					}
					else
						m_SkillLevelMax[index] = atoi(csValue) * 10;
				}
			}
			break;
		}
	}
	return bStatus ? true : false;
}

bool CTSkillClass::Write(CStdioFile &csfOutput)
{
	CString csLine;
	csLine.Format("NAME=%s\n", this->m_csName);
	csfOutput.WriteString(csLine);
	if ( this->m_StatMax[0] != (WORD) -1 )
	{
		csLine.Format("STR=%ld\n", this->m_StatMax[0]);
		csfOutput.WriteString(csLine);
	}
	if ( this->m_StatMax[1] != (WORD) -1 )
	{
		csLine.Format("INT=%ld\n", this->m_StatMax[1]);
		csfOutput.WriteString(csLine);
	}
	if ( this->m_StatMax[2] != (WORD) -1 )
	{
		csLine.Format("DEX=%ld\n", this->m_StatMax[2]);
		csfOutput.WriteString(csLine);
	}
	if ( this->m_dwPlotFlags != 0 )
	{
		csLine.Format("PLOTFLAGS=0%8x\n", this->m_dwPlotFlags);
		csfOutput.WriteString(csLine);
	}
	for (int i = 0; i < 50; i++)
	{
		if ( this->m_SkillLevelMax[i] != (WORD) -1)
		{
			csLine.Format("%s=%5.1f\n", pSkillNames[i], (float) (this->m_SkillLevelMax[i] / 10.0));
			csfOutput.WriteString(csLine);
		}
	}
	return true;
}

const char * pSkillKeys [] =
{
	"KEY",
	"TITLE",
	"PROMPT_MSG",
	"FAIL_MSG",
	"DELAY",
	"STAT_STR",
	"STAT_INT",
	"STAT_DEX",
	"BONUS_STR",
	"BONUS_INT",
	"BONUS_DEX",
	"SKILL_STAT",
	"ADV_RATE",
};

CTSkillDef::CTSkillDef()
{
	m_bType = TYPE_SKILL;
	m_csKey = "";
	m_csTitle = "";
	m_csTargetPrompt = "";
	m_csFailMsg = "";
	m_wDelay = 0;
	for ( int i = 0 ; i < 3; i++ )
	{
		m_bStat[i] = 0;
		m_bStatBonus[i] = 0;
	}
	m_bSkillStat = 0;
}

CTSkillDef::~CTSkillDef()
{
}

bool CTSkillDef::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	CString csLine;
	while (bStatus)
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( ! bStatus )
			break;
		CString csKey, csValue;
		csLine = csLine.SpanExcluding("//");
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		if ( csLine.Find("=") == -1 )
			continue;
		csKey = csLine.Left(csLine.Find("="));
		if ( csLine.Find("=") != csLine.GetLength() )
			csValue = csLine.Mid(csLine.Find("=") + 1);
		csValue.TrimLeft(); csValue.TrimRight();
		csValue.Remove(0x0d);
		switch( FindTable(csKey, pSkillKeys, 13) )
		{
		case 0:
			m_csKey = csValue; break;
		case 1:
			m_csTitle = csValue; break;
		case 2:
			m_csTargetPrompt = csValue; break;
		case 3:
			m_csFailMsg = csValue; break;
		case 4:
			m_wDelay = (WORD) atoi(csValue); break;
		case 5:
			m_bStat[STAT_STR] = (BYTE) atoi(csValue); break;
		case 6:
			m_bStat[STAT_INT] = (BYTE) atoi(csValue); break;
		case 7:
			m_bStat[STAT_DEX] = (BYTE) atoi(csValue); break;
		case 8:
			m_bStatBonus[0] = (BYTE) atoi(csValue); break;
		case 9:
			m_bStatBonus[1] = (BYTE) atoi(csValue); break;
		case 10:
			m_bStatBonus[2] = (BYTE) atoi(csValue); break;
		case 11:
			m_bSkillStat = (BYTE) atoi(csValue); break;
		case 12:
			m_Adv.Load(csValue); break;
		default:
			// This is something I don't know about...what to do?
			break;
		}
	}
	return bStatus ? true : false;
}

bool CTSkillDef::WriteBasic(CStdioFile &csfOutput)
{
	if ( ! CTObject::Write(csfOutput) )
		return false;
	CString csLine;
	csLine.Format("KEY=%s\n", this->m_csKey);
	csfOutput.WriteString(csLine);
	return true;
}

bool CTSkillDef::Write(CStdioFile &csfOutput)
{
	CString csLine;
	for ( int i = 0; i < 13; i++ )
	{
		CString csValue;
		switch (i)
		{
		case 0:
			csValue = m_csKey; break;
		case 1:
			csValue = m_csTitle; break;
		case 2:
			csValue = m_csTargetPrompt; break;
		case 3:
			csValue = m_csFailMsg; break;
		case 4:
			if ( m_wDelay )
				csValue.Format("%ld", m_wDelay);
			break;
		case 5:
			csValue.Format("%ld", m_bStat[0]); break;
		case 6:
			csValue.Format("%ld", m_bStat[1]); break;
		case 7:
			csValue.Format("%ld", m_bStat[2]); break;
		case 8:
			csValue.Format("%ld", m_bStatBonus[0]); break;
		case 9:
			csValue.Format("%ld", m_bStatBonus[1]); break;
		case 10:
			csValue.Format("%ld", m_bStatBonus[2]); break;
		case 11:
			csValue.Format("%ld", m_bSkillStat); break;
		case 12:
			m_Adv.Format(csValue); break;
		}
		csLine.Format("%s=%s\n", pSkillKeys[i], csValue);
		csfOutput.WriteString(csLine);
	}
	return true;
}

const char * pPotionKeys [] = 
{
	"NAME",
	"ITEM",
	"REAGENT",
	"REAGENT_AMT",
	"SKILL_MIN",
	"COLOR"
};

CTPotionDef::CTPotionDef()
{
	m_bType = TYPE_POTION;
	m_csName = "";
	m_csPotionID = "";
	m_csRegID = "";
	m_iRegReq = 0;
	m_iSkillReq = 0;
	m_csColor = "";
}

CTPotionDef::~CTPotionDef()
{
}

bool CTPotionDef::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	CString csLine;
	while ( bStatus )
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		csLine = csLine.SpanExcluding("//");
		if ( csLine == "" )
			continue;
		if ( csLine.Find("=") == -1 )
			continue;
		CString csKey, csValue;
		csKey = csLine.Left(csLine.Find("="));
		if ( csLine.Find("=") != csLine.GetLength() )
			csValue = csLine.Mid(csLine.Find("=") + 1);
		csValue.Remove(0x0d);
		switch (FindTable(csKey, pPotionKeys, 6))
		{
		case 0:
			m_csName = csValue; break;
		case 1:
			m_csPotionID = csValue; break;
		case 2:
			m_csRegID = csValue; break;
		case 3:
			m_iRegReq = atoi(csValue); break;
		case 4:
			if ( csValue.Find(".") != -1 )
			{
				csValue.Replace(".", "");
				m_iSkillReq = atoi(csValue);
			}
			else
				m_iSkillReq = atoi(csValue) * 10;
			break;
		case 5:
			m_csColor = csValue; break;
		default:
			break;
		}
	}
	return bStatus ? true : false;
}

bool CTPotionDef::Write(CStdioFile &csfOutput)
{
	CString csLine;
	for ( int i = 0; i < 6; i++ )
	{
		CString csValue;
		switch (i)
		{
		case 0:
			csValue = m_csName; break;
		case 1:
			csValue.Format("%s", m_csPotionID); break;
		case 2:
			csValue.Format("%s", m_csRegID); break;
		case 3:
			csValue.Format("%ld", m_iRegReq); break;
		case 4:
			csValue.Format("%.1f", (float)(m_iSkillReq / 10.0 )); break;
		case 5:
			csValue.Format("%s", m_csColor); break;
		default:
			break;
		}
		csLine.Format("%s=%s\n", pPotionKeys[i], csValue);
		csfOutput.WriteString(csLine);
	}
	return true;
}

const char * pSpellKeys [] =
{
	"CASTTIME",
	"DURATION_HI",
	"DURATION_LO",
	"EFFECT_HI",
	"EFFECT_ID",
	"EFFECT_LO",
	"FLAGS",
	"NAME",
	"REAGENTS",
	"RUNE_ITEM",
	"RUNES",
	"SCROLL_ITEM",
	"SOUND",
};

CTSpellDef::CTSpellDef()
{
	m_bType = TYPE_SPELL;
	this->m_csName = "";
	this->m_csReags = "";
	this->m_csRunes = "";
	this->m_wCastTime = 0;
	this->m_wDurationTimeHi = 0;
	this->m_wDurationTimeLo = 0;
	this->m_wEffectHi = 0;
	this->m_csEffectID = "";
	this->m_wEffectLo = 0;
	this->m_csScrollID = "";
	this->m_csSound = "";
	this->m_csRuneID = "";
	this->m_wFlags = 0;
}

CTSpellDef::~CTSpellDef()
{
}

bool CTSpellDef::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	CString csLine;
	while ( bStatus )
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		csLine = csLine.SpanExcluding("//");
		if ( csLine == "" )
			continue;
		if ( csLine.Find("=") == -1 )
			continue;
		CString csKey, csValue;
		csKey = csLine.Left(csLine.Find("="));
		if ( csLine.Find("=") != csLine.GetLength() )
			csValue = csLine.Mid(csLine.Find("=") + 1);
		csValue.Remove(0x0d);
		switch (FindTable(csKey, pSpellKeys, 13 ) )
		{
		case 0:
			this->m_wCastTime = (WORD) atoi(csValue); break;
		case 1:
			this->m_wDurationTimeHi = (WORD) atoi(csValue); break;
		case 2:
			this->m_wDurationTimeLo = (WORD) atoi(csValue); break;
		case 3:
			this->m_wEffectHi = (WORD) DefLookup(csValue); break;
		case 4:
			this->m_csEffectID = csValue; break;
		case 5:
			this->m_wEffectLo = (WORD) DefLookup(csValue); break;
		case 6:
			this->ReadFlags(csValue); break;
		case 7:
			csValue.Remove('"');
			this->m_csName = csValue; break;
		case 8:
			this->m_csReags = csValue; break;
		case 9:
			this->m_csRuneID = csValue; break;
		case 10:
			this->m_csRunes = csValue; break;
		case 11:
			this->m_csScrollID = csValue; break;
		case 12:
			this->m_csSound = csValue; break;
		default:
			break;
		}
	}
	return bStatus ? true : false;
}

bool CTSpellDef::WriteBasic(CStdioFile &csfOutput)
{
	if ( ! CTObject::Write(csfOutput) )
		return false;
	CString csLine;
	csLine.Format("NAME=%s\n", this->m_csName);
	csfOutput.WriteString(csLine);
	return true;
}

bool CTSpellDef::Write(CStdioFile &csfOutput)
{
	CString csLine;
	csLine.Format("NAME=%s\n", this->m_csName);
	csfOutput.WriteString(csLine);
	csLine.Format("SOUND=%s\n", this->m_csSound);
	csfOutput.WriteString(csLine);
	csLine.Format("RUNES=%s\n", this->m_csRunes);
	csfOutput.WriteString(csLine);
	csLine.Format("CASTTIME=%ld\n", this->m_wCastTime);
	csfOutput.WriteString(csLine);
	csLine.Format("REAGENTS=%s\n", this->m_csReags);
	csfOutput.WriteString(csLine);
	csLine.Format("RUNE_ITEM=%s\n", this->m_csRuneID);
	csfOutput.WriteString(csLine);
	csLine.Format("SCROLL_ITEM=%s\n", this->m_csScrollID);
	csfOutput.WriteString(csLine);
	if ( this->m_wFlags )
	{
		csfOutput.WriteString("FLAGS=");
		csLine = "";
		FormatFlags(csLine);
		csfOutput.WriteString(csLine);
		csfOutput.WriteString("\n");
	}
	else
		csfOutput.WriteString("FLAGS=0\n");
	csLine.Format("EFFECT_ID=%s\n", this->m_csEffectID);
	csfOutput.WriteString(csLine);
	csLine.Format("EFFECT_LO=%ld\n", this->m_wEffectLo);
	csfOutput.WriteString(csLine);
	csLine.Format("EFFECT_HI=%ld\n", this->m_wEffectHi);
	csfOutput.WriteString(csLine);
	return true;
}

const char * pSpellFlags [] =
{
	"SPELLFLAG_DIR_ANIM", //  0x0001	// Evoke type cast or directed. (animation)
	"SPELLFLAG_TARG_OBJ", //  0x0002	// Need to target an object or char ?
	"SPELLFLAG_TARG_CHAR", // 0x0004	// Needs to target a living thing
	"SPELLFLAG_TARG_XYZ", //  0x0008	// Can just target a location.
	"SPELLFLAG_HARM", //		0x0010	// The spell is in some way harmfull.
	"SPELLFLAG_FX_BOLT", //	0x0020	// Effect is a bolt to the target.
	"SPELLFLAG_FX_TARG", //	0x0040	// Effect is at the target.
	"SPELLFLAG_FIELD", //		0x0080	// create a field of stuff. (fire,poison,wall)
	"SPELLFLAG_SUMMON", //	0x0100	// summon a creature.
	"SPELLFLAG_GOOD", //		0x0200	// The spell is a good spell. u intend to help to receiver.
	"", // 0x0400
	"", // 0x0800
	"", // 0x1000
	"", // 0x2000
	"", // 0x4000
	"SPELLFLAG_DISABLED", //	0x8000
};

void CTSpellDef::ReadFlags(CString csLine)
{
	int index;
	while ( csLine.Find("|") != -1 )
	{
		CString csTemp = csLine.Left(csLine.Find("|"));
		csTemp.TrimLeft(); csTemp.TrimRight();
		index = FindTable(csTemp, pSpellFlags, 16);
		if ( index != -1 )
			this->m_wFlags |= 1 << index;
		csTemp = csLine.Mid(csLine.Find("|") + 1);
		csLine = csTemp;
	}
	csLine.TrimLeft(); csLine.TrimRight();
	index = FindTable(csLine, pSpellFlags, 16);
	if ( index != -1 )
		this->m_wFlags |= 1 << index;
}

void CTSpellDef::FormatFlags(CString &csLine)
{
	for (int i = 0; i < 16; i++)
	{
		WORD wTest = 1 << i;
		if ( m_wFlags & wTest )
		{
			CString csTemp = csLine;
			if ( csLine != "" )
				csLine.Format("%s | %s", csTemp, pSpellFlags[i]);
			else
				csLine.Format("%s", pSpellFlags[i]);
		}
	}
}

CSpawnRec::CSpawnRec()
{
	this->m_csID = "";
	this->m_iWeight = 1;
}

CSpawnRec::~CSpawnRec()
{
}

const char * pSpawnKeys [] =
{
	"CATEGORY",
	"SUBSECTION",
	"DESCRIPTION",
	"NAME",
	"ID",
	"WEIGHT",
};

CTSpawnDef::CTSpawnDef()
{
	m_bType = TYPE_SPAWN;
	this->m_csName = "";
	this->m_iTotalWeight = 0;
	this->m_csCategory = "Spawn Groups";
	this->m_csSubsection = "<none>";
	this->m_csDescription = "<unnamed>";
	this->m_iIndex = 0;
}

CTSpawnDef::~CTSpawnDef()
{
	for ( int i = 0; i < this->m_Members.GetSize(); i++ )
	{
		CSpawnRec * pRec = (CSpawnRec *) this->m_Members.GetAt(i);
		if ( pRec )
			delete pRec;
	}
}

bool CTSpawnDef::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	CString csLine;
	CSpawnRec * pSpawn = NULL;
	while ( bStatus )
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		csLine = csLine.SpanExcluding("//");
		if ( csLine == "" )
			continue;
		if ( csLine.Find("=") == -1 )
			continue;
		CString csKey, csValue;
		csKey = csLine.Left(csLine.Find("="));
		if ( csLine.Find("=") != csLine.GetLength() )
			csValue = csLine.Mid(csLine.Find("=") + 1);
		csValue.Remove(0x0d);
		switch (FindTable(csKey, pSpawnKeys, 6))
		{
		case 0:
			m_csCategory = csValue; break;
		case 1:
			m_csSubsection = csValue; break;
		case 2:
			m_csDescription = csValue; break;
		case 3:
			m_csName = csValue; break;
		case 4:
			pSpawn = new (CSpawnRec);
			if ( pSpawn )
			{
				pSpawn->m_csID = csValue;
				this->m_Members.Add(pSpawn);
			}
			break;
		case 5:
			if ( pSpawn )
				pSpawn->m_iWeight = atoi(csValue);
			break;
		}
	}
	for (int i = 0; i < this->m_Members.GetSize(); i++)
	{
		CSpawnRec * pSpawn = (CSpawnRec *) this->m_Members.GetAt(i);
		if ( pSpawn )
			this->m_iTotalWeight += pSpawn->m_iWeight;
	}
	return bStatus ? true : false;
}

bool CTSpawnDef::WriteBasic(CStdioFile &csfOutput)
{
	if ( !CTObject::Write(csfOutput) )
		return false;
	CString csLine;
	csLine.Format("CATEGORY=%s\n", m_csCategory);
	csfOutput.WriteString(csLine);
	csLine.Format("SUBSECTION=%s\n", m_csSubsection);
	csfOutput.WriteString(csLine);
	csLine.Format("DESCRIPTION=%s\n", m_csDescription);
	csfOutput.WriteString(csLine);
	csLine.Format("NAME=%s\n", m_csName);
	csfOutput.WriteString(csLine);
	return true;
}

bool CTSpawnDef::Write(CStdioFile &csfOutput)
{
	CString csLine;
	if ( ! WriteBasic(csfOutput) )
		return false;
	for ( int i = 0; i < this->m_Members.GetSize(); i++ )
	{
		CSpawnRec * pSpawn = (CSpawnRec *) this->m_Members.GetAt(i);
		if ( pSpawn )
		{
			csLine.Format("ID=%s\n", pSpawn->m_csID);
			csfOutput.WriteString(csLine);
			if ( pSpawn->m_iWeight > 1 )
			{
				csLine.Format("WEIGHT=%ld\n", pSpawn->m_iWeight);
				csfOutput.WriteString(csLine);
			}
		}
	}
	return true;
}

const char * pTables [] =
{
	"OBSCENE",
	"BLOCKEMAIL",
	"PLEVEL",
	"EVENTS",
	"SPEECH",
	"ADVANCE",
	"STARTS",
	"RUNES",
	"ORE",
	"SKILLCLASS",
	"SKILL",
	"POTION",
	"SPELL",
	"SPAWN",
	"NOTOTITLES",
	"RESOURCES",
};

CTables::CTables()
{
	this->m_resources.m_csTableName = "RESOURCES";
	this->m_obscene.m_csTableName = "OBSCENE";
	this->m_blockEmail.m_csTableName = "BLOCKEMAIL";
	this->m_events.m_csTableName = "EVENTS";
	this->m_Speech.m_csTableName = "SPEECH";
	this->m_runes.m_csTableName = "RUNES";
	this->m_skillClasses.m_csBaseName = "SKILLCLASS";
	this->m_skills.m_csBaseName = "SKILL";
	this->m_potions.m_csBaseName = "POTION";
	this->m_spells.m_csBaseName = "SPELL";
	this->m_spawns.m_csBaseName = "SPAWN";
	this->m_notoTitles.m_csTableName = "NOTOTITLES";
}

CTables::~CTables()
{
	this->Unload();
}

bool CTables::Read()
{
	Main->m_log.Add("Loading tables");
	CWaitCursor hourglass;
	CString csScpPath = GetScriptPath();
	CFileReadProgress dlgProgress;
	dlgProgress.Create(IDD_FILEREAD_PROGRESS);

	// Grab the custom file from the remote server if necessary.
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
#ifdef _AXIS
		CFileStatus st;
		if ( !CFile::GetStatus(RMT_TABLE_FILE, st) )
		{
			CRemoteConnection rcRemote;
			if (rcRemote.IsValid())
				rcRemote.GetCustomFile("graytables");
		}
#endif
	}
	
	this->m_spawns.m_bHexIndex = true;
	this->m_spawns.m_dwIndexOffset = 0x8000;

	CString csMessage;

	BOOL bStatus = TRUE;
	CStdioFile csfTables;
	CString csFilename;
	if ( Main->GetProfileType() == PROFILE_REMOTE )
		csFilename.Format("%s\\%s", Main->GetProfileString(), RMT_TABLE_FILE);
	else if ( csScpPath == "" )
		csFilename = Main->GetProfileType() == PROFILE_REMOTE ? RMT_TABLE_FILE : TABLE_FILE;
	else
		csFilename.Format("%s%s", csScpPath, TABLE_FILE);
	if ( !csfTables.Open(csFilename, CFile::modeRead | CFile::shareDenyNone) )
	{
		if ( csFilename.Find("\\") == -1 )
		{
			CString csTemp = csFilename;
			char szPath[MAX_PATH];
			memset(&szPath[0], 0x00, sizeof(szPath) );
			getcwd(&szPath[0], sizeof(szPath));
			csFilename.Format("%s\\%s", szPath, csFilename);
		}
		Main->m_log.Add("Unable to open file %s", csFilename);
		csMessage.Format("Unable to open file %s for input.", csFilename);
		AfxMessageBox(csMessage, MB_OK | MB_ICONEXCLAMATION);
		return false;
	}

	csMessage.Format("Reading %s", csFilename);
	dlgProgress.m_csMessage.SetWindowText(csMessage);
	dlgProgress.SetPos(0);

	DWORD dwFileLength = csfTables.GetLength();
	int scalefactor = dwFileLength / 0xefff;
	scalefactor += 1;
	dlgProgress.SetRange(0, (short)(dwFileLength / scalefactor));

	while (bStatus)
	{
		CString csLine;
		bStatus = csfTables.ReadString(csLine);
		if ( !bStatus )
			break;
		dlgProgress.SetPos(csfTables.Seek(0, CFile::current) / scalefactor);
		csLine.TrimLeft();
		csLine.TrimRight();
		csLine = csLine.SpanExcluding("//");
		if (csLine == "")
			continue;
		if (csLine.Find("[") != -1)
		{
			// Which one is this?
			CString csKey, csIndex, csTemp;
			csTemp = csLine.Mid(csLine.Find("[") + 1);
			csTemp = csTemp.SpanExcluding("]");
			if ( csTemp.Find(" ") != -1 && csTemp.Find(" ") != csTemp.GetLength())
			{
				csKey = csTemp.SpanExcluding(" ");
				csIndex = csTemp.Mid(csTemp.Find(" ") + 1);
			}
			else
			{
				csKey = csTemp;
				csIndex = "";
			}
			int iIndex = atoi(csIndex);
			switch ( FindTable(csKey, pTables, 16) )
			{
			case 0:	// Obscene
				this->m_obscene.Read(csfTables);
				break;
			case 1: // Blockemail
				this->m_blockEmail.Read(csfTables);
				break;
			case 2: // Plevel N
				this->m_pLevel[iIndex - 1].m_csTableName.Format("%s %s", csKey, csIndex);
				this->m_pLevel[iIndex - 1].Read(csfTables);
				break;
			case 3: // Events
				this->m_events.Read(csfTables);
				break;
			case 4: // Speech
				this->m_Speech.Read(csfTables);
				break;
			case 5: // Advance
				{
					for ( int i = 0; i < 3; i++ )
					{
						bStatus = csfTables.ReadString(csLine);
						CString csKey = csLine.SpanExcluding("=");
						CString csValue = csLine.Mid(csLine.Find("=") + 1);
						csKey.MakeUpper();
						int index = -1;
						if ( csKey == "STR" )
							index = 0;
						if ( csKey == "INT" )
							index = 1;
						if ( csKey == "DEX" )
							index = 2;
						if ( index == -1 )
							continue;
						this->m_statAdvance[index].Load(csValue);
					}
				}
				break;
			case 6: // Starts
				{
					bool bContinue = true;
					while ( bContinue )
					{
						CTStartLocation * pStart = new (CTStartLocation);
						bContinue = pStart->Read(csfTables);
						if ( bContinue )
							this->m_starts.Add(pStart);
						else
							delete pStart;
					}
				}
				break;
			case 7: // Runes
				this->m_runes.Read(csfTables);
				break;
			case 8: // Ore
				{
					while ( bStatus )
					{
						DWORD dwPosition = csfTables.GetPosition();
						bStatus = csfTables.ReadString(csLine);
						if ( csLine.Find("[") != -1 )
						{
							csfTables.Seek( (LONG) (dwPosition - csfTables.GetPosition()), CFile::current );
							break;
						}
						csLine = csLine.SpanExcluding("//");
						csLine.TrimLeft(); csLine.TrimRight();
						if ( csLine == "" )
							continue;
						CTOreDef * pOre = new (CTOreDef);
						pOre->Load(csLine);
						this->m_ores.Add(pOre);
					}
				}
				break;
			case 9: // Skillclass N
				{
					CTSkillClass * pSkillClass = new (CTSkillClass);
					pSkillClass->Read(csfTables);
					this->m_skillClasses.SetAtGrow(iIndex, pSkillClass);
				}
				break;
			case 10: // Skill
				{
					CTSkillDef * pSkill = new (CTSkillDef);
					pSkill->Read(csfTables);
					this->m_skills.SetAtGrow(iIndex, pSkill);
				}
				break;
			case 11: // Potion
				{
					CTPotionDef * pPotion = new (CTPotionDef);
					pPotion->Read(csfTables);
					this->m_potions.SetAtGrow(iIndex, pPotion);
				}
				break;
			case 12: // Spell
				{
					CTSpellDef * pSpell = new (CTSpellDef);
					pSpell->Read(csfTables);
					this->m_spells.SetAtGrow(iIndex, pSpell);
				}
				break;
			case 13: // Spawn
				{
					CTSpawnDef * pSpawn = new (CTSpawnDef);
					pSpawn->m_iIndex = ahextoi(csIndex);
					pSpawn->Read(csfTables);
					this->m_spawns.SetAtGrow(ahextoi(csIndex) - 0x8000, (CObject *) pSpawn);
				}
				break;
			case 14: // Nototitles
				this->m_notoTitles.Read(csfTables);
				break;
			case 15: // Resources
				this->m_resources.Read(csfTables);
				break;
			}
		}
	}
	csfTables.Close();
	dlgProgress.EndDialog(IDOK);
	return true;
}

bool CTables::Write()
{
	Main->m_log.Add("Writing tables");
	CWaitCursor hourglass;
	CStdioFile csfOutput;
	if ( ! csfOutput.Open(TMP_TABLE_FILE, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText) )
	{
		Main->m_log.Add("Unable to open temporary file while saving tables");
		AfxMessageBox("Unable to open temporary file while saving tables.  Save Failed.", MB_OK | MB_ICONSTOP);
		return false;
	}
	WriteFileHeader(&csfOutput, TABLE_FILE);

	this->m_resources.Write(csfOutput);
	this->m_obscene.Write(csfOutput);
	this->m_blockEmail.Write(csfOutput);
	for ( int i = 0; i < 6; i++ )
		m_pLevel[i].Write(csfOutput);
	this->m_events.Write(csfOutput);
	this->m_Speech.Write(csfOutput);
	csfOutput.WriteString("[ADVANCE]\n");
	for ( i = 0; i < 3; i++ )
	{
		switch (i)
		{
		case STAT_STR:
			csfOutput.WriteString("STR="); break;
		case STAT_INT:
			csfOutput.WriteString("INT="); break;
		case STAT_DEX:
			csfOutput.WriteString("DEX="); break;
		}
		this->m_statAdvance[i].Write(csfOutput);
	}
	csfOutput.WriteString("\n");
	csfOutput.WriteString("[STARTS]\n");
	for ( i = 0; i < m_starts.GetSize(); i++ )
	{
		CTObject * pObject = (CTObject *) m_starts.GetAt(i);
		if ( pObject )
			pObject->Write(csfOutput);
	}
	csfOutput.WriteString("\n");
	this->m_runes.Write(csfOutput);
	csfOutput.WriteString("[ORE]\n");
	for ( i = 0; i < m_ores.GetSize(); i++ )
	{
		CTObject * pObject = (CTObject *) m_ores.GetAt(i);
		if ( pObject )
			pObject->Write(csfOutput);
	}
	csfOutput.WriteString("\n");
	this->m_skillClasses.Write(csfOutput);
	this->m_skills.Write(csfOutput);
	this->m_potions.Write(csfOutput);
	this->m_spells.Write(csfOutput);
	this->m_spawns.Write(csfOutput);
	this->m_notoTitles.Write(csfOutput);

	csfOutput.Close();
	return UpdateFiles( TMP_TABLE_FILE, TABLE_FILE, RMT_TABLE_FILE, "graytables" );
}

void CTables::Unload()
{
	Main->m_log.Add("Unloading tables");
	for ( int i = 0; i < this->m_ores.GetSize(); i++ )
	{
		CTOreDef * pOre = (CTOreDef *) this->m_ores.GetAt(i);
		if ( pOre )
			delete pOre;
	}
	this->m_ores.RemoveAll();
	for ( i = 0; i < this->m_skillClasses.GetSize(); i++ )
	{
		CTSkillClass * pSkillclass = (CTSkillClass *) this->m_skillClasses.GetAt(i);
		if ( pSkillclass )
			delete pSkillclass;
	}
	this->m_skillClasses.RemoveAll();
	for ( i = 0; i < this->m_skills.GetSize(); i++ )
	{
		CTSkillDef * pSkill = (CTSkillDef *) this->m_skills.GetAt(i);
		if ( pSkill )
			delete pSkill;
	}
	this->m_skills.RemoveAll();
	for ( i = 0; i < this->m_potions.GetSize(); i++ )
	{
		CTPotionDef * pPotion = (CTPotionDef *) this->m_potions.GetAt(i);
		if ( pPotion )
			delete pPotion;
	}
	this->m_potions.RemoveAll();
	for ( i = 0; i < this->m_spells.GetSize(); i++ )
	{
		CTSpellDef * pSpell = (CTSpellDef *) this->m_spells.GetAt(i);
		if ( pSpell )
			delete pSpell;
	}
	this->m_spells.RemoveAll();
	for ( i = 0; i < this->m_spawns.GetSize(); i++ )
	{
		CSObject * pSpawn = (CSObject *) this->m_spawns.GetAt(i);
		if ( pSpawn )
			delete pSpawn;
	}
	this->m_spawns.RemoveAll();
	for ( i = 0; i < this->m_starts.GetSize(); i++ )
	{
		CTStartLocation * pStart = (CTStartLocation * ) this->m_starts.GetAt(i);
		if ( pStart )
			delete pStart;
	}
	this->m_starts.RemoveAll();
	this->m_blockEmail.RemoveAll();
	this->m_events.RemoveAll();
	this->m_notoTitles.RemoveAll();
	this->m_obscene.RemoveAll();
	for ( i = 0; i < 6; i++ )
		this->m_pLevel[i].RemoveAll();
	this->m_runes.RemoveAll();
	this->m_Speech.RemoveAll();
	this->m_resources.RemoveAll();
}
