/*
 $Id: TABLES.H,v 1.5 2002/05/06 04:11:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// tables.h
//
// Defines the Table classes
//

#ifndef __TABLES_H__
#define __TABLES_H__

#include "common.h"

class CTStringTable : public CStringArray, public CTObject
{
public:
	CTStringTable();
	~CTStringTable();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);

	CString m_csTableName;
};

class CTObArray : public CScriptArray
{
public:
	CTObArray();
	~CTObArray();
	bool Write(CStdioFile &csfOutput);

	CString m_csBaseName;
	bool m_bHexIndex;
	DWORD m_dwIndexOffset;
};

class CAdvanceDef
{
public:
	CAdvanceDef();
	~CAdvanceDef();
	int m_Rate[3];
	void Load(CString csLine);
	void Write(CStdioFile &csfOutput);
	void Format(CString &csOutput);
};

class CTStartLocation : public CTObject
{
public:
	CTStartLocation();
	~CTStartLocation();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);

	CString m_csTown;
	CString m_csLocation;
	CString m_csCoords;
};

class CTOreDef : public CTObject
{
public:
	CTOreDef();
	~CTOreDef();
	void Load(CString csLine);
	bool Write(CStdioFile &csfOutput);

	CString m_csName;
	CString m_csColor;	// what color is the ore. (ores and metals are unique defined by color)
	int m_iMinSkill;		// Mining/Smelting skill needed to find it.
	int m_iVeinChance;		// rareity in the random world. (may be higher locally)
	int m_iMinAmount;		// How much to get of it.
	int m_iMaxAmount;
	CString m_csIngot;	// The ingot item ??? This should just be a colored / named item ?
};

class CTSkillClass : public CTObject
{
public:
	CTSkillClass();
	~CTSkillClass();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);

	CString m_csName;	// The name of this skill class.
	WORD m_StatMax[3];	// STAT_BASE_QTY
	WORD m_SkillLevelMax[50];

	// Triggers that come with the skillclass 
	DWORD m_dwPlotFlags;
	CStringArray m_Events;	// Action or motivation type indexes.
};

class CTSkillDef : public CTObject
{
public:
	CTSkillDef();
	~CTSkillDef();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);
	bool WriteBasic(CStdioFile &csfOutput);

	CString m_csKey;
	CString m_csTitle;	// title one gets if it's your specialty.

	CString m_csTargetPrompt;	// targetting prompt.
	CString m_csFailMsg;			// Message on skill fail

	WORD m_wDelay;	//	The base delay for the skill. (tenth of seconds)

	// Stat effects.
	// You will tend toward these stat vals if you use this skill a lot.
	BYTE m_bStat[3];	// STAT_STR, STAT_INT, STAT_DEX
	BYTE m_bStatBonus[3]; // % of each stat toward success at skill
	BYTE m_bSkillStat; // % of skill toward success at skill (heh funny sounding eh?)
	CAdvanceDef m_Adv; // OSI defined "skill curve"
};

class CTPotionDef : public CTObject
{
public:
	CTPotionDef();
	~CTPotionDef();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);

	CString m_csName;
	CString m_csPotionID;// resulting bottle type.
	CString m_csRegID;		// Reagent code.
	int m_iRegReq;			// Amount of reagent required.
	int m_iSkillReq;		// Minuimum required skill level for this potion.
	CString m_csColor;	// For nonstandard potions (COLOR_DEFAULT for normal ones)
};

class CTSpellDef : public CTObject
{
private:
#define SPELLFLAG_DIR_ANIM  0x0001	// Evoke type cast or directed. (animation)
#define SPELLFLAG_TARG_OBJ  0x0002	// Need to target an object or char ?
#define SPELLFLAG_TARG_CHAR 0x0004	// Needs to target a living thing
#define SPELLFLAG_TARG_XYZ  0x0008	// Can just target a location.
#define SPELLFLAG_HARM		0x0010	// The spell is in some way harmfull.
#define SPELLFLAG_FX_BOLT	0x0020	// Effect is a bolt to the target.
#define SPELLFLAG_FX_TARG	0x0040	// Effect is at the target.
#define SPELLFLAG_FIELD		0x0080	// create a field of stuff. (fire,poison,wall)
#define SPELLFLAG_SUMMON	0x0100	// summon a creature.
#define SPELLFLAG_GOOD		0x0200	// The spell is a good spell. u intend to help to receiver.
#define SPELLFLAG_DISABLED	0x8000

public:
	CTSpellDef();
	~CTSpellDef();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);
	bool WriteBasic(CStdioFile &csfOutput);

	CString m_csName;
	CString m_csSound;			// What noise does it make when done.
	CString m_csRunes;	// Letter Runes for Words of power.
	CString m_csReags;	// What reagents does it take ?
	CString m_csRuneID;		// The rune graphic for this.
	CString m_csScrollID;		// The scroll graphic item for this.
	CString m_csEffectID;	// Animation effect ID
	WORD m_wEffectLo;	// Damage or time or effect based on skill of caster.
	WORD m_wEffectHi;	// Damage or time of effect based on 100% magery
	WORD m_wCastTime;	// In tenths.
	WORD m_wDurationTimeLo;
	WORD m_wDurationTimeHi;

	WORD m_wFlags;

	void ReadFlags(CString csLine);
	void FormatFlags(CString & csLine);
};

class CSpawnRec : public CTObject
{
public:
	CSpawnRec();
	~CSpawnRec();
	CString m_csID;
	int m_iWeight;
};

class CTSpawnDef : public CSObject
{
public:
	CTSpawnDef();
	~CTSpawnDef();
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);
	bool WriteBasic(CStdioFile &csfOutput);

	int m_iIndex;
	int m_iTotalWeight;
	CPtrArray m_Members;
	CString m_csName;	// Name the group.
};

class CTables
{
public:
	void Unload();
	CTables();
	~CTables();
	bool Read();
	bool Write();

	CTStringTable m_resources;
	CTStringTable m_obscene;
	CTStringTable m_blockEmail;
	CTStringTable m_pLevel[6];
	CTStringTable m_events;
	CTStringTable m_Speech;;
	CAdvanceDef m_statAdvance[3];
	CScriptArray m_starts;
	CTStringTable m_runes;
	CScriptArray m_ores;
	CTObArray m_skillClasses;
	CTObArray m_skills;
	CTObArray m_potions;
	CTObArray m_spells;
	CTObArray m_spawns;
	CTStringTable m_notoTitles;
	CTStringTable m_Sphere;
	CTStringTable m_Servers;
	CTStringTable m_BlockIP;
};
#endif // __TABLES_H__
