/*
 $Id: adddestination.cpp,v 1.5 2002/11/09 03:56:07 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// AddDestination.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "AddDestination.h"
#include "AddValueDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAddDestination dialog


CAddDestination::CAddDestination(CWnd* pParent /*=NULL*/)
	: CDialog(CAddDestination::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAddDestination)
	m_csCategory = _T("");
	m_csDescription = _T("");
	m_csSubsection = _T("");
	m_csXCoord = _T("");
	m_csYCoord = _T("");
	m_csZCoord = _T("");
	m_csPlane = _T("");
	//}}AFX_DATA_INIT
}


void CAddDestination::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAddDestination)
	DDX_Control(pDX, IDC_SUBSECTION, m_ccbSubsection);
	DDX_Control(pDX, IDC_DESCRIPTION, m_ceDescription);
	DDX_Control(pDX, IDC_CATEGORY, m_ccbCategory);
	DDX_CBString(pDX, IDC_CATEGORY, m_csCategory);
	DDX_Text(pDX, IDC_DESCRIPTION, m_csDescription);
	DDX_CBString(pDX, IDC_SUBSECTION, m_csSubsection);
	DDX_Text(pDX, IDC_XCOORD, m_csXCoord);
	DDX_Text(pDX, IDC_YCOORD, m_csYCoord);
	DDX_Text(pDX, IDC_ZCOORD, m_csZCoord);
	DDX_Text(pDX, IDC_PLANE, m_csPlane);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAddDestination, CDialog)
	//{{AFX_MSG_MAP(CAddDestination)
	ON_CBN_SELCHANGE(IDC_CATEGORY, OnSelchangeCategory)
	ON_BN_CLICKED(IDC_ADD_CATEGORY, OnAddCategory)
	ON_BN_CLICKED(IDC_ADD_SUBSECTION, OnAddSubsection)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddDestination message handlers

void CAddDestination::OnOK() 
{
	// TODO: Add extra validation here
	// Make sure stuff has been entered
	UpdateData(true);
	if (m_ccbCategory.GetCurSel() == CB_ERR)
	{
		AfxMessageBox("No category selected!", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	if (m_ccbSubsection.GetCurSel() == CB_ERR)
	{
		AfxMessageBox("No subsection selected!", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	m_csDescription.TrimLeft();
	if (m_csDescription == "")
	{
		AfxMessageBox("Must have text in the description field", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	m_csXCoord.TrimLeft();
	if ((m_csXCoord == "") || (atoi(m_csXCoord) < 0) || (atoi(m_csXCoord) > 768 * 8) || (m_csXCoord.SpanIncluding("0123456789-") != m_csXCoord))
	{
		AfxMessageBox("Invalid value for X coordinate", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	m_csYCoord.TrimLeft();
	if ((m_csYCoord == "") || (atoi(m_csYCoord) < 0) || (atoi(m_csYCoord) > 512 * 8) || (m_csYCoord.SpanIncluding("0123456789-") != m_csYCoord))
	{
		AfxMessageBox("Invalid value for Y coordinate", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	m_csZCoord.TrimLeft();
	if ((m_csZCoord == "") || (m_csZCoord.SpanIncluding("0123456789-") != m_csZCoord) || (atoi(m_csZCoord) < -128) || (atoi(m_csZCoord) > 128))
	{
		AfxMessageBox("Invalid value for Z coordinate", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	m_csPlane.TrimLeft();
	if ((m_csPlane == "") || (m_csPlane.SpanIncluding("0123456789-") != m_csPlane) || (atoi(m_csPlane) < 0) || (atoi(m_csPlane) > 255))
	{
		AfxMessageBox("Invalid value for map plane", MB_OK | MB_ICONEXCLAMATION);
		return;
	}


	CDialog::OnOK();
}

BOOL CAddDestination::OnInitDialog() 
{
	CDialog::OnInitDialog();

	UpdateData(false);
	// Fill in the Category Box
	if (!m_pCategoryList->IsEmpty())
	{
		POSITION pos = m_pCategoryList->GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) m_pCategoryList->GetNext(pos);
			int iIndex = m_ccbCategory.AddString(pCategory->m_csName);
			m_ccbCategory.SetItemData(iIndex, 0);
		}
		m_ccbCategory.SetCurSel(0);
		this->OnSelchangeCategory();
	}

	this->m_ccbCategory.SelectString(-1, m_csCategory);
	this->OnSelchangeCategory();
	this->m_ccbSubsection.SelectString(-1, m_csSubsection);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAddDestination::OnSelchangeCategory() 
{
	if ( m_ccbCategory.GetCurSel() == -1 )
		return;
	CString csCategory;
	m_ccbCategory.GetLBText(m_ccbCategory.GetCurSel(), csCategory);
	if ( (csCategory != "") && (m_ccbCategory.GetItemData(m_ccbCategory.GetCurSel()) == 0))
	{
		m_ccbSubsection.ResetContent();
		CCategory * pCategory = FindCategory(m_pCategoryList, csCategory);
		if (pCategory == NULL)
			return;
		if (!pCategory->m_SubsectionList.IsEmpty())
		{
			POSITION pos = pCategory->m_SubsectionList.GetHeadPosition();
			while (pos != NULL)
			{
				CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(pos);
				int iIndex = m_ccbSubsection.AddString(pSubsection->m_csName);
				m_ccbSubsection.SetItemData(iIndex, 0);
			}
		}
		m_ccbSubsection.SetCurSel(0);
	}
	else
	{
		m_ccbSubsection.ResetContent();
		m_ccbSubsection.SetCurSel(-1);
	}
}

void CAddDestination::OnAddCategory() 
{
	CAddValueDialog dlg;
	if (dlg.DoModal() == IDOK)
	{
		if (m_ccbCategory.FindStringExact(0, dlg.m_csValue) != CB_ERR)
		{
			AfxMessageBox("Category already exists", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		int iIndex = m_ccbCategory.AddString(dlg.m_csValue);
		m_ccbCategory.SetItemData(iIndex, 1);
		m_ccbCategory.SetCurSel(iIndex);
		this->OnSelchangeCategory();
	}
	return;
}

void CAddDestination::OnAddSubsection() 
{
	CAddValueDialog dlg;
	if (dlg.DoModal() == IDOK)
	{
		if (m_ccbSubsection.FindStringExact(0, dlg.m_csValue) != CB_ERR)
		{
			AfxMessageBox("Subsection already exists", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		int iIndex = m_ccbSubsection.AddString(dlg.m_csValue);
		m_ccbSubsection.SetItemData(iIndex, 1);
		m_ccbSubsection.SetCurSel(iIndex);
	}
	return;
}
