/*
 $Id: common.cpp,v 1.22 2003/01/06 07:23:56 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/


#include "stdafx.h"
#include "common.h"
#include "citem.h"
#include "cnpc.h"
#include "cscriptobjects.h"
#include "remoteconnection.h"
#include <direct.h>

const char * ItemTypeName[] =
{
	"  NORMAL  ",
	"CONTAINER",		// any unlocked container or corpse. CContainer based
	"CONTAINER_LOCKED",	// 2 = 
	"DOOR",				// 3 = door can be opened
	"DOOR_LOCKED",		// 4 = a locked door.
	"KEY",				// 5 = 
	"LIGHT_LIT",			// 6 = Local Light giving object
	"LIGHT_OUT",			// 7 = Can be lit.
	"FOOD",				// 8 = edible food. (poisoned food ?)
	"FOOD_RAW",			// 9 = Must cook raw meat unless your an animal.
	"ARMOR",				// 10 = some type of armor. (no real action)
	"WEAPON_MACE_SMITH",	// 11 = Can be used for smithing 
	"WEAPON_MACE_SHARP",	// 12 = war axe can be used to cut/chop trees.
	"WEAPON_SWORD",		// 13 = 
	"WEAPON_FENCE",		// 14 = can't be used to chop trees. (make kindling)
	"WEAPON_BOW",		// 15 = bow or xbow
	"WAND",			    // 16 = A magic storage item
	"TELEPAD",			// 17 = walk on teleport
	"SWITCH",			// 18 = this is a switch which effects some other object in the world.
	"BOOK",				// 19 = read this book. (static text)
	"RUNE",				// 20 = can be marked and renamed as a recall rune.
	"BOOZE",				// 21 = booze
	"POTION",			// 22 = Some magic effect.	
	"FIRE",				// 23 = It will burn you.
	"CLOCK",				// 24 = or a wristwatch
	"TRAP",				// 25 = walk on trap.
	"TRAP_ACTIVE",		// 26 = some animation 
	"MUSICAL",			// 27 = a musical instrument.
	"SPELL",				// 28 = magic spell effect.
	"GEM",				// 29 = no use yet 
	"WATER",				// 30 = This is water (fishable)
	"CLOTHING",			// 31 = All cloth based wearable stuff",
	"SCROLL",			// 32 = magic scroll
	"CARPENTRY",			// 33 = tool of some sort.
	"SPAWN_CHAR",		// 34 = spawn object. should be invis also.
	"GAME_PIECE",		// 35 = can't be removed from game.
	"PORTCULIS",			// 36 = Z delta moving gate. (open)
	"FIGURINE",			// 37 = magic figure that turns into a creature when activated.
	"SHRINE",			// 38 = can res you
	"MOONGATE",			// 39 = linked to other moon gates (hard coded locations)	
	"CHAIR",				// 40 = Any sort of a chair item. we can sit on.
	"FORGE",				// 41 = used to smelt ore", blacksmithy etc.
	"ORE",				// 42 = smelt to ingots.
	"LOG",				// 43 = make into furniture etc.
	"TREE",				// 44 = can be chopped.
	"ROCK",				// 45 = can be mined for ore.
	"CARPENTRY_CHOP",	// 46 = tool of some sort.
	"MULTI",				// 47 = multi part object like house or ship.
	"REAGENT",			// 48 = alchemy when clicked ?
	"SHIP",				// 49 = this is a SHIP MULTI
	"SHIP_PLANK",		
	"SHIP_SIDE",			// 51 = Should extend to make a plank
	"SHIP_SIDE_LOCKED",
	"SHIP_TILLER",		// 53 = Tiller man on the ship.
	"EQ_TRADE_WINDOW",		// 54 = container for the trade window.
	"FISH",				// 55 = fish can e cut up.
	"SIGN_GUMP",			// 56 = Things like grave stones and sign plaques
	"STONE_GUILD",		// 57 = Guild/town stones
	"ANIM_ACTIVE",		// 58 = active anium that will recycle when done.
	"ADVANCE_GATE",		// 59 = advance gate. m_AdvanceGateType = CREID_TYPE
	"CLOTH",				// 60 = bolt or folded cloth
	"HAIR",
	"BEARD",				// 62 = just for grouping purposes.
	"INGOT",				// 63 = Ingot.
	"COIN",				// 64 = coin of some sort. gold or otherwise.
	"PLANT",				// 65 = a plant that will regrow. picked type.
	"DRINK",				// 66 = some sort of drink (non booze)
	"ANVIL",				// 67 = for repair.
	"PORT_LOCKED",		// 68 = this portcullis must be triggered.
	"SPAWN_ITEM",		// 69 = spawn other items.
	"TELESCOPE",			// 70 = big telescope pic.
	"BED_EW",			// 71 = EW facing bed.
	"BED_NS",			// 72 = NS facing bed
	"MAP",				// 73 = Map object with pins.
	"EQ_MEMORY_OBJ",		// 74 = A Char has a memory link to some object. (I am fighting with someone. This records the fight.)
	"WEAPON_MACE_STAFF",	// 75 = staff type of mace. or just other type of mace.
	"EQ_HORSE",			// 76 = equipped horse object represents a riding horse to the client.
	"COMM_CRYSTAL",		// 77 = communication crystal.
	"GAME_BOARD",		// 78 = this is a container of pieces.
	"TRASH",				// 79 = delete any object dropped on it.
	"CANNON_MUZZLE",		// 80 = cannon muzzle. NOT the other cannon parts.
	"CANNON",			// 81 = the reset of the cannon.
	"CANNON_BALL",
	"PLANT_FRUITING",	// 83 = fruit can be picked off this.
	"PLANT_IMATURE",		// 84 = can't be picked yet.
	"RING_REAGENTS",		// 85 = ring of reagents.
	"CRYSTAL_BALL",
	"CASHIERS_CHECK",	// 87 = represents large amounts of money.
	"MESSAGE",			// 88 = user written message item. (for bboard ussually)
	"REAGENT_RAW",		// 89 = Freshly grown reagents...not processed yet.
	"EQ_CLIENT_LINGER",	// 90 = Change the brain type for some time.
	"DREAM_GATE",		// 91 = Push you to another shard. (no items transfered", client insta logged out)
	"ITEM_STONE",		// 92 = Double click for items
	"METRONOME",			// 93 = ticks once every n secs.
	"EXPLOSION",			// 94 = async explosion.
	"EQ_NPC_SCRIPT",		// 95 = Script npc actions in the form of a book.
	"WEB",
	"GRASS",
	"AROCK",				// 98 = a rock or boulder. can be thrown by giants.
	"TRACKER",			// 99 = points to a linked object.
	"SOUND",				// 100 = this is a sound source.
	"STONE_TOWN",		// 101 = Town stones. everyone free to join.
	"WEAPON_MACE_CROOK",	// 102
	"WEAPON_MACE_PICK",	// 103
	"LEATHER",			// 104 = Leather or skins of some sort.(not wearable)
	"SHIP_OTHER",		// 105 = some other part of a ship.
	"BBOARD",			// 106 = a container and bboard object.
	"SPELLBOOK",			// 107 = spellbook (with spells)
	"CORPSE",			// 108 = special type of item.
	"TRACK_ITEM",		// 109 - track a id or type of item.
	"TRACK_CHAR",		// 110 = track a char or range of char id's
	"WEAPON_ARROW",		// 111
	"WEAPON_BOLT",		// 112
	"EQ_VENDOR_BOX",		// 113 = an equipped vendor .
	"EQ_BANK_BOX",		// 114 = an equipped bank box
	"DEED",			// 115
	"LOOM",			// 116
	"BEE_HIVE",		// 117
	"ARCHERY_BUTTE",	// 118
	"EQ_MURDER_COUNT",	// 119 = my murder count flag.
	"EQ_STUCK",		// 120
	"TRAP_INACTIVE",	// 121 = a safe trap.
	"STONE_ROOM",		// 122 = house type stone. (for mapped house regions)
	"BANDAGE",			// 123 = can be used for healing.
	"CAMPFIRE",		// 124 = this is a fire but a small one.
	"MAP_BLANK",		// 125 = blank map.
	"SPY_GLASS",		// 126
	"SEXTANT",			// 127
	"SCROLL_BLANK",	// 128
	"FRUIT",			// 129
	"WATER_WASH",		// 130 = water that will not contain fish. (for washing or drinking)
	"WEAPON_AXE",		// 131 = not the same as a sword. but uses swordsmanship skill
	"WEAPON_XBOW",		// 132
	"SPELLICON",		// 133
	"DOOR_OPEN",		// 134 = You can walk through doors that are open.
	"MEAT_RAW",		// 135 = raw (uncooked meat) or part of a corpse.
	"GARBAGE",			// 136 = this is junk.
	"KEYRING",			// 137
	"TABLE",			// 138 = a table top
	"FLOOR",			// 139
	"ROOF",			// 140
	"FEATHER",			// 141 = a birds feather
	"WOOL",			// 142 = Wool cut frm a sheep.
	"FUR",				// 143
	"BLOOD",			// 144 = blood of some creature
	"FOLIAGE",			// 145 = does not go invis when reaped. but will if eaten
	"GRAIN",			// 146
	"SCISSORS",		// 147
	"THREAD",			// 148
	"YARN",			// 149
	"SPINWHEEL",		// 150
	"BANDAGE_BLOOD",	// 151 = must be washed in water to get bandage back.
	"FISH_POLE",		// 152
	"SHAFT",			// 153 = used to make arrows and xbolts
	"LOCKPICK",		// 154
	"KINDLING",		// 155 = lit to make campfire
	"TRAIN_DUMMY",		// 156
	"TRAIN_PICKPOCKET",// 157
	"BEDROLL",			// 158
	"BELLOWS",			// 159
	"HIDE",			// 160 = hides are cured to make leather.
	"CLOTH_BOLT",		// 161 = must be cut up to make cloth squares.
	"LUMBER",			// 162 = logs are plained into decent lumber
	"PITCHER",			// 163
	"PITCHER_EMPTY",	// 164
	"DYE_VAT",			// 165
	"DYE",				// 166
	"POTION_EMPTY",	// 167 = empty bottle.
	"MORTAR",			// 168
	"HAIR_DYE",		// 169
	"SEWING_KIT",		// 170
	"TINKER_TOOLS",	// 171
	"WALL",			// 172 = wall of a structure.
	"WINDOW",			// 173 = window for a structure.
	"COTTON",			// 174 = Cotton from the plant
	"BONE",			// 175
	"EQ_SCRIPT",		// 176
	"SHIP_HOLD",		// 177 = ships hold.
	"SHIP_HOLD_LOCK",	// 178
	"LAVA",			// 179
	"SHIELD",			// 180 = equippable armor.
	"JEWELRY",			// 181 = equippable.
	"DIRT",			// 182 = a patch of dirt where i can plant something
};

const char * SpellNames[] =
{
	"  NONE  ",

	"Clumsy",
	"Create_Food",
	"Feeblemind",
	"Heal",
	"Magic_Arrow",
	"Night_Sight",
	"Reactive_Armor",
	"Weaken",

	// 2nd
	"Agility",	// 9
	"Cunning",
	"Cure",
	"Harm",
	"Magic_Trap",
	"Magic_Untrap",
	"Protection",
	"Strength",	// 16

	// 3rd
	"Bless",		// 17
	"Fireball",
	"Magic_Lock",
	"Poison",
	"Telekin",
	"Teleport",
	"Unlock",
	"Wall_of_Stone",	// 24

	// 4th
	"Arch_Cure",	// 25
	"Arch_Prot",
	"Curse",
	"Fire_Field",
	"Great_Heal",
	"Lightning",	// 30
	"Mana_Drain",
	"Recall",

		// 5th
	"Blade_Spirit",
	"Dispel_Field",
	"Incognito",		// 35
	"Magic_Reflect",
	"Mind_Blast",
	"Paralyze",
	"Poison_Field",
	"Summon",			// 40

		// 6th
	"Dispel",
	"Energy_Bolt",
	"Explosion",
	"Invis",
	"Mark",
	"Mass_Curse",
	"Paralyze_Field",
	"Reveal",

		// 7th
	"Chain_Lightning",
	"Energy_Field",
	"Flame_Strike",
	"Gate_Travel",
	"Mana_Vamp",
	"Mass_Dispel",
	"Meteor_Swarm",
	"Polymorph",

		// 8th
	"Earthquake",
	"Vortex",
	"Resurrection",	// 59
	"Air_Elem",
	"Daemon",
	"Earth_Elem",
	"Fire_Elem",
	"Water_Elem",	// 64

	// Necro
	"Summon_Undead",
	"Animate_Dead",
	"Bone_Armor",
	"Light",
	"Fire_Bolt"		// 69

};

const char * knownItemTags[]	 =
{
	"NAME",
	"DYE",
	"REPAIR",
	"REQSTR",
	"MOREP",
	"TYPE",
	"CATEGORY",
	"SUBSECTION",
	"DESCRIPTION",
	"ID",
	"COLOR",
	"WEIGHT",
	"ARMOR",
	"BUYVALUE",
	"SELLVALUE",
	"DAM",
	"ATT",
	"HITPOINTS",
	"DUPEITEM",
	"ATTR",
	"SPEED",
	"SKILL",
	"MORE",
	"MORE1",
	"MORE2",
	"ONTRIGGER",
	"DUPELIST",
	"DEFNAME",
	"TIMER",
	"NOINDEX",
	"TWOHANDS",
	"COMPONENT",
	"MULTIREGION",
	"MATERIAL",
};

const char * knownMapTags[] = 
{
	"GROUP",								// 0
	"TITLE",
	"P",
	"RECT",
	"MIDILIST",
	"MAGICDAMAGE",
	"MAGIC",
	"MARK",
	"RECALLIN",
	"RECALLOUT",
	"GATE",								// 10
	"GATEIN",
	"GATEOUT",
	"BUILDABLE",
	"NOBUILD",
	"GUARDED",
	"GUARDOWNER",
	"RAINCHANCE",
	"COLDCHANCE",
	"LOCAL",
	"GLOBAL",						// 20
	"SOUNDS",
	"NODECAY",
	"SAFE",
	"UNDERGROUND",				// 24
	"ANNOUNCEMENT",
	"ANNOUNCE",
	"TRIGGER",				// 27
	"FLAGS",				// 28
	"TRIGPERIODIC",			// 29
};

const char * knownINITags [] =
{
	"FILES",	// 
	"MULFILES",
	"SCPFILES",
	"WORLDSAVE",
	"LOG",

	"PROFILE",
	"VERBOSE",
	"GUESTS",
	"WEBPAGESRC",		// m_sWebPageFilePath2
	"SECURE",
	"HEARALL",
	"SAVEPERIOD",
	"MAPCACHETIME",
	"REAGENTSREQUIRED",
	"DUNGEONLIGHT",
	"MONSTERFIGHT",
	"SPEECHFILES",
	"DECAYTIMER",
	"BANKMAXITEMS",
	"BANKMAXWEIGHT",
	"WOPPLAYER",
	"WOPSTAFF",
	"DEADSOCKETTIME",
	"DEBUGFLAGS",
	"GUARDSINSTANTKILL",
	"ARRIVEDEPARTMSG",  // 25
	"CORPSEPLAYERDECAY",
	"CORPSENPCDECAY",	// 27
	"SNOOPCRIMINAL",
	"FREESERVER",
	"FREESHARD",
	"CLIENTLINGER",
	"FREEZERESTARTTIME",		// m_iFreezeRestartTime
	"WEBPAGEUPDATE",
	"WEBPAGEDST",
	"MINCHARDELETETIME",
	"NPCTRAINPERCENT",			// m_iTrainSkillPercent
	"CLIENTVERSION",
	"REGISTERSERVER",			// m_sRegisterServer
	"CHARTAGS",					// m_fCharTags
	"VENDORMAXSELL",			// m_iVendorMaxSell
	"BACKUPLEVELS",				// m_iSaveBackupLevels
	"SAVEBACKGROUND",			// m_iSaveBackgroundTime
	"FORCEGARBAGECOLLECT",		// m_fSaveGarbageCollect
	"GAMEMINUTELENGTH",			// m_iGameMinuteLength
	"NOWEATHER",				// m_fNoWeather
	"EQUIPPEDCAST",				// m_fEquippedCast
	"FLIPDROPPEDITEMS",			// m_fFlipDroppedItems
	"STATCAP",					// m_iMaxSumOfStats
	"STATAVG",					// m_iAvgSumOfStats
	"REAGENTLOSSFAIL",			// m_fReagentLossFail
	"NTSERVICE",				// m_fUseNTService
	"POLLSERVERS",				// m_iPollServers
	"MAGICUNLOCKDOOR",		// m_iMagicUnlockDoor
	"MAINLOGSERVER",		// m_fMainLogServer
	"MURDERMINCOUNT",		// m_iMurderMinCount;		// amount of murders before we get title.
	"MURDERDECAYTIME",		// m_iMurderDecayTime;
	"CLIENTS",
	"CLIENTMAX",			// m_nClientsMax
	"WEBCLIENTLISTFORM",	// m_sWebClientListFormat
	"WEBSERVERLISTFORM",	// m_sWebServListFormat
	"MAXCHARSPERACCOUNT",	// m_iMaxCharsPerAccount
	"LOGMASK",				// GetLogMask
	"LIGHTNIGHT",			// m_iLightNight
	"NPCTRAINMAX",			// m_iTrainSkillMax
	"MAXCOMPLEXITY",			// m_iMaxComplexity

	// These are for the server references...
	"TZ",
	"TIMEZONE",
	"EMAIL",
	"ADMIN",
	"ADMINEMAIL",	// 	m_sEMail
	"URL",			// m_sURL

	// These are new
	"PLAYERNEUTRAL",
	"GUARDLINGER",
	"CRIMINALTIMER",
	"HITPOINTPERCENTONREZ",
	"MONSTERFEAR",
	"ACCTFILES",
	"LOOTINGISACRIME",
	"RESPONSIBLEOWNERS",
	"HELPINGCRIMINALSISACRIME",
	"PLAYERGHOSTSOUNDS",
	"AUTONEWBIEKEYS",
	"MAXBASESKILL",
	"RUNNINGPENALTY",
	"STAMINALOSSATWEIGHT",
};

const char * knownAcctTags [] = 
{
	"ACCOUNT",
	"BLOCK",
	"CHATNAME",
	"COMMENT",
	"EMAIL",
	"EMAILFAIL",
	"EMAILMSG",
	"FIRSTCONNECTDATE",
	"FIRSTIP",
	"GUEST",
	"JAIL",
	"LANG",
	"LASTCHARUID",
	"LASTCONNECTDATE",
	"LASTCONNECTTIME",
	"LASTIP",
	"LEVEL",
	"PASSWORD",
	"PLEVEL",
	"PRIV",
	"T2A",
	"TOTALCONNECTTIME",
};

const char * knownPotionTags [] = 
{
	"NAME",
	"ITEM",
	"REAGENT",
	"REAGENT_AMT",
	"SKILL_MIN",
	"COLOR",
};

const char * knownSpawnTags [] =
{
	"DEFNAME",
	"CATEGORY",
	"SUBSECTION",
	"DESCRIPTION",
	"ID",
	"WEIGHT",
};

const char * Resources [] = 
{
	"BW",
	"BL",
	"BP",
	"BM",
	"BS",
	"VB",
	"BO",
	"BR",
	"DA",
	"FD",
	"DB",
	"EC",
	"GA",
	"GI",
	"MR",
	"EN",
	"NS",
	"OB",
	"PI",
	"PU",
	"SA",
	"SS",
	"SE",
	"VA",
	"DW",
	"WH",
	"ME",
	"FE",
	"HI",
	"LE",
	"FU",
	"WO",
	"IN",
	"IG",
	"IS",
	"IC",
	"LO",
	"SH",
	"BT",
	"SC",
	"CL",
	"HA",
	"FL",
};

const char * ResourceNames [] =
{
	"Batwing",
	"Blackmoor",
	"Black Pearl",
	"Blood Moss",
	"Bloodspawn",
	"Vial of Blood",
	"Bone",
	"Brimstone",
	"Daemon Bone",
	"Fertile Dirt",
	"Dragon's Blood",
	"Executioner's Cap",
	"Garlic",
	"Ginseng",
	"Mandrake Root",
	"Eye of Newt",
	"Nightshade",
	"Obsidian",
	"Pig Iron",
	"Pumice",
	"Sulfurous Ash",
	"Spider Silk",
	"Serpent Scales",
	"Volcanic Ash",
	"Dead Wood",
	"Wyrm's Heart",
	"Meat",
	"Feathers",
	"Hides",
	"Leather",
	"Furs",
	"Wool",
	"Iron Ingots",
	"Gold Ingots",
	"Silver Ingots",
	"Copper Ingots",
	"Logs",
	"Shafts",
	"Empty Bottle",
	"Blank Scroll",
	"Cloth",
	"Hay",
	"Flour",
};


CIntRange::CIntRange()
{
	this->min = 0;
	this->max = 0;
}

void CIntRange::ReadRange(const char *pszStr)
{
	min = parseRange(pszStr, false);
	max = parseRange(pszStr, true);
	if ( min > max )
	{
		int tmp = min;
		min = max;
		max = tmp;
	}
	return;
}

CString CIntRange::WriteRange()
{
	CString csOut;
	if ( min != max )
		csOut.Format("{%ld %ld}", min, max);
	else
		csOut.Format("%ld", min);
	return csOut;
}

void CIntRange::Copy(CIntRange *pSrc)
{
	this->max = pSrc->max;
	this->min = pSrc->min;
}

CFloatRange::CFloatRange()
{
	min = 0.0;
	max = 0.0;
}

void CFloatRange::ReadRange(const char *pszStr)
{
	min = fParseRange(pszStr, false);
	max = fParseRange(pszStr, true);
	if ( min > max )
	{
		float tmp = min;
		min = max;
		max = tmp;
	}
	return;
}

CString CFloatRange::WriteRange()
{
	CString csOut;
	if ( min == max )
		csOut.Format("{%.2f %.2f}", min, max);
	else
		csOut.Format("%.2f", min);
	return csOut;
}

void CVendorItem::Copy(CVendorItem *pSrc)
{
	this->m_csName = pSrc->m_csName;
	this->m_iProbability = pSrc->m_iProbability;
	this->m_irAmount.max = pSrc->m_irAmount.max;
	this->m_irAmount.min = pSrc->m_irAmount.min;
	this->m_pItem = pSrc->m_pItem;
}

CTemplate::~CTemplate()
{
	// Clear out all of the loot items
	POSITION pos = this->m_olItemList.GetHeadPosition();
	while ( pos != NULL )
	{
		CNPCLootItem * pItem = (CNPCLootItem * ) this->m_olItemList.GetNext( pos );
		if ( pItem != NULL )
			delete ( pItem );
	}
}

bool CTemplate::WriteBasic(CStdioFile &csfOutput)
{
	if ( !CSObject::Write(csfOutput) )
		return false;
	return true;
}

void CTemplate::Write( CStdioFile * csfOut )
{
	CString csLine;
	// Write the ID
	csLine.Format("[%s]\n", this->m_csValue );
	csfOut->WriteString( csLine );
	// Write the name
	csLine.Format("DEFNAME=%s\n", this->m_csDescription );
	csfOut->WriteString( csLine );
	// Write the container, if it exists
	if ( this->m_dwContainerID != 0 )
	{
		csLine.Format("CONTAINER=0%x\n", m_dwContainerID);
		csfOut->WriteString( csLine );
	}
	else if ( this->m_csContainer != "" )
	{
		csLine.Format("CONTAINER=%s\n", this->m_csContainer );
		csfOut->WriteString( csLine );
	}
	if ( this->m_bNoIndex )
		csfOut->WriteString("NOINDEX\n");
	// Write the more value
	if ( this->m_csContainerMore != "" )
	{
		csfOut->WriteString( this->m_csContainerMore );
		csfOut->WriteString("\n");
	}
	// Time for the items
	POSITION pos = this->m_olItemList.GetHeadPosition();
	while ( pos != NULL )
	{
		CNPCLootItem * pItem = ( CNPCLootItem * ) this->m_olItemList.GetNext( pos );
		if ( pItem == NULL )
			break;
		CString csType;
		switch ( this->m_iBuySell )
		{
		case -1:
			csType = "SELL"; break;
		case 1:
			csType = "BUY"; break;
		case 0:
		default:
			csType = "ITEM"; break;
		}
		if ( pItem->m_csName != "" )
			csLine.Format( "%s=%s", csType, pItem->m_csName );
		else
			csLine.Format( "%s=%s", csType, pItem->m_csWeight );
		csfOut->WriteString( csLine );
		if ( pItem->m_iProbability != 1 )
		{
			csLine.Format( ",R%ld", pItem->m_iProbability );
			csfOut->WriteString( csLine );
		}
		csfOut->WriteString("\n");
		if ( pItem->m_irAmount.min != 1 && pItem->m_irAmount.max != 1 )
		{
			if ( pItem->m_irAmount.min != pItem->m_irAmount.max )
				csLine.Format( "AMOUNT={%ld %ld}\n", pItem->m_irAmount.min, pItem->m_irAmount.max );
			else
				csLine.Format( "AMOUNT=%ld\n", pItem->m_irAmount.min );
			csfOut->WriteString( csLine );
		}
		if ( pItem->m_csColor != "" )
		{
			csLine.Format( "COLOR=%s\n", pItem->m_csColor );
			csfOut->WriteString( csLine );
		}
		if ( pItem->m_csMore != "" )
		{
			csfOut->WriteString( pItem->m_csMore );
			csfOut->WriteString( "\n" );
		}
	}
	csfOut->WriteString("\n");
	return;
}

bool CTemplate::Read(CStdioFile &csfIn)
{
	CNPCLootItem * pCurrentItem = NULL;
	BOOL bStatus = true;
	while ( bStatus )
	{
		CString csLine;
		DWORD dwCur = csfIn.GetPosition();
		bStatus = csfIn.ReadString( csLine );
		if ( ! bStatus )
			return bStatus ? true : false;
		csLine.TrimLeft();
		csLine.TrimRight();
		csLine.MakeUpper();
		csLine.Remove(0x0d);
		if ( csLine != "" )
		{
			// Is this the beginning of a new template?
			if ( csLine.Find("[") != -1 )
			{
				// Yep...sure is
				csfIn.Seek((LONG) (dwCur - csfIn.GetPosition()), CFile::current);
				return bStatus ? true : false;
			}
			else if ( csLine.Find("//") != -1 )
			{
				if ( this->m_csDescription == "" && this->m_dwContainerID == 0 )
				{
					// This is the template name
					CString csName = csLine.Mid( csLine.Find("//") + 2 );
					csName.TrimLeft();
					csName.TrimRight();
					this->m_csDescription = csName;
				}
				else if ( pCurrentItem == NULL )
				{
					// This belongs to the container
					CString csTemp;
					if ( this->m_csContainerMore == "" )
						csTemp = csLine;
					else
						csTemp.Format("%s\n%s", this->m_csContainerMore, csLine );
					this->m_csContainerMore = csTemp;
					continue;
				}
				else
				{
					if ( this->m_csDescription != "" )
					{
						// This belongs to the item.
						CString csTemp;
						if ( pCurrentItem->m_csMore == "" )
							csTemp = csLine;
						else
							csTemp.Format("%s\n%s", pCurrentItem->m_csMore, csLine );
						pCurrentItem->m_csMore = csTemp;
						continue;
					}
				}
			}
			else if ( csLine.Find("DEFNAME") != -1 )
				this->m_csDef = csLine.Mid(csLine.Find("=") + 1);
			else if ( csLine.Find("CONTAINER") != -1 )
			{
				pCurrentItem = NULL;
				CString csContainer = csLine.Mid( csLine.Find("=") + 1 );
				this->m_dwContainerID = Main->m_pScripts->m_aDefs.Find( csContainer );
				if ( this->m_dwContainerID == 0 )
				{
					this->m_csContainer = csContainer;
					this->m_dwContainerID = 0;
				}
			}
			else if ( csLine.Find("NOINDEX") != -1 )
				this->m_bNoIndex = true;
			else if ( csLine.Find("ITEM") != -1 )
			{
				pCurrentItem = new ( CNPCLootItem );
				CString csItem = csLine.Mid( csLine.Find("=") + 1 );
				if ( csItem.Find("{") != -1 )
					pCurrentItem->m_csWeight = csItem;
				else
				{
					pCurrentItem->m_csName = csItem.SpanExcluding(",");
					if ( csItem.Find(",") != -1 )
					{
						CString csMore = csItem.Mid( csItem.Find(",") + 1 );
						if (csMore != "")
						{
							if (csMore.Find(",") != -1)
							{
								CString csProb;
								CString csAmount = csMore.Left(csMore.Find(","));
								pCurrentItem->m_irAmount.ReadRange(csAmount);
								csProb = csMore.Mid(csAmount.GetLength() + 1);
								if (csProb != "")
								{
									if (csProb.Find("R") != -1)
									{
										csProb = csProb.Mid(csProb.Find("R") + 1);
										pCurrentItem->m_iProbability = atoi(csProb);
									}
								}
							}
							else
							{
								if (csMore.Find("R") == -1)
									pCurrentItem->m_irAmount.ReadRange(csMore);
								else
								{
									pCurrentItem->m_irAmount.ReadRange("1");
									CString csProb = csMore.Mid(csMore.Find("R") + 1);
									pCurrentItem->m_iProbability = atoi(csProb);
								}
							}
						}
					}
				}
				if ( this->m_olItemList.IsEmpty() )
					this->m_olItemList.AddHead( pCurrentItem );
				else
					this->m_olItemList.AddTail( pCurrentItem );
			}
			else if ( csLine.Find("BUY") != -1 )
			{
				pCurrentItem = new ( CNPCLootItem );
				CString csItem = csLine.Mid( csLine.Find("=") + 1 );
				if ( csItem.Find("{") != -1 )
					pCurrentItem->m_csWeight = csItem;
				else
				{
					pCurrentItem->m_csName = csItem.SpanExcluding(",");
					if ( csItem.Find(",") != -1 )
					{
						CString csMore = csItem.Mid( csItem.Find(",") + 1 );
						if (csMore != "")
						{
							if (csMore.Find(",") != -1)
							{
								CString csProb;
								CString csAmount = csMore.Left(csMore.Find(","));
								pCurrentItem->m_irAmount.ReadRange(csAmount);
								csProb = csMore.Mid(csAmount.GetLength() + 1);
								if (csProb != "")
								{
									if (csProb.Find("R") != -1)
									{
										csProb = csProb.Mid(csProb.Find("R") + 1);
										pCurrentItem->m_iProbability = atoi(csProb);
									}
								}
							}
							else
							{
								if (csMore.Find("R") == -1)
									pCurrentItem->m_irAmount.ReadRange(csMore);
								else
								{
									pCurrentItem->m_irAmount.ReadRange("1");
									CString csProb = csMore.Mid(csMore.Find("R") + 1);
									pCurrentItem->m_iProbability = atoi(csProb);
								}
							}
						}
					}
				}
				if ( this->m_olItemList.IsEmpty() )
					this->m_olItemList.AddHead( pCurrentItem );
				else
					this->m_olItemList.AddTail( pCurrentItem );
				if ( this->m_iBuySell == 0 )
					this->m_iBuySell = 1;
			}
			else if ( csLine.Find("SELL") != -1 )
			{
				pCurrentItem = new ( CNPCLootItem );
				CString csItem = csLine.Mid( csLine.Find("=") + 1 );
				if ( csItem.Find("{") != -1 )
					pCurrentItem->m_csWeight = csItem;
				else
				{
					pCurrentItem->m_csName = csItem.SpanExcluding(",");
					if ( csItem.Find(",") != -1 )
					{
						CString csMore = csItem.Mid( csItem.Find(",") + 1 );
						if (csMore != "")
						{
							if (csMore.Find(",") != -1)
							{
								CString csProb;
								CString csAmount = csMore.Left(csMore.Find(","));
								pCurrentItem->m_irAmount.ReadRange(csAmount);
								csProb = csMore.Mid(csAmount.GetLength() + 1);
								if (csProb != "")
								{
									if (csProb.Find("R") != -1)
									{
										csProb = csProb.Mid(csProb.Find("R") + 1);
										pCurrentItem->m_iProbability = atoi(csProb);
									}
								}
							}
							else
							{
								if (csMore.Find("R") == -1)
									pCurrentItem->m_irAmount.ReadRange(csMore);
								else
								{
									pCurrentItem->m_irAmount.ReadRange("1");
									CString csProb = csMore.Mid(csMore.Find("R") + 1);
									pCurrentItem->m_iProbability = atoi(csProb);
								}
							}
						}
					}
				}
				if ( this->m_olItemList.IsEmpty() )
					this->m_olItemList.AddHead( pCurrentItem );
				else
					this->m_olItemList.AddTail( pCurrentItem );
				if ( this->m_iBuySell == 0 )
					this->m_iBuySell = -1;
			}
			else if ( csLine.Find("COLOR") != -1 )
			{
				if ( pCurrentItem == NULL )
				{
					// this belongs to the container
					CString csTemp;
					if ( this->m_csContainerMore == "" )
						csTemp = csLine;
					else
						csTemp.Format("%s\n%s", this->m_csContainerMore, csLine );
					this->m_csContainerMore = csTemp;
				}
				else
				{
					CString csColor = csLine.Mid( csLine.Find("=") + 1 );
					pCurrentItem->m_csColor = csColor;
				}
			}
			else if ( csLine.Find("AMOUNT") != -1 )
			{
				// Containers can't have amounts.
				if ( pCurrentItem == NULL )
					continue;
				// Read the amount.
				CString csAmount = csLine.Mid( csLine.Find("=") + 1 );
				pCurrentItem->m_irAmount.ReadRange( csAmount );
			}
			else
			{
				// It's something else
				if ( pCurrentItem == NULL )
				{
					// This belongs to the container
					CString csTemp;
					if ( this->m_csContainerMore == "" )
						csTemp = csLine;
					else
						csTemp.Format("%s\n%s", this->m_csContainerMore, csLine );
					this->m_csContainerMore = csTemp;
					continue;
				}
				else
				{
					if ( this->m_csDescription != "" )
					{
						// This belongs to the item.
						CString csTemp;
						if ( pCurrentItem->m_csMore == "" )
							csTemp = csLine;
						else
							csTemp.Format("%s\n%s", pCurrentItem->m_csMore, csLine );
						pCurrentItem->m_csMore = csTemp;
						continue;
					}
				}
				CString csName = csLine.Mid( csLine.Find("//") + 2 );
				csName.TrimLeft();
				csName.TrimRight();
				this->m_csDescription = csName;
			}
		}
	}
	return bStatus ? true : false;
}

void CNPCLootItem::Copy(CNPCLootItem *pCopy)
{
	this->m_csColor = pCopy->m_csColor;
	this->m_csName = pCopy->m_csName;
	this->m_csWeight = pCopy->m_csWeight;
	this->m_iProbability = pCopy->m_iProbability;
	this->m_irAmount.max = pCopy->m_irAmount.max;
	this->m_irAmount.min = pCopy->m_irAmount.min;
	this->m_csMore = pCopy->m_csMore;
	this->m_bNewbie = pCopy->m_bNewbie;
	this->m_pItem = pCopy->m_pItem;
}

CMessage::CMessage()
{
	this->m_dwLParam = 0;
	this->m_dwMessage = 0;
	this->m_dwWParam = 0;
}

void CMessage::Write( CFile * pFile )
{
	pFile->Write( (void *) &m_dwMessage, sizeof(DWORD) );
	pFile->Write( (void *) &m_dwWParam, sizeof(DWORD) );
	pFile->Write( (void *) &m_dwLParam, sizeof(DWORD) );
}

void CMessage::Read( CFile * pFile )
{
	pFile->Read( (void *) &m_dwMessage, sizeof(DWORD) );
	pFile->Read( (void *) &m_dwWParam, sizeof(DWORD) );
	pFile->Read( (void *) &m_dwLParam, sizeof(DWORD) );
}

CSequence::CSequence()
{
}

CSequence::~CSequence()
{
	while ( ! this->m_olMessages.IsEmpty() )
	{
		CMessage * pMessage = (CMessage *) this->m_olMessages.RemoveHead();
		if ( pMessage )
			delete pMessage;
	}
	while ( ! this->m_olRawMessages.IsEmpty() )
	{
		CMessage * pMessage = (CMessage *) this->m_olRawMessages.RemoveHead();
		if ( pMessage )
			delete pMessage;
	}
}

void CSequence::Write( CFile * pFile )
{
	POSITION pos = this->m_olMessages.GetHeadPosition();
	while ( pos != NULL )
	{
		CMessage * pMessage = (CMessage *) this->m_olMessages.GetNext( pos );
		if ( pMessage )
			pMessage->Write( pFile );
	}
	CMessage nullMsg;
	nullMsg.Write( pFile );
	pos = this->m_olRawMessages.GetHeadPosition();
	while ( pos != NULL )
	{
		CMessage * pMessage = (CMessage *) this->m_olRawMessages.GetNext( pos );
		if ( pMessage )
			pMessage->Write( pFile );
	}
	nullMsg.Write( pFile );
}

void CSequence::Read( CFile * pFile )
{
	while ( true )
	{
		CMessage * pNew = new ( CMessage );
		pNew->Read( pFile );
		if ( pNew->m_dwMessage != 0 )
		{
			if ( this->m_olMessages.IsEmpty() )
				this->m_olMessages.AddHead( pNew );
			else
				this->m_olMessages.AddTail( pNew );
		}
		else
		{
			delete pNew;
			break;
		}
	}
	while ( true )
	{
		CMessage * pNew = new ( CMessage );
		pNew->Read( pFile );
		if ( pNew->m_dwMessage != 0 )
		{
			if ( this->m_olRawMessages.IsEmpty() )
				this->m_olRawMessages.AddHead( pNew );
			else
				this->m_olRawMessages.AddTail( pNew );
		}
		else
		{
			delete pNew;
			break;
		}
	}

}

CString CSequence::Format()
{
	CString csOut;
	
	POSITION pos = this->m_olMessages.GetHeadPosition();
	while ( pos != NULL )
	{
		CMessage * pMsg = (CMessage *) this->m_olMessages.GetNext( pos );
		if ( pMsg != NULL )
		{
			CString csText;
			switch( pMsg->m_dwMessage )
			{
			case WM_CHAR:
				if ( pMsg->m_dwWParam >= 'A' && pMsg->m_dwWParam <= 'z' )
					csText = (char) pMsg->m_dwWParam;
				else if ( pMsg->m_dwWParam > 0 && pMsg->m_dwWParam < 32 )
					csText.Format("<Ctrl>%c", (char) pMsg->m_dwWParam + 'A' - 1 );
				else if ( pMsg->m_dwWParam == VK_SPACE )
					csText = " ";
				break;
			case WM_SYSCHAR:
				csText.Format("<Alt>%c", (char) pMsg->m_dwWParam );
				break;
			case WM_USER:
				csText.Format("<sleep>%ld", (long) pMsg->m_dwWParam );
			case WM_KEYDOWN:
				if ( pMsg->m_dwWParam >= 0x70 && pMsg->m_dwWParam <= 0x7B )
					csText.Format("<F%ld>", (pMsg->m_dwWParam - 0x70 + 1) );
				break;
			}
			csOut+= csText;
		}
	}
	return csOut;
}

CMacro::CMacro()
{
}

CMacro::~CMacro()
{
	while ( ! this->m_olSequences.IsEmpty() )
	{
		CSequence * pSequence = (CSequence *) this->m_olSequences.RemoveHead();
		if ( pSequence )
			delete pSequence;
	}
}

void CMacro::Write( CFile * pFile )
{
	POSITION pos = this->m_olSequences.GetHeadPosition();
	while ( pos != NULL )
	{
		CSequence * pSequence = (CSequence *) this->m_olSequences.GetNext( pos );
		if ( pSequence )
			pSequence->Write( pFile );
	}
}

void CMacro::Read( CFile * pFile )
{
	while ( true )
	{
		CSequence * pSequence = new ( CSequence );
		pSequence->Read( pFile );
		if ( ! pSequence->m_olMessages.IsEmpty() )
		{
			if ( this->m_olSequences.IsEmpty() )
				this->m_olSequences.AddHead( pSequence );
			else
				this->m_olSequences.AddTail( pSequence );
		}
		else
		{
			delete pSequence;
			break;
		}
	}
}

CAccount::CAccount()
{
	m_iPLevel = 1;
	m_wPrivs = 0;
}

CSkillMenuGroup::CSkillMenuGroup()
{
	m_csaComments.RemoveAll();
	m_csaRestest.RemoveAll();
	m_csaSkillReqs.RemoveAll();
}

CSkillMenuGroup::~CSkillMenuGroup()
{
	POSITION pos = this->m_olChildren.GetHeadPosition();
	while ( pos != NULL )
	{
		CSObject * pObject = (CSObject *) m_olChildren.GetNext(pos);
		CSkillMenuItem * pItem = dynamic_cast <CSkillMenuItem *> (pObject);
		if ( pItem )
		{
			delete pItem;
			continue;
		}
		CSkillMenuGroup * pGroup = dynamic_cast <CSkillMenuGroup *> (pObject);
		if ( pGroup )
		{
			try
			{
				delete pGroup;
			}
			catch(...)
			{
			}
			continue;
		}
		else
			delete pObject;
	}
	m_csaSkillReqs.RemoveAll();
	m_csaRestest.RemoveAll();
	m_csaComments.RemoveAll();
}
const char * SkillMenuTags[] =
{
	"ONOPTION",
	"RESTEST",
	"TEST",
	"SKILLMENU",
};

void CSkillMenuGroup::Read(CStringArray *psaData)
{
	for (int i = 0; i < psaData->GetSize(); i++)
	{
		CString csLine = psaData->GetAt(i);
		if ( csLine == "" )
			continue;
		CString csTag = csLine.SpanExcluding(" =");
		CString csValue;
		if (csTag.GetLength() < csLine.GetLength() )
			csValue = csLine.Mid(csTag.GetLength()+1);
		csValue.TrimLeft();
		csValue.TrimRight();
		switch (FindTable(csTag, SkillMenuTags, 4, sizeof(char*)))
		{
		case 0:	// ONOPTION
			// Parse this into the graphic and the name
			m_csGraphic = csValue.SpanExcluding(" ");
			if ( m_csGraphic.GetLength() < csValue.GetLength() )
				m_csName = csValue.Mid(m_csGraphic.GetLength() + 1);
			break;
		case 1: // RESTEST
			// This could have multiple resources...need to parse it.
			while (csValue != "")
			{
				CString csAmount, csResource, csTemp;
				csTemp = csValue.SpanExcluding(",");
				if ( csTemp.GetLength() < csValue.GetLength() )
					csValue = csValue.Mid(csTemp.GetLength() + 1);
				else
					csValue = "";
				CString csResString;
				if ( csTemp.Find(" ") == -1 )
				{
					csAmount = csTemp;
					csTemp = csValue.SpanExcluding(",");
					if ( csTemp.GetLength() < csValue.GetLength() )
						csValue = csValue.Mid(csTemp.GetLength() + 1);
					else
						csValue = "";
					csResource = csTemp;
					csResString.Format("%s,%s", csAmount, csResource);
				}
				else
					csResString = csTemp;
				m_csaRestest.Add(csResString);
			}
			break;
		case 2:	// TEST
			m_csaSkillReqs.Add(csValue);
			break;
		case 3:	// SKILLMENU
			m_csInternalName.Format("%s %s", m_csSkillGroup, csValue);
			break;
		}
	}
}

void CSkillMenuGroup::Write(CStdioFile *psfOut)
{
	CString csLine;
	csLine.Format("ONOPTION=%s %s\n", m_csGraphic, m_csName);
	psfOut->WriteString(csLine);
	for ( int i = 0; i < m_csaRestest.GetSize(); i++ )
	{
		csLine.Format("RESTEST=%s\n", m_csaRestest.GetAt(i));
		psfOut->WriteString(csLine);
	}
	for ( i = 0; i < m_csaSkillReqs.GetSize(); i++ )
	{
		csLine.Format("TEST %s\n", m_csaSkillReqs.GetAt(i));
		psfOut->WriteString(csLine);
	}
	CString csIndex = m_csInternalName.Mid(m_csInternalName.Find(" ") + 1);
	csLine.Format("SKILLMENU=%s\n\n", csIndex);
	psfOut->WriteString(csLine);
}

void CSkillMenuGroup::WriteMenu(CStdioFile *psfOut)
{
	CString csLine;
	csLine.Format("[%s]\n", m_csInternalName);
	psfOut->WriteString(csLine);
	csLine.Format("%s\n\n", m_csName);
//	for (int i = 0; i < this->m_csaComments.GetSize(); i++)
//	{
//		psfOut->WriteString(m_csaComments.GetAt(i));
//		psfOut->WriteString("\n");
//	}
	psfOut->WriteString(csLine);
	POSITION pos = m_olChildren.GetHeadPosition();
	while ( pos != NULL )
	{
		CSObject * pObject = (CSObject *) m_olChildren.GetNext(pos);
		CSkillMenuGroup * pGroup = dynamic_cast <CSkillMenuGroup *> (pObject);
		if ( pGroup != NULL )
		{
			pGroup->Write(psfOut);
			continue;
		}
		CSkillMenuItem * pItem = dynamic_cast <CSkillMenuItem *> (pObject);
		if ( pItem != NULL )
		{
			pItem->Write(psfOut);
			continue;
		}
	}
	pos = m_olChildren.GetHeadPosition();
	while ( pos != NULL )
	{
		CSObject * pObject = (CSObject *) m_olChildren.GetNext(pos);
		CSkillMenuGroup * pGroup = dynamic_cast <CSkillMenuGroup *> (pObject);
		if ( pGroup != NULL )
			pGroup->WriteMenu(psfOut);
	}
}

CSkillRootMenu::~CSkillRootMenu()
{
	m_oSubmenus.RemoveAll();
}

CSkillMenuItem::CSkillMenuItem()
{
	m_bReplicate = false;
	m_csaComments.RemoveAll();
	m_csaRestest.RemoveAll();
	m_csaSkillReqs.RemoveAll();
}

CSkillMenuItem::~CSkillMenuItem()
{
	m_csaComments.RemoveAll();
	m_csaRestest.RemoveAll();
	m_csaSkillReqs.RemoveAll();
}

const char * SkillItemTags[] =
{
	"ONOPTION",
	"RESOURCE",
	"TEST",
	"MAKEITEM",
	"SUMMON",
	"DRAWMAP",
	"POLY",
	"REPAIR",
	"REPLICATE",
};

void CSkillMenuItem::Read(CStringArray *psaData)
{
	for (int i = 0; i < psaData->GetSize(); i++)
	{
		CString csLine = psaData->GetAt(i);
		if ( csLine == "" )
			continue;
		CString csTag = csLine.SpanExcluding(" =");
		CString csValue;
		if (csTag.GetLength() < csLine.GetLength() )
			csValue = csLine.Mid(csTag.GetLength()+1);
		csValue.TrimLeft();
		csValue.TrimRight();
		switch (FindTable(csTag, SkillItemTags, 9, sizeof(char*)))
		{
		case 0:	// ONOPTION
			// Parse this into the graphic and the name
			m_csGraphic = csValue.SpanExcluding(" ");
			if ( m_csGraphic.GetLength() < csValue.GetLength() )
				m_csName = csValue.Mid(m_csGraphic.GetLength() + 1);
			break;
		case 1: // RESOURCE
			// This could have multiple resources...need to parse it.
			while (csValue != "")
			{
				CString csAmount, csResource, csTemp;
				csTemp = csValue.SpanExcluding(",");
				if ( csTemp.GetLength() < csValue.GetLength() )
					csValue = csValue.Mid(csTemp.GetLength() + 1);
				else
					csValue = "";
				CString csResString;
				if ( csTemp.Find(" ") == -1 )
				{
					csAmount = csTemp;
					csTemp = csValue.SpanExcluding(",");
					if ( csTemp.GetLength() < csValue.GetLength() )
						csValue = csValue.Mid(csTemp.GetLength() + 1);
					else
						csValue = "";
					csResource = csTemp;
					csResString.Format("%s,%s", csAmount, csResource);
				}
				else
					csResString = csTemp;
				m_csaRestest.Add(csResString);
			}
			break;
		case 2:	// TEST
			m_csaSkillReqs.Add(csValue);
			break;
		case 3:	// MAKEITEM
			m_csTarget = csValue;
			m_option_type = OPT_MAKEITEM;
			break;
		case 4:	// SUMMON
			m_csTarget = csValue;
			m_option_type = OPT_SUMMON;
			break;
		case 5: //DRAWMAP
			m_csTarget = csValue;
			m_option_type = OPT_DRAWMAP;
			break;
		case 6:	//POLY
			m_csTarget = csValue;
			m_option_type = OPT_POLY;
			break;
		case 7:	// REPAIR
			m_option_type = OPT_REPAIR;
			break;
		case 8: // REPLICATE
			m_bReplicate = atoi(csValue) ? true : false;
			break;
		}
	}
}

void CSkillMenuItem::Write(CStdioFile *psfOut)
{
	CString csLine;
	csLine.Format("ONOPTION=%s %s\n", m_csGraphic, m_csName);
	psfOut->WriteString(csLine);
	if ( m_option_type == OPT_MAKEITEM && m_bReplicate)
		psfOut->WriteString("REPLICATE=1\n");
	for ( int i = 0; i < m_csaRestest.GetSize(); i++ )
	{
		csLine.Format("RESOURCE=%s\n", m_csaRestest.GetAt(i));
		psfOut->WriteString(csLine);
	}
	for ( i = 0; i < m_csaSkillReqs.GetSize(); i++ )
	{
		csLine.Format("TEST %s\n", m_csaSkillReqs.GetAt(i));
		psfOut->WriteString(csLine);
	}
	switch (m_option_type)
	{
	case OPT_MAKEITEM:
	default:
		csLine.Format("MAKEITEM=%s\n", m_csTarget);
		break;
	case OPT_SUMMON:
		csLine.Format("SUMMON=%s\n", m_csTarget);
		break;
	case OPT_DRAWMAP:
		csLine.Format("DRAWMAP=%s\n", m_csTarget);
		break;
	case OPT_POLY:
		csLine.Format("POLY %s\n", m_csTarget);
		break;
	case OPT_REPAIR:
		csLine.Format("REPAIR\n");
		break;
	}
	psfOut->WriteString(csLine);
	psfOut->WriteString("\n");
}
void WriteFileHeader(CStdioFile * psfOutput, CString csFilename)
{
	psfOutput->WriteString("//****************************************************************************\n");
	psfOutput->WriteString("//SPHERE by : Menasoft 1997-2001\n");
	psfOutput->WriteString("//www.sphereserver.com\n");
	psfOutput->WriteString("// All SPHERE script files and formats are copyright Menasoft & Partners.\n");
	psfOutput->WriteString("// This file may be freely edited for personal use, but may not be distributed\n");
	psfOutput->WriteString("// in whole or in part, in any format without express written permission from\n");
	psfOutput->WriteString("// Menasoft & Partners.  All donations and contributions \n");
	psfOutput->WriteString("// become the property of Menasoft & Partners.\n");
	psfOutput->WriteString("//****************************************************************************\n");
	CString csLine;
	csLine.Format("// %s written by %s\n", csFilename, Main->GetVersionTitle());
	psfOutput->WriteString(csLine);
	CTime time;
	time = CTime::GetCurrentTime();
	csLine.Format("// Last modified ");
	psfOutput->WriteString(csLine);
	csLine = time.Format("%B %d, %Y at %H:%M:%S\n\n");
	psfOutput->WriteString(csLine);
}

CPotion::CPotion()
{
	m_iAmount = 0;
	m_fSkill = 0.0;
}

CWebpageDef::CWebpageDef()
{
	this->m_iUpdateInterval = 30;
	this->m_csDest = "";
	this->m_csSrc = "";
	this->m_csClientList = "<tr><td>%NAME%</td><td>%REGION.NAME%</td></tr>";
	this->m_csServerList = "<tr><td>%URLLINK%</td><td>%STATUS%</td></tr>";
}

CDef::CDef()
{
	m_dwValueMin = 0;
	m_dwValueMax = 0;
	m_bCustom = false;
}

CDef::~CDef()
{
}

void CDef::Read(CString csLine)
{
	CString csTemp;
	csTemp = csLine.SpanExcluding(" \t");
	csTemp.MakeUpper();
	this->m_csValue = csTemp;
	if ( this->m_csValue.GetLength() != csLine.GetLength() )
		csTemp = csLine.Mid(this->m_csValue.GetLength() + 1);
	else
		return;
	csTemp.TrimLeft();
	csTemp.TrimRight();
	if ( csTemp.Find("{") != -1 )
		this->m_expression.Read(csTemp);
	else
	{
		CString csTemp2;
		if ( csTemp.Find(" ") != -1)
			csTemp2 = csTemp.Mid(csTemp.Find(" ") + 1);
		csTemp2.TrimLeft();
		csTemp2.TrimRight();
		if ( csTemp.GetAt(0) == '0' )
			this->m_dwValueMin = ahextoi(csTemp);
		else
			this->m_dwValueMin = atoi(csTemp);
		if ( csTemp2.GetLength() > 0 )
		{
			if ( csTemp2.GetAt(0) == '0' )
				this->m_dwValueMax = ahextoi(csTemp2);
			else
				this->m_dwValueMax = atoi(csTemp2);
		}
		else
			this->m_dwValueMax = this->m_dwValueMin;

	}
}

void CDef::WriteDef(CStdioFile &csfOutput)
{
	CString csLine;
	csLine.Format("%s\t", m_csValue);
	csfOutput.WriteString(csLine);
	if ( m_expression.m_csMember.GetSize() != 0 )
		csLine.Format("%s\n", m_expression.Write() );
	else
	{
		if ( m_dwValueMax == m_dwValueMin )
			csLine.Format("%ld\n", m_dwValueMin);
		else
			csLine.Format("%ld %ld\n", m_dwValueMin, m_dwValueMax);
	}
	csfOutput.WriteString(csLine);
}

DWORD CDef::Evaluate()
{
	if ( m_expression.m_csMember.GetSize() != 0 )
		return m_expression.Evaluate();
	else
	{
		if ( m_dwValueMax == m_dwValueMin )
			return m_dwValueMax;
		else
		{
			srand(time(NULL));
			return ((DWORD) (rand() % (m_dwValueMax - m_dwValueMin) ) + m_dwValueMin);
		}
	}
}

CSObject::CSObject()
{
	this->m_csCategory = "<none>";
	this->m_csSubsection = "<none>";
	this->m_csDescription = "<unnamed>";
	this->m_bNoIndex = false;
}

CSObject::~CSObject()
{
}

void CSObject::Copy(CSObject *pObject)
{
	this->m_csCategory = pObject->m_csCategory;
	this->m_csSubsection = pObject->m_csSubsection;
	this->m_bNoIndex = pObject->m_bNoIndex;
	this->m_bType = pObject->m_bType;
	this->m_csDescription = pObject->m_csDescription;
	this->m_csDupeItem = pObject->m_csDupeItem;
	this->m_csFilename = pObject->m_csFilename;
	this->m_dwFileOffset = pObject->m_dwFileOffset;
	this->m_csValue = pObject->m_csValue;
}

const char * pSObjectTags [] =
{
	"CATEGORY",
	"SUBSECTION",
	"DESCRIPTION",
	"NOINDEX",
	"DUPEITEM",
};

bool CSObject::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	while ( bStatus )
	{
		DWORD dwCur = csfInput.GetPosition();
		CString csLine;
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") == 0 )
		{
			csfInput.Seek((LONG)(dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		if ( csLine.CompareNoCase("NOINDEX") == 0 )
			this->m_bNoIndex = true;
		if ( ( csLine.Find("=") == -1) || (csLine.Find("=") == csLine.GetLength()))
			continue;
		CString csTag = csLine.SpanExcluding("=");
		CString csValue = csLine.Mid(csLine.Find("=") + 1);
		csTag.TrimLeft(); csTag.TrimRight();
		csValue.TrimLeft(); csValue.TrimRight();
		switch (FindTable(csTag, pSObjectTags, 5))
		{
		case 0:
			this->m_csCategory = csValue;
			break;
		case 1:
			this->m_csSubsection = csValue;
			break;
		case 2:
			this->m_csDescription = csValue;
			break;
		case 4:
			this->m_csDupeItem = csValue;
			break;
		default:
			{
				if ( csTag.Find("DEFNAME") == 0 )
				{
					CDef * pDef = new CDef;
					pDef->m_csFilename = this->m_csFilename;
					pDef->m_csValue = csValue;
					pDef->m_expression.Read(this->m_csValue);
					int iOld = Main->m_pScripts->m_aDefs.Find(pDef->m_csValue);
					if ( iOld != -1 )
					{
						CDef * pOld = (CDef *) Main->m_pScripts->m_aDefs.GetAt(iOld);
						Main->m_pScripts->m_aDefs.RemoveAt(iOld);
						delete pOld;
					}
					Main->m_pScripts->m_aDefs.Insert(pDef);
				}
			}
		}
	}
	return bStatus ? true : false;
}

bool CSObject::Write(CStdioFile &csfOutput)
{
	if ( ! CTObject::Write(csfOutput) )
		return false;
	CString csLine;
	if ( this->m_csDupeItem != "" )
	{
		csLine.Format("DUPEITEM=%s\n", this->m_csDupeItem);
		csfOutput.WriteString(csLine);
	}
	if ( ! this->m_bNoIndex )
	{
		if ( this->m_csCategory != "<none>" )
		{
			csLine.Format("CATEGORY=%s\n", this->m_csCategory);
			csfOutput.WriteString(csLine);
		}
		if ( this->m_csSubsection != "<none>" )
		{
			csLine.Format("SUBSECTION=%s\n", this->m_csSubsection);
			csfOutput.WriteString(csLine);
		}
		if ( this->m_csDescription != "<unnamed>" )
		{
			csLine.Format("DESCRIPTION=%s\n", this->m_csDescription);
			csfOutput.WriteString(csLine);
		}
	}
	else
		csfOutput.WriteString("NOINDEX");
	return true;
}

CScriptArray::CScriptArray()
{
}

CScriptArray::~CScriptArray()
{
	this->RemoveAll();
}

int CScriptArray::Insert(CTObject * pScript)
{
	if ( pScript->m_csValue == "" )
	{
		delete pScript;
		return -1;
	}
	int iLower = 0;
	int iUpper = this->GetUpperBound();
	if ( iUpper == -1 )
	{
		this->InsertAt(0, pScript);
		return 0;
	}
	int iIndex = 0;
	int iCompare = 0;
	while ( iLower <= iUpper )
	{
		iIndex = (iUpper + iLower ) / 2;
		CSObject * pTest = (CSObject *) this->GetAt(iIndex);
		if ( pTest->m_csValue.CompareNoCase(pScript->m_csValue) > 0 )
		{
			iCompare = 0;
			iUpper = iIndex - 1;
		}
		else
		{
			iCompare = 1;
			iLower = iIndex + 1;
		}
	}
	iIndex += iCompare;
	this->InsertAt(iIndex, (void *) pScript);
	return iIndex;
}

int CScriptArray::Find(CString csName)
{
	if ( this->GetSize() == 0 )
		return -1;
	int iLower = 0;
	int iUpper = this->GetUpperBound();
	int iIndex = 0;
	while ( iLower <= iUpper )
	{
		iIndex = (iUpper + iLower ) / 2;
		CSObject * pTest = (CSObject *) this->GetAt(iIndex);
		CString csExisting = pTest->m_csValue;
		if ( csExisting.CompareNoCase(csName) == 0 )
			return iIndex;
		if ( csExisting.CompareNoCase(csName) > 0 )
			iUpper = iIndex - 1;
		else
			iLower = iIndex + 1;
	}
	return -1;
}

void CScriptArray::Remove(CTObject * pScript)
{
	int iIndex = this->Find(pScript->m_csValue);
	if ( iIndex != -1 )
	{
		RemoveAt(iIndex);
		delete pScript;
		this->FreeExtra();
	}
}

void CScriptArray::RemoveAll()
{
	/*
	for (int i = 0; i < this->GetSize(); i++)
	{
		CSObject * pScript = (CSObject *) this->GetAt(i);
		if ( pScript )
			delete pScript;
	}
	*/
	CPtrArray::RemoveAll();
}

CTObject * CScriptArray::Get(CString csName)
{
	int iIndex = Find(csName);
	if ( iIndex != -1 )
		return (CTObject *) GetAt(iIndex);
	else
	{
		// Try to look this up as a def
		DWORD dwDef = DefLookup(csName);
		if ( dwDef != 0 )
		{
			CString csDef;
			csDef.Format("%04x", dwDef);
			iIndex = Find(csDef);
			if ( iIndex == -1 )
			{
				csDef.Format("0%04x", dwDef);
				iIndex = Find(csDef);
			}
			if ( iIndex != -1 )
				return (CTObject *) GetAt(iIndex);
		}
	}
	return NULL;

}

CStringBlock::CStringBlock()
{
	this->m_bIgnoreBlanks = false;
	this->m_bIgnoreComments = false;
	this->m_bStringsOnly = false;
	this->m_bWriteStringCount = false;
}

CStringBlock::~CStringBlock()
{
	CStringArray::RemoveAll();
}

bool CStringBlock::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	while ( bStatus )
	{
		CString csLine;
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek( (LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			break;
		}
		if ( m_bIgnoreBlanks && csLine == "" )
			continue;
		if ( m_bIgnoreComments )
		{
			csLine = csLine.SpanExcluding("//");
			if ( csLine == "" )
				continue;
		}
		if ( m_bStringsOnly )
		{
			csLine = csLine.SpanExcluding("0123456789");
			if ( csLine == "" )
				continue;
		}
		this->Add(csLine);
	}
	return bStatus ? true : false;
}

bool CStringBlock::Write(CStdioFile &csfOutput)
{
	CString csLine;
	csLine.Format("[%s]\n", this->m_csID);
	csfOutput.WriteString(csLine);
	if ( m_bWriteStringCount )
	{
		csLine.Format("%ld\n", this->GetSize());
		csfOutput.WriteString(csLine);
	}
	for ( int i = 0; i < this->GetSize(); i++ )
	{
		csLine.Format("%s\n", this->GetAt(i));
		csfOutput.WriteString(csLine);
	}
	if ( m_bIgnoreBlanks )
		csfOutput.WriteString("\n");
	return true;
}

CAnimFrame::CAnimFrame()
{
	this->m_wHeight = 0;
	this->m_wWidth = 0;
	this->m_wImageCenterX = 0;
	this->m_wImageCenterY = 0;
	this->m_waPixels.SetSize(0);
}

CExpression::CExpression()
{
}

CExpression::~CExpression()
{
	this->m_csMember.RemoveAll();
}

void CExpression::Read(CString csExp)
{
	CString csTemp;
	if ( csExp.Find("{") != -1 )
		csTemp = csExp.Mid(csExp.Find("{") + 1);
	else
		csTemp = csExp;
	csTemp.TrimRight(" \t");
	if ( csTemp.ReverseFind('}') != -1 )
		csTemp = csTemp.Left(csTemp.ReverseFind('}'));
	while (csTemp != "")
	{
		csTemp.TrimLeft();
		CString csValue = csTemp.SpanExcluding(" \t");
		if ( csValue != "" )
		{
			this->m_csMember.Add(csValue);
			if ( csTemp.GetLength() > csValue.GetLength() )
			{
				CString csTmp2 = csTemp.Mid(csValue.GetLength() + 1);
				csTemp = csTmp2;
			}
			else
				csTemp = "";
		}
	}
}

CString CExpression::Write()
{
	CString csOutput;
	for ( int i = 0; i < this->m_csMember.GetSize(); i++ )
	{
		if ( !i )
			csOutput.Format("{%s", m_csMember.GetAt(i));
		else
		{
			CString csTemp;
			csTemp.Format("%s %s", csOutput, m_csMember.GetAt(i));
			csOutput = csTemp;
		}
	}
	CString csTemp;
	csTemp.Format("%s}", csOutput);
	csOutput = csTemp;
	return csOutput;
}

int CExpression::m_counter = 0;

DWORD CExpression::Evaluate()
{
	m_counter++;
	if ( m_counter > 100 )
	{
		// To much recursion
		m_counter = 0;
		return 0;
	}
	DWORD dwValue = 0;
	srand(time(NULL));
	// How many values do we have?
	switch ( m_csMember.GetSize() )
	{
	case 0:
		break;
	case 1:
		dwValue = DefLookup(m_csMember.GetAt(0));
		break;
	case 2:
		{
			DWORD dwMin = DefLookup(m_csMember.GetAt(0));
			DWORD dwMax = DefLookup(m_csMember.GetAt(1));
			if ( dwMax != dwMin )
				dwValue = dwMin + (rand() % (dwMax - dwMin));
			else
				dwValue = dwMin;
		}
		break;
	default:
		{
			// This one is a bit tougher
			int iTotalWeight = 0;
			int iState = 0;
			for ( int i = 0; i < m_csMember.GetSize(); i++ )
			{
				switch (iState)
				{
				case 0:
					// First value
					iState++;
					break;
				case 1:
					// Second value of a range, or the weight
					if ( ! DefLookup(m_csMember.GetAt(i)) || DefLookup(m_csMember.GetAt(i)) >= DefLookup(m_csMember.GetAt(i-1)) )
					{
						// Second value
						iState++;
						break;
					}
				case 2:
					iTotalWeight += DefLookup(m_csMember.GetAt(i));
					iState = 0;
					break;
				}
			}
			// Now figure out which value to pick
			if ( iTotalWeight == 0 )
				return 0;
			iTotalWeight = rand() % iTotalWeight + 1;
			// Now loop to that value
			iState = 0;
			int iQty = m_csMember.GetSize();
			for ( i = 0; i < iQty; i++ )
			{
				switch (iState)
				{
				case 0:
					// First value
					iState++;
					break;
				case 1:
					// Second value of a range, or the weight
					if ( ! DefLookup(m_csMember.GetAt(i)) || DefLookup(m_csMember.GetAt(i)) >= DefLookup(m_csMember.GetAt(i-1)) )
					{
						// Second value
						iState++;
						break;
					}
				case 2:
					iTotalWeight -= DefLookup(m_csMember.GetAt(i));
					if ( iTotalWeight > 0 )
						iState = 0;
					break;
				}
				if ( iTotalWeight <= 0 )
					break;
			}
			if ( iState == 0 )
				dwValue = DefLookup(m_csMember.GetAt(0));
			else if ( iState == 1 )
				dwValue = DefLookup(m_csMember.GetAt(i-1));
			else
			{
				DWORD dwRand = DefLookup(m_csMember.GetAt(i-1)) - DefLookup(m_csMember.GetAt(i-2));
				if ( dwRand != 0 )
					dwValue = DefLookup(m_csMember.GetAt(i-2)) + (rand() % dwRand );
				else
					dwValue = DefLookup(m_csMember.GetAt(i-2));
			}
		}
	}
	m_counter = 0;
	return dwValue;
}

bool CTrigger::Read(CStdioFile &csfIn)
{
	BOOL bStatus = TRUE;
	while ( bStatus )
	{
		DWORD dwCur = csfIn.GetPosition();
		CString csLine;
		bStatus = csfIn.ReadString(csLine);
		if ( !bStatus )
			break;
		if (csLine.Find("[") == 0)
		{
			csfIn.Seek((LONG)(csfIn.GetPosition() - dwCur), CFile::current);
			break;
		}
		this->m_csaEvents.Add(csLine);
	}
	return bStatus ? true : false;
}



bool CWorldItem::Read(CStdioFile &csfInput)
{
	LoadBase();
	static const char * szWorldItemKeys [] =
	{
		"SERIAL",
		"NAME",
		"COLOR",
		"TIMER",

		"DISPID",
		"AMOUNT",
		"TYPE",
		"LINK",
		"ATTR",
		"MORE1",
		"MORE2",
		"MOREP",
		"LAYER",
		"CONT",
		"P",

		"REGION.FLAGS",
		"AGE",
	};

	BOOL bStatus = TRUE;
	while ( bStatus )
	{
		DWORD dwStart = csfInput.GetPosition();
		CString csLine;
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") == 0 )
		{
			csfInput.Seek((LONG) dwStart, CFile::begin);
			break;
		}
		CString csKey, csValue;
		csKey = csLine.SpanExcluding("=");
		if ( csLine.Find("=") != -1 )
			csValue = csLine.Mid(csLine.Find("=") + 1);
		if ( csKey == "" )
			continue;
		switch( FindTable(csKey, &szWorldItemKeys[0], 15) )
		{
		case 0:
			m_dwSerial = (DWORD) ahextoi(csValue); break;
		case 1:
			m_csName = csValue; break;
		case 2:
			m_wColor = (WORD) ahextoi(csValue); break;
		case 3:
			m_dwTimer = (DWORD) ahextoi(csValue); break;
		case 4:
			m_wArtID = (WORD) ahextoi(csValue); break;
		case 5:
			m_wAmount = (WORD) atoi(csValue); break;
		case 6:
			m_dwItemType = (DWORD) DefLookup(csValue); break;
		case 7:
			m_dwLink = (DWORD) ahextoi(csValue); break;
		case 8:
			m_dwAttr = (DWORD) ahextoi(csValue); break;
		case 9:
			m_dwMore1 = (DWORD) ahextoi(csValue); break;
		case 10:
			m_dwMore2 = (DWORD) ahextoi(csValue); break;
		case 11:
			// Need to parse this data out into the X, Y, and Z components.
			{
				CString csTemp = csValue;
				CString csComponent = csTemp.SpanExcluding(",");
				if ( csComponent.GetAt(0) == '0' )
					m_wMoreX = (WORD) ahextoi(csComponent);
				else
					m_wMoreX = (WORD) atoi(csComponent);
				if ( csTemp.Find(",") != -1 )
				{
					csValue = csTemp.Mid(csComponent.GetLength() + 1);
					csTemp = csValue;
					csComponent = csTemp.SpanExcluding(",");
					if ( csComponent.GetAt(0) == '0' )
						m_wMoreY = (WORD) ahextoi(csComponent);
					else
						m_wMoreY = (WORD) atoi(csComponent);
					if ( csTemp.Find(",") != -1 )
					{
						csValue = csTemp.Mid(csComponent.GetLength() + 1);
						csTemp = csValue;
						csComponent = csTemp.SpanExcluding(",");
						if ( csComponent.GetAt(0) == '0' )
							m_bMoreZ = (BYTE) ahextoi(csComponent);
						else
							m_bMoreZ = (BYTE) atoi(csComponent);
					}
				}
			}
		case 12:
			m_bLayer = (BYTE) atoi(csValue); break;
		case 13:
			m_dwContainer = (DWORD) ahextoi(csValue); break;
		case 14:
			// Need to parse this data out into the X, Y, and Z components.
			{
				CString csTemp = csValue;
				CString csComponent = csTemp.SpanExcluding(",");
				if ( csComponent.GetAt(0) == '0' )
					m_pX = (WORD) ahextoi(csComponent);
				else
					m_pX = (WORD) atoi(csComponent);
				if ( csTemp.Find(",") != -1 )
				{
					csValue = csTemp.Mid(csComponent.GetLength() + 1);
					csTemp = csValue;
					csComponent = csTemp.SpanExcluding(",");
					if ( csComponent.GetAt(0) == '0' )
						m_pY = (WORD) ahextoi(csComponent);
					else
						m_pY = (WORD) atoi(csComponent);
					if ( csTemp.Find(",") != -1 )
					{
						csValue = csTemp.Mid(csComponent.GetLength() + 1);
						csTemp = csValue;
						csComponent = csTemp.SpanExcluding(",");
						if ( csComponent.GetAt(0) == '0' )
							m_pZ = (BYTE) ahextoi(csComponent);
						else
							m_pZ = (BYTE) atoi(csComponent);
					}
				}
			}
		case 15:
			m_wRegionFlags = (WORD) ahextoi(csValue); break;
		case 16:
			m_dwAge = ahextoi(csValue); break;
		default:
			// Main->m_log.Add("WARNING: Unhandled key \'%s\' in file %s", csKey, csfInput.GetFilePath() );
			break;
		}
	}
	return true;
}

CWorldItem::CWorldItem()
{
	m_bStaticFlag = 0;
	m_csBaseID = _T("");
	m_wArtID = 0;
	m_dwSerial = 0;
	m_wColor = 0;
	m_dwLink = 0;
	m_dwMore1 = 0;
	m_dwMore2 = 0;
	m_wMoreX = 0;
	m_wMoreY = 0;
	m_bMoreZ = 0;
	m_bLayer = 0;
	m_dwContainer = 0;
	m_dwAttr = 0;
	m_wAmount = 0;
	m_pX = 0;
	m_pY = 0;
	m_pZ = 0;
	m_csName = _T("");
	m_dwTimer = 0;
	m_dwItemType = 0;
	m_wRegionFlags = 0;
	m_dwAge = 0;
}

bool CWorldItem::LoadBase()
{
	int idx = Main->m_pScripts->m_aItems.Find(this->m_csBaseID);
	if ( idx == -1 )
	{
		DWORD dwDef = DefLookup(m_csBaseID);
		if ( dwDef != 0 )
		{
			CString csDef;
			csDef.Format("0%x", dwDef);
			idx = Main->m_pScripts->m_aItems.Find(csDef);
		}
		if ( idx == -1 )
		{
			Main->m_log.Add("Unable to find base item script for worlditem %s", m_csBaseID);
			return false;
		}
	}
	CSObject * pObject = (CSObject *) Main->m_pScripts->m_aItems.GetAt(idx);
	if ( !pObject )
		return false;
	CItem base;
	base.CopyBase(pObject);
	base.Load();

	this->m_dwItemType = base.m_dwType;
	this->m_dwMore1 = base.m_dwMore1;
	this->m_dwMore2 = base.m_dwMore2;
	this->m_dwAttr = (DWORD) base.m_wAttr;
	// this->m_csName = base.m_csItemName;  -- Don't do this...we should use the tiledata for the name unless the name is set in the worldfile 

	return true;
}

CUOStaticItem::CUOStaticItem()
{
	m_dwBlockIndex = 0;
	memset(&m_blockData, 0x00, sizeof(tUOStaticRec));
	m_bFlag = 0;
}

bool CUOStaticItem::Read(CFile &cfInput)
{
	DWORD dwRead = cfInput.Read(&m_blockData, sizeof(tUOStaticRec));
	return ( dwRead == sizeof(tUOStaticRec) ? true : false );
}

DWORD CWorldItem::GetMapBlock()
{
	int XBlock, YBlock;
	XBlock = m_pX / 8;
	YBlock = m_pY / 8;
	return ( XBlock * 512 + YBlock );
}

BYTE CWorldItem::GetBlockX()
{
	return ( m_pX % 8 );
}

BYTE CWorldItem::GetBlockY()
{
	return ( m_pY % 8 );
}

int parseRange(const char * pszStr, bool bMinMax)
{
	CString csVal(pszStr);
	csVal.Replace(".", "");
	csVal.Replace("{", "");
	csVal.Replace("}", "");
	csVal.TrimLeft();
	csVal.TrimRight();

	int iSign = csVal.Find("-");
	CString csLeftStr;
	if (iSign == 0)
	{
		CString csTmp;
		csTmp = csVal.Right(csVal.GetLength() - 1);
		csLeftStr.Format("-%s", csTmp.SpanIncluding("0123456789"));
	}
	else
		csLeftStr = csVal.SpanIncluding("0123456789");
	if ( csVal.GetLength() > csLeftStr.GetLength() )
		csVal = csVal.Mid(csLeftStr.GetLength() + 1);
	else
		csVal = "";
	// Check to see if we have a delimiter here
	if ( csVal.GetLength() > 1 && csVal.GetAt(0) == '-' && csVal.GetAt(1) == ' ' )
		// it's just a delimeter.
		csVal = csVal.Mid(1);
	csVal.TrimLeft();
	csVal.TrimRight();
	if (bMinMax == 0 || (csVal == ""))
		return (atoi(csLeftStr));
	else
		return (atoi(csVal));
}

float fParseRange(const char * pszStr, bool bMinMax)
{
	CString csVal(pszStr);
	csVal.Replace("{", "");
	csVal.Replace("}", "");
	csVal.TrimLeft();
	csVal.TrimRight();

	// if bMinMax = 0, we are looking for the minimum value
	CString csLeftStr = csVal.SpanIncluding("0123456789.");
	if ( csVal.GetLength() > csLeftStr.GetLength() )
		csVal = csVal.Mid(csLeftStr.GetLength() + 1);
	else
		csVal = "";
	csVal.Replace("-", "");
	csVal.TrimLeft();
	csVal.TrimRight();
	if (bMinMax == 0)
		return (float) (atof(csLeftStr));
	else
		return (float) (atof(csVal));
}

CCategory * FindCategory(CPtrList * pPtrList, CString csName, bool bCreate)
{
	CCategory * pCategory = NULL;
	if (!pPtrList->IsEmpty())
	{
		POSITION pos = pPtrList->GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pTest = (CCategory *) pPtrList->GetNext(pos);
			if (pTest->m_csName == csName)
			{
				pCategory = pTest;
				break;
			}
		}
	}
	if (pCategory == NULL && bCreate == true)
	{
		// Category didn't exist.  Create a new one
		pCategory = new (CCategory);
		pCategory->m_csName = csName;
		if (pPtrList->IsEmpty())
			pPtrList->AddHead(pCategory);
		else
			pPtrList->AddTail(pCategory);
	}
	return pCategory;
}

CSubsection * FindSubsection(CCategory * pCategory, CString csName, bool bCreate)
{
	CSubsection * pSubsection = NULL;
	if (!pCategory->m_SubsectionList.IsEmpty())
	{
		POSITION pos = pCategory->m_SubsectionList.GetHeadPosition();
		while (pos != NULL)
		{
			CSubsection * pTest = (CSubsection *) pCategory->m_SubsectionList.GetNext(pos);
			if (pTest->m_csName == csName)
			{
				pSubsection = pTest;
				break;
			}
		}
	}
	if (pSubsection == NULL && bCreate == true)
	{
		// Subsection didn't exist.  Create a new one
		pSubsection = new (CSubsection);
		pSubsection->m_csName = csName;
		if (pCategory->m_SubsectionList.IsEmpty())
			pCategory->m_SubsectionList.AddHead(pSubsection);
		else
			pCategory->m_SubsectionList.AddTail(pSubsection);
	}
	return pSubsection;
}

CSObject * FindItem(CSubsection * pSubsection, CString csName, bool bCreate)
{
	CItem * pItem = NULL;
	if (!pSubsection->m_ItemList.IsEmpty())
	{
		POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
		while (pos != NULL)
		{
			CItem * pTest = (CItem *) pSubsection->m_ItemList.GetNext(pos);
			if (pTest->m_csDescription == csName)
			{
				pItem = pTest;
				break;
			}
		}
	}
	if (pItem == NULL && bCreate == true)
	{
		// Item didn't exist.  Create a new one
		pItem = new (CItem);
		pItem->m_csDescription = csName;
		if (pSubsection->m_ItemList.IsEmpty())
			pSubsection->m_ItemList.AddHead(pItem);
		else
			pSubsection->m_ItemList.AddTail(pItem);
	}
	return pItem;
}

int FindTable( const char * pFind, const char * const * ppTable, int iCount, int iElemSize )
{
	for ( int i=0; i<iCount; i++ )
	{
		if ( ! strcmpi( *ppTable, pFind ))
			return( i );
		ppTable = (const char * const *)((( const BYTE*) ppTable ) + iElemSize );
	}
	return( -1 );
}

DWORD ahextoi( const char * pszStr ) // Convert hex string to integer
{
	// Unfortunatly the library func cant handle the number FFFFFFFF
	// char * sstop; return( strtol( s, &sstop, 16 ));

	GETNONWHITESPACE( pszStr );

	DWORD val = 0;
	while ( 1 )
	{
		char ch = *pszStr;
		if ( ch >= '0' && ch <= '9' )
			ch -= '0';
		else
		{
			ch |= 0x20;	// toupper()
			if ( ch > 'f' || ch <'a' ) break;
			ch -= 'a' - 10;
		}
		val *= 0x10;
		val += ch;
		pszStr ++;
	}
	return( val );
}

DWORD DefLookup(CString csValue)
{
	if ( csValue == "" )
		return 0;
	if ( !isalpha(csValue.GetAt(0)) )
	{
		// This isn't really a def.
		if ( csValue.GetAt(0) == '0' )
			return (DWORD) ahextoi(csValue);
		else
			return (DWORD) atoi(csValue);
	}
	csValue.MakeUpper();

	int iIndex = Main->m_pScripts->m_aDefs.Find(csValue);
	if ( iIndex != -1 )
	{
		CDef * pDef = (CDef *) Main->m_pScripts->m_aDefs.GetAt(iIndex);
		if ( pDef )
			return pDef->Evaluate();
	}
	return 0;
}

CString NumToDef( DWORD dwID )
{
	CString csOutput;
	// Try to open the Defs file
	CStdioFile csfDefs;
	CString csScpPath = GetScriptPath();
	CString csFile;
	if ( csScpPath == "" )
		csFile = DEFS_FILE;
	else
		csFile.Format("%s%s", csScpPath, DEFS_FILE);
	if ( csfDefs.Open( csFile, CFile::modeRead | CFile::shareDenyNone ) )
	{
		CString csLine;
		BOOL bStatus = true;
		while ( bStatus )
		{
			bStatus = csfDefs.ReadString( csLine );
			if ( ! bStatus )
				break;
			csLine.TrimLeft(); csLine.TrimRight();
			csLine = csLine.SpanExcluding( "//" );
			if ( csLine != "" )
			{
				CString csDef = csLine.SpanExcluding(" \t");
				CString csValue = csLine.Mid( csDef.GetLength() + 1 );
				csDef.TrimRight();
				csValue.TrimLeft(); csValue.TrimRight();
				DWORD dwValue = atoi(csValue);
				if ( dwValue == dwID )
				{
					csOutput = csDef;
					csfDefs.Close();
					return csOutput;
				}
			}
		}
		csfDefs.Close();
	}
	if ( csOutput == "" )
		csOutput.Format( "%05x", dwID );
	return csOutput;
}

CString ItemLookup(CPtrList * pCategoryList, DWORD dwID)
{
	// Return the name of the item with the given ID within the entire category list
	CCategory * pCategory = NULL;
	if (!pCategoryList->IsEmpty())
	{
		POSITION CategoryPos = pCategoryList->GetHeadPosition();
		while (CategoryPos != NULL)
		{
			CCategory * pCategory = (CCategory *) pCategoryList->GetNext(CategoryPos);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION SubsectionPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (SubsectionPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(SubsectionPos);
					if (!pSubsection->m_ItemList.IsEmpty())
					{
						POSITION ItemPos = pSubsection->m_ItemList.GetHeadPosition();
						while (ItemPos != NULL)
						{
							CItem * pItem = (CItem *) pSubsection->m_ItemList.GetNext(ItemPos);
							if (ahextoi(pItem->m_csValue) == dwID)
								return pItem->m_csDescription;
						}
					}
				}
			}
		}
	}
	return("");
}

CString NPCLookup(CPtrList * pCategoryList, DWORD dwID)
{
	// Return the name of the item with the given ID within the entire category list
	CCategory * pCategory = NULL;
	if (!pCategoryList->IsEmpty())
	{
		POSITION CategoryPos = pCategoryList->GetHeadPosition();
		while (CategoryPos != NULL)
		{
			CCategory * pCategory = (CCategory *) pCategoryList->GetNext(CategoryPos);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION SubsectionPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (SubsectionPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(SubsectionPos);
					if (!pSubsection->m_ItemList.IsEmpty())
					{
						POSITION ItemPos = pSubsection->m_ItemList.GetHeadPosition();
						while (ItemPos != NULL)
						{
							CNPC * pNPC = (CNPC *) pSubsection->m_ItemList.GetNext(ItemPos);
							if (ahextoi(pNPC->m_csValue) == dwID)
								return pNPC->m_csDescription;
						}
					}
				}
			}
		}
	}
	return("");
}

bool UpdateFiles(CString csNewFile, CString csOldFile, CString csRemoteFile, CString csRemoteKey)
{
	bool bStatus = false;
#ifdef _AXIS
	if ( Main->GetProfileType() == PROFILE_REMOTE )
	{
		bStatus = ReplaceFiles( csNewFile, csRemoteFile );
		if ( bStatus )
		{
			CRemoteConnection rc;
			if ( rc.IsValid() )
				rc.SendCustomFile( csRemoteKey, csRemoteFile );
		}
	}
	else
	{
		CRemoteConsole * pConsole = Main->m_pRConsole;
		bool bDeleteConsole = false;
		if (pConsole == NULL)
		{
			pConsole = new (CRemoteConsole);
			bDeleteConsole = true;
		}
		pConsole->Pause();
		int i = 0;
		while (pConsole->m_bCommandPending)
		{
			i++;
			Sleep(50);
			if (i > 100)
				break;
		}
		bStatus = ReplaceFiles( csNewFile, csOldFile );
		pConsole->Resync();
		i = 0;
		while (pConsole->m_bCommandPending)
		{
			i++;
			Sleep(50);
			if (i > 100)
				break;
		}
		if (bDeleteConsole)
			delete pConsole;
	}
#endif
	return bStatus;
}

bool IsDef(CString csValue)
{
	CString csTemp = csValue.SpanIncluding("0123456789abcdefABCDEF");
	if ( csTemp == csValue )
		return false;
	else
		return true;
}

CString GetScriptPath()
{
	CStdioFile csfIni;
	CString csPath;
	char szPath[MAX_PATH];
	memset(szPath, 0x00, sizeof(szPath));
	if ( Main->m_csRootDirectory == "" )
		getcwd(szPath, sizeof(szPath));
	else
		sprintf(szPath, "%s", Main->m_csRootDirectory);
	strcat(szPath, "\\");
	csPath = _T(szPath);
	CString csIni = csPath + _T(INI_FILE);
	BOOL bStatus = csfIni.Open(csIni, CFile::modeRead | CFile::shareDenyNone );
	if ( !bStatus )
		return csPath;
	while ( bStatus )
	{
		CString csLine;
		bStatus = csfIni.ReadString(csLine);
		if ( ! bStatus )
			break;
		if ( csLine == "" )
			continue;
		csLine.MakeLower();
		if ( csLine.Find("scpfiles") == 0 )
		{
			csPath = csLine.Mid(9);
			csPath.TrimRight();
			break;
		}
	}
	csfIni.Close();
	if ( csPath.Right(1) != "\\" )
		csPath += "\\";
	return csPath;
}

DWORD ScaleColor(WORD wColor)
{
	DWORD dwNewColor;
	
	dwNewColor = ((((((wColor >> 10) & 0x01f) * 0x0ff / 0x01f))
		| (((((wColor >> 5) & 0x01f) * 0x0ff / 0x01f)) << 8)
		| ((((wColor & 0x01f) * 0x0ff / 0x01f)) << 16)));
	return (dwNewColor);
}

bool ReplaceFiles( CString csNewFile, CString csOldFile )
{
	// Most likely, the old file name doesn't have the path associated with it.
	if (csOldFile.Find(":") == -1 && csOldFile.Find("ini") == -1 && csOldFile.Find("custloc.scp") == -1)
	{
		CString csPath = GetScriptPath();
		if ( csPath != "" )
		{
			CString csTemp = csOldFile;
			csOldFile.Format("%s%s", csPath, csTemp);
		}
	}
	// Loop through this a few times
	int i = 0;
	bool bSuccess = false;
	while ( i < 5 && !bSuccess )
	try
	{
		CFileStatus st;
		if ( CFile::GetStatus( csOldFile, st ) )
			CFile::Remove( csOldFile );
		CFile::Rename( csNewFile, csOldFile );
		bSuccess = true;
	}
	catch ( CFileException * e )
	{
		i++;
		if ( i < 5 )
		{
			Sleep( 500 );
			e->Delete();
		}
		else
		{
			CString csMessage;
			char szCause[MAX_BUFFER];
			memset(szCause, 0x00, sizeof(szCause));
			e->GetErrorMessage( &szCause[0], MAX_BUFFER - 1 );
			csMessage.Format("An exception occured while moving files.\n\n%s\n\nThis typically is caused by the Sphere server having the file locked open.\nAxis tries to pause the Sphere server before doing this, but failed.\n", szCause);
			csMessage+= "Make sure Axis is able to connect to the Sphere server with the Remote Console\nor pause the Sphere server manually before attempting a save.\n";
			csMessage+= "\n\nAxis can attempt to rename the existing file, and move in the new file.\nWould you like to try this?";
			e->Delete();
			if ( AfxMessageBox( csMessage, MB_YESNO | MB_ICONQUESTION ) == IDYES )
			{
				CString csBackup;
				CTime ct = CTime::GetCurrentTime();
				csBackup.Format("%s.%s", csOldFile, ct.Format("%Y%m%d%H%M%S"));
				try
				{
					CFile::Rename( csOldFile, csBackup );
					CFile::Rename( csNewFile, csOldFile );
					CString csMessage;
					csMessage.Format("Rename was successful.  Tne backup file is now stored in\n%s\n\nYou may need to manually pause and resync your\nserver for the changes to take effect.", csBackup);
					AfxMessageBox(csMessage, MB_OK | MB_ICONINFORMATION);
					return true;
				}
				catch ( CFileException * e )
				{
					e->Delete();
					AfxMessageBox("Rename failed.", MB_OK | MB_ICONSTOP );
					return false;
				}
			}
			return false;
		}
	}
	return true;
}
