/*
 $Id: ntservice.cpp,v 1.3 2002/05/01 03:33:11 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// ntservice.cpp

#ifdef WIN32

#include "ntservice.h"
#include "messages.h"

// The service is a global function
CService * pService = NULL;	// there can be only one instance of this per application

void CService::Install()
{
	char szPath[_MAX_PATH * 2];
	char szErr[256];

	// Try to determine the name and path of this application.
	if (GetModuleFileName(NULL, szPath, sizeof(szPath)) == 0)
	{
		printf("Unable to install %s as a service - %s\n", m_szTitle, GetLastErrorText(szErr, sizeof(szErr)));
		return;
	}

	// Try to open the Service Control Manager
	SC_HANDLE schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if ( ! schSCManager)
	{
		printf("OpenSCManager() failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
		return;
	}

	// Try to create the service

	SC_HANDLE schService = CreateService(
		schSCManager,					// handle of the Service Control Manager
		m_szTitle,				// Internal name of the service (used when controlling the service using "net start" or "netsvc")
		m_szTitle,			// Display name of the service (displayed in the Control Panel | Services page)
		SERVICE_ALL_ACCESS,
		SERVICE_WIN32_OWN_PROCESS, 
		SERVICE_AUTO_START,				// Start automatically when the OS starts 
		SERVICE_ERROR_NORMAL,
		szPath,							// Path and filename of this executable
		NULL, 
		NULL, 
		NULL, 
		NULL, 
		NULL
	);
	if ( ! schService )
	{
		printf("CreateService() failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
bailout1:
		CloseServiceHandle(schSCManager);
		return;
	}

	// Register the application for event logging
	HKEY hk;
	DWORD dwData;
	// Try to create the registry key containing information about this application
	char szKey[MAX_PATH];
	sprintf(szKey, "System\\CurrentControlSet\\Services\\EventLog\\Application\\%s", m_szServiceName);
	if (RegCreateKey(HKEY_LOCAL_MACHINE, szKey, &hk))
		printf("RegCreateKey() for Event Log failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
	else
	{
		// Try to create the registry key containing the name of the EventMessageFile
		//  Replace the name of the exe with the name of the dll in the szPath variable
		if (RegSetValueEx(hk, "EventMessageFile", 0, REG_EXPAND_SZ, (LPBYTE) szPath, strlen(szPath) + 1))
			printf("RegSetValueEx() for \"EventMessageFile\" failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
		else
		{
			// Try to create the registry key containing the types of errors this application will generate
			dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_INFORMATION_TYPE | EVENTLOG_WARNING_TYPE;
			if (RegSetValueEx(hk, "TypesSupported", 0, REG_DWORD, (LPBYTE) &dwData, sizeof(DWORD)))
				printf("RegSetValueEx() for \"TypesSupported\" failed -%s\n", GetLastErrorText(szErr, sizeof(szErr)));
		}
		// Close the Registry Key handle
		RegCloseKey(hk);
	}

	// Set the working path for the application
	sprintf(szKey,  "System\\CurrentControlSet\\Services\\%s\\Parameters", m_szTitle);
	if ( RegCreateKey(HKEY_LOCAL_MACHINE, szKey, &hk))
	{
		printf("RegCreateKey() for \"Parameters\" failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
bailout2:
		CloseServiceHandle(schService);
		goto bailout1;
	}

	memset(&szPath[0], 0x00, sizeof(szPath));
	GetModuleFileName(NULL, &szPath[0], sizeof(szPath));
	for (int i = strlen(szPath); i > 0; i-- )
	{
		if ( szPath[i] == '\\' )
		{
			szPath[i] = 0x00;
			break;
		}
	}

	if (RegSetValueEx(hk, "WorkingPath", 0, REG_SZ, (const unsigned char *) &szPath[0], strlen(szPath)))
		printf("RegSetValueEx() for \"WorkingPath\" failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));

	printf( "%s installed for server %s\n", m_szServiceName, m_szServerName);

	goto bailout2;

}

void CService::Remove()
{
	SC_HANDLE schService;

	char szErr[256];

	// Try to open the Service Control Manager
	SC_HANDLE schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (schSCManager)
	{
		// Try to obtain the handle of this service
		schService = OpenService(schSCManager, m_szTitle, SERVICE_ALL_ACCESS);
		if (schService)
		{
			// Check to see if the service is started, if so try to stop it.
			if (ControlService(schService, SERVICE_CONTROL_STOP, &m_ssStatus))
			{
				if (!m_bDebug)
				{
					AddToMessageLog(m_szTitle, NULL, APP_SERVICE_STOPPING, EVENTLOG_INFORMATION_TYPE);
				}
				printf("Stopping %s.", m_szServiceName);
				Sleep(1000);
				// wait for the service to stop.
				while (QueryServiceStatus(schService, &m_ssStatus))
				{
					if (GetState() == SERVICE_STOP_PENDING)
					{
						printf(".");
						Sleep(1000);
					}
					else
						break;
				}
				if (GetState() == SERVICE_STOPPED)
					printf("\n%s has stopped.\n", m_szServiceName);
				else
					printf("\n%s failed to stop.\n", m_szServiceName);
			}

			// Remove the service
			if (DeleteService(schService))
				printf( "%s has been removed.\n", m_szTitle);
			else
				printf( "DeleteService() failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
			// Close the handle to this service
			CloseServiceHandle(schService);
		}
		else
			printf("OpenService() failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
		// Close the SCM handle
		CloseServiceHandle(schSCManager);
	}
	else
		printf("OpenSCManager() failed - %s\n", GetLastErrorText(szErr, sizeof(szErr)));
}

int CService::Debug(int argc, char **argv)
{
	this->m_bDebug = true;
	printf("Debugging %s.\n", m_szTitle);
	SetConsoleCtrlHandler( ControlHandler, TRUE );
	return Start( argc, argv );
}

void CService::StartMain(DWORD dwArgc, LPTSTR * lpszArgv)
{
	SetState(SERVICE_START_PENDING);
	m_sshStatusHandle = RegisterServiceCtrlHandler( m_szTitle, ServiceCtrl );
	if ( m_sshStatusHandle == NULL )
	{
		if ( m_bDebug )
			printf("Unable to Register Control Handler.  Terminating.\n");
		else
			AddToMessageLog(m_szTitle, "Unable to Register Control Handler. Terminating.", APP_SERVICE_START_FAILED, EVENTLOG_ERROR_TYPE );
		return;
	}
	ReportStatus();
	int rc = Start(dwArgc, lpszArgv);
	m_ssStatus.dwCurrentState = SERVICE_STOPPED;
	m_ssStatus.dwWin32ExitCode = rc;
	ReportStatus();
}

int CService::Start(DWORD dwArgc, LPTSTR * lpszArgv)
{
	int rc = 0;
	char szMsg[MAX_BUFFER];
	HANDLE hEvents[2] = { NULL, NULL };
	PSECURITY_DESCRIPTOR pSD = NULL;
	HANDLE hPipe;

	if ( this->m_bDebug )
		printf("Service Start pending...");
	else
	{
		AddToMessageLog(m_szTitle, NULL, APP_SERVICE_STARTING, EVENTLOG_INFORMATION_TYPE );
	}

	this->ReportStatus();
	
	this->m_hStopEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
	
	ReportStatus();
	// Create Event Objects;
	hEvents[0] = m_hStopEvent;
	hEvents[1] = CreateEvent( NULL, TRUE, FALSE, NULL );
	if ( hEvents[1]  == NULL )
	{
		rc = -1;
		goto bailout;
	}

	ReportStatus();
	// Create Security Descriptor
	pSD = (PSECURITY_DESCRIPTOR) malloc ( SECURITY_DESCRIPTOR_MIN_LENGTH );
	if ( pSD == NULL )
	{
		rc = -2;
		goto bailout;
	}
	if ( ! InitializeSecurityDescriptor( pSD, SECURITY_DESCRIPTOR_REVISION ) )
	{
		rc = -3;
		goto bailout;
	}
	// Add a NULL descriptor ACL to the security descriptor
	if ( ! SetSecurityDescriptorDacl( pSD, TRUE, (PACL) NULL, FALSE) )
	{
		rc = -3;
		goto bailout;
	}
	SECURITY_ATTRIBUTES sa;
	sa.nLength = sizeof(sa);
	sa.lpSecurityDescriptor = pSD;
	sa.bInheritHandle = TRUE;
	ReportStatus();
	// Create a named pipe
	char lpszPipeName[MAX_PATH];
	sprintf(lpszPipeName, "\\\\.\\pipe\\%s", this->m_szTitle);
	hPipe = CreateNamedPipe(lpszPipeName, FILE_FLAG_OVERLAPPED | PIPE_ACCESS_DUPLEX, PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT, 1, 0, 0,  1000, &sa);
	if ( hPipe == INVALID_HANDLE_VALUE )
	{
		if ( this->m_bDebug )
		{
			printf("%s could not create a named pipe.\n", this->m_szDisplayName );
			printf("This is usually due to another instance of this program running\non this machine.\nTerminating\n.");
		}
		else
		{
			char szErr[MAX_BUFFER];
			this->AddToMessageLog(m_szTitle, GetLastErrorText(szErr, sizeof(szErr)), APP_SERVICE_CREATEPIPE_FAILED, EVENTLOG_ERROR_TYPE );
		}
		rc = -4;
		goto bailout;
	}
	ReportStatus();
	// Set the working path
	HKEY hKey;
	char szKey[MAX_PATH];
	sprintf(szKey, "System\\CurrentControlSet\\Services\\%s\\Parameters", this->m_szTitle );
	char szGlobalPath[MAX_PATH];
	memset(&szGlobalPath[0], 0x00, sizeof(szGlobalPath));
	if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, szKey, 0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )
	{
		unsigned long lType;
		unsigned long lSize;
		RegQueryValueEx( hKey, "WorkingPath", 0, &lType, (unsigned char *) &szGlobalPath[0], &lSize);

	}
	ReportStatus();
	if ( szGlobalPath[0] = 0x00 )
	{
		GetModuleFileName( NULL, &szGlobalPath[0], sizeof(szGlobalPath) );
		for ( int i = strlen(szGlobalPath); i > 0; i-- )
		{
			if ( szGlobalPath[i] == '\\' )
			{
				szGlobalPath[i] = 0x00;
				break;
			}
		}
	}
	_chdir( szGlobalPath );
	// Now we are going to call the main function....
	SetState(SERVICE_RUNNING);
	ReportStatus();
	if ( m_bDebug )
		printf("started.\n");
	else
	{
		AddToMessageLog( m_szTitle, NULL, APP_SERVICE_STARTED, EVENTLOG_INFORMATION_TYPE );
	}
	rc = service_main( dwArgc, lpszArgv );
	// When this function returns, the service has stopped.
	sprintf(szMsg, "The exit code is %ld", rc );
	if ( this->m_bDebug )
		printf("%s\n", szMsg);
	else
	{
		if ( rc == 0 )
			AddToMessageLog( m_szTitle, NULL, APP_SERVICE_STOPPED, EVENTLOG_INFORMATION_TYPE );
		else
			AddToMessageLog( m_szTitle, szMsg, APP_SERVICE_TERMINATED, EVENTLOG_ERROR_TYPE );
	}
	free(pSD);
	CloseHandle(hEvents[1]);
	CloseHandle(m_hStopEvent);
	return rc;

bailout:
	switch ( rc )
	{
	case -4:
	case -3:
			free(pSD);
	case -2:
			CloseHandle(hEvents[1]);
	case -1:
			CloseHandle(m_hStopEvent);
	case 0:
	default:
		break;
	}
	sprintf(szMsg, "An unexpected error occurred (rc = %ld).\n", rc);
	if ( this->m_bDebug )
		printf(szMsg);
	else
	{
		AddToMessageLog( m_szTitle, szMsg, APP_SERVICE_START_FAILED, EVENTLOG_ERROR_TYPE );
	}
	return rc;

}

void CService::Stop()
{
	this->m_ssStatus.dwCurrentState = SERVICE_STOP_PENDING;
	this->ReportStatus();
	if ( this->m_hStopEvent )
		SetEvent(m_hStopEvent);
}

BOOL WINAPI CService::ControlHandler(DWORD dwCtrlType)
{
	switch ( dwCtrlType )
	{
	case CTRL_C_EVENT:
	case CTRL_BREAK_EVENT:
		pService->Stop();
		return (TRUE);
	}
	pService->m_ssStatus.dwWaitHint = 3000;
	pService->ReportStatus();
	return (FALSE);
}

void WINAPI CService::ServiceCtrl(DWORD dwCtrlCode)
{
	switch ( dwCtrlCode )
	{
	case SERVICE_CONTROL_STOP:
	case SERVICE_CONTROL_SHUTDOWN:
		if ( !pService->m_bDebug )
			pService->AddToMessageLog( pService->m_szTitle, NULL, APP_SERVICE_STOPPING, EVENTLOG_INFORMATION_TYPE );
		pService->m_ssStatus.dwCurrentState = SERVICE_STOP_PENDING;
		pService->Stop();
		break;
	case SERVICE_CONTROL_INTERROGATE:
		break;
	default:
		break;
	}
	pService->ReportStatus();
}

void WINAPI CService::ServiceMain(DWORD dwArgc, LPTSTR * lpszArgv)
{
	pService->StartMain( dwArgc, lpszArgv );
}

void CService::Dispatch()
{
	m_bDebug = FALSE;
	printf("\nStartServiceCtrlDispatcher being called.\n");
	printf("This may take serveral seconds... Please wait.\n");

	char szTmp[256];
	sprintf( szTmp, "%s",  m_szTitle );
	SERVICE_TABLE_ENTRY dispatchTable[] = 
	{
		{ szTmp, (LPSERVICE_MAIN_FUNCTION) ServiceMain },
		{ NULL, NULL },
	};

	if (!StartServiceCtrlDispatcher(dispatchTable))
	{
		char szErr[256];
		AddToMessageLog(m_szTitle, GetLastErrorText(szErr, sizeof(szErr)), APP_SERVICE_START_FAILED, EVENTLOG_ERROR_TYPE);
	}
}

BOOL CService::ReportStatus()
{
	if ( this->m_bDebug )
		return TRUE;	// don't report to the SCM if we are debugging.
	if ( m_sshStatusHandle == NULL )
		return TRUE;

	this->m_ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;

	if ( this->m_ssStatus.dwCurrentState == SERVICE_START_PENDING )
		m_ssStatus.dwControlsAccepted = 0;
	else
		m_ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
	static DWORD dwCheckPoint = 1;
	if ( ( m_ssStatus.dwCurrentState == SERVICE_RUNNING ) || ( m_ssStatus.dwCurrentState == SERVICE_STOPPED ) )
	{
		m_ssStatus.dwCheckPoint = 0;
		m_ssStatus.dwWaitHint = 0;
	}
	else
	{
		m_ssStatus.dwCheckPoint = dwCheckPoint++;
		m_ssStatus.dwWaitHint = 5000;
	}
	BOOL rc = SetServiceStatus( m_sshStatusHandle, &this->m_ssStatus );

	if ( rc == 0 )
	{
		char szMsg[MAX_PATH];
		sprintf(szMsg, "rc = %ld, Err = %ld", rc, GetLastError());
		AddToMessageLog( szMsg, NULL, APP_SERVICE_SETSTATUS_FAILED, EVENTLOG_WARNING_TYPE ); 
	}

	return rc;
}

void CService::AddToMessageLog(LPTSTR lpszMsg1, LPTSTR lpszMsg2, DWORD dwEventID, WORD wType)
{
	HANDLE hEventSource = RegisterEventSource( NULL, m_szServiceName );
	char * pMsgs[2];
	pMsgs[0] = lpszMsg1;
	pMsgs[1] = lpszMsg2;
	int nMsgs;
	if ( lpszMsg1 == NULL )
		nMsgs = 0;
	else if ( lpszMsg2 == NULL )
		nMsgs = 1;
	else
		nMsgs = 2;
	
	if ( hEventSource != NULL )
	{
		ReportEvent(hEventSource, wType, 0, dwEventID, NULL, nMsgs, 0, (const char **) pMsgs,  NULL);
		DeregisterEventSource(hEventSource);
	}
}

LPTSTR CService::GetLastErrorText(LPTSTR lpszBuf, DWORD dwSize)
{
	LPTSTR lpszTemp = NULL;
	DWORD dwRet = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ARGUMENT_ARRAY,
		NULL, GetLastError(), LANG_NEUTRAL, (LPTSTR)&lpszTemp, 0, NULL);
	if ( !dwRet || ((long)dwSize < (long)dwRet+14))
		lpszBuf[0] = 0x00;
	else
	{
		lpszTemp[strlen(lpszTemp) - 2] = 0x00;
		sprintf(lpszBuf, "%s (0x%x)", lpszTemp, GetLastError());
	}
	if ( lpszTemp)
		LocalFree((HLOCAL) lpszTemp);
	return lpszBuf;
}

void CService::SetName(char *pName)
{
	sprintf(m_szServerName, pName);
	if (m_szServerName[0] != 0x00 )
		sprintf(m_szTitle, "%s - %s", m_szDisplayName, pName);
	else
		sprintf(m_szTitle, "%s", m_szDisplayName);
}

CService::CService()
{
	this->m_bDebug = TRUE;
	this->m_hStopEvent = NULL;
	this->m_sshStatusHandle = NULL;
	sprintf( this->m_szServerName, "" );
	sprintf( this->m_szServiceName, "unnamed" );
	sprintf( this->m_szDisplayName, "unnamed service" );
	sprintf( this->m_szTitle, m_szDisplayName );
	this->m_ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	this->m_ssStatus.dwCurrentState = SERVICE_STOPPED;
	this->m_ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
	this->m_ssStatus.dwWin32ExitCode = NO_ERROR;
	this->m_ssStatus.dwCheckPoint = 0;
	this->m_ssStatus.dwWaitHint = 5000;
	this->m_ssStatus.dwServiceSpecificExitCode = 0;
}

CService::CService(LPTSTR szName, LPTSTR szDisplayName, LPTSTR szServerName)
{
	CService();
	sprintf( this->m_szServiceName, szName );
	sprintf( this->m_szDisplayName, szDisplayName );
	sprintf( this->m_szServerName, szServerName );
	if ( szServerName[0] == 0x00 )
		sprintf( this->m_szTitle, szDisplayName );
	else
		sprintf( m_szTitle, "%s - %s", szDisplayName, szServerName );
}

CService::~CService()
{

}

int _cdecl main(int argc, char * argv[])
{
	// Create the service object
	CService service("axissvr", "Axis Server", "" );
	pService = &service;
	pService->ReportStatus();
	// Find the path
	char szPath[MAX_PATH];
	GetModuleFileName( NULL, &szPath[0], sizeof(szPath) );
	if ( szPath[0] != 0 )
	{
		for ( int i = strlen(szPath); i > 0; i-- )
		{
			if ( szPath[i] == '\\' )
			{
				szPath[i] = 0x00;
				break;
			}
		}
		_chdir(szPath);
	}
	// Find the server name
	FILE * pIniFile = fopen("sphere.ini", "r");
	if ( pIniFile != NULL )
	{
		char szLine[MAX_BUFFER];
		while ( !feof(pIniFile) )
		{
			fReadLine( pIniFile, &szLine[0], sizeof(szLine) );
			if ( strstr( szLine, "[SERVERS]") != NULL )
			{
				fReadLine( pIniFile, &szLine[0], sizeof(szLine) );
				pService->SetName( &szLine[0] );
				break;
			}
		}
		fclose(pIniFile);
	}
	pService->ReportStatus();
	pIniFile = fopen("hogsvr.cfg", "r");
	bool bUseService = false;
	OSVERSIONINFO osInfo;
	osInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if ( ! GetVersionEx(&osInfo) || osInfo.dwPlatformId != VER_PLATFORM_WIN32_NT )
	{
		// We aren't running NT
		return service_main(argc, argv);
	}
	if ( pIniFile != NULL )
	{
		char szLine[MAX_BUFFER];
		while ( !feof(pIniFile) )
		{
			fReadLine( pIniFile, &szLine[0], sizeof(szLine) );
			if ( strstr( szLine, "NTSERVICE" ) == &szLine[0] )
			{
				bUseService = true;
				break;
			}
		}
		fclose(pIniFile);
	}
	pService->ReportStatus();
	if ( ! bUseService )
	{
		// Not running as an NT Service
		return service_main(argc, argv);
	}
	if ( (argc > 1) && ( (*argv[1] == '-') || ( *argv[1] == '/')))
	{
		if ( ! strcmpi((argv[1] + 1), "install") )
		{
			pService->Install();
			return 0;
		}
		else if ( ! strcmpi((argv[1] + 1), "remove") )
		{
			pService->Remove();
			return 0;
		}
		else if ( ! strcmpi((argv[1] + 1), "debug") )
			return pService->Debug( argc, argv );
	}
	printf("axissvr -install\t\t\tto install the service\n");
	printf("axissvr -remove\t\t\tto remove the service\n");
	printf("axissvr -debug\t\t\tto debug the service\n");
	pService->Dispatch();
	pService->SetState(SERVICE_STOPPED);
	pService->ReportStatus();
	return -1;
}

#endif