/*
 $Id: remoteconnection.cpp,v 1.20 2002/05/20 19:14:43 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// RemoteConnection.cpp
// Source file for CRemoteConnection class

#include "stdafx.h"
#ifdef _AXIS
#include "drewsky.h"
#else
#include "SphereScript.h"
#endif
#include "RemoteConnection.h"
#include "RemoteConsole.h"
#include "cnpc.h"
#include "citem.h"

int fReadLine(FILE * pFile, char * pszBuf, int iSize)
{
	memset(pszBuf, 0x00, iSize);
	int iCount = 0;
	char c = 0x00;
	while (c != '\n' && iCount < (iSize - 1) && !feof(pFile))
	{
		c = fgetc(pFile);
		if (c != '\n' && c != -1)
		{
			*(pszBuf + iCount) = c;
			iCount++;
		}
	}
	if (feof(pFile) && iCount == 0)
		return -1;
	return iCount;
}

const char * pszFiles [] = {
	"graychar2",
	"grayitem2",
	"graydefs",
	"graymap",
	"graytables",
	"graytemplate",
	"graymagic",
	"grayskill",
	"grayspee",
	"grayname",
	"graytrig",
	"grayini",
	"grayacct",
	"graynpcs",
	"grayitems",
};

CRemoteConnection::CRemoteConnection()
{
	// Use default profile
	ReadProfile(Main->GetProfileString());
	Initialize();
}

CRemoteConnection::CRemoteConnection(CString csProfile)
{
	ReadProfile(csProfile);
	Initialize();
}

CRemoteConnection::~CRemoteConnection()
{
	Send("QUIT", 4);
	Main->m_log.Add("Closing connection to remote server");
	shutdown(m_sock.m_hSocket, 2);
	closesocket(m_sock.m_hSocket);
	m_sock.m_hSocket = INVALID_SOCKET;
	DeleteProgressWindow();
}

bool CRemoteConnection::Logon()
{
	Main->m_log.Add("Attempting to log on to axis server");
	char szBuffer[MAX_BUFFER];
	ZeroMemory(szBuffer, sizeof(szBuffer));
	int rc = Receive(szBuffer, sizeof(szBuffer));
	if ( rc < 0 )
	{
		Main->m_log.Add("Didn't receive initial message from axis server");
		return false;
	}
	Send(m_csUsername, strlen(m_csUsername));
	ZeroMemory(szBuffer, sizeof(szBuffer));
	rc = Receive(szBuffer, sizeof(szBuffer));
	if ( rc < 0 )
	{
		Main->m_log.Add("Didn't receive response for user name on login");
		return false;
	}
	Send(m_csPassword, strlen(m_csPassword));
	ZeroMemory(szBuffer, sizeof(szBuffer));
	rc = Receive(szBuffer, sizeof(szBuffer));
	if ( rc < 0 )
	{
		Main->m_log.Add("Didn't receive response for password on login");
		return false;
	}
	if (strstr(szBuffer, "Login successful.") != NULL)
	{
		// Try to get the protocol level;
		if ( strstr(szBuffer, "Protocol 1.0") != NULL )
			this->m_protocolLevel = 1;
		else if ( strstr(szBuffer, "Protocol 2.0") != NULL )
			this->m_protocolLevel = 2;
		else
			this->m_protocolLevel = 0;
		Main->m_log.Add("Login successful...protocol level is %ld.0", m_protocolLevel);
		return true;
	}
	else
	{
		CString csError;
		csError.Format("Could not log on to remote server.\nThe server returned the following error:\n\n%s", szBuffer);
		AfxMessageBox(csError, MB_OK | MB_ICONSTOP);
		Main->m_log.Add("Unable to log on to remote server.  Error was %s", szBuffer);
		return false;
	}
}

void CRemoteConnection::GetSpawnpoints(CStringArray *pSpawns)
{
	if (!m_bIsValid)
		return;
	Main->m_log.Add("Retrieving spawnpoints from remote server");
	Main->m_pScripts->LoadItems();
	Main->m_pScripts->LoadNPCs();
	Main->m_pScripts->LoadTables();
	Main->m_pScripts->LoadTemplates();
	char szBuffer[MAX_BUFFER + sizeof(int)];
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	char crlf [] = {0x0d, 0x0a, 0x00};
	sprintf(szBuffer, "GSPN");

	CString csMessage;
	csMessage.Format("Attempting to read spawn information from remote server");
	if ( m_pDlg )
		m_pDlg->m_csMessage.SetWindowText(csMessage);
	
	Send(szBuffer, strlen(szBuffer));
	ZeroMemory(szBuffer, sizeof(szBuffer));
	int result = Receive(szBuffer, sizeof(szBuffer));
	if (strstr(szBuffer, "Error -- Could not open world file") != NULL)
	{
		CString csErr;
		Main->m_log.Add("Error:  %s", szBuffer);
		csErr.Format("The remote server reported an error while retrieving spawnpoints:\n\n%s", szBuffer);
		AfxMessageBox(csErr, MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	if (result == SOCKET_ERROR)
	{
		Main->m_log.Add("Socket error: %ld - %s", WSAGetLastError(), strerror(WSAGetLastError()));
		AfxMessageBox("Socket error communicating with remote server.", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	long lLength = atoi(szBuffer);
	if ( lLength == 0 && szBuffer[0] != '0')
	{
		Main->m_log.Add("Buffer is empty");
		return;
	}
	if ( m_pDlg )
	{
		m_pDlg->SetRange32(0, (int) lLength);
		m_pDlg->SetPos(0);
	}
	DWORD dwBytesRead = 0;

	bool bEndOfFile = false;
	char * pBuffer = new char [lLength + 1];
	memset(pBuffer, 0x00, lLength + 1);
	while ( dwBytesRead < (DWORD) lLength )
	{
		int iRead = Receive(pBuffer + dwBytesRead, MAX_BUFFER);
		if ( iRead > 0 )
		{
			dwBytesRead += iRead;
			csMessage.Format("Received %ld of %ld bytes of spawn information", dwBytesRead, lLength);
			if ( m_pDlg )
			{
				m_pDlg->m_csMessage.SetWindowText(csMessage);
				m_pDlg->SetPos((int) dwBytesRead);
			}
		}
		else
		{
			AfxMessageBox("WARNING: Not all spawn information was read", MB_OK | MB_ICONEXCLAMATION);
			break;
		}

	}
	CString csBuffer = _T(pBuffer);
	CString csToken = csBuffer.SpanExcluding(crlf);
	while (csBuffer != "")
	{
		CString csLine;
		csLine.Format("%s", csToken);
		if (csLine.Compare(".") != 0)
		{
			// Find the ID
			CString csType = csLine.Left(csLine.Find(","));
			CString csID = csLine.Mid(csLine.Find(",") + 1);
			csID = csID.SpanExcluding(",");
			CString csOldID = csID;
			DWORD dwID = ahextoi(csID);
			if (csType == "ITEM")
			{
				// It's either an item or a template....
				int iIndex = -1;
				if ( isdigit(csID[0]) )
				{
					CString csIndex;
					if ( csID[0] == '0' )
						csIndex = csID;
					else
						csIndex.Format("0%x", atoi(csID));
					csID = csIndex;
				}
				if ( !Main->m_pScripts->m_bNewFormat )
				{
					DWORD dwID = ahextoi(csID);
					csID.Format("%04x", dwID);
				}
				iIndex = Main->m_pScripts->m_aItems.Find(csID);
				if ( iIndex != -1 )
				{
					// It's an item
					CItem * pItem = (CItem *) Main->m_pScripts->m_aItems.GetAt(iIndex);
					if ( pItem )
						csID = pItem->m_csDescription;
				}
				else
				{
					// It's a template
					iIndex = Main->m_pScripts->m_aTemplates.Find(csOldID);
					if ( iIndex != -1 )
					{
						CTemplate * pTemplate = (CTemplate *) Main->m_pScripts->m_aTemplates.GetAt(iIndex);
						if ( pTemplate )
							csID = pTemplate->m_csDescription;
					}
				}
			}
			else
			{
				// It's an NPC
				int iIndex = -1;
				if ( isdigit(csID[0]) )
				{
					CString csIndex;
					if ( csID[0] == '0' )
						csIndex = csID;
					else
						csIndex.Format("0%x", atoi(csID));
					csID = csIndex;
				}
				if ( !Main->m_pScripts->m_bNewFormat )
				{
					DWORD dwID = ahextoi(csID);
					csID.Format("%04x", dwID);
				}
				iIndex = Main->m_pScripts->m_aNPCs.Find(csID);
				if ( iIndex != -1 )
				{
					CNPC * pNPC = (CNPC *) Main->m_pScripts->m_aNPCs.GetAt(iIndex);
					if ( pNPC )
						csID = pNPC->m_csDescription;
				}
				else
				{
					// It's a spawn group
					iIndex = Main->m_pScripts->m_tables.m_spawns.Find(csID);
					if ( iIndex != -1 )
					{
						CTSpawnDef * pSpawn = (CTSpawnDef *) Main->m_pScripts->m_tables.m_spawns.GetAt(iIndex);
						if ( pSpawn )
							csID = pSpawn->m_csDescription;
					}
				}
			}
			csLine.Replace(csOldID, csID);
			pSpawns->Add(csLine);
		}
		else
			bEndOfFile = true;
		if (csBuffer.Find(crlf) != -1)
			csBuffer = csBuffer.Right(csBuffer.GetLength() - (csBuffer.Find(crlf) + 2));
		else
			csBuffer = "";
		csToken = csBuffer.SpanExcluding(crlf);
	}
	delete [] pBuffer;
	return;
}

void CRemoteConnection::GetCustomFile(CString csFile, CString csLocalFile)
{
	if (!this->m_bIsValid)
		return;
	CString csMessage;
	CString csCmd;
	CString csRemoteFileName;
	switch(FindTable(csFile, pszFiles, 15))
	{
	case 0:
		csRemoteFileName = RMT_NPC_FILE;
		csCmd = "GCHR";
		break;
	case 1:
		csRemoteFileName = RMT_ITEM_FILE;
		csCmd = "GITM";
		break;
	case 2:
		csRemoteFileName = RMT_DEFS_FILE;
		csCmd = "GDFS";
		break;
	case 3:
		if ( m_protocolLevel == 0 )
		{
			// server doesn't support the new map file
			AfxMessageBox("Remote AxisSvr does not support the new map file retrieval.\nPlease update axissvr on the remote machine.", MB_OK | MB_ICONSTOP);
			return;
		}
		csRemoteFileName = RMT_MAP_FILE;
		csCmd = "GMAP";
		break;
	case 4:
		{
			if ( Main->m_pScripts->m_bNewFormat )
				csCmd.Format("GFIL%s", csFile);
			else
			{
				csRemoteFileName = RMT_TABLE_FILE;
				csCmd = "GTBL";
			}
		}
		break;
	case 5:
		csRemoteFileName = RMT_TEMPLATE_FILE;
		csCmd = "GTPL";
		break;
	case 6:
		csRemoteFileName = RMT_MAGIC_FILE;
		csCmd = "GMAG";
		break;
	case 7:
		csRemoteFileName = RMT_SKILL_FILE;
		csCmd = "GSKL";
		break;
	case 8:
		csRemoteFileName = RMT_SPEECH_FILE;
		csCmd = "GSPE";
		break;
	case 9:
		csRemoteFileName = RMT_NAME_FILE;
		csCmd = "GNAM";
		break;
	case 10:
		csRemoteFileName = RMT_TRIGGER_FILE;
		csCmd = "GTRG";
		break;
	case 11:
		csRemoteFileName = RMT_INI_FILE;
		csCmd = "GINI";
		break;
	case 12:
		csRemoteFileName = RMT_ACCU_FILE;
		csCmd = "GACT";
		break;
	default:
		csCmd.Format("GFIL%s", csFile);
		break;
	}

	CStdioFile cfOutput;
	if ( csLocalFile != "" )
	{
		if ( csLocalFile.GetAt(0) == '\\' )
			csRemoteFileName = csLocalFile.Mid(1);
		else
			csRemoteFileName = csFile;
	}
	else if (csRemoteFileName == "" )
		csRemoteFileName = csFile;
	if ( csLocalFile == "" )
		csLocalFile.Format("%s\\%s", Main->GetProfileString(), csRemoteFileName);
	csLocalFile.Replace(':', '!');
	csLocalFile.Replace('/', '\\');
	csLocalFile.Replace("\\\\", "\\");
	csRemoteFileName.Replace('/', '\\');
	csRemoteFileName.Replace("\\\\", "\\");
	csMessage.Format("Attempting to read file %s from remote server", csRemoteFileName);
	if ( m_pDlg )
		m_pDlg->m_csMessage.SetWindowText(csMessage);

	Main->m_log.Add("Attempting to retrieve file %s (%s) from remote server", csRemoteFileName, csLocalFile);
	// Make sure this directory exists
	CString csDirectory = csLocalFile;
	if ( csDirectory.Find('\\') != -1 )
	{
		csDirectory.SetAt(csDirectory.ReverseFind('\\'), 0x00);
		// We may have to recursively create the directory
		{
			int iSlash = 0;
			while ( iSlash >= 0 )
			{
				iSlash = csDirectory.Find("\\", iSlash + 1);
				if ( iSlash == -1 )
				{
					CreateDirectory(csDirectory, NULL);
					break;
				}
				CString csPath = csDirectory.Left(iSlash);
				CreateDirectory(csPath, NULL);
			}
		}
	}
	if (!cfOutput.Open(csLocalFile, CFile::modeWrite | CFile::modeCreate | CFile::shareDenyWrite))
		return;
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	char szBuffer[MAX_BUFFER + 4];
	sprintf(szBuffer, csCmd);
	Send(szBuffer, strlen(szBuffer));
	ZeroMemory(szBuffer, sizeof(szBuffer));
	Receive(szBuffer, sizeof(szBuffer));
	long lLength = atoi(szBuffer);
	if ( lLength == 0 )
	{
		Main->m_log.Add("File length is 0");
		return;
	}
	if ( m_pDlg )
	{
		m_pDlg->SetRange32(0, (int) lLength);
		m_pDlg->SetPos(0);
	}
	DWORD dwBytesRead = 0;
	while (strcmp(szBuffer, szEOF) != 0)
	{
		ZeroMemory(szBuffer, sizeof(szBuffer));
		int result = Receive(szBuffer, sizeof(szBuffer));
		if (result <= 0)
		{
			cfOutput.Close();
			CFile::Remove(csLocalFile);
			CString csMessage;
			Main->m_log.Add("Socket error %ld while receiving file %s", WSAGetLastError(), csRemoteFileName);
			Main->m_log.Add("%s", strerror(WSAGetLastError()));
			csMessage.Format("Socket Error %ld while receiving file %s\n%s", WSAGetLastError(), csRemoteFileName, strerror(WSAGetLastError()));
			AfxMessageBox(csMessage, MB_OK);
			break;
		}
		if (result == SOCKET_ERROR)
		{
			Main->m_log.Add("Socket Error after %ld of %ld bytes in file %s.  Error %ld - %s", dwBytesRead, lLength, csRemoteFileName, WSAGetLastError(), strerror(WSAGetLastError()));
			break;
		}
		if (strstr(szBuffer, "Could not open") != NULL)
		{
			Main->m_log.Add("Error returned from remote server while processing command %s - %s", csCmd, szBuffer);
			break;
		}
		if (strstr(szBuffer, "Command unknown") != NULL)
		{
			Main->m_log.Add("Error returned from remote server while processing command %s - %s", csCmd, szBuffer);
			break;
		}
		dwBytesRead += result;
		csMessage.Format("Received %ld of %ld bytes for file %s", dwBytesRead, lLength, csRemoteFileName);
		if ( m_pDlg )
		{
			m_pDlg->m_csMessage.SetWindowText(csMessage);
			m_pDlg->SetPos((int) dwBytesRead);
		}

		if (strcmp(szBuffer, szEOF) != 0)
		{
			CString csBuffer = _T(szBuffer);
			CString csEOL;
			csEOL.Format("%c%c", 0x0d, 0x0a);
			csBuffer.Replace(csEOL, "\n");
			cfOutput.WriteString(csBuffer);
			sprintf(szBuffer, "%ld", result);
			Sleep(10);
			Send(szBuffer, strlen(szBuffer));
		}
		//Main->m_log.Add("Received %ld of %ld bytes for file %s", dwBytesRead, lLength, csRemoteFileName);
	}
	cfOutput.Close();
	if (strcmp(szBuffer, szEOF) != 0)
	{
		// Didn't get the entire file....delete the temporary file
		Main->m_log.Add("EOF message never received from remote server for file %s", csRemoteFileName);
		try
		{
			CFile::Remove(csLocalFile);
		}
		catch (CFileException * e)
		{
			e->Delete();
		}
	}
	if ( m_pDlg )
		m_pDlg->SetPos((int) lLength);
	return;
}

bool CRemoteConnection::SendCustomFile(CString csFileName, CString csTmpFile, CSObject * pObject )
{
	if (!m_bIsValid)
		return false;
	// Connect to the remote server ala GrayMon to pause the server.
	CString csMessage;
	CRemoteConsole * pConsole = Main->m_pRConsole;
	bool bDeleteConsole = false;
	CString csFile = csFileName;
	if ( csTmpFile != RMT_ACCT_FILE && !Main->m_pScripts->m_bNewFormat )
	{
		if (pConsole == NULL)
		{
			pConsole = new (CRemoteConsole);
			bDeleteConsole = true;
		}
		pConsole->Pause();
	}

	CString csInput;
	if (csTmpFile != "")
		csInput = csTmpFile;
	else
		csInput = csFileName + ".scp";
	csFileName.Replace('!', ':');
	CStdioFile csfInput;
	if (csfInput.Open(csInput, CFile::modeRead | CFile::shareDenyNone))
	{
		char szBuffer[MAX_BUFFER];
		CString csCmd;
		switch(FindTable(csFileName, pszFiles, 15))
		{
		case 0:
			csCmd = "SCHR";
			break;
		case 1:
			csCmd = "SITM";
			break;
		case 2:
			csCmd = "SDFS";
			break;
		case 3:
			if ( m_protocolLevel == 0 )
			{
				// server doesn't support the new map file
				AfxMessageBox("Remote AxisSvr does not support the new map file updates.\nPlease update axissvr on the remote machine.", MB_OK | MB_ICONSTOP);
				pConsole->Resync();
				return false;
			}
			csCmd = "SMAP";
			break;
		case 4:
			csCmd = "STBL";
			break;
		case 5:
			csCmd = "STPL";
			break;
		case 6:
			csCmd = "SMAG";
			break;
		case 7:
			csCmd = "SSKL";
			break;
		case 8:
			csCmd = "SSPE";
			break;
		case 9:
			csCmd = "SNAM";
			break;
		case 10:
			csCmd = "STRG";
			break;
		case 11:
			csCmd = "SINI";
			break;
		case 12:
			csCmd = "SACT";
			break;
		case 13:
			csCmd = "SCH2";
			break;
		case 14:
			csCmd = "SIT2";
			break;
		default:
			if (csFile.Find(Main->GetProfileString()) != -1 )
				csFileName = csFile.Mid(csFile.Find(Main->GetProfileString()) + Main->GetProfileString().GetLength() + 1);
			else
				csFileName = csFile;
		csCmd.Format("SFIL%s", csFileName);
			break;
		}
		CFileStatus st;
		CFile::GetStatus(csInput, st);
		Main->m_log.Add("Sending file %s (%s) to remote server", csTmpFile, csFileName );
		csMessage.Format("Sending file %s to remote server", csFileName);
		if ( m_pDlg )
		{
			m_pDlg->m_csMessage.SetWindowText(csMessage);
			m_pDlg->SetPos(0);
			m_pDlg->SetRange32(0, st.m_size );
		}
		this->Send(csCmd, csCmd.GetLength());
		memset(&szBuffer[0], 0x00, sizeof(szBuffer));
		int rc = Receive(szBuffer, sizeof(szBuffer));
		if ( strstr(szBuffer, "Ready to receive") == NULL )
		{
			// Can't update the file.
			CString csMessage;
			Main->m_log.Add("Error while sending file - %s", szBuffer);
			csMessage.Format("Error occurred while sending file:\n%s", szBuffer);
			AfxMessageBox(csMessage, MB_OK | MB_ICONEXCLAMATION);
			pConsole->Resync();
			return false;

		}
		if (rc <= 0)
		{
			pConsole->Resync();  // Re-sync the server.
			return false;
		}
		int iBytesRead = 1;
		memset(&szBuffer[0], 0x00, sizeof(szBuffer));
		char crlf[] = {0x0d, 0x0a, 0x00};
		DWORD dwBytesSent = 0;
		while (iBytesRead)
		{
			iBytesRead = 0;
			char szLine[MAX_PATH];
			while (iBytesRead < (sizeof(szBuffer) - MAX_PATH - 2) )
			{
				memset(&szLine[0], 0x00, sizeof(szLine));
				LPTSTR pStr = csfInput.ReadString(szLine, MAX_PATH);
				if (pStr != NULL)
				{
					int iBytes = strlen(szLine);
					iBytesRead += iBytes + 1;
					sprintf(szBuffer, "%s%s", szBuffer, szLine);
				}
				else
					break;
			}
			if (iBytesRead)
			{
				if (iBytesRead < MAX_BUFFER)
					szBuffer[iBytesRead] = 0x00;
				this->Send(szBuffer, iBytesRead);
				memset(&szBuffer, 0x00, sizeof(szBuffer));
				char szInput[5];
				memset(&szInput, 0x00, sizeof(szInput));
				this->Receive(&szInput[0], sizeof(szInput));
				if ( atoi(szInput) != iBytesRead )
				{
					Main->m_log.Add("Ack failed while sending file %s - %ld != %ld", csFileName, atoi(szInput), iBytesRead);
					AfxMessageBox("Ack failed while sending file", MB_OK | MB_ICONEXCLAMATION);
					break;
				}
			}
			dwBytesSent += iBytesRead;
			csMessage.Format("Sent %ld of %ld bytes for file %s", dwBytesSent, st.m_size, csFileName);
			if ( m_pDlg )
			{
				m_pDlg->SetPos((int) dwBytesSent);
				m_pDlg->m_csMessage.SetWindowText(csMessage);
			}
		}
		csfInput.Close();
		char szEOF[] = {0x0d, 0x0a, '.', 0x0d, 0x0a};
		this->Send(szEOF, strlen(szEOF));
		memset( &szBuffer[0], 0x00, sizeof(szBuffer) );
		rc = Receive(szBuffer, sizeof(szBuffer));
		if ( rc > 0 )
		{
			if ( csCmd == "SCHR" || csCmd == "SITM" )
			{
				// Find out what item this is
				if ( strstr(szBuffer, "ID=") != NULL )
				{
					char * pszID = strstr(szBuffer, "ID=");
					DWORD dwID = ahextoi( pszID + 3 );
					if ( csfInput.Open(csInput, CFile::modeRead | CFile::shareDenyNone) )
					{
						CStdioFile csfOutput;
						if ( ! csfOutput.Open( "newitem.tmp", CFile::modeWrite | CFile::modeCreate | CFile::shareDenyNone) )
							csfInput.Close();
						else
						{
							CString csLine;
							BOOL bStatus = true;
							while ( bStatus )
							{
								bStatus = csfInput.ReadString( csLine );
								if ( ! bStatus )
									break;
								if ( csLine.Find("[]") != -1 )
									csLine.Format("[%04x]", dwID );
								csfOutput.WriteString( csLine );
								csfOutput.WriteString("\n");
							}
							csfInput.Close();
							csfOutput.Close();
							CFile::Remove( csInput );
							CFile::Rename( "newitem.tmp", csInput );
							if ( csCmd == "SCHR" )
							{
								CNPC * pNPC = (CNPC *) pObject;
								pNPC->m_csValue.Format( "%04x", dwID );
							}
							else
							{
								CItem * pItem = (CItem *) pObject;
								pItem->m_csValue.Format( "%04x", dwID );

							}
						}
					}
				}
			}
		}
	}
	Sleep(1000);
	if ( csTmpFile != RMT_ACCT_FILE && !Main->m_pScripts->m_bNewFormat )
		pConsole->Resync();
	if (bDeleteConsole)
		delete pConsole;
	return true;
}

DWORD CRemoteConnection::GetRemotePort()
{
	if (!this->IsValid())
		return 0;
	Main->m_log.Add("Attempting to receive remote port");
	this->Send("PORT", 4);
	char szData[MAX_BUFFER];
	ZeroMemory(&szData[0], sizeof(szData));
	Receive(szData, sizeof(szData));
	if (szData[0] != 0x00)
	{
		Main->m_log.Add("Received port successfully");
		return ((DWORD) atoi(szData));
	}
	else
		return 0;
}

int CRemoteConnection::Send(const void *buffer, int length)
{
	char szNewBuffer[MAX_BUFFER + sizeof(int)];
	memset(szNewBuffer, 0x00, sizeof(szNewBuffer));
	int * header = (int *) &szNewBuffer;
	*header = length;
	//memcpy((void *) szNewBuffer, (void *)length, sizeof(int) );
	memcpy(&szNewBuffer[sizeof(int)], buffer, length);
	int nBytes = 0;
	int i = 0;
	while (nBytes < length)
	{
		int nSent = 0;
		if ( nBytes )
			nSent = m_sock.Send( &szNewBuffer[nBytes + sizeof(int)], length + sizeof(int) - nBytes );
		else
			nSent = m_sock.Send( szNewBuffer, length + sizeof(int) );
		if ( nSent < 0 )
		{
			Main->m_log.Add("Socket error while sending data to remote server: %ld - %s", WSAGetLastError(), strerror(WSAGetLastError()));
			return nBytes;
		}
		if ( i > 100 )
			return nBytes;
		i++;
		nBytes += nSent - sizeof(int);
		if ( nBytes < length)
			Sleep(10);
	}
	return nBytes;
}

int CRemoteConnection::Receive(char *buffer, int max_buffer)
{
	int nBytes = 0;
	int i = 0;
	// Pull off the length of the incoming packet.
	struct timeval timeout;

	int iLength = 0;
	while ( !iLength )
	{
		FD_SET fd;
		FD_ZERO(&fd);
		FD_SET(m_sock.m_hSocket, &fd);
		timeout.tv_sec = 0;
		timeout.tv_usec = 1000;
		int rc = select(m_sock.m_hSocket + 1, &fd, 0, 0, &timeout);
		if (rc < 0)
		{
			Main->m_log.Add("Socket error on select(): %ld - %s", WSAGetLastError(), strerror(WSAGetLastError()));
			return rc;
		}
		if ( i++ > m_iReceiveTimeout )
		{
			Main->m_log.Add("Timeout exceeded while receiving incoming packet.");
			return -3;
		}
		if ( rc == 0 )
			continue;
		m_sock.Receive(&iLength, sizeof(int));
	}
	if ( iLength == 0 )
	{
		Main->m_log.Add("Length of incoming packet is 0");
		return 0;
	}
	i = 0;
	while ( nBytes < iLength )
	{
		FD_SET fd;
		FD_ZERO(&fd);
		FD_SET(m_sock.m_hSocket, &fd);
		struct timeval timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = 1000;
		int rc = select(m_sock.m_hSocket + 1, &fd, 0, 0, &timeout);
		if (rc < 0)
		{
			Main->m_log.Add("Socket error on select(): %ld - %s", WSAGetLastError(), strerror(WSAGetLastError()));
			return rc;
		}
		if ( rc == 0 )
			continue;
		int nRead = m_sock.Receive( buffer + nBytes, iLength - nBytes );
		if ( nRead < 0 )
		{
			Main->m_log.Add("Socket error while receiving data from remote server: %ld - %s", WSAGetLastError(), strerror(WSAGetLastError()));
			return nRead;
		}
		if ( i++ > m_iReceiveTimeout )
		{
			Main->m_log.Add("Timeout exceeded while receiving data from remote server.");
			return -3;
		}
		nBytes += nRead;
		if ( nBytes < iLength )
			Sleep(10);
	}
	return iLength;
}

void CRemoteConnection::StartTUS()
{
	Main->m_log.Add("Attempting to start remote server");
	char szCmd[MAX_PATH];
	sprintf(szCmd, "STRT");
	this->Send(szCmd, strlen(szCmd));
}

void CRemoteConnection::GetUpdates()
{
	// First check to see if we need to update the tables file
	if ( m_pDlg )
	{
		m_pDlg->m_csMessage.SetWindowText("Checking for file updates");
		m_pDlg->SetPos(0);
	}
	if ( !this->m_bIsValid )
	{
		Main->m_log.Add("Unable to connect to axissvr.  Trying to read existing local copies");
		// Try to read files that are already here...
		CString csFile;
		csFile.Format("%s\\%s", Main->GetProfileString(), INI_FILE);
		Main->m_pScripts->LoadFile(csFile);
		Main->m_pScripts->m_config.Read();
		csFile.Format("%s\\%s", Main->GetProfileString(), TABLE_FILE);
		if ( !Main->m_pScripts->LoadFile(csFile) )
		{
			CString csScpPath = Main->m_pScripts->m_config.m_sSCPBaseDir;
			if ( csScpPath.GetLength() > 0 && ( csScpPath.GetAt(csScpPath.GetLength() - 1) == '\\' || csScpPath.GetAt(csScpPath.GetLength() -1) == '/' ) )
				csScpPath.SetAt(csScpPath.GetLength() - 1, 0x00);
			csFile.Format("%s\\%s\\%s", Main->GetProfileString(), csScpPath, TABLE_FILE);
			Main->m_pScripts->LoadFile(csFile);
		}
		return;
	}
	CString csMessage;
	Main->m_log.Add("Checking for file updates");
	char szCmd[MAX_BUFFER];
	// Check to see if the INI file has been updated
	sprintf(szCmd, "GFCS%s", INI_FILE);
	Send(szCmd, strlen(szCmd));
	char szBuffer[MAX_BUFFER];
	memset(&szBuffer[0], 0x00, sizeof(szBuffer));
	Receive(&szBuffer[0], sizeof(szBuffer));
	unsigned int iChk = atoi(szBuffer);
	CString csFileName;
	csFileName.Format("%s\\%s", Main->GetProfileString(), INI_FILE);
	csMessage.Format("Checking for an updated version of %s", INI_FILE);
	if ( m_pDlg )
		m_pDlg->m_csMessage.SetWindowText(csMessage);
	unsigned int iTest = GetChecksum(csFileName);
	if ( iChk != iTest )
		GetCustomFile(INI_FILE, csFileName);
	Main->m_pScripts->LoadFile(csFileName);
	Main->m_pScripts->m_config.Read();
	csMessage.Format("Checking for an updated version of %s", TABLE_FILE);
	if ( m_pDlg )
		m_pDlg->m_csMessage.SetWindowText(csMessage);
	sprintf(szCmd, "GFCS%s", TABLE_FILE);
	Send(szCmd, strlen(szCmd));
	memset(&szBuffer[0], 0x00, sizeof(szBuffer));
	Receive(&szBuffer[0], sizeof(szBuffer));
	iChk = atoi(szBuffer);
	csFileName.Format("%s\\%s", Main->GetProfileString(), TABLE_FILE);
	CreateDirectory(Main->GetProfileString(), NULL);
	iTest = GetChecksum(csFileName);
	if ( iChk != iTest )
	{
		Main->m_log.Add("Checksum for file %s did not match...local = %ld, remote = %ld", csFileName, iTest, iChk);
		GetCustomFile(TABLE_FILE, csFileName);
	}
	CString csScpPath = Main->m_pScripts->m_config.m_sSCPBaseDir;
	if ( csScpPath.GetLength() > 0 && ( csScpPath.GetAt(csScpPath.GetLength() - 1) == '\\' || csScpPath.GetAt(csScpPath.GetLength() -1) == '/' ) )
		csScpPath.SetAt(csScpPath.GetLength() - 1, 0x00);
	if ( csScpPath.GetLength() == 0 )
		csScpPath = ".";
	if ( !Main->m_pScripts->LoadFile(csFileName) )
	{
		// Maybe spheretables.scp is actually in the scripts directory
		CString csRemoteFile;
		sprintf(szCmd, "GFCS%s\\%s", csScpPath, TABLE_FILE);
		csRemoteFile.Format("%s\\%s", csScpPath, TABLE_FILE);
		csFileName.Format("%s\\%s\\%s", Main->GetProfileString(), csScpPath, TABLE_FILE);
		csMessage.Format("Checking for an updated version of %s", TABLE_FILE);
		if ( m_pDlg )
			m_pDlg->m_csMessage.SetWindowText(csMessage);
		Send(szCmd, strlen(szCmd));
		memset(&szBuffer[0], 0x00, sizeof(szBuffer));
		Receive(&szBuffer[0], sizeof(szBuffer));
		iChk = atoi(szBuffer);
		iTest = GetChecksum(csFileName);
		if ( iChk != iTest )
		{
			Main->m_log.Add("Checksum for file %s did not match...local = %ld, remote = %ld", csFileName, iTest, iChk);
			GetCustomFile(csRemoteFile, csFileName);
		}
		csFileName.Replace(':', '!');
		Main->m_pScripts->LoadFile(csFileName);
	}
	// Check the files in the RESOURCES section to see if we need to grab them.
	/*
	char szScriptPath [MAX_PATH];
	memset(&szScriptPath[0], 0x00, sizeof(szScriptPath));
	CString csConfigFile;
	csConfigFile.Format("%s\\%s", Main->GetProfileString(), INI_FILE);
	GetPrivateProfileString("SPHERE", "SCPFILES", "", &szScriptPath[0], sizeof(szScriptPath), csConfigFile);
	*/
	// Get a list of files from the server -- Don't do this if we are in old script mode
	for ( int i = 0; i < Main->m_pScripts->m_tables.m_resources.GetSize(); i++ )
	{
		CString csFilename = Main->m_pScripts->m_tables.m_resources.GetAt(i);
		if ( csFilename == "" )
			continue;
		CString csFile;
		if ( csFilename.GetAt(0) == '\\' || csFilename.GetAt(0) == '/' )
			csFile = csFilename.Mid(1);
		else
			csFile = csFilename;
		csFile.Replace('/', '\\');
		csFile.Replace(':', '!');
		/*
		if ( csFilename.ReverseFind('/') != -1 )
			csFile = csFilename.Mid(csFilename.ReverseFind('/') + 1);
		else if (csFilename.ReverseFind('\\') != -1 )
			csFile = csFilename.Mid(csFilename.ReverseFind('\\') + 1);
		else
			csFile = csFilename;
		*/
		// This could be a directory!
		if ( csFile.ReverseFind('\\') == csFile.GetLength() - 1)
		{
			CStringArray files, directories;
			GetDirectory(csFilename, &files, &directories);
			for ( int j = 0; j < files.GetSize(); j++ )
			{
				CString csNewFile;
				csNewFile.Format("%s%s", csFilename, files.GetAt(j));
				Main->m_pScripts->m_tables.m_resources.Add(csNewFile);
			}
		}
		else
		{
			CString csLocalFile;
			if ( csFile.Find('\\') == -1 )
			{
				csLocalFile.Format("%s\\%s\\%s", Main->GetProfileString(), csScpPath, csFile);
				csFilename.Format("%s\\%s", csScpPath, csFilename);
			}
			else
				csLocalFile.Format("%s\\%s", Main->GetProfileString(), csFile);
			sprintf(szCmd, "GFCS%s", csFilename);
			Send(szCmd, strlen(szCmd));
			memset(&szBuffer[0], 0x00, sizeof(szBuffer));
			Receive(&szBuffer[0], sizeof(szBuffer));
			iChk = atoi(szBuffer);
			csMessage.Format("Checking for an updated version of %s", csFilename);
			if ( m_pDlg )
				m_pDlg->m_csMessage.SetWindowText(csMessage);
			iTest = GetChecksum(csLocalFile);
			if ( iChk != iTest )
			{
				Main->m_log.Add("Checksum for file %s did not match...local = %ld, remote = %ld", csLocalFile, iTest, iChk);
				GetCustomFile(csFilename, csLocalFile);
			}
		}
	}
	/* We shouldn't try to get non-resource scripts from the remote server
	if ( Main->m_pScripts->m_tables.m_resources.GetSize() && !Main->m_dwResourcesOnly )
	{
		CStringArray saFiles;
		CStringArray saDirs;

		GetDirectory(csScpPath.GetAt(0) == 0x00 ? "." : csScpPath, &saFiles, &saDirs);
		for ( int i = 0; i < saFiles.GetSize(); i++ )
		{
			// Make sure the file isn't in the RESOURCES section
			CString csMessage;
			csMessage.Format("Checking for updated version of file %s", saFiles.GetAt(i));
			if ( m_pDlg )
			{
				m_pDlg->m_csMessage.SetWindowText(csMessage);
				m_pDlg->SetPos(0);
			}
			CString csFile = saFiles.GetAt(i);
			if ( csFile.Right(4) == ".scp")
			{
				CString csScript;
				if ( csScpPath.GetAt(0) != 0x00 )
				{
					if ( csScpPath.Find("/") != -1 )
						csScript.Format("%s/%s", csScpPath, csFile );
					else
						csScript.Format("%s\\%s", csScpPath, csFile);
				}
				else
					csScript = csFile;
				sprintf(szCmd, "GFCS%s", csScript);
				Send(szCmd, strlen(szCmd));
				memset(&szBuffer[0], 0x00, sizeof(szBuffer));
				Receive(&szBuffer[0], sizeof(szBuffer));
				iChk = atoi(szBuffer);
				CString csLocalFile;
				if ( m_pDlg )
					m_pDlg->m_csMessage.SetWindowText(csMessage);
				// csLocalFile never set????
				// What should it be?
				iTest = GetChecksum(csLocalFile);
				if ( iChk != iTest )
				{
					Main->m_log.Add("Checksum for file %s did not match...local = %ld, remote = %ld", csLocalFile, iTest, iChk);
					GetCustomFile(csScript);
				}
			}
		}
	}
	*/

}

unsigned int CRemoteConnection::GetChecksum(const char *pszFilename)
{
	if ( m_pDlg )
		m_pDlg->SetPos(0);
	unsigned int chk = 0;
	if ( pszFilename == NULL || pszFilename[0] == 0x00)
		return chk;
	CString csFilename = _T(pszFilename);
	csFilename.Replace(':', '!');
	Main->m_log.Add("Computing checksum for file %s", csFilename);
	// Use the same code from the server
	FILE * pFile;
	pFile = fopen( LPCTSTR(csFilename), "r" );
	if ( ! pFile )
		return chk;
	char szLine[8192];
	int line = 0;
	while ( !feof(pFile) )
	{
		fReadLine(pFile, szLine, sizeof(szLine));
		unsigned int linesum = 0;
		for ( int i = 0; i < (int) strlen(szLine); i++ )
		{
			char c = szLine[i];
			if ( c != 0x0d && c != 0x0a )
				linesum += (unsigned int) c;
		}
		chk += (linesum & line);
		line++;
	}
	fclose(pFile);
	return chk;
}

void CRemoteConnection::GetDirectory(const char * pszPath, CStringArray * pFilelist, CStringArray * pDirlist)
{
	CString csMessage;
	csMessage.Format("Retrieving directory listing for %s", pszPath);
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	if ( m_pDlg )
	{
		m_pDlg->m_csMessage.SetWindowText(csMessage);
		m_pDlg->SetPos(0);
	}
	if ( !this->m_bIsValid )
		return;
	Main->m_log.Add("Retrieving directory listing for %s", pszPath);
	char szCmd[MAX_BUFFER];
	sprintf(szCmd, "GDIR%s", pszPath);
	Send(szCmd, strlen(szCmd));
	char * pszBuffer = new char [MAX_BUFFER];
	memset(pszBuffer, 0x00, MAX_BUFFER);
	int iBytes = MAX_BUFFER;
	int iLen = 0;
	int iRead = 1;
	while ( iRead != 0 )
	{
		char szBuffer[MAX_BUFFER];
		ZeroMemory(szBuffer, sizeof(szBuffer));
		iRead = Receive(szBuffer, sizeof(szBuffer));
		if (iRead <= 0)
			break;
		if ( strstr(szBuffer, szEOF) != NULL )
			break;
		if ( iLen + iRead > iBytes )
		{
			char * pszNewBuffer = new char [MAX_BUFFER + iBytes];
			memset(pszNewBuffer, 0x00, MAX_BUFFER + iBytes);
			memcpy(pszNewBuffer, pszBuffer, iBytes);
			iBytes += MAX_BUFFER;
			delete [] pszBuffer;
			pszBuffer = pszNewBuffer;
		}
		memcpy(pszBuffer + iLen, &szBuffer[0], iRead);
		iLen += iRead;
	}
	// Grab the file info
	bool bContinue = true;
	char * pCur = pszBuffer;
	while (bContinue)
	{
		int iType;
		memcpy(&iType, pCur, sizeof(int));
		pCur+= sizeof(int);
		if ( iType != 0 )
		{
			char szFile[MAX_PATH];
			sprintf(szFile, pCur);
			pCur+= (strlen(szFile) + 1);
			CString csFile = _T(szFile);
			if ( iType & FILE_ATTRIBUTE_DIRECTORY )
				pDirlist->Add(csFile);
			else
				pFilelist->Add(csFile);
		}
		else
			bContinue = false;
	}
	delete [] pszBuffer;
	return;
}

void CRemoteConnection::DeleteProgressWindow()
{
	if ( m_pDlg && m_bDeleteWindow )
	{
		m_pDlg->EndDialog(0);
		delete m_pDlg;
	}
	m_pDlg = NULL;
}

void CRemoteConnection::SetDialog(CFileReadProgress * pDlg)
{
	DeleteProgressWindow(); 
	m_bDeleteWindow = false;
	m_pDlg = pDlg;
}

void CRemoteConnection::Initialize()
{
	Main->m_log.Add("Connecting to remote server on IP %s, port %ld", m_csIP, m_iRemotePort);
	m_bDeleteWindow = false;
	m_pDlg = NULL;
	if ( m_pDlg == NULL )
	{
		m_bDeleteWindow = true;
		m_pDlg = new CFileReadProgress;
		m_pDlg->Create(IDD_FILEREAD_PROGRESS);
	}
	if ( m_pDlg )
		m_pDlg->m_csMessage.SetWindowText("Connecting to remote server...");
	m_sock.m_hSocket = socket(AF_INET, SOCK_STREAM, 0);
	setsockopt(m_sock.m_hSocket, SOL_SOCKET, SO_DONTLINGER, "1", 1);
	if ( m_sock.Connect(m_csIP, m_iRemotePort) )
		m_bIsValid = true;
	else
		m_bIsValid = false;
	// For some reason, if we don't wait here, we'll likely kill hogsvr with a broken pipe.
	if ( m_bIsValid )
		m_bIsValid = this->Logon();
	else
	{
		CString csMessage;
		Main->m_log.Add("Unable to connect to axis server running on IP %s, port %ld", m_csIP, m_iRemotePort);
		csMessage.Format("Unable to connect to axissvr running on IP %s, port %ld.", m_csIP, m_iRemotePort );
		AfxMessageBox(csMessage, MB_OK | MB_ICONSTOP );
	}
}

void CRemoteConnection::ReadProfile(CString csProfile)
{
	CString csKey = _T("SOFTWARE\\Menasoft\\GM Tools\\Remote Profiles\\") + csProfile;
	m_iRemotePort = atoi(Main->GetRegistryString("HogPort", "4006", HKEY_LOCAL_MACHINE, csKey));
	m_csIP = Main->GetRegistryString("ServerIP", "", HKEY_LOCAL_MACHINE, csKey);
	m_csPassword = Encrypt(Main->GetRegistryString("Password", "", HKEY_LOCAL_MACHINE, csKey));
	m_csUsername = Main->GetRegistryString("Username", "", HKEY_LOCAL_MACHINE, csKey);
	m_iReceiveTimeout = Main->GetRegistryDword("ReceiveTimeout", 60000, HKEY_LOCAL_MACHINE, csKey);
}

void CRemoteConnection::UpdateFile(CString csRemoteName)
{
	if ( !m_bIsValid )
		return;
	CString csFile;
	if ( csRemoteName.GetAt(0) == '\\' || csRemoteName.GetAt(0) == '/' )
		csFile = csRemoteName.Mid(1);
	else
		csFile = csRemoteName;
	csFile.Replace('/', '\\');
	csFile.Replace(':', '!');
	// Is this a directory?
	if ( csFile.ReverseFind('\\') == csFile.GetLength() - 1)
		return;
	else
	{
		CString csLocalFile;
		csLocalFile.Format("%s\\%s", Main->GetProfileString(), csFile);
		char szCmd[MAX_PATH];
		sprintf(szCmd, "GFCS%s", csRemoteName);
		Send(szCmd, strlen(szCmd));
		char szBuffer[MAX_PATH];
		memset(&szBuffer[0], 0x00, sizeof(szBuffer));
		Receive(&szBuffer[0], sizeof(szBuffer));
		int iChk = atoi(szBuffer);
		CString csMessage;
		csMessage.Format("Checking for an updated version of %s", csRemoteName);
		if ( m_pDlg )
			m_pDlg->m_csMessage.SetWindowText(csMessage);
		int iTest = GetChecksum(csLocalFile);
		if ( iChk != iTest )
		{
			Main->m_log.Add("Checksum for file %s did not match...local = %ld, remote = %ld", csLocalFile, iTest, iChk);
			GetCustomFile(csRemoteName, csLocalFile);
		}
	}
}
