/*
 $Id: serverconfigdlg.cpp,v 1.5 2002/05/14 20:03:35 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// ServerConfigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "ServerConfigDlg.h"
#include "AddServer.h"
#include "RemoteConnection.h"
#include "RemoteConsole.h"
#include "EditValue.h"
#include "EditFlagsDlg.h"
#include "SpawnMessageDlg.h"
#include <direct.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// ServerConfigDlg dialog

const char * INIDescriptions [] =
{
	"UO File Path",	// 
	"Script File Path",
	"Save Path",
	"Log File Path",

	"Enable Profiling",
	"Start in Verbose Mode",
	"Number of Guests",
	"Start in Secure Mode",
	"Enable Hearall",
	"Save Period",
	"Map Cache Time",
	"Reagents Required",
	"Dungeon Light",
	"Monster Fight",
	"Speech File Path",
	"Item Decay Timer",
	"Max Bank Items",
	"Max Bank Weight",
	"Words of Power (Player)",
	"Words of Power (Staff)",
	"Dead Socket Timeout",
	"Debug Flags",
	"Guards Kill Instantly",
	"Show Arrive Depart Message",  // 25
	"Player Corpse Decay Timer",
	"NPC Corpse Decay Timer",	// 27
	"Snooping Criminal Chance",
	"Auto Account Creation",
	"Client Linger Time",
	"Freeze Restart Time",		// m_iFreezeRestartTime
	"Min Character Delete Time",
	"NPC Train Percent",			// m_iTrainSkillPercent
	"Client Version",
	"Central Login Server",			// m_sRegisterServer
	"Show Char Tags",					// m_fCharTags
	"Vendor max # of items to sell at once",			// m_iVendorMaxSell
	"Number of backup levels",				// m_iSaveBackupLevels
	"Background saves enabled",			// m_iSaveBackgroundTime
	"Force garbage collection",		// m_fSaveGarbageCollect
	"Game minute length",			// m_iGameMinuteLength
	"Weather effects off",				// m_fNoWeather
	"Casting while equipped allowed",				// m_fEquippedCast
	"Flip Dropped Items",			// m_fFlipDroppedItems
	"Stat Cap",					// m_iMaxSumOfStats
	"Stat Average",					// m_iAvgSumOfStats
	"Consume Reagents on Failure",			// m_fReagentLossFail
	"Run as NT Service",				// m_fUseNTService
	"Peer Server Poll Interval",				// m_iPollServers
	"Magic Unlock Door chance",		// m_iMagicUnlockDoor
	"Server is central login server",		// m_fMainLogServer
	"Murder min count",		// m_iMurderMinCount;		// amount of murders before we get title.
	"Murder decay time",		// m_iMurderDecayTime;
	"Max Clients",			// m_nClientsMax
	"Max characters per account",	// m_iMaxCharsPerAccount
	"Log File mask",				// GetLogMask
	"Nighttime light level",			// m_iLightNight
	"Max NPC Train Percent",			// m_iTrainSkillMax
	"Max Sector Complexity",			// m_iMaxComplexity
	"Time Zone",
	"Admin E-Mail",
	"Server URL",
	"Minimum neutral player karma",
	"Guard linger time",
	"Criminal timer",
	"Percent of Hitpoints on Ressurection",
	"Enable Monster Fear AI",
	"Account Files",
	"Looting is a crime",
	"Responsible owners",
	"Helping criminals is a crime",
	"Player ghost sounds",
	"Auto-Newbie keys",
	"Max base skill",
	"Running Penalty",
	"Stamina loss at weight",
};

CServerConfigDlg::CServerConfigDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CServerConfigDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(ServerConfigDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	
	this->m_csPollServers = "0";
	this->m_csRegisterServer = "menace.ne.mediaone.net";
	this->m_bMainLogServer = false;
	this->m_csMapCacheTime = "120";
	this->m_dwDebugFlags = 0;
	this->m_dwLogMask = 0;
	this->m_bSecure = false;
	this->m_csFreezeRestartTime = "100";
	this->m_bRequireReagents = true;
	this->m_bReagentLossFail = true;
	this->m_bWOPPlayer = true;
	this->m_bWOPStaff = true;
	this->m_csMagicUnlockDoor = "1000";
	this->m_csDecayTimer = "1800";
	this->m_csCorpsePlayerDecay = "2700";
	this->m_csCorpseNPCDecay = "900";
	this->m_csGuests = "0";
	this->m_bArriveDepartMsg = true;
	this->m_bFreeServer = false;
	this->m_csClientLinger = "60";
	this->m_csDeadSocketTime = "600";
	this->m_csMinCharDeleteTime = "259200";
	this->m_csMaxCharsPerAccount = "5";
	this->m_csBackupLevels = "3";
	this->m_csSavePeriod = "900";
	this->m_bForceGarbageCollect = false;
	this->m_csWebPageUpdate = "120";
	this->m_csWebClientListFormat = "<tr><td>%NAME%</td><td>%REGION.NAME%</td></tr>\\n";
	this->m_csWebServerListFormat = "<tr><td>%SERVNAME%</td><td>%STATUS%</td></tr>\\n";
	this->m_bMonsterFight = true;
	this->m_csDungeonLight = "17";
	this->m_csLightNight = "17";
	this->m_csBankMaxItems = "1000";
	this->m_csBankMaxWeight = "4000";
	this->m_bGuardsInstantKill = false;
	this->m_csSnoopCriminal = "500";
	this->m_csNPCTrainPercent = "50";
	this->m_csTrainSkillMax = "500";
	this->m_bCharTags = true;
	this->m_csVendorMaxSell = "30";
	this->m_csGameMinuteLength = "8";
	this->m_csSpeechFiles = "speech";
	this->m_bNoWeather = false;
	this->m_bFlipDroppedItems = true;
	this->m_csStatAvg = "300";
	this->m_csStatCap = "350";
	this->m_csMurderMinCount = "0";
	this->m_csMurderDecayTime = "28800";
	this->m_csMaxComplexity = "16";
	this->m_iTimeZone = 0;
	this->m_csPlayerNeutral = "-2000";
	this->m_csGuardLinger = "1";
	this->m_csCriminalTimer = "3";
	this->m_csHitpointPercentOnRez = "10";
	this->m_bMonsterFear = 0;
	this->m_csAcctFiles = "";
	this->m_bLootingIsACrime = true;
	this->m_bResponsibleOwners = true;
	this->m_bHelpingCriminalsIsACrime = true;
	this->m_bPlayerGhostSounds = true;
	this->m_bAutoNewbieKeys = true;
	this->m_csMaxBaseSkill = "250";
	this->m_csRunningPenalty = "50";
	this->m_csStaminaLossAtWeight = "90";
}


void CServerConfigDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CServerConfigDlg)
	DDX_Control(pDX, IDC_REMOVEVALUE, m_cbRemoveValue);
	DDX_Control(pDX, IDC_ADDVALUE, m_cbAddValue);
	DDX_Control(pDX, IDC_CONFIG_LIST, m_clcValues);
	DDX_Control(pDX, IDC_CONFIG_TREE, m_ctcSections);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CServerConfigDlg, CDialog)
	//{{AFX_MSG_MAP(CServerConfigDlg)
	ON_BN_CLICKED(IDOK, OnOk)
	ON_NOTIFY(TVN_SELCHANGED, IDC_CONFIG_TREE, OnSelchangedConfigTree)
	ON_BN_CLICKED(IDC_ADDVALUE, OnAddvalue)
	ON_BN_CLICKED(IDC_REMOVEVALUE, OnRemovevalue)
	ON_NOTIFY(NM_DBLCLK, IDC_CONFIG_LIST, OnDblclkConfigList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ServerConfigDlg message handlers

BOOL CServerConfigDlg::OnInitDialog() 
{
	CWaitCursor hourglass;
	CDialog::OnInitDialog();
	
	// Read the gray.ini file
	CStdioFile csfIniFile;
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CRemoteConnection remote;
		remote.GetCustomFile("grayini");
	}
	if (csfIniFile.Open(Main->GetProfileType() == PROFILE_REMOTE ? RMT_INI_FILE : INI_FILE, CFile::modeRead | CFile::shareDenyNone))
	{
		CString csLine;
		BOOL bStatus = true;
		while (bStatus)
		{
			bStatus = csfIniFile.ReadString(csLine);
			if (bStatus)
			{
				if ( csLine.Find("//") != -1 )
					csLine = csLine.SpanExcluding("//");
				if (csLine != "")
				{
					if (csLine.Find("[") != 0)
					{
						CString csTag, csValue;
						csTag = csLine.SpanExcluding("=");
						csValue = csLine.Mid(csLine.Find("=") + 1);
						LoadVal(csTag, csValue);
					}
					else
					{
						// This is either the SERVER or BLOCKIP section
						if ( csLine.Find("[SERVERS]") == 0 )
							bStatus = ReadServerInfo(csfIniFile);
						if ( csLine.Find("[BLOCKIP]") == 0 )
							bStatus = ReadBlockedIPs(csfIniFile);
						if ( csLine.Find("[WEBPAGE") == 0 )
						{
							CWebpageDef * pPage = new (CWebpageDef);
							ReadWebpageInfo(csfIniFile, pPage);
							m_oWebpages.Add( pPage );
						}
					}
				}
			}
		}
		csfIniFile.Close();
	}
	else
	{
		CString csMessage;
		csMessage.Format("Unable to open %s", INI_FILE);
		AfxMessageBox(csMessage, MB_OK | MB_ICONEXCLAMATION);
		return TRUE;
	}
	InitializeSections();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CServerConfigDlg::LoadVal(CString csTag, CString csValue)
{
	csTag.MakeUpper();
	switch (FindTable(csTag, knownINITags, 86))
	{
	case 0:	// FILES
	case 1: // MULFILES
		this->m_csUOFiles = csValue; break;
	case 2: // SCPFILES
		this->m_csScriptFiles = csValue; break;
	case 3: // WORLDSAVE
		this->m_csWorldSave = csValue; break;
	case 4: // LOG
		this->m_csLog = csValue; break;
	case 5: // PROFILE
		this->m_bProfile = atoi(csValue) == 0 ? false : true; break;
	case 6: // VERBOSE
		this->m_bVerbose = atoi(csValue) == 0 ? false : true; break;
	case 7: // GUESTS
		this->m_csGuests = csValue; break;
	case 8: // WEBPAGEFILE2
		break;
	case 9: // SECURE
		this->m_bSecure = atoi(csValue) == 0 ? false : true; break;
	case 10: // HEARALL
		this->m_bHearall = atoi(csValue) == 0 ? false : true; break;
	case 11: // SAVEPERIOD
		this->m_csSavePeriod = csValue; break;
	case 12: // MAPCACHETIME
		this->m_csMapCacheTime = csValue; break;
	case 13: // REAGENTSREQUIRED
		this->m_bRequireReagents = atoi(csValue) == 0 ? false : true; break;
	case 14: // DUNGEONLIGHT
		this->m_csDungeonLight = csValue; break;
	case 15: // MONSTERFIGHT
		this->m_bMonsterFight = atoi(csValue) == 0 ? false : true; break;
	case 16: // SPEECHFILES
		this->m_csSpeechFiles = csValue; break;
	case 17: // DECAYTIMER
		this->m_csDecayTimer = csValue; break;
	case 18: // BANKMAXITEMS
		this->m_csBankMaxItems = csValue; break;
	case 19: // BANKMAXWEIGHT
		this->m_csBankMaxWeight = csValue; break;
	case 20: // WOPPLAYER
		this->m_bWOPPlayer = atoi(csValue) == 0 ? false : true; break;
	case 21: // WOPSTAFF
		this->m_bWOPStaff = atoi(csValue) == 0 ? false : true; break;
	case 22: // DEADSOCKETTIME
		this->m_csDeadSocketTime = csValue; break;
	case 23: // DEBUGFLAGS
		if ( csValue[0] == '0' )
			this->m_dwDebugFlags = ahextoi(csValue);
		else
			this->m_dwDebugFlags = atoi(csValue);
		break;
	case 24: // GUARDSINSTANTKILL
		this->m_bGuardsInstantKill = atoi(csValue) == 0 ? false : true; break;
	case 25: // ARRIVEDEPARTMSG
		this->m_bArriveDepartMsg = atoi(csValue) == 0 ? false : true; break;
	case 26: // CORPSEPLAYERDECAY
		this->m_csCorpsePlayerDecay = csValue; break;
	case 27: // CORPSENPCDECAY
		this->m_csCorpseNPCDecay = csValue; break;
	case 28: // SNOOPCRIMINAL
		this->m_csSnoopCriminal = csValue; break;
	case 29: // FREESHARD
	case 30: // FREESERVER
		this->m_bFreeServer = atoi(csValue) == 0 ? false : true; break;
	case 31: // CLIENTLINGER
		this->m_csClientLinger = csValue; break;
	case 32: // FREEZERESTARTTIME
		this->m_csFreezeRestartTime = csValue; break;
	case 33: // WEBPAGEUPDATE
		break;
	case 34: // WEBPAGEFILE
		break;
	case 35: // MINCHARDELETETIME
		this->m_csMinCharDeleteTime = csValue; break;
	case 36: // NPCTRAINPERCENT
		this->m_csNPCTrainPercent = csValue; break;
	case 37: // CLIENTVERSION
		this->m_csClientVersion = csValue; break;
	case 38: // REGISTERSERVER
		this->m_csRegisterServer = csValue; break;
	case 39: // CHARTAGS
		this->m_bCharTags = atoi(csValue) == 0 ? false : true; break;
	case 40: // VENDORMAXSELL
		this->m_csVendorMaxSell = csValue; break;
	case 41: // BACKUPLEVELS
		this->m_csBackupLevels = csValue; break;
	case 42: // SAVEBACKGROUND
		this->m_bSaveBackground = atoi(csValue) == 0 ? false : true; break;
	case 43: // FORCEGARBAGECOLLECT
		this->m_bForceGarbageCollect = atoi(csValue) == 0 ? false : true; break;
	case 44: // GAMEMINUTELENGTH
		this->m_csGameMinuteLength = csValue; break;
	case 45: // NOWEATHER
		this->m_bNoWeather = atoi(csValue) == 0 ? false : true; break;
	case 46: // EQUIPPEDCAST
		this->m_bEquippedCast = atoi(csValue) == 0 ? false : true; break;
	case 47: // FLIPDROPPEDITEMS
		this->m_bFlipDroppedItems = atoi(csValue) == 0 ? false : true; break;
	case 48: // STATCAP
		this->m_csStatCap = csValue; break;
	case 49: // STATAVG
		this->m_csStatAvg = csValue; break;
	case 50: // REAGENTLOSSFAIL
		this->m_bReagentLossFail = atoi(csValue) == 0 ? false : true; break;
	case 51: // NTSERVICE
		this->m_bNTService = atoi(csValue) == 0 ? false : true; break;
	case 52: // POLLSERVERS
		this->m_csPollServers = csValue; break;
	case 53: // MAGICUNLOCKDOOR
		this->m_csMagicUnlockDoor = csValue; break;
	case 54: // MAINLOGSERVER
		this->m_bMainLogServer = atoi(csValue) == 0 ? false : true; break;
	case 55: // MURDERMINCOUNT
		this->m_csMurderMinCount = csValue; break;
	case 56: // MURDERDECAYTIME
		this->m_csMurderDecayTime = csValue; break;
	case 57: // CLIENTS
	case 58: // CLIENTMAX
		this->m_csClientsMax = csValue; break;
	case 59: // WEBCLIENTLISTFORM
		break;
	case 60: // WEBSERVERLISTFORM
		break;
	case 61: // MAXCHARSPERACCOUNT
		this->m_csMaxCharsPerAccount = csValue; break;
	case 62: // LOGMASK
		if ( csValue[0] == '0' )
			this->m_dwLogMask = ahextoi(csValue);
		else
			this->m_dwLogMask = atoi(csValue);
		break;
	case 63: // LIGHTNIGHT
		this->m_csLightNight = csValue; break;
	case 64: // NPCTRAINMAX
		this->m_csTrainSkillMax = csValue; break;
	case 65: // MAXCOMPLEXITY
		this->m_csMaxComplexity = csValue; break;
	case 66: // TZ
	case 67: // TIMEZONE
		this->m_iTimeZone = atoi(csValue); break;
	case 68: // EMAIL
	case 69: // ADMIN
	case 70: // ADMINEMAIL
		this->m_csAdminEMail = csValue; break;
	case 71: // URL
		this->m_csURL = csValue; break;
	case 72: // PLAYERNEUTRAL
		this->m_csPlayerNeutral = csValue; break;
	case 73: // GUARDLINGER
		this->m_csGuardLinger = csValue; break;
	case 74: // CRIMINALTIMER
		this->m_csCriminalTimer = csValue; break;
	case 75: // HITPOINTPERCENTONREZ
		this->m_csHitpointPercentOnRez = csValue; break;
	case 76: // MONSTERFEAR
		this->m_bMonsterFear = atoi(csValue) == 0 ? false : true; break;
	case 77: // ACCTFILES
		this->m_csAcctFiles = csValue; break;
	case 78: // LOOTINGISACRIME
		this->m_bLootingIsACrime = atoi(csValue) == 0 ? false : true; break;
	case 79: // RESPONSIBLEOWNERS
		this->m_bResponsibleOwners = atoi(csValue) == 0 ? false : true; break;
	case 80: // HELPINGCRIMINALSISACRIME
		this->m_bHelpingCriminalsIsACrime = atoi(csValue) == 0 ? false : true; break;
	case 81: // PLAYERGHOSTSOUNDS
		this->m_bPlayerGhostSounds = atoi(csValue) == 0 ? false : true; break;
	case 82: // AUTONEWBIEKEYS
		this->m_bAutoNewbieKeys = atoi(csValue) == 0 ? false : true; break;
	case 83: // MAXBASESKILL
		this->m_csMaxBaseSkill = csValue; break;
	case 84: // RUNNINGPENALTY
		this->m_csRunningPenalty = csValue; break;
	case 85: // STAMINALOSSATWEIGHT
		this->m_csStaminaLossAtWeight = csValue; break;
	default:
		break;
	}
}

BOOL CServerConfigDlg::ReadServerInfo(CStdioFile &csfFile)
{
	BOOL bStatus = true;
	CString csLine;
	int i = 0;
	while ( bStatus )
	{
		bStatus = csfFile.ReadString( csLine );
		if ( csLine == "" )
			break;
		// This should be the server name
		this->m_csServerName.InsertAt( i, csLine );
		bStatus = csfFile.ReadString( csLine );
		this->m_csServerIP.InsertAt( i, csLine );
		bStatus = csfFile.ReadString( csLine );
		this->m_csServerPort.InsertAt( i, csLine );
		i++;
	}
	return bStatus;
}

void CServerConfigDlg::OnOk() 
{
	CWaitCursor hourglass;
	if ( SaveConfig() )
	{
		CDialog::OnOK();
		return;
	}
}

bool CServerConfigDlg::SaveConfig()
{
	bool bTagHandled[86];
	for ( int i = 0; i < 86; i++ )
		bTagHandled[i] = false;
	CStdioFile csfIni, csfOld;
	if (!csfIni.Open(TMP_INI_FILE, CFile::modeWrite | CFile::modeCreate | CFile::shareDenyWrite))
	{
		AfxMessageBox("ERROR: Unable to open file sphereini.tmp for output.", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}
	if ( !csfOld.Open(Main->GetProfileType() == PROFILE_REMOTE ? RMT_INI_FILE : INI_FILE, CFile::modeRead | CFile::shareDenyNone))
	{
		AfxMessageBox("ERROR: Unable to open file sphere.ini for input.", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}
	BOOL bStatus = TRUE;
	while ( bStatus )
	{
		CString csLine;
		bStatus = csfOld.ReadString( csLine );
		if ( !bStatus )
			break;
		if ( csLine.Find("[SERVERS]") == 0 )
		{
			// We need to take care of the tags we haven't handled yet.
			for ( i = 0; i < 86; i++ )
			{
				if ( !bTagHandled[i] )
					WriteTag( &csfIni, knownINITags[i], "" );
			}
			// Now write the server list
			csfIni.WriteString("\n[SERVERS]\n");
			for ( int i = 0; i < this->m_csServerName.GetSize(); i++ )
			{
				csfIni.WriteString(this->m_csServerName.GetAt(i)); csfIni.WriteString("\n");
				csfIni.WriteString(this->m_csServerIP.GetAt(i)); csfIni.WriteString("\n");
				csfIni.WriteString(this->m_csServerPort.GetAt(i)); csfIni.WriteString("\n");
			}
			csfIni.WriteString("\n");
			// Now write the blocked IPs
			csfIni.WriteString("[BLOCKIP]\n");
			for ( i = 0; i < this->m_csaBlockedIPs.GetSize(); i++ )
			{
				csfIni.WriteString(this->m_csaBlockedIPs.GetAt(i));
				csfIni.WriteString("\n");
			}
			csfIni.WriteString("\n");
			// Finally, write the webpages
			for ( i = 0; i < this->m_oWebpages.GetSize(); i++ )
			{
				CWebpageDef * pPage = (CWebpageDef *) this->m_oWebpages.GetAt(i);
				if ( pPage )
				{
					CString csLine;
					csLine.Format("[WEBPAGE %ld]\n", i + 1);
					csfIni.WriteString(csLine);
					csLine.Format("WEBPAGEUPDATE=%ld\n", pPage->m_iUpdateInterval);
					csfIni.WriteString(csLine);
					csLine.Format("WEBPAGESRC=%s\n", pPage->m_csSrc);
					csfIni.WriteString(csLine);
					csLine.Format("WEBPAGEDST=%s\n", pPage->m_csDest);
					csfIni.WriteString(csLine);
					csLine.Format("WEBCLIENTLISTFORM=%s\n", pPage->m_csClientList);
					csfIni.WriteString(csLine);
					csLine.Format("WEBSERVERLISTFORM=%s\n", pPage->m_csServerList);
					csfIni.WriteString(csLine);
					csfIni.WriteString("\n");
				}
			}
			break;
		}
		if ( csLine == "" || csLine.Find("//") == 0 || csLine.Find("=") == -1)
		{
			csfIni.WriteString( csLine );
			csfIni.WriteString("\n");
			continue;
		}
		CString csTag;
		csTag = csLine.SpanExcluding("=");
		WriteTag( &csfIni, csTag, csLine );
		int index = FindTable( csTag, knownINITags, 86 );
		if ( index != -1 )
			bTagHandled[index] = true;
	}
	
	csfIni.Close();
	csfOld.Close();
	// Now we need to pause the server....
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		ReplaceFiles(TMP_INI_FILE, RMT_INI_FILE);
		CRemoteConnection remote;
		bool bSuccess = remote.SendCustomFile("grayini", RMT_INI_FILE);
		if (!bSuccess)
		{
			AfxMessageBox("Error occurred while sending file to the remote server.", MB_OK | MB_ICONEXCLAMATION);
			return false;
		}
	}
	else
	{
		CRemoteConsole * pConsole = Main->m_pRConsole;
		bool bDeleteConsole = false;
		if (pConsole == NULL)
		{
			pConsole = new (CRemoteConsole);
			bDeleteConsole = true;
		}
		pConsole->Pause();
		int i = 0;
		while (pConsole->m_bCommandPending)
		{
			i++;
			Sleep(50);
			if (i > 100)
				break;
		}
		ReplaceFiles(TMP_INI_FILE, INI_FILE);
		pConsole->Resync();
		i = 0;
		while (pConsole->m_bCommandPending)
		{
			i++;
			Sleep(50);
			if (i > 100)
				break;
		}
		if (bDeleteConsole)
			delete pConsole;
	}
	return true;

}

BOOL CServerConfigDlg::ReadBlockedIPs(CStdioFile &csfFile)
{
	BOOL bStatus = true;
	CString csLine;
	int i = 0;
	while ( bStatus )
	{
		bStatus = csfFile.ReadString( csLine );
		if ( csLine == "" )
			break;
		this->m_csaBlockedIPs.InsertAt( i, csLine );
		i++;
	}
	return bStatus;
}


void CServerConfigDlg::InitializeSections()
{
	this->m_ctcSections.InsertItem("Main Options", 0, 0, TVI_ROOT, TVI_LAST );
	this->m_ctcSections.InsertItem("Servers", 0, 0, TVI_ROOT, TVI_LAST );
	this->m_ctcSections.InsertItem("Blocked IPs", 0, 0, TVI_ROOT, TVI_LAST );
	for ( int i = 0; i < this->m_oWebpages.GetSize(); i++ )
	{
		CWebpageDef * pPage = (CWebpageDef *) this->m_oWebpages.GetAt(i);
		if ( pPage )
		{
			CString csTag;
			csTag.Format("Webpage %ld", i+1);
			HTREEITEM hItem = this->m_ctcSections.InsertItem(csTag, 0, 0, TVI_ROOT, TVI_LAST );
			this->m_ctcSections.SetItemData( hItem, (DWORD) pPage );
		}
	}
	return;
}

void CServerConfigDlg::DisplayMainOptions()
{
	this->m_cbAddValue.SetWindowText("Add Webpage");
	this->m_cbRemoveValue.EnableWindow( FALSE );

	CString csTempValue;

	this->m_clcValues.InsertColumn( 0, "Setting", LVCFMT_LEFT, 200, 0 );
	this->m_clcValues.InsertColumn( 1, "Value", LVCFMT_LEFT, 125, 1 );

	this->m_clcValues.InsertItem( 0, "Path to UO Mul Files", 0 );
	this->m_clcValues.SetItemText( 0, 1, m_csUOFiles );
	this->m_clcValues.InsertItem( 1, "Path to Script Files", 0 );
	this->m_clcValues.SetItemText( 1, 1, m_csScriptFiles );
	this->m_clcValues.InsertItem( 2, "Path to Sphere Save Files", 0 );
	this->m_clcValues.SetItemText( 2, 1, m_csWorldSave );
	this->m_clcValues.InsertItem( 3, "Path to Log Files", 0 );
	this->m_clcValues.SetItemText( 3, 1, m_csLog );
	this->m_clcValues.InsertItem( 4, "Enable Profiling", 0 );
	this->m_clcValues.SetItemText( 4, 1, m_bProfile ? "true" : "false" );
	this->m_clcValues.InsertItem( 5, "Start in Verbose Mode", 0 );
	this->m_clcValues.SetItemText( 5, 1, m_bVerbose ? "true" : "false" );
	this->m_clcValues.InsertItem( 6, "Number of Guest Accounts allowed", 0 );
	this->m_clcValues.SetItemText( 6, 1, m_csGuests );
	this->m_clcValues.InsertItem( 7, "Start in Secure Mode", 0 );
	this->m_clcValues.SetItemText( 7, 1, m_bSecure ? "true" : "false" );
	this->m_clcValues.InsertItem( 8, "Hear all that is said", 0 );
	this->m_clcValues.SetItemText( 8, 1, m_bHearall ? "true" : "false" );
	this->m_clcValues.InsertItem( 9, "Time between world saves (minutes)", 0 );
	this->m_clcValues.SetItemText( 9, 1, m_csSavePeriod );
	this->m_clcValues.InsertItem( 10, "Time to keep map sections in cache (seconds)", 0 );
	this->m_clcValues.SetItemText( 10, 1, m_csMapCacheTime );
	this->m_clcValues.InsertItem( 11, "Reagents required to cast spells", 0 );
	this->m_clcValues.SetItemText( 11, 1, m_bRequireReagents ? "true" : "false" );
	this->m_clcValues.InsertItem( 12, "Dungeon Light Level (0=bright 30=pitch black)", 0 );
	this->m_clcValues.SetItemText( 12, 1, m_csDungeonLight );
	this->m_clcValues.InsertItem( 13, "Monsters fight each other", 0 );
	this->m_clcValues.SetItemText( 13, 1, m_bMonsterFight ? "true" : "false" );
	this->m_clcValues.InsertItem( 14, "Path to Speech Files", 0 );
	this->m_clcValues.SetItemText( 14, 1, m_csSpeechFiles );
	this->m_clcValues.InsertItem( 15, "Item Decay Timer (minutes)", 0 );
	this->m_clcValues.SetItemText( 15, 1, m_csDecayTimer );
	this->m_clcValues.InsertItem( 16, "Maximum items allowed in bank boxes", 0 );
	this->m_clcValues.SetItemText( 16, 1, m_csBankMaxItems );
	this->m_clcValues.InsertItem( 17, "Maximum weight allowed in bank boxes", 0 );
	this->m_clcValues.SetItemText( 17, 1, m_csBankMaxWeight );
	this->m_clcValues.InsertItem( 18, "Words of Power displayed for Players", 0 );
	this->m_clcValues.SetItemText( 18, 1, m_bWOPPlayer ? "true" : "false" );
	this->m_clcValues.InsertItem( 19, "Words of Power displayed for Staff", 0 );
	this->m_clcValues.SetItemText( 19, 1, m_bWOPStaff ? "true" : "false" );
	this->m_clcValues.InsertItem( 20, "Dead Socket Timeout (minutes)", 0 );
	this->m_clcValues.SetItemText( 20, 1, m_csDeadSocketTime );
	this->m_clcValues.InsertItem( 21, "Debug Flags", 0 );
	csTempValue.Format("0x%04x", m_dwDebugFlags);
	this->m_clcValues.SetItemText( 21, 1, csTempValue );
	this->m_clcValues.InsertItem( 22, "Guards Kill Instantly", 0 );
	this->m_clcValues.SetItemText( 22, 1, m_bGuardsInstantKill ? "true" : "false" );
	this->m_clcValues.InsertItem( 23, "Show Arrive/Depart Messages", 0 );
	this->m_clcValues.SetItemText( 23, 1, m_bArriveDepartMsg ? "true" : "false" );
	this->m_clcValues.InsertItem( 24, "Player corpse decay time (minutes)", 0 );
	this->m_clcValues.SetItemText( 24, 1, m_csCorpsePlayerDecay );
	this->m_clcValues.InsertItem( 25, "NPC corpse decay time (minutes)", 0 );
	this->m_clcValues.SetItemText( 25, 1, m_csCorpseNPCDecay );
	this->m_clcValues.InsertItem( 26, "1 in # chance of getting flagged criminal when snooping", 0 );
	this->m_clcValues.SetItemText( 26, 1, m_csSnoopCriminal );
	this->m_clcValues.InsertItem( 27, "Automatically create new accounts", 0 );
	this->m_clcValues.SetItemText( 27, 1, m_bFreeServer ? "true" : "false" );
	this->m_clcValues.InsertItem( 28, "Time for players logged out in non-safe areas to remain in the world (seconds)", 0 );
	this->m_clcValues.SetItemText( 28, 1, m_csClientLinger );
	this->m_clcValues.InsertItem( 29, "Time to wait for a frozen loop before restarting (seconds)", 0 );
	this->m_clcValues.SetItemText( 29, 1, m_csFreezeRestartTime );
	this->m_clcValues.InsertItem( 30, "Minimum age of characters before deletion is allowed (seconds)", 0 );
	this->m_clcValues.SetItemText( 30, 1, m_csMinCharDeleteTime );
	this->m_clcValues.InsertItem( 31, "% of NPC Skill that they can teach up to (100 = 10.0; 1000 for up to their value)", 0 );
	this->m_clcValues.SetItemText( 31, 1, m_csNPCTrainPercent );
	this->m_clcValues.InsertItem( 32, "Client Version", 0 );
	this->m_clcValues.SetItemText( 32, 1, m_csClientVersion );
	this->m_clcValues.InsertItem( 33, "IP Address of server to register with (leave blank for none)", 0 );
	this->m_clcValues.SetItemText( 33, 1, m_csRegisterServer );
	this->m_clcValues.InsertItem( 34, "Display character tags ([NPC], [hallu], etc)", 0 );
	this->m_clcValues.SetItemText( 34, 1, m_bCharTags ? "true" : "false" );
	this->m_clcValues.InsertItem( 35, "Maximum number of an item that NPCs will sell at a time", 0 );
	this->m_clcValues.SetItemText( 35, 1, m_csVendorMaxSell );
	this->m_clcValues.InsertItem( 36, "Number of levels of backups to be kept (7 files per level)", 0 );
	this->m_clcValues.SetItemText( 36, 1, m_csBackupLevels );
	this->m_clcValues.InsertItem( 37, "Perform worldsaves in background", 0 );
	this->m_clcValues.SetItemText( 37, 1, m_bSaveBackground ? "true" : "false" );
	this->m_clcValues.InsertItem( 38, "Force garbage collection before worldsave", 0 );
	this->m_clcValues.SetItemText( 38, 1, m_bForceGarbageCollect ? "true" : "false" );
	this->m_clcValues.InsertItem( 39, "Length of game minute in realworld seconds", 0 );
	this->m_clcValues.SetItemText( 39, 1, m_csGameMinuteLength );
	this->m_clcValues.InsertItem( 40, "Weather Effects off", 0 );
	this->m_clcValues.SetItemText( 40, 1, m_bNoWeather ? "true" : "false" );
	this->m_clcValues.InsertItem( 41, "Allow players to have item equipped when casting", 0 );
	this->m_clcValues.SetItemText( 41, 1, m_bEquippedCast ? "true" : "false" );
	this->m_clcValues.InsertItem( 42, "Flip items when dropped", 0 );
	this->m_clcValues.SetItemText( 42, 1, m_bFlipDroppedItems ? "true" : "false" );
	this->m_clcValues.InsertItem( 43, "Maximum sum of stats (absolute) for a player", 0 );
	this->m_clcValues.SetItemText( 43, 1, m_csStatCap );
	this->m_clcValues.InsertItem( 44, "Maximum sum of stats before stat atrophy", 0 );
	this->m_clcValues.SetItemText( 44, 1, m_csStatAvg );
	this->m_clcValues.InsertItem( 45, "Reagents consumed upon spell failure", 0 );
	this->m_clcValues.SetItemText( 45, 1, m_bReagentLossFail ? "true" : "false" );
	this->m_clcValues.InsertItem( 46, "Run as a service under Windows NT", 0 );
	this->m_clcValues.SetItemText( 46, 1, m_bNTService ? "true" : "false" );
	this->m_clcValues.InsertItem( 47, "Peer server poll interval (minutes)", 0 );
	this->m_clcValues.SetItemText( 47, 1, m_csPollServers );
	this->m_clcValues.InsertItem( 48, "1 in n Chance of magically unlocking doors and chests (0 = no chance)", 0 );
	this->m_clcValues.SetItemText( 48, 1, m_csMagicUnlockDoor );
	this->m_clcValues.InsertItem( 49, "This server is a central log in server", 0 );
	this->m_clcValues.SetItemText( 49, 1, m_bMainLogServer ? "true" : "false" );
	this->m_clcValues.InsertItem( 50, "Minimum murder count before player receives 'murderer' title (0 = never)", 0 );
	this->m_clcValues.SetItemText( 50, 1, m_csMurderMinCount );
	this->m_clcValues.InsertItem( 51, "Time for murders to be forgotten (0 = never)", 0 );
	this->m_clcValues.SetItemText( 51, 1, m_csMurderDecayTime );
	this->m_clcValues.InsertItem( 52, "Maximum # of clients that can be connected at a time", 0 );
	this->m_clcValues.SetItemText( 52, 1, m_csClientsMax );
	this->m_clcValues.InsertItem( 53, "Maximum # of characters per account", 0 );
	this->m_clcValues.SetItemText( 53, 1, m_csMaxCharsPerAccount );
	this->m_clcValues.InsertItem( 54, "Log File Mask", 0 );
	csTempValue.Format("0x%04x", m_dwLogMask);
	this->m_clcValues.SetItemText( 54, 1, csTempValue );
	this->m_clcValues.InsertItem( 55, "Light level for nighttime (0=full daylight, 30=pitch black)", 0 );
	this->m_clcValues.SetItemText( 55, 1, m_csLightNight );
	this->m_clcValues.InsertItem( 56, "Maximum skill level for NPCs to train to (100 = 10.0)", 0 );
	this->m_clcValues.SetItemText( 56, 1, m_csTrainSkillMax );
	this->m_clcValues.InsertItem( 57, "Maximum number of spawned NPCs per sector", 0 );
	this->m_clcValues.SetItemText( 57, 1, m_csMaxComplexity );
	this->m_clcValues.InsertItem( 58, "Time Zone offset from GMT (EST = 5, PST = 8)", 0 );
	csTempValue.Format("%ld", m_iTimeZone);
	this->m_clcValues.SetItemText( 58, 1, csTempValue );
	this->m_clcValues.InsertItem( 59, "Administrator's e-mail address", 0 );
	this->m_clcValues.SetItemText( 59, 1, m_csAdminEMail );
	this->m_clcValues.InsertItem( 60, "Server URL", 0 );
	this->m_clcValues.SetItemText( 60, 1, m_csURL );
	this->m_clcValues.InsertItem( 61, "Minimum karma for a player to remain neutral", 0 );
	this->m_clcValues.SetItemText( 61, 1, m_csPlayerNeutral );
	this->m_clcValues.InsertItem( 62, "Time for called guards to linger (minutes)", 0 );
	this->m_clcValues.SetItemText( 62, 1, m_csGuardLinger );
	this->m_clcValues.InsertItem( 63, "Time for a criminal flag to be in effect (minutes)", 0 );
	this->m_clcValues.SetItemText( 63, 1, m_csCriminalTimer );
	this->m_clcValues.InsertItem( 64, "Percent of max hitpoints to heal on resurrection", 0 );
	this->m_clcValues.SetItemText( 64, 1, m_csHitpointPercentOnRez );
	this->m_clcValues.InsertItem( 65, "Enable Monster Fear AI", 0 );
	csTempValue.Format("%s", this->m_bMonsterFear ? "true" : "false" );
	this->m_clcValues.SetItemText( 65, 1, csTempValue );
	this->m_clcValues.InsertItem( 66, "Account Files path", 0 );
	this->m_clcValues.SetItemText( 66, 1, this->m_csAcctFiles );
	this->m_clcValues.InsertItem( 67, "Looting is a crime", 0 );
	csTempValue.Format("%s", this->m_bLootingIsACrime ? "true" : "false" );
	this->m_clcValues.SetItemText( 67, 1, csTempValue );
	this->m_clcValues.InsertItem( 68, "Owners are responsible for the actions of their pets/hirelings", 0 );
	csTempValue.Format("%s", this->m_bResponsibleOwners ? "true" : "false" );
	this->m_clcValues.SetItemText( 68, 1, csTempValue );
	this->m_clcValues.InsertItem( 69, "Helping Criminals is a crime", 0 );
	csTempValue.Format("%s", this->m_bHelpingCriminalsIsACrime ? "true" : "false" );
	this->m_clcValues.SetItemText( 69, 1, csTempValue );
	this->m_clcValues.InsertItem( 70, "Player ghosts make eerie sounds", 0 );
	csTempValue.Format("%s", this->m_bPlayerGhostSounds ? "true" : "false" );
	this->m_clcValues.SetItemText( 70, 1, csTempValue );
	this->m_clcValues.InsertItem( 71, "Automatical newbie keys", 0 );
	csTempValue.Format("%s", this->m_bAutoNewbieKeys ? "true" : "false" );
	this->m_clcValues.SetItemText( 71, 1, csTempValue );
	this->m_clcValues.InsertItem( 72, "Maximum base skills at character creation", 0 );
	this->m_clcValues.SetItemText( 72, 1, m_csMaxBaseSkill );
	this->m_clcValues.InsertItem( 73, "Penalty to stamina calculation for running", 0 );
	this->m_clcValues.SetItemText( 73, 1, m_csRunningPenalty );
	this->m_clcValues.InsertItem( 74, "Stamina loss begins when weight approaches this percent", 0 );
	this->m_clcValues.SetItemText( 74, 1, m_csStaminaLossAtWeight );

	return;
}

void CServerConfigDlg::DisplayServers()
{
	this->m_cbAddValue.SetWindowText("Add");
	this->m_cbRemoveValue.EnableWindow( TRUE );

	this->m_clcValues.InsertColumn( 0, "Server Name", LVCFMT_LEFT, 200, 0 );
	this->m_clcValues.InsertColumn( 1, "IP Address", LVCFMT_LEFT, 75, 1 );
	this->m_clcValues.InsertColumn( 2, "Port", LVCFMT_LEFT, 50, 2 );

	for ( int i = 0; i < this->m_csServerName.GetSize(); i++ )
	{
		this->m_clcValues.InsertItem( i, this->m_csServerName.GetAt(i), 0 );
		this->m_clcValues.SetItemText( i, 1, this->m_csServerIP.GetAt(i) );
		this->m_clcValues.SetItemText( i, 2, this->m_csServerPort.GetAt(i) );
	}
}

void CServerConfigDlg::DisplayBlockedIPs()
{
	this->m_cbAddValue.SetWindowText("Add");
	this->m_cbRemoveValue.EnableWindow( TRUE );

	this->m_clcValues.InsertColumn( 0, "IP Address", LVCFMT_LEFT, 125, 0 );

	for ( int i = 0; i < this->m_csaBlockedIPs.GetSize(); i++ )
		this->m_clcValues.InsertItem( i, this->m_csaBlockedIPs.GetAt(i), 0 );
}

void CServerConfigDlg::OnSelchangedConfigTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	// Determine which section has been selected
	HTREEITEM hSelItem = this->m_ctcSections.GetSelectedItem();
	CString csText = this->m_ctcSections.GetItemText( hSelItem );

	while ( this->m_clcValues.DeleteColumn( 0 ) );
	this->m_clcValues.DeleteAllItems();

	if ( csText == "Main Options" )
		DisplayMainOptions();
	else if ( csText == "Servers" )
		DisplayServers();
	else if ( csText == "Blocked IPs" )
		DisplayBlockedIPs();
	else
	{
		CWebpageDef * pPage = (CWebpageDef *) this->m_ctcSections.GetItemData(hSelItem);
		if ( pPage )
			DisplayWebpage( pPage );
	}
	*pResult = 0;
}

void CServerConfigDlg::OnAddvalue() 
{
	// Find out which section we are in
	HTREEITEM hSelItem = this->m_ctcSections.GetSelectedItem();
	CString csText = this->m_ctcSections.GetItemText( hSelItem );

	if ( csText == "Servers" )
		AddServer();
	else if ( csText == "Blocked IPs" )
		AddBlockedIP();
	else
	{
		// Adding a webpage
		CWebpageDef * pPage = new (CWebpageDef);
		if ( pPage )
		{
			int iIndex = this->m_oWebpages.Add(pPage);
			CString csName;
			csName.Format("Webpage %ld", iIndex+1);
			HTREEITEM hItem = this->m_ctcSections.InsertItem(csName, 0, 0, TVI_ROOT, TVI_LAST);
			this->m_ctcSections.SetItemData(hItem, (DWORD) pPage);
			this->m_ctcSections.SelectItem(hItem);
		}
	}
}

void CServerConfigDlg::OnRemovevalue() 
{
	// Find out which section we are in
	HTREEITEM hSelItem = this->m_ctcSections.GetSelectedItem();
	CString csText = this->m_ctcSections.GetItemText( hSelItem );

	if ( csText == "Main Options" )
		return;
	if ( csText == "Servers" )
		RemoveServer();
	if ( csText == "Blocked IPs" )
		RemoveBlockedIP();
	if ( csText.Find("Webpage") == 0 )
	{
		// Removing a webpage
		CWebpageDef * pPage = (CWebpageDef *) this->m_ctcSections.GetItemData(hSelItem);
		for ( int i = 0; i < this->m_oWebpages.GetSize(); i++)
		{
			CWebpageDef * pTest = (CWebpageDef *) this->m_oWebpages.GetAt(i);
			if ( pTest == pPage )
			{
				this->m_oWebpages.RemoveAt( i, 1 );
				delete pPage;
				break;
			}
		}
		this->m_ctcSections.DeleteAllItems();
		this->m_clcValues.DeleteAllItems();
		this->m_cbRemoveValue.EnableWindow(FALSE);
		this->m_cbAddValue.SetWindowText("Add Webpage");
		this->InitializeSections();
	}

}

void CServerConfigDlg::OnDblclkConfigList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Find out which section we are in
	HTREEITEM hSelItem = this->m_ctcSections.GetSelectedItem();
	CString csText = this->m_ctcSections.GetItemText( hSelItem );

	if ( csText == "Main Options" )
		EditMainOptions();
	else if ( csText == "Servers" )
		EditServers();
	else if ( csText == "Blocked IPs" )
		EditBlockedIPs();
	else
	{
		CWebpageDef * pPage = (CWebpageDef *) this->m_ctcSections.GetItemData( hSelItem );
		if ( pPage )
			EditWebpage(pPage);
	}
	*pResult = 0;
}

void CServerConfigDlg::EditMainOptions()
{
	// Find out which option is selected
	int iSel = this->m_clcValues.GetNextItem( -1, LVNI_SELECTED );
	if ( iSel == -1 )
		return;
	CString csValue = this->m_clcValues.GetItemText( iSel, 1 );
	if ( iSel != 21 && iSel != 54 )
	{
		CEditValueDlg dlg;
		dlg.m_csCaption = INIDescriptions[iSel];
		switch (iSel)
		{
		case 0:
		case 1:
		case 2:
		case 3:
		case 6:
		case 9:
		case 10:
		case 12:
		case 14:
		case 15:
		case 16:
		case 17:
		case 20:
		case 24:
		case 25:
		case 26:
		case 28:
		case 29:
		case 30:
		case 31:
		case 32:
		case 33:
		case 35:
		case 36:
		case 39:
		case 43:
		case 44:
		case 47:
		case 48:
		case 50:
		case 51:
		case 52:
		case 53:
		case 55:
		case 56:
		case 57:
		case 58:
		case 59:
		case 60:
		case 61:
		case 62:
		case 63:
		case 64:
		case 66:
		case 72:
		case 73:
		case 74:
			dlg.m_bTextMode = true;
			dlg.m_csTextValue = csValue;
			break;
		case 4:
		case 5:
		case 7:
		case 8:
		case 11:
		case 13:
		case 18:
		case 19:
		case 22:
		case 23:
		case 27:
		case 34:
		case 37:
		case 38:
		case 40:
		case 41:
		case 42:
		case 45:
		case 46:
		case 49:
		case 65:
		case 67:
		case 68:
		case 69:
		case 70:
		case 71:
			dlg.m_bTextMode = false;
			if ( csValue == "true" )
				dlg.m_bEnabled = true;
			else
				dlg.m_bEnabled = false;
			break;
		}
		if ( dlg.DoModal() == IDOK )
		{
			if ( dlg.m_bTextMode )
				csValue = dlg.m_csTextValue;
			else
				csValue = dlg.m_bEnabled ? "true" : "false";
			switch (iSel)
			{
			case 0:
				m_csUOFiles = csValue; break;
			case 1:
				m_csScriptFiles = csValue; break;
			case 2:
				m_csWorldSave = csValue; break;
			case 3:
				m_csLog = csValue; break;
			case 4:
				m_bProfile = dlg.m_bEnabled; break;
			case 5:
				m_bVerbose = dlg.m_bEnabled; break;
			case 6:
				m_csGuests = csValue; break;
			case 7:
				m_bSecure = dlg.m_bEnabled; break;
			case 8:
				m_bHearall = dlg.m_bEnabled; break;
			case 9:
				m_csSavePeriod = csValue; break;
			case 10:
				m_csMapCacheTime = csValue; break;
			case 11:
				m_bRequireReagents = dlg.m_bEnabled; break;
			case 12:
				m_csDungeonLight = csValue; break;
			case 13:
				m_bMonsterFight = dlg.m_bEnabled; break;
			case 14:
				m_csSpeechFiles = csValue; break;
			case 15:
				m_csDecayTimer = csValue; break;
			case 16:
				m_csBankMaxItems = csValue; break;
			case 17:
				m_csBankMaxWeight = csValue; break;
			case 18:
				m_bWOPPlayer = dlg.m_bEnabled; break;
			case 19:
				m_bWOPStaff = dlg.m_bEnabled; break;
			case 20:
				m_csDeadSocketTime = csValue; break;
			case 21:
				break;
			case 22:
				m_bGuardsInstantKill = dlg.m_bEnabled; break;
			case 23:
				m_bArriveDepartMsg = dlg.m_bEnabled; break;
			case 24:
				m_csCorpsePlayerDecay = csValue; break;
			case 25:
				m_csCorpseNPCDecay = csValue; break;
			case 26:
				m_csSnoopCriminal = csValue; break;
			case 27:
				m_bFreeServer = dlg.m_bEnabled; break;
			case 28:
				m_csClientLinger = csValue; break;
			case 29:
				m_csFreezeRestartTime = csValue; break;
			case 30:
				m_csMinCharDeleteTime = csValue; break;
			case 31:
				m_csNPCTrainPercent = csValue; break;
			case 32:
				m_csClientVersion = csValue; break;
			case 33:
				m_csRegisterServer = csValue; break;
			case 34:
				m_bCharTags = dlg.m_bEnabled; break;
			case 35:
				m_csVendorMaxSell = csValue; break;
			case 36:
				m_csBackupLevels = csValue; break;
			case 37:
				m_bSaveBackground = dlg.m_bEnabled; break;
			case 38:
				m_bForceGarbageCollect = dlg.m_bEnabled; break;
			case 39:
				m_csGameMinuteLength = csValue; break;
			case 40:
				m_bNoWeather = dlg.m_bEnabled; break;
			case 41:
				m_bEquippedCast = dlg.m_bEnabled; break;
			case 42:
				m_bFlipDroppedItems = dlg.m_bEnabled; break;
			case 43:
				m_csStatCap = csValue; break;
			case 44:
				m_csStatAvg = csValue; break;
			case 45:
				m_bReagentLossFail = dlg.m_bEnabled; break;
			case 46:
				m_bNTService = dlg.m_bEnabled; break;
			case 47:
				m_csPollServers = csValue; break;
			case 48:
				m_csMagicUnlockDoor = csValue; break;
			case 49:
				m_bMainLogServer = dlg.m_bEnabled; break;
			case 50:
				m_csMurderMinCount = csValue; break;
			case 51:
				m_csMurderDecayTime = csValue; break;
			case 52:
				m_csClientsMax = csValue; break;
			case 53:
				m_csMaxCharsPerAccount = csValue; break;
			case 54:
				break;
			case 55:
				m_csLightNight = csValue; break;
			case 56:
				m_csTrainSkillMax = csValue; break;
			case 57:
				m_csMaxComplexity = csValue; break;
			case 58:
				m_iTimeZone = atoi(csValue); break;
			case 59:
				m_csAdminEMail = csValue; break;
			case 60:
				m_csURL = csValue; break;
			case 61:
				m_csPlayerNeutral = csValue; break;
			case 62:
				m_csGuardLinger = csValue; break;
			case 63:
				m_csCriminalTimer = csValue; break;
			case 64:
				m_csHitpointPercentOnRez = csValue; break;
			case 65:
				m_bMonsterFear = dlg.m_bEnabled; break;
			case 66:
				m_csAcctFiles = csValue; break;
			case 67:
				m_bLootingIsACrime = dlg.m_bEnabled; break;
			case 68:
				m_bResponsibleOwners = dlg.m_bEnabled; break;
			case 69:
				m_bHelpingCriminalsIsACrime = dlg.m_bEnabled; break;
			case 70:
				m_bPlayerGhostSounds = dlg.m_bEnabled; break;
			case 71:
				m_bAutoNewbieKeys = dlg.m_bEnabled; break;
			case 72:
				m_csMaxBaseSkill = csValue; break;
			case 73:
				m_csRunningPenalty = csValue; break;
			case 74:
				m_csStaminaLossAtWeight = csValue; break;
			default:
				break;
			}
		}
		else
			return;
	}
	else
	{
		CEditFlagsDlg dlg;
		dlg.m_csCaption = INIDescriptions[iSel];
		switch (iSel)
		{
			case 21:
				dlg.m_iFlagTable = 0;
				dlg.m_dwFlags = this->m_dwDebugFlags;
				break;
			case 54:
				dlg.m_iFlagTable = 1;
				dlg.m_dwFlags = this->m_dwLogMask;
				break;
		}
		if ( dlg.DoModal() == IDOK )
		{
			switch (iSel)
			{
			case 21:
				this->m_dwDebugFlags = dlg.m_dwFlags;
				break;
			case 54:
				this->m_dwLogMask = dlg.m_dwFlags;
				break;
			}
			csValue.Format("0x%04x", dlg.m_dwFlags );
		}
		else
			return;
	}
	this->m_clcValues.SetItemText( iSel, 1, csValue );
	return;
}

void CServerConfigDlg::EditServers()
{
	int iSel = this->m_clcValues.GetNextItem( -1, LVNI_SELECTED );
	if ( iSel == -1 )
		return;
	CAddServer dlg;
	dlg.m_csServerName = this->m_clcValues.GetItemText( iSel, 0 );
	dlg.m_csServerIP = this->m_clcValues.GetItemText( iSel, 1 );
	dlg.m_csServerPort = this->m_clcValues.GetItemText( iSel, 2 );
	if ( dlg.DoModal() == IDOK )
	{
		this->m_clcValues.SetItemText( iSel, 0, dlg.m_csServerName );
		this->m_clcValues.SetItemText( iSel, 1, dlg.m_csServerIP );
		this->m_clcValues.SetItemText( iSel, 2, dlg.m_csServerPort );
		this->m_csServerName.SetAt( iSel, dlg.m_csServerName );
		this->m_csServerIP.SetAt( iSel, dlg.m_csServerIP );
		this->m_csServerPort.SetAt( iSel, dlg.m_csServerPort );
	}
}

void CServerConfigDlg::EditBlockedIPs()
{
	int iSel = this->m_clcValues.GetNextItem( -1, LVNI_SELECTED );
	if ( iSel == -1 )
		return;
	CString csValue = this->m_clcValues.GetItemText( iSel, 0 );
	CEditValueDlg dlg;
	dlg.m_csCaption = "Edit Blocked IP";
	dlg.m_csTextValue = csValue;
	dlg.m_bTextMode = true;
	if ( dlg.DoModal() == IDOK )
	{
		this->m_csaBlockedIPs.SetAt( iSel, dlg.m_csTextValue );
		this->m_clcValues.SetItemText( iSel, 0, dlg.m_csTextValue );
	}
}

void CServerConfigDlg::AddServer()
{
	CAddServer dlg;
	if ( dlg.DoModal() == IDOK )
	{
		int iCount = this->m_clcValues.GetItemCount();
		int iIndex = this->m_clcValues.InsertItem( iCount, dlg.m_csServerName, 0 );
		this->m_clcValues.SetItemText( iIndex, 1, dlg.m_csServerIP );
		this->m_clcValues.SetItemText( iIndex, 2, dlg.m_csServerPort );
		this->m_csServerName.Add( dlg.m_csServerName );
		this->m_csServerIP.Add( dlg.m_csServerIP );
		this->m_csServerPort.Add( dlg.m_csServerPort );
	}
}

void CServerConfigDlg::RemoveServer()
{
	POSITION pos = this->m_clcValues.GetFirstSelectedItemPosition();
	int iDeletes = 0;
	while ( pos != NULL )
	{
		int iSel = this->m_clcValues.GetNextSelectedItem( pos );
		this->m_clcValues.DeleteItem( iSel - iDeletes );
		this->m_csServerName.RemoveAt( iSel - iDeletes );
		this->m_csServerIP.RemoveAt( iSel - iDeletes );
		this->m_csServerPort.RemoveAt( iSel -iDeletes );
		iDeletes++;
	}
}

void CServerConfigDlg::AddBlockedIP()
{
	CEditValueDlg dlg;
	dlg.m_csCaption = "Add Blocked IP";
	dlg.m_bTextMode = true;
	if ( dlg.DoModal() == IDOK )
	{
		int iCount = this->m_clcValues.GetItemCount();
		int iIndex = this->m_clcValues.InsertItem( iCount, dlg.m_csTextValue, 0 );
		this->m_csaBlockedIPs.Add( dlg.m_csTextValue );
	}
}

void CServerConfigDlg::RemoveBlockedIP()
{
	POSITION pos = this->m_clcValues.GetFirstSelectedItemPosition();
	int iDeletes = 0;
	while ( pos != NULL )
	{
		int iSel = this->m_clcValues.GetNextSelectedItem( pos );
		this->m_clcValues.DeleteItem( iSel - iDeletes );
		this->m_csaBlockedIPs.RemoveAt( iSel - iDeletes );
		iDeletes++;
	}
}

void CServerConfigDlg::WriteTag(CStdioFile * file, CString csTag, CString csLine)
{
	CString csOutput;
	switch (FindTable( csTag, knownINITags, 86 ))
	{
	case 0:
		csOutput.Format("%s=%s\n", csTag, this->m_csUOFiles); break;
	case 1:
		break;
	case 2:
		csOutput.Format("%s=%s\n", csTag, this->m_csScriptFiles); break;
	case 3:
		csOutput.Format("%s=%s\n", csTag, this->m_csWorldSave); break;
	case 4:
		csOutput.Format("%s=%s\n", csTag, this->m_csLog); break;
	case 5:
		csOutput.Format("%s=%ld\n", csTag, this->m_bProfile ? 1 : 0); break;
	case 6:
		csOutput.Format("%s=%ld\n", csTag, this->m_bVerbose ? 1 : 0); break;
	case 7:
		csOutput.Format("%s=%s\n", csTag, this->m_csGuests); break;
	case 8:
		break;
	case 9:
		csOutput.Format("%s=%ld\n", csTag, this->m_bSecure ? 1 : 0); break;
	case 10:
		csOutput.Format("%s=%ld\n", csTag, this->m_bHearall ? 1 : 0); break;
	case 11:
		csOutput.Format("%s=%s\n", csTag, this->m_csSavePeriod); break;
	case 12:
		csOutput.Format("%s=%s\n", csTag, this->m_csMapCacheTime); break;
	case 13:
		csOutput.Format("%s=%ld\n", csTag, this->m_bRequireReagents ? 1 : 0); break;
	case 14:
		csOutput.Format("%s=%s\n", csTag, this->m_csDungeonLight); break;
	case 15:
		csOutput.Format("%s=%ld\n", csTag, this->m_bMonsterFight ? 1 : 0); break;
	case 16:
		csOutput.Format("%s=%s\n", csTag, this->m_csSpeechFiles); break;
	case 17:
		csOutput.Format("%s=%s\n", csTag, this->m_csDecayTimer); break;
	case 18:
		csOutput.Format("%s=%s\n", csTag, this->m_csBankMaxItems); break;
	case 19:
		csOutput.Format("%s=%s\n", csTag, this->m_csBankMaxWeight); break;
	case 20:
		csOutput.Format("%s=%ld\n", csTag, this->m_bWOPPlayer ? 1 : 0); break;
	case 21:
		csOutput.Format("%s=%ld\n", csTag, this->m_bWOPStaff ? 1 : 0); break;
	case 22:
		csOutput.Format("%s=%s\n", csTag, this->m_csDeadSocketTime); break;
	case 23:
		csOutput.Format("%s=0%04x\n", csTag, this->m_dwDebugFlags); break;
	case 24:
		csOutput.Format("%s=%ld\n", csTag, this->m_bGuardsInstantKill ? 1 : 0); break;
	case 25:
		csOutput.Format("%s=%ld\n", csTag, this->m_bArriveDepartMsg ? 1 : 0); break;
	case 26:
		csOutput.Format("%s=%s\n", csTag, this->m_csCorpsePlayerDecay); break;
	case 27:
		csOutput.Format("%s=%s\n", csTag, this->m_csCorpseNPCDecay); break;
	case 28:
		csOutput.Format("%s=%s\n", csTag, this->m_csSnoopCriminal); break;
	case 29:
		csOutput.Format("%s=%s\n", csTag, this->m_bFreeServer); break;
	case 30:
		break;
	case 31:
		csOutput.Format("%s=%s\n", csTag, this->m_csClientLinger); break;
	case 32:
		csOutput.Format("%s=%s\n", csTag, this->m_csFreezeRestartTime); break;
	case 33:
		break;
	case 34:
		break;
	case 35:
		csOutput.Format("%s=%s\n", csTag, this->m_csMinCharDeleteTime); break;
	case 36:
		csOutput.Format("%s=%s\n", csTag, this->m_csNPCTrainPercent); break;
	case 37:
		csOutput.Format("%s=%s\n", csTag, this->m_csClientVersion); break;
	case 38:
		csOutput.Format("%s=%s\n", csTag, this->m_csRegisterServer); break;
	case 39:
		csOutput.Format("%s=%ld\n", csTag, this->m_bCharTags ? 1 : 0); break;
	case 40:
		csOutput.Format("%s=%s\n", csTag, this->m_csVendorMaxSell); break;
	case 41:
		csOutput.Format("%s=%s\n", csTag, this->m_csBackupLevels); break;
	case 42:
		csOutput.Format("%s=%ld\n", csTag, this->m_bSaveBackground ? 1 : 0); break;
	case 43:
		csOutput.Format("%s=%ld\n", csTag, this->m_bForceGarbageCollect ? 1 : 0); break;
	case 44:
		csOutput.Format("%s=%s\n", csTag, this->m_csGameMinuteLength); break;
	case 45:
		csOutput.Format("%s=%ld\n", csTag, this->m_bNoWeather ? 1 : 0); break;
	case 46:
		csOutput.Format("%s=%ld\n", csTag, this->m_bEquippedCast ? 1 : 0); break;
	case 47:
		csOutput.Format("%s=%ld\n", csTag, this->m_bFlipDroppedItems ? 1 : 0); break;
	case 48:
		csOutput.Format("%s=%s\n", csTag, this->m_csStatCap); break;
	case 49:
		csOutput.Format("%s=%s\n", csTag, this->m_csStatAvg); break;
	case 50:
		csOutput.Format("%s=%ld\n", csTag, this->m_bReagentLossFail ? 1 : 0); break;
	case 51:
		csOutput.Format("%s=%ld\n", csTag, this->m_bNTService ? 1 : 0); break;
	case 52:
		csOutput.Format("%s=%s\n", csTag, this->m_csPollServers); break;
	case 53:
		csOutput.Format("%s=%s\n", csTag, this->m_csMagicUnlockDoor); break;
	case 54:
		csOutput.Format("%s=%ld\n", csTag, this->m_bMainLogServer ? 1 : 0); break;
	case 55:
		csOutput.Format("%s=%s\n", csTag, this->m_csMurderMinCount); break;
	case 56:
		csOutput.Format("%s=%s\n", csTag, this->m_csMurderDecayTime); break;
	case 57:
		break;
	case 58:
		csOutput.Format("%s=%s\n", csTag, this->m_csClientsMax); break;
	case 59:
		break;
	case 60:
		break;
	case 61:
		csOutput.Format("%s=%s\n", csTag, this->m_csMaxCharsPerAccount); break;
	case 62:
		csOutput.Format("%s=0%04x\n", csTag, this->m_dwLogMask); break;
	case 63:
		csOutput.Format("%s=%s\n", csTag, this->m_csLightNight); break;
	case 64:
		csOutput.Format("%s=%s\n", csTag, this->m_csTrainSkillMax); break;
	case 65:
		csOutput.Format("%s=%s\n", csTag, this->m_csMaxComplexity); break;
	case 66:
		break;
	case 67:
		csOutput.Format("%s=%ld\n", csTag, this->m_iTimeZone); break;
	case 68:
		break;
	case 69:
		break;
	case 70:
		csOutput.Format("%s=%s\n", csTag, this->m_csAdminEMail); break;
	case 71:
		csOutput.Format("%s=%s\n", csTag, this->m_csURL); break;
	case 72:
		csOutput.Format("%s=%s\n", csTag, this->m_csPlayerNeutral); break;
	case 73:
		csOutput.Format("%s=%s\n", csTag, this->m_csGuardLinger); break;
	case 74:
		csOutput.Format("%s=%s\n", csTag, this->m_csCriminalTimer); break;
	case 75:
		csOutput.Format("%s=%s\n", csTag, this->m_csHitpointPercentOnRez); break;
	case 76:
		csOutput.Format("%s=%ld\n", csTag, this->m_bMonsterFear ? 1 : 0); break;
	case 77:
		csOutput.Format("%s=%s\n", csTag, this->m_csAcctFiles); break;
	case 78:
		csOutput.Format("%s=%ld\n", csTag, this->m_bLootingIsACrime ? 1 : 0); break;
	case 79:
		csOutput.Format("%s=%ld\n", csTag, this->m_bResponsibleOwners ? 1 : 0); break;
	case 80:
		csOutput.Format("%s=%ld\n", csTag, this->m_bHelpingCriminalsIsACrime ? 1 : 0); break;
	case 81:
		csOutput.Format("%s=%ld\n", csTag, this->m_bPlayerGhostSounds ? 1 : 0); break;
	case 82:
		csOutput.Format("%s=%ld\n", csTag, this->m_bAutoNewbieKeys ? 1 : 0); break;
	case 83:
		csOutput.Format("%s=%s\n", csTag, this->m_csMaxBaseSkill); break;
	case 84:
		csOutput.Format("%s=%s\n", csTag, this->m_csRunningPenalty); break;
	case 85:
		csOutput.Format("%s=%s\n", csTag, this->m_csStaminaLossAtWeight); break;
	default:
		csOutput.Format("%s\n", csLine );
		break;
	}
	if ( csOutput != "" )
		file->WriteString(csOutput);
}

BOOL CServerConfigDlg::ReadWebpageInfo(CStdioFile &csfInput, CWebpageDef *pPage)
{
	BOOL bStatus = true;
	CString csLine;
	while ( bStatus )
	{
		bStatus = csfInput.ReadString( csLine );
		if ( !bStatus )
			break;
		if ( csLine == "" )
			break;
		CString csTag = csLine.SpanExcluding("=");
		CString csValue;
		if ( csTag.GetLength() < ( csLine.GetLength() + 1 ))
			csValue = csLine.Mid(csTag.GetLength() + 1);
		switch(FindTable(csTag, knownINITags, 86, sizeof(char*)))
		{
		case 8:
			// source file
			pPage->m_csSrc = csValue; break;
		case 33:
			// Update Interval
			pPage->m_iUpdateInterval = atoi(csValue); break;
		case 34:
			// destination page
			pPage->m_csDest = csValue; break;
		case 59:
			// Client list
			pPage->m_csClientList = csValue; break;
		case 60:
			// Server list
			pPage->m_csServerList = csValue; break;
		default:
			break;
		}
	}
	return bStatus;

}

CServerConfigDlg::~CServerConfigDlg()
{
	for ( int i = 0; i < this->m_oWebpages.GetSize(); i++ )
	{
		CWebpageDef * pPage = (CWebpageDef *) this->m_oWebpages.GetAt( i );
		if ( pPage )
			delete pPage;
	}
	this->m_oWebpages.RemoveAll();
	m_csServerName.RemoveAll();
	m_csServerIP.RemoveAll();
	m_csServerPort.RemoveAll();
	m_csaBlockedIPs.RemoveAll();
}

void CServerConfigDlg::DisplayWebpage(CWebpageDef *pPage)
{
	this->m_cbAddValue.SetWindowText("Add Webpage");
	this->m_cbRemoveValue.EnableWindow( TRUE );

	this->m_clcValues.InsertColumn( 0, "Setting", LVCFMT_LEFT, 100, 0 );
	this->m_clcValues.InsertColumn( 1, "Value", LVCFMT_LEFT, 250, 1 );

	CString csTemp;
	this->m_clcValues.InsertItem( 0, "Update Interval", 0 );
	csTemp.Format("%ld", pPage->m_iUpdateInterval);
	this->m_clcValues.SetItemText( 0, 1, csTemp );
	this->m_clcValues.InsertItem( 1, "Template File", 0 );
	this->m_clcValues.SetItemText( 1, 1, pPage->m_csSrc );
	this->m_clcValues.InsertItem( 2, "Destination File", 0 );
	this->m_clcValues.SetItemText( 2, 1, pPage->m_csDest );
	this->m_clcValues.InsertItem( 3, "Client list format", 0 );
	this->m_clcValues.SetItemText( 3, 1, pPage->m_csClientList );
	this->m_clcValues.InsertItem( 4, "Server list format", 0 );
	this->m_clcValues.SetItemText( 4, 1, pPage->m_csServerList );
}

void CServerConfigDlg::EditWebpage(CWebpageDef *pPage)
{
	int iSel = this->m_clcValues.GetNextItem( -1, LVNI_SELECTED );
	if ( iSel == -1 )
		return;
	CString csValue = this->m_clcValues.GetItemText( iSel, 1 );
	CEditValueDlg dlg;
	CString csCaption;
	switch ( iSel )
	{
	case 0:
		csCaption = "Update Interval (minutes)"; break;
	case 1:
		csCaption = "Webpage Source (template)"; break;
	case 2:
		csCaption = "Webpage Output File (local path)"; break;
	case 3:
		csCaption = "Client List entry format (HTML)"; break;
	case 4:
		csCaption = "Server List entry format (HTML)"; break;
	}
	dlg.m_csCaption = csCaption;
	dlg.m_csTextValue = csValue;
	dlg.m_bTextMode = true;
	if ( dlg.DoModal() == IDOK )
	{
		this->m_clcValues.SetItemText( iSel, 1, dlg.m_csTextValue );
		switch ( iSel )
		{
		case 0:
			pPage->m_iUpdateInterval = atoi(dlg.m_csTextValue); break;
		case 1:
			pPage->m_csSrc = dlg.m_csTextValue; break;
		case 2:
			pPage->m_csDest = dlg.m_csTextValue; break;
		case 3:
			pPage->m_csClientList = dlg.m_csTextValue; break;
		case 4:
			pPage->m_csServerList = dlg.m_csTextValue; break;
		}
	}
}