/*
 $Id: traveltab.cpp,v 1.30 2003/01/06 07:04:50 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// TravelTab.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "FileReadProgress.h"
#include "TravelTab.h"
#include "AddDestination.h"
#include "WorldMapDlg.h"
#include "RemoteConnection.h"
#include "SpawnList.h"
#include "EditGraymapDlg.h"
#include "MapCoordinatesDlg.h"
#include "SearchCritDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTravelTab property page

IMPLEMENT_DYNCREATE(CTravelTab, CPropertyPage)

int CTravelTab::iLocSort = 1;
int CTravelTab::iCoordSort = 1;

CTravelTab::CTravelTab() : CPropertyPage(CTravelTab::IDD)
{
	//{{AFX_DATA_INIT(CTravelTab)
	//}}AFX_DATA_INIT
	this->m_iLastCustLocIndex = 0;
	m_bSortOrder = false;	// false is descending
	m_bWorldMapCreated = false;
	m_bWorldMapVisible = false;
	m_pEditDlg = NULL;
	m_bColorsLoaded = false;
	m_crosshairIndex = -1;
	m_iCatSeq = 0;
}

CTravelTab::~CTravelTab()
{
	Unload();
}

void CTravelTab::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTravelTab)
	DDX_Control(pDX, IDC_MAPPLANE, m_ceMapPlane);
	DDX_Control(pDX, IDC_EDIT_LOC, m_cbEditLoc);
	DDX_Control(pDX, IDC_LOCALSPAWNS, m_cbLocalSpawns);
	DDX_Control(pDX, IDC_LISTSPAWNS, m_cbWorldSpawns);
	DDX_Control(pDX, IDC_COORDS, m_csCoords);
	DDX_Control(pDX, IDC_LOCATIONS, m_clcLocations);
	DDX_Control(pDX, IDC_CATEGORYTREE, m_ctcCategoryTree);
	DDX_Control(pDX, IDC_REMOVE_LOC, m_cbRemoveLoc);
	DDX_Control(pDX, IDC_TRAVELMAP, m_TravelMap);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTravelTab, CPropertyPage)
	//{{AFX_MSG_MAP(CTravelTab)
	ON_BN_CLICKED(IDC_GO, OnGo)
	ON_BN_CLICKED(IDC_SEND, OnSend)
	ON_BN_CLICKED(IDC_ADD_LOC, OnAddLoc)
	ON_BN_CLICKED(IDC_REMOVE_LOC, OnRemoveLoc)
	ON_NOTIFY(TVN_SELCHANGED, IDC_CATEGORYTREE, OnSelchangedCategorytree)
	ON_NOTIFY(NM_CLICK, IDC_LOCATIONS, OnClickLocations)
	ON_NOTIFY(LVN_COLUMNCLICK, IDC_LOCATIONS, OnColumnclickLocations)
	ON_WM_LBUTTONDOWN()
	ON_BN_CLICKED(IDC_WHERE, OnWhere)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_LOCATIONS, OnItemchangedLocations)
	ON_BN_CLICKED(IDC_LISTSPAWNS, OnListspawns)
	ON_BN_CLICKED(IDC_LOCALSPAWNS, OnLocalspawns)
	ON_BN_CLICKED(IDC_EDITGRAYMAP, OnEditgraymap)
	ON_BN_CLICKED(IDC_RELOADSPAWNS, OnReloadspawns)
	ON_BN_CLICKED(IDC_EDIT_LOC, OnEditLoc)
	ON_WM_RBUTTONDOWN()
	ON_BN_CLICKED(IDC_MOREP, OnMorep)
	ON_BN_CLICKED(IDC_ZOOMOUT, OnZoomout)
	ON_BN_CLICKED(IDC_ZOOMIN, OnZoomin)
	ON_BN_CLICKED(IDC_WORLDMAP, OnWorldmap)
	ON_BN_CLICKED(IDC_FIND, OnFind)
	ON_EN_CHANGE(IDC_MAPPLANE, OnChangeMapplane)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTravelTab message handlers

BOOL CTravelTab::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	// Set up the travel map window
	int X, Y, zoom;
	X = Main->GetRegistryDword("TravelTab Center X", 3077);
	Y = Main->GetRegistryDword("TravelTab Center Y", 2048);
	zoom = Main->GetRegistryDword("TravelTab Zoom", 0);

	m_TravelMap.SetZoomLevel((short) zoom);
	m_TravelMap.SetDrawStatics(1);
	Recenter((short) X, (short) Y);

	LoadScripts();
	this->m_clcLocations.InsertColumn(0, "Description", LVCFMT_LEFT, 180, -1);
	this->m_clcLocations.InsertColumn(1, "Coordinates", LVCFMT_LEFT, 100, -1);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTravelTab::OnGo() 
{
	CString csVal;
	this->m_csCoords.GetWindowText(csVal);
	CString csCmd;
	CString csPlane;
	m_ceMapPlane.GetWindowText(csPlane);
	if ( Main->m_bSeerMode )
	{
		csVal.Replace(',', ' ');
		csCmd.Format(".tp (%s)", csVal);
	}
	else
	{
		if ( csPlane != "" )
			csCmd.Format(".go %s, %s", csVal, csPlane);
		else
			csCmd.Format(".go %s", csVal);
	}
	SendToUO(csCmd);
}

void CTravelTab::OnSend() 
{
	CString csVal, csCmd;
	this->m_csCoords.GetWindowText(csVal);
	CString csPlane;
	m_ceMapPlane.GetWindowText(csPlane);
	if ( Main->m_bSeerMode )
	{
		AfxMessageBox("You can't send others while in Seer mode.  =(", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	else
	{
		if ( csPlane != "" )
			csCmd.Format(".x.go %s, %s", csVal, csPlane);
		else
			csCmd.Format(".x.go %s", csVal);
	}
	SendToUO(csCmd);
}

void CTravelTab::OnAddLoc() 
{
	CAddDestination dlgAdd;
	dlgAdd.m_pCategoryList = &Main->m_pScripts->m_olLocations;
	// Compare the selected coords with the displayed coords
	int iSel = m_clcLocations.GetNextItem(-1, LVNI_SELECTED);
	if (iSel != -1)
	{
		CString csSelectedCoords = m_clcLocations.GetItemText(iSel, 1);
		CString csDisplayedCoords;
		m_csCoords.GetWindowText(csDisplayedCoords);
		if (csDisplayedCoords != csSelectedCoords)
		{
			// Pass these off to the dialog.
			dlgAdd.m_csXCoord = csDisplayedCoords.Left(csDisplayedCoords.Find(","));
			dlgAdd.m_csYCoord = csDisplayedCoords.Mid(csDisplayedCoords.Find(",") + 1);
			dlgAdd.m_csZCoord = dlgAdd.m_csYCoord.Mid(dlgAdd.m_csYCoord.Find(",") + 1);
			dlgAdd.m_csYCoord = dlgAdd.m_csYCoord.SpanExcluding(",");
			dlgAdd.m_csPlane = dlgAdd.m_csZCoord.Mid(dlgAdd.m_csZCoord.Find(",") + 1);
			dlgAdd.m_csZCoord = dlgAdd.m_csZCoord.SpanExcluding(",");

		}
	}
	else
	{
		CString csDisplayedCoords;
		m_csCoords.GetWindowText(csDisplayedCoords);
		dlgAdd.m_csXCoord = csDisplayedCoords.Left(csDisplayedCoords.Find(","));
		dlgAdd.m_csYCoord = csDisplayedCoords.Mid(csDisplayedCoords.Find(",") + 1);
		dlgAdd.m_csZCoord = dlgAdd.m_csYCoord.Mid(dlgAdd.m_csYCoord.Find(",") + 1);
		dlgAdd.m_csYCoord = dlgAdd.m_csYCoord.SpanExcluding(",");
		dlgAdd.m_csPlane = dlgAdd.m_csZCoord.Mid(dlgAdd.m_csZCoord.Find(",") + 1);
		dlgAdd.m_csZCoord = dlgAdd.m_csZCoord.SpanExcluding(",");
	}
	if (dlgAdd.DoModal() == IDOK)
	{
		CString csCategory = dlgAdd.m_csCategory;
		CString csSubsection = dlgAdd.m_csSubsection;
		CString csDescription = dlgAdd.m_csDescription;
		CString csX = dlgAdd.m_csXCoord;
		CString csY = dlgAdd.m_csYCoord;
		CString csZ = dlgAdd.m_csZCoord;
		CString csP = dlgAdd.m_csPlane;
		CString csPoint;
		csPoint.Format("%s,%s,%s,%s", csX, csY, csZ, csP);
		// Make sure an entry with this name doesn't already exist
		CCategory * pCategory = FindCategory(&Main->m_pScripts->m_olLocations, csCategory);
		CSubsection * pSubsection = NULL;
		CSObject * pObj = NULL;
		if (pCategory != NULL)
		{
			pSubsection = FindSubsection(pCategory, csSubsection);
			if (pSubsection != NULL)
			{
				POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
				while ( pos )
				{
					CSObject * pObject = (CSObject *) pSubsection->m_ItemList.GetNext(pos);
					if ( pObject )
					{
						CLocation * pLoc = dynamic_cast <CLocation *> (pObject);
						if ( pLoc )
						{
							if ( pLoc->m_csDescription == csDescription )
							{
								pObj = (CSObject *) pLoc;
								break;
							}
							continue;
						}
						CArea * pArea = dynamic_cast <CArea *> (pObject);
						if ( pArea )
						{
							if ( pArea->m_csName == csDescription )
							{
								pObj = (CSObject *) pArea;
								break;
							}
							continue;
						}
					}
				}
			}
		}
		
		if (pObj != NULL)
		{
			AfxMessageBox("An entry with that name already exists.", MB_OK);
			return;
		}
		CLocation * pLoc = new (CLocation);
		pLoc->m_bCustom = true;
		pLoc->m_csCategory = csCategory;
		pLoc->m_csSubsection = csSubsection;
		pLoc->m_csDescription = csDescription;
		pLoc->m_csPoint = csPoint;

		Main->m_pScripts->m_aLocations.Add(pLoc);
		Main->m_pScripts->SaveLocations();
		pCategory = FindCategory(&Main->m_pScripts->m_olLocations, pLoc->m_csCategory);
		pSubsection = FindSubsection(pCategory, pLoc->m_csSubsection);
		if ( pSubsection->m_ItemList.IsEmpty() )
			pSubsection->m_ItemList.AddHead( pLoc );
		else
			pSubsection->m_ItemList.AddTail( pLoc );

		// The data should be saved now, so we'll add this to the list 
		// Actually, just reload the whole darn thing
		FillCategoryTree();
		SelectTreeItem(csCategory, csSubsection);
	}
	else 
		return;
}

void CTravelTab::OnRemoveLoc() 
{
	CString csCategory;
	CString csSubsection;
	CString csDescription;
	CString csPoint;

	HTREEITEM hCategory, hSubsection;
	hSubsection = this->m_ctcCategoryTree.GetSelectedItem();
	if (hSubsection != NULL)
	{
		hCategory = this->m_ctcCategoryTree.GetParentItem(hSubsection);
		if (hCategory == NULL)
			return;
	}

	csCategory = this->m_ctcCategoryTree.GetItemText(hCategory);
	csSubsection = this->m_ctcCategoryTree.GetItemText(hSubsection);
	int iItemSel = this->m_clcLocations.GetNextItem(-1, LVNI_SELECTED);
	if (iItemSel == -1)
		return;
	
	CCategory * pCategory = FindCategory(&Main->m_pScripts->m_olLocations, csCategory, false);
	CSubsection * pSubsection = FindSubsection(pCategory, csSubsection, false);

	CLocation * pLoc = (CLocation *) this->m_clcLocations.GetItemData(iItemSel);

	POSITION pos = pSubsection->m_ItemList.Find((CObject*) pLoc);
	if ( pos )
		pSubsection->m_ItemList.RemoveAt(pos);

	for ( int i = 0; i < Main->m_pScripts->m_aLocations.GetSize(); i++ )
	{
		CLocation * pTest = (CLocation *) Main->m_pScripts->m_aLocations.GetAt(i);
		if ( pTest == pLoc )
		{
			Main->m_pScripts->m_aLocations.RemoveAt(i);
			delete pLoc;
			break;
		}
	}
	FillCategoryTree();
	SelectTreeItem(csCategory, csSubsection);
	Main->m_pScripts->SaveLocations();
}

void CTravelTab::OnSelchangedCategorytree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here

	this->m_clcLocations.SetHotItem(-1);

	HTREEITEM hSelectedItem = NULL;
	HTREEITEM hParentItem = NULL;
	this->m_clcLocations.DeleteAllItems();
	this->m_cbRemoveLoc.EnableWindow(FALSE);
	this->m_cbEditLoc.EnableWindow(FALSE);

	// Do the bold/nobold thing
	if (pNMTreeView != NULL)
	{
		HTREEITEM hOldItem = pNMTreeView->itemOld.hItem;
		if (hOldItem != NULL)
			this->m_ctcCategoryTree.SetItemState(hOldItem, 0, TVIS_BOLD);
	}

	hSelectedItem = this->m_ctcCategoryTree.GetSelectedItem();
	this->m_ctcCategoryTree.SetItemState(hSelectedItem, TVIS_BOLD, TVIS_BOLD);
	hParentItem = this->m_ctcCategoryTree.GetParentItem(hSelectedItem);

	if (hParentItem != NULL)
	{
		CString csParent = this->m_ctcCategoryTree.GetItemText(hParentItem);
		CString csSelected = this->m_ctcCategoryTree.GetItemText(hSelectedItem);
		CCategory * pCategory = FindCategory(&Main->m_pScripts->m_olLocations, csParent);
		if (pCategory == NULL)
			return;
		CSubsection * pSubsection = FindSubsection(pCategory, csSelected);
		if (pSubsection == NULL)
			return;
		if (!pSubsection->m_ItemList.IsEmpty())
		{
			POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
			int iCount = 0;
			while (pos != NULL)
			{
				CSObject * pObject = (CSObject *) pSubsection->m_ItemList.GetNext(pos);
				CArea * pArea = dynamic_cast <CArea *> (pObject);
				if ( pArea )
				{
					this->m_clcLocations.InsertItem(iCount, pArea->m_csDescription, 0);
					this->m_clcLocations.SetItemText(iCount, 1, pArea->m_csPoint);
					this->m_clcLocations.SetItemData(iCount, (DWORD)pArea);
					iCount++;
				}
				else
				{
					CLocation * pLoc = dynamic_cast <CLocation *> (pObject);
					if ( pLoc )
					{
						this->m_clcLocations.InsertItem(iCount, pLoc->m_csDescription, 0);
						this->m_clcLocations.SetItemText(iCount, 1, pLoc->m_csPoint);
						this->m_clcLocations.SetItemData(iCount, (DWORD)pLoc);
						iCount++;
					}
				}
			}
		}
	}

	iLocSort = 1;
	iCoordSort = 1;
	m_clcLocations.SortItems(CompareFunc, 0);
	iLocSort = -1;
	*pResult = 0;
}

void CTravelTab::OnClickLocations(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMHDR *pJunk = pNMHDR;
	int iSelIndex = this->m_clcLocations.GetNextItem(-1, LVNI_SELECTED);
	if (iSelIndex == -1)
	{
		m_csCoords.SetWindowText("");
		Recenter(3077, 2048);
		return;
	}
	else
	{
		this->m_clcLocations.SetHotItem(iSelIndex);
		CString Coords = this->m_clcLocations.GetItemText(iSelIndex, 1);
		if (Coords.Find(",") == Coords.ReverseFind(','))
		{
			CString csTemp = Coords;
			Coords.Format("%s", csTemp);
		}
		short x, y, z, m;
		CString csTemp = Coords.SpanExcluding(",");
		x = (short) atoi(csTemp);
		csTemp = Coords.Mid(Coords.Find(",") + 1);
		CString csY = csTemp.SpanExcluding(",");
		y = (short) atoi(csY);
		if ( csTemp.Find(",") != -1 )
		{
			csTemp = csTemp.Mid(csTemp.Find(",") + 1);
			CString csZ = csTemp.SpanExcluding(",");
			z = (short) atoi(csZ);
			if ( csTemp.Find(",") != -1 )
			{
				Coords = csTemp;
				csTemp = Coords.Mid(Coords.Find(",") + 1);
				csTemp = csTemp.SpanExcluding(",");
				m = (short) atoi(csTemp);
			}
			else
				m = 0;
			CString csPlane;
			csPlane.Format("%ld", m);
			m_ceMapPlane.SetWindowText(csPlane);
		}
		else
		{
			z = 256;
			m = 256;
		}
		Recenter(x, y, z, m);
	}
	CSObject * pObject = (CSObject *) this->m_clcLocations.GetItemData(iSelIndex);
	CLocation * pLoc = dynamic_cast <CLocation *> (pObject);
	if ( pLoc )
	{
		this->m_cbRemoveLoc.EnableWindow(pLoc->m_bCustom ? TRUE : FALSE);
		this->m_cbEditLoc.EnableWindow(pLoc->m_bCustom ? TRUE : FALSE);
	}
	else
	{
		this->m_cbRemoveLoc.EnableWindow(FALSE);
		this->m_cbEditLoc.EnableWindow(FALSE);
	}
	*pResult = 0;
}

void CTravelTab::OnColumnclickLocations(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	m_clcLocations.SortItems(CompareFunc, pNMListView->iSubItem);
	if (pNMListView->iSubItem == 0)
	{
		switch(iLocSort)
		{
		case -1:
			iLocSort = 1;
			break;
		case 0:
			iLocSort = 1;
			break;
		case 1:
			iLocSort = -1;
			break;
		}
	}
	else
	{
		switch(iCoordSort)
		{
		case -1:
			iCoordSort = 1;
			break;
		case 0:
			iCoordSort = 1;
			break;
		case 1:
			iCoordSort = -1;
			break;
		}
	}
	*pResult = 0;
}

int CALLBACK CTravelTab::CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CSObject * pObject1 = (CSObject *)lParam1;
	CSObject * pObject2 = (CSObject *)lParam2;
	CString csName1, csName2;
	CString csPoint1, csPoint2;

	CArea * pArea = dynamic_cast <CArea *> (pObject1);
	if ( pArea )
	{
		csName1 = pArea->m_csName;
		csPoint1 = pArea->m_csPoint;
	}
	CLocation * pLoc = dynamic_cast <CLocation *> (pObject1);
	if ( pLoc )
	{
		csName1 = pLoc->m_csDescription;
		csPoint1 = pLoc->m_csPoint;
	}

	pArea = dynamic_cast <CArea *> (pObject2);
	if ( pArea )
	{
		csName2 = pArea->m_csName;
		csPoint2 = pArea->m_csPoint;
	}
	pLoc = dynamic_cast <CLocation *> (pObject2);
	if ( pLoc )
	{
		csName2 = pLoc->m_csDescription;
		csPoint2 = pLoc->m_csPoint;
	}

	if (lParamSort == 0)
	{
		if (iLocSort == 0)
			return (lParam1 < lParam2 ? 1 : -1);
		if (iLocSort == 1)
			return (csName1 < csName2 ? -1 : 1);
		if (iLocSort == -1)
			return (csName1 < csName2 ? 1 : -1);
	}
	else
	{
		if (iCoordSort == 0)
			return (lParam1 < lParam2 ? 1 : -1);
		if (iCoordSort == 1)
			return (csPoint1 < csPoint2 ? -1 : 1);
		if (iCoordSort == -1)
			return (csPoint1 < csPoint2 ? 1 : -1);
	}
	return 0;

}

void CTravelTab::OnLButtonDown(UINT nFlags, CPoint point) 
{

	CPropertyPage::OnLButtonDown(nFlags, point);
}

void CTravelTab::OnWhere() 
{
	if ( Main->m_bSeerMode )
		SendToUO(".wh");
	else
		SendToUO(".where");	
}

void CTravelTab::FillCategoryTree()
{
	HTREEITEM CategoryParent;
	HTREEITEM SubsectionParent;
	TV_INSERTSTRUCT InsertItem;

	m_ctcCategoryTree.DeleteAllItems();

	if (!Main->m_pScripts->m_olLocations.IsEmpty())
	{
		POSITION pos = Main->m_pScripts->m_olLocations.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) Main->m_pScripts->m_olLocations.GetNext(pos);
			InsertItem.item.mask = TVIF_TEXT;
			InsertItem.item.pszText = (char *)LPCTSTR(pCategory->m_csName);
			InsertItem.item.cchTextMax = pCategory->m_csName.GetLength();
			InsertItem.hParent = NULL;
			InsertItem.hInsertAfter = TVI_SORT;
			CategoryParent = this->m_ctcCategoryTree.InsertItem(&InsertItem);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION sPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (sPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(sPos);
					InsertItem.item.mask = TVIF_TEXT;
					InsertItem.item.pszText = (char *)LPCTSTR(pSubsection->m_csName);
					InsertItem.item.cchTextMax = pSubsection->m_csName.GetLength();
					InsertItem.hParent = CategoryParent;
					InsertItem.hInsertAfter = TVI_SORT;
					SubsectionParent = this->m_ctcCategoryTree.InsertItem(&InsertItem);
				}
			}
		}
		m_iCatSeq = Main->m_pScripts->m_iMCatSeq;
	}
}

void CTravelTab::SelectTreeItem(CString csParent, CString csChild)
{
	HTREEITEM hParent = m_ctcCategoryTree.GetFirstVisibleItem();
	HTREEITEM hChild;
	
	while (hParent != NULL)
	{
		CString csPText = m_ctcCategoryTree.GetItemText(hParent);
		if (csPText == csParent)
		{
			hChild = m_ctcCategoryTree.GetChildItem(hParent);
			while (hChild != NULL)
			{
				CString csCText = m_ctcCategoryTree.GetItemText(hChild);
				if (csCText == csChild)
				{
					m_ctcCategoryTree.SelectItem(hChild);
					return;
				}
				hChild = m_ctcCategoryTree.GetNextSiblingItem(hChild);
			}
			// Couldn't find the child...
			m_ctcCategoryTree.SelectItem(hParent);
			return;
		}
		hParent = m_ctcCategoryTree.GetNextSiblingItem(hParent);
	}
}

LRESULT CTravelTab::DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if (message == WM_USER)
	{
		if (m_pEditDlg != NULL)
		{
			// pass this message on to the EditGraymap Dialog
			m_pEditDlg->PostMessage(WM_USER, wParam);
		}
		else
		{
			// Calculate the X and Y coordinates of the block that we just got sent
			short X, Y, Z;
			CString csCoords;
			if ( wParam < (768 * 512 ) )
			{
				X = (wParam / 512) * 8;
				Y = (wParam % 512) * 8;
				Z = m_TravelMap.GetMapHeight(X, Y);
				csCoords.Format("%ld,%ld,%d, 0", X, Y, Z);
				m_TravelMap.SetMapFile(0);
				m_ceMapPlane.SetWindowText("0");
			}
			else
			{
				wParam %= (768 * 512);
				X = (wParam / 512 ) * 8;
				Y = (wParam % 512 ) * 8;
				Z = m_TravelMap.GetMapHeight(X, Y);
				csCoords.Format("%ld, %ld, %d, 2", X, Y, Z);
				m_TravelMap.SetMapFile(2);
				m_ceMapPlane.SetWindowText("2");
			}
			this->m_csCoords.SetWindowText(csCoords);
			Recenter(X, Y);
		}
		return TRUE;
	}	
	return CPropertyPage::DefWindowProc(message, wParam, lParam);
}

void CTravelTab::OnItemchangedLocations(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here

	if (pNMListView->uNewState & LVNI_SELECTED)
	{
		// this is the one that has been selected
		OnClickLocations(pNMHDR, pResult);
		return;
	}
	
	*pResult = 0;
}

void CTravelTab::OnListspawns() 
{
	CSpawnList cdSpawnList;
	
	cdSpawnList.m_csTitle = "Spawn Points for the entire world";
	cdSpawnList.m_pcsaSpawns = &this->m_csaSpawns;
	cdSpawnList.DoModal();
}

void CTravelTab::GetLocalSpawnpoints(HWND hWnd)
{
	CFileReadProgress * pProgress = (CFileReadProgress *) CFileReadProgress::FromHandle(hWnd);
	pProgress->m_csMessage.SetWindowText("Loading Spawnpoints");
	char crlf[] = {0x0d, 0x0a, 0x00};
	char szBuf[MAX_BUFFER];
	memset(szBuf, 0x00, sizeof(szBuf));
	// First we need to find the location of the world files
	CStdioFile csfIniFile;
	CString csIni = Main->m_csRootDirectory + _T(INI_FILE);
	if (!csfIniFile.Open(csIni, CFile::modeRead | CFile::shareDenyNone))
	{
		sprintf(szBuf, "Error -- Could not open %s%s", csIni, crlf);
		return;
	}
	CString csPath;
	CString csLine;
	BOOL bStatus = true;
	while(bStatus)
	{
		bStatus = csfIniFile.ReadString(csLine);
		csLine.TrimLeft();
		csLine.TrimRight();
		if (csLine != "")
		{
			if (csLine.Find("WORLDSAVE") == 0)
			{
				csPath = csLine.Right(csLine.GetLength() - 10);
				break;
			}
		}
	}
	csfIniFile.Close();

	CString csSaveFile;

	if (csPath.GetLength() != 0 && csPath[csPath.GetLength() - 1] != '\\')
		csSaveFile.Format("%s\\%s", csPath, WORLD_FILE);
	else
		csSaveFile.Format("%s%s", csPath, WORLD_FILE);

	CStdioFile csfWorldFile;
	if (!csfWorldFile.Open(csSaveFile, CFile::modeRead | CFile::shareDenyNone))
	{
		sprintf(szBuf, "Error -- Could not open world file %s %s", csSaveFile, crlf);
		AfxMessageBox(szBuf, MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	// Get the length of the file
	DWORD dwFileLen = csfWorldFile.GetLength();
	DWORD dwFileBegin = csfWorldFile.GetPosition();
	DWORD lineCount = 0;
	pProgress->SetRange32(0, dwFileLen);
	pProgress->SetPos(0);
	csLine.MakeUpper();
	bStatus = true;
	bool bWorldGem = false;
	while (bStatus)
	{
		CString csType = "<unknown>";
		CString csID = "<invalid>";
		CString csCoords = "-1,-1,-1,0";
		bStatus = csfWorldFile.ReadString(csLine);
		if ( (lineCount++ % 1000) == 0 )
		{
			DWORD dwFilePos = csfWorldFile.GetPosition();
			DWORD dwOffset = dwFilePos - dwFileBegin;
			pProgress->SetPos( dwOffset );
		}
		csLine = csLine.SpanExcluding(crlf);
		CString csLineUpper = csLine;
		csLineUpper.MakeUpper();
		if ( csLineUpper.Find("[WORLDITEM I_WORLDGEM_BIT") != -1)
			bWorldGem = true;
		else if ( csLineUpper.Find("[") != -1 )
			bWorldGem = false;
		if (csLineUpper.Find("TYPE=34") != -1 || csLineUpper.Find("TYPE=T_SPAWN_CHAR") != -1 || csLineUpper.Find("TYPE=034") != -1 || ( csLineUpper.Find("DISPID=I_PET_") != -1 && bWorldGem ) )
		{
			// this is a NPC spawn point
			char szOutput[80];
			csType = "NPC";
			while (bStatus)
			{
				bStatus = csfWorldFile.ReadString(csLine);
				csLine = csLine.SpanExcluding(crlf);
				if (csLine != "")
				{
					if (csLine[0] == '[')
					{
						DWORD dwID = ahextoi(csID);
						// It's an NPC
						int iIndex = Main->m_pScripts->m_aNPCs.Find(csID);
						if ( iIndex != -1 )
						{
							CNPC * pNPC = (CNPC *) Main->m_pScripts->m_aNPCs.GetAt(iIndex);
							if ( pNPC )
								csID = pNPC->m_csDescription;
						}
						else
						{
							// It's a spawn group
							iIndex = Main->m_pScripts->m_tables.m_spawns.Find(csID);
							if ( iIndex != -1 )
							{
								CTSpawnDef * pSpawn = (CTSpawnDef *) Main->m_pScripts->m_tables.m_spawns.GetAt(iIndex);
								if ( pSpawn )
									csID = pSpawn->m_csDescription;
							}
						}
						sprintf(szOutput, "%s,%s,%s", csType, csID, csCoords);
						this->m_csaSpawns.Add(szOutput);
						csType = "<unknown>";
						csID = "<invalid>";
						csCoords = "-1,-1,-1,0";
						break;
					}
				}
				if (csLine.Find("MORE1=") != -1)
					csID = csLine.Right(csLine.GetLength() - 6);
				if (csLine.Find("MORE=") != -1)
					csID = csLine.Right(csLine.GetLength() - 5);
				if (csLine.Find("P=") != -1)
				{
					if (csLine.Find("MOREP=") == -1)
						csCoords = csLine.Right(csLine.GetLength() - 2);
				}
			}
		}
		if (csLineUpper.Find("TYPE=69") != -1 || csLineUpper.Find("TYPE=T_SPAWN_ITEM") != -1 || csLineUpper.Find("TYPE=069") != -1)
		{
			// This is an item spawn point
			char szOutput[80];
			csType = "ITEM";
			while (bStatus)
			{
				bStatus = csfWorldFile.ReadString(csLine);
				csLine = csLine.SpanExcluding(crlf);
				if (csLine != "")
				{
					if (csLine[0] == '[')
					{
						// It's either an item or a template....
						int iIndex = Main->m_pScripts->m_aItems.Find(csID);
						if ( iIndex != -1 )
						{
							// It's an item
							CItem * pItem = (CItem *) Main->m_pScripts->m_aItems.GetAt(iIndex);
							if ( pItem )
								csID = pItem->m_csDescription;
						}
						else
						{
							// It's a template
							iIndex = Main->m_pScripts->m_aTemplates.Find(csID);
							if ( iIndex != -1 )
							{
								CTemplate * pTemplate = (CTemplate *) Main->m_pScripts->m_aTemplates.GetAt(iIndex);
								if ( pTemplate )
									csID = pTemplate->m_csDescription;
							}
						}
						sprintf(szOutput, "%s,%s,%s", csType, csID, csCoords);
						this->m_csaSpawns.Add(szOutput);
						csType = "<unknown>";
						csID = "<invalid>";
						csCoords = "-1,-1,-1,0";
						break;
					}
				}
				if (csLine.Find("MORE1=") != -1)
					csID = csLine.Right(csLine.GetLength() - 6);
				if (csLine.Find("MORE=") != -1)
					csID = csLine.Right(csLine.GetLength() - 5);
				if (csLine.Find("P=") != -1)
				{
					if (csLine.Find("MOREP=") == -1)
						csCoords = csLine.Right(csLine.GetLength() - 2);
				}
			}
		}
	}
	csfWorldFile.Close();
	return;
}

void CTravelTab::OnLocalspawns() 
{
	CSpawnList cdSpawnList;
	
	cdSpawnList.m_csTitle = "Spawn Points displayed on the radar map";
	cdSpawnList.m_pcsaSpawns = &this->m_csaLocalSpawns;
	cdSpawnList.DoModal();
}

void CTravelTab::OnEditgraymap() 
{
	CEditGraymapDlg dlg;
	m_pEditDlg = &dlg;
	memcpy(&(dlg.m_dwColorMap[0]), this->m_dwColorMap, sizeof(this->m_dwColorMap));
	if ( this->m_bWorldMapCreated )
		this->m_dlgWorldMap.m_pParent = &dlg;
	dlg.DoModal();
	if ( this->m_bWorldMapCreated )
		this->m_dlgWorldMap.m_pParent = this;
	m_pEditDlg = NULL;
}

void CTravelTab::Unload()
{
}

void CTravelTab::LoadScripts()
{
	if ( Main->GetProfileType() == PROFILE_NONE )
		return;
	CWaitCursor hourglass;
	Main->m_pScripts->CategorizeLocations();


	CFileReadProgress dlgProgress;
	dlgProgress.Create(IDD_FILEREAD_PROGRESS);

	if (Main->m_dwShowSpawnpoints)
		OnReloadspawns();
	dlgProgress.EndDialog(IDOK);

	// Now add all of the categories to the category list box
	FillCategoryTree();

}

void CTravelTab::OnReloadspawns() 
{
	if ( !Main->m_dwShowSpawnpoints )
		return;
	CFileReadProgress dlgProgress;
	dlgProgress.Create(IDD_FILEREAD_PROGRESS);

	if ( this->m_csaSpawns.GetSize() > 0 )
		this->m_csaSpawns.RemoveAll();

	if ( this->m_csaLocalSpawns.GetSize() > 0 )
		this->m_csaLocalSpawns.RemoveAll();

	m_TravelMap.RemoveDrawObjects();
	// Add the crosshair back in
	CString csCoords;
	m_csCoords.GetWindowText(csCoords);
	short X, Y, Z, M;
	CString csTemp = csCoords;
	X = (short) atoi(csTemp.SpanExcluding(","));
	csTemp = csTemp.Mid(csTemp.Find(",") + 1);
	Y = (short) atoi(csTemp.SpanExcluding(","));
	if ( csTemp.Find(",") != -1 )
	{
		csTemp = csTemp.Mid(csTemp.Find(",") + 1);
		Z = (short) atoi(csTemp.SpanExcluding(","));
		if ( csTemp.Find(",") != -1 )
		{
			csTemp = csTemp.Mid(csTemp.Find(",") + 1);
			M = (short) atoi(csTemp.SpanExcluding(","));
		}
		else
			M = m_TravelMap.GetMapFile();
	}
	else
		Z = 0;
	m_crosshairIndex = m_TravelMap.AddDrawObject(X, Y, 1, 3, 0x00FFFFFF);

	if ( Main->GetProfileType() == PROFILE_REMOTE || ( Main->GetProfileType() == PROFILE_LOCAL && Main->m_dwRemoteSpawns ) )
	{
		CRemoteConnection remote;
		remote.GetSpawnpoints(&this->m_csaSpawns);
	}
	else
		GetLocalSpawnpoints(dlgProgress.GetSafeHwnd());
	if ( m_bWorldMapCreated )
	{
		m_dlgWorldMap.m_csaSpawns.RemoveAll();
		m_dlgWorldMap.m_csaSpawns.Copy(m_csaSpawns);
		m_dlgWorldMap.RedrawWindow();
	}
	UpdateSpawnDisplay();
	Recenter();

	dlgProgress.EndDialog(IDOK);
}

void CTravelTab::OnEditLoc() 
{
	int iSel = m_clcLocations.GetNextItem(-1, LVNI_SELECTED);
	if (iSel == -1)
	{
		// This shouldn't happen...but just in case.
		return;
	}

	CSObject * pObject = (CSObject *) m_clcLocations.GetItemData(iSel);
	CLocation * pLoc = dynamic_cast <CLocation *> (pObject);
	if ( !pLoc )
		return;

	CAddDestination dlgAdd;
	dlgAdd.m_pCategoryList = &Main->m_pScripts->m_olLocations;

	CString csDisplayedCoords;
	m_csCoords.GetWindowText(csDisplayedCoords);
	dlgAdd.m_csDescription = pLoc->m_csDescription;
	dlgAdd.m_csCategory = pLoc->m_csCategory;
	dlgAdd.m_csSubsection = pLoc->m_csSubsection;
	dlgAdd.m_csXCoord = csDisplayedCoords.Left(csDisplayedCoords.Find(","));
	dlgAdd.m_csYCoord = csDisplayedCoords.Mid(csDisplayedCoords.Find(",") + 1);
	dlgAdd.m_csZCoord = dlgAdd.m_csYCoord.Mid(dlgAdd.m_csYCoord.Find(",") + 1);
	dlgAdd.m_csYCoord = dlgAdd.m_csYCoord.SpanExcluding(",");
	dlgAdd.m_csPlane = dlgAdd.m_csZCoord.Mid(dlgAdd.m_csZCoord.Find(",") + 1);
	dlgAdd.m_csZCoord = dlgAdd.m_csZCoord.SpanExcluding(",");
	if (dlgAdd.DoModal() == IDOK)
	{
		CString csCategory = dlgAdd.m_csCategory;
		CString csSubsection = dlgAdd.m_csSubsection;
		CString csDescription = dlgAdd.m_csDescription;
		CString csX = dlgAdd.m_csXCoord;
		CString csY = dlgAdd.m_csYCoord;
		CString csZ = dlgAdd.m_csZCoord;
		CString csP = dlgAdd.m_csPlane;
		CString csPoint;
		csPoint.Format("%s,%s,%s,%s", csX, csY, csZ, csP);
		// Make sure an entry with this name doesn't already exist
		CCategory * pCategory = FindCategory(&Main->m_pScripts->m_olLocations, csCategory);
		CSubsection * pSubsection = NULL;
		CSObject * pObj = NULL;
		if (pCategory != NULL)
		{
			pSubsection = FindSubsection(pCategory, csSubsection);
			if (pSubsection != NULL)
			{
				POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
				while ( pos )
				{
					CSObject * pObject = (CSObject *) pSubsection->m_ItemList.GetNext(pos);
					if ( pObject )
					{
						CLocation * pLoc = dynamic_cast <CLocation *> (pObject);
						if ( pLoc )
						{
							if ( pLoc->m_csDescription == csDescription )
							{
								pObj = (CSObject *) pLoc;
								break;
							}
							continue;
						}
						CArea * pArea = dynamic_cast <CArea *> (pObject);
						if ( pArea )
						{
							if ( pArea->m_csName == csDescription )
							{
								pObj = (CSObject *) pArea;
								break;
							}
							continue;
						}
					}
				}
			}
		}
		
		if (pObj != NULL && pObj != pLoc )
		{
			AfxMessageBox("An entry with that name already exists.", MB_OK);
			return;
		}
		pLoc->m_bCustom = true;
		pLoc->m_csCategory = csCategory;
		pLoc->m_csSubsection = csSubsection;
		pLoc->m_csDescription = csDescription;
		pLoc->m_csPoint = csPoint;

		Main->m_pScripts->SaveLocations();
		Main->m_pScripts->RemoveLocationCategories();
		Main->m_pScripts->CategorizeLocations();

		// The data should be saved now, so we'll add this to the list 
		// Actually, just reload the whole darn thing
		FillCategoryTree();
		SelectTreeItem(csCategory, csSubsection);
	}
	else 
		return;	
}

void CTravelTab::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CPropertyPage::OnRButtonDown(nFlags, point);
}

void CTravelTab::OnMorep() 
{
	CString csCmd, csCoords;
	m_csCoords.GetWindowText(csCoords);
	csCmd.Format(".set morep %s", csCoords);
	SendToUO(csCmd);
}

BEGIN_EVENTSINK_MAP(CTravelTab, CPropertyPage)
    //{{AFX_EVENTSINK_MAP(CTravelTab)
	ON_EVENT(CTravelTab, IDC_TRAVELMAP, -605 /* MouseDown */, OnMouseDownTravelmap, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	ON_EVENT(CTravelTab, IDC_TRAVELMAP, -606 /* MouseMove */, OnMouseMoveTravelmap, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	ON_EVENT(CTravelTab, IDC_TRAVELMAP, -607 /* MouseUp */, OnMouseUpTravelmap, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CTravelTab::OnMouseDownTravelmap(short Button, short Shift, long x, long y) 
{
	// Make sure we're in the map control.
	CRect rect;
	m_TravelMap.GetClientRect(&rect);
	if ( x < 0 || x > rect.Width() || y < 0 || y > rect.Height() )
		return;
	// Which button?
	switch ( Button )
	{
	case 1:
		{
			short xc, yc, z;
			xc = (short) x;
			yc = (short) y;
			m_TravelMap.CtrlToMap(&xc, &yc);
			CString csCoords;
			z = m_TravelMap.GetMapHeight(xc, yc);
			csCoords.Format("%d,%d,%d", xc, yc, z);
			this->m_csCoords.SetWindowText(csCoords);
			Recenter(xc, yc);
			break;
		}
	case 2:
		{
			CMapCoordinatesDlg dlg;
			CString csCoords;
			this->m_csCoords.GetWindowText(csCoords);
			if (csCoords != "")
			{
				CString csX, csY;
				csX = csCoords.Left(csCoords.Find(","));
				csY = csCoords.Mid(csCoords.Find(",") + 1);
				dlg.m_iX = atoi(csX);
				dlg.m_iY = atoi(csY);
			}
			else
			{
				dlg.m_iX = 0;
				dlg.m_iY = 0;
			}
			if ( dlg.DoModal() == IDOK )
			{
				short z = m_TravelMap.GetMapHeight(dlg.m_iX, dlg.m_iY);
				csCoords.Format("%ld,%ld,z", dlg.m_iX, dlg.m_iY, z);
				this->m_csCoords.SetWindowText(csCoords);
				Recenter((short) dlg.m_iX, (short) dlg.m_iY);
			}
			// Since we popped up a dialog, we need to have the mouse up event fire
			CPoint pt(x, y);
			m_TravelMap.SendMessage(WM_RBUTTONUP, 0, (LPARAM) &pt);
		}
		break;
	default:
		break;
	}
}

void CTravelTab::OnMouseMoveTravelmap(short Button, short Shift, long x, long y) 
{

}

void CTravelTab::OnMouseUpTravelmap(short Button, short Shift, long x, long y) 
{
	return;
}

void CTravelTab::Recenter(short x, short y, short z, short m)
{
	if ( m == 256 )
	{
		CString csPlane;
		m_ceMapPlane.GetWindowText(csPlane);
		m = atoi(csPlane);
	}
	if ( m == 2 )
		m_TravelMap.SetMapFile(2);
	else
		m_TravelMap.SetMapFile(0);
 	CString csPlane;
	csPlane.Format("%ld", m);
	//m_ceMapPlane.SetWindowText(csPlane);
	m_TravelMap.RemoveDrawObjectAt(m_crosshairIndex);
	m_crosshairIndex = m_TravelMap.AddDrawObject(x, y, 1, 3, 0x00FFFFFF);
	m_TravelMap.SetCenter(x, y);
	Main->PutRegistryDword("TravelTab Center X", (DWORD) x);
	Main->PutRegistryDword("TravelTab Center Y", (DWORD) y);
	Main->PutRegistryDword("TravelTab Zoom", (DWORD) m_TravelMap.GetZoomLevel());

	// Only do this if a z coordinate hasn't been specified.
	CString csCoords;
	m_csCoords.GetWindowText(csCoords);
	if ( z > 255 )
		csCoords.Format("%d, %d, %d", x, y, m_TravelMap.GetMapHeight(x, y));
	else
		csCoords.Format("%d, %d, %d", x, y, z);
	m_csCoords.SetWindowText(csCoords);

	m_csaLocalSpawns.RemoveAll();
	CRect mapRect;
	CRect clientRect;
	m_TravelMap.GetClientRect(&clientRect);
	mapRect.left = m_TravelMap.CtrlToMapX(0);
	mapRect.top = m_TravelMap.CtrlToMapY(0);
	mapRect.right = m_TravelMap.CtrlToMapX((short)clientRect.right);
	mapRect.bottom = m_TravelMap.CtrlToMapY((short)clientRect.bottom);

	for ( int i = 0; i < m_csaSpawns.GetSize(); i++ )
	{
		CString csType, csID, csX, csY, csZ, csM;
		CString csSpawn = m_csaSpawns.GetAt(i);
		csType = csSpawn.SpanExcluding(",");
		CString csTemp = csSpawn.Mid(csSpawn.Find(",") + 1);
		csID = csTemp.SpanExcluding(",");
		csTemp = csTemp.Mid(csTemp.Find(",") + 1);
		csX = csTemp.SpanExcluding(",");
		csTemp = csTemp.Mid(csTemp.Find(",") + 1);
		csY = csTemp.SpanExcluding(",");
		if ( csTemp.Find(",") != -1 )
		{
			csTemp = csTemp.Mid(csTemp.Find(",") + 1);
			csZ = csTemp.SpanExcluding(",");
			if ( csTemp.Find(",") != -1 )
			{
				csTemp = csTemp.Mid(csTemp.Find(",") + 1);
				csM = csTemp.SpanExcluding(",");
			}
		}
		x = (short) atoi(csX);
		y = (short) atoi(csY);
		z = (short) atoi(csZ);
		int plane = (short) atoi(csM);

		CPoint pt(x, y);
		if ( mapRect.PtInRect(pt) && ( (plane == m) || (plane == 255) ) )
			m_csaLocalSpawns.Add(csSpawn);
	}
}

void CTravelTab::OnZoomout() 
{
	m_TravelMap.ZoomOut();
	Recenter();
}

void CTravelTab::OnZoomin() 
{
	m_TravelMap.ZoomIn();
	Recenter();
}

void CTravelTab::Recenter()
{
	short x, y;
	m_TravelMap.GetCenter(&x, &y);
	Recenter(x, y, 256, 256);
}

void CTravelTab::OnWorldmap() 
{
	if (!m_bWorldMapCreated)
	{
		m_dlgWorldMap.m_csaSpawns.Copy(m_csaSpawns);
		m_dlgWorldMap.Create(IDD_WORLDMAP, this);
		m_dlgWorldMap.m_pParent = this;
		m_bWorldMapCreated = true;
	}
	if (!m_bWorldMapVisible)
	{
		m_dlgWorldMap.ShowWindow(SW_SHOW);
		m_bWorldMapVisible = true;
	}
	else
	{
		m_dlgWorldMap.ShowWindow(SW_HIDE);
		m_bWorldMapVisible = false;
	}	
}

BOOL CTravelTab::OnSetActive() 
{
	TRACE("CTravelTab::OnSetActive()\n");
	if ( Main->m_pScripts->m_iMCatSeq != m_iCatSeq )
		this->FillCategoryTree();

	// Make sure the paths are set correctly
	if ( csUOPath != "" )
		m_TravelMap.SetClientPath(csUOPath);
	CString csRegKey;
	CString csValue;
	csValue = Main->GetRegistryString("MulPath", "" );
	if ( csValue != "" )
		m_TravelMap.SetClientPath(csValue);
	csRegKey.Format("Mul%ld", VERFILE_MAP0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_TravelMap.SetCustomMulPath(VERFILE_MAP0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_MAP2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_TravelMap.SetCustomMulPath(VERFILE_MAP2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_TravelMap.SetCustomMulPath(VERFILE_STAIDX0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_TravelMap.SetCustomMulPath(VERFILE_STATICS0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_TravelMap.SetCustomMulPath(VERFILE_STAIDX2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_TravelMap.SetCustomMulPath(VERFILE_STATICS2, csValue);

	if ( Main->m_dwShowSpawnpoints )
	{
		// Make sure we've loaded them
		if ( m_csaSpawns.GetSize() == 0 )
			OnReloadspawns();
		else
			UpdateSpawnDisplay();
	}
	else
	{
		// Remove the draw objects
		m_TravelMap.RemoveDrawObjects();
	}

	this->Recenter();

	return CPropertyPage::OnSetActive();
}

void CTravelTab::OnFind() 
{
	CSearchCritDlg dlg;
	if ( dlg.DoModal() != IDOK )
		return;
	Find( dlg.m_csValue, dlg.m_iCrit );
	return;	
}

CSObject * CTravelTab::Find(CString csString, int iCrit, bool bSelect, bool bPrevious)
{
	CSObject * pSelection = NULL;
	if ( csString != "" )
	{
		// this is a new search
		this->m_csSearchValue = csString;
		this->m_iSearchCrit = iCrit;
		this->m_pSearchCategory = NULL;
		this->m_pSearchSubsection = NULL;
		this->m_pSearchItem = NULL;
	}
	POSITION catPos;
	if ( this->m_pSearchCategory )
		catPos = Main->m_pScripts->m_olLocations.Find( this->m_pSearchCategory );
	else
	{
		if ( !bPrevious )
			catPos = Main->m_pScripts->m_olLocations.GetHeadPosition();
		else
			catPos = Main->m_pScripts->m_olLocations.GetTailPosition();
	}
	while ( catPos != NULL )
	{
		CCategory * pCategory;
		if ( !bPrevious )
			pCategory = (CCategory *) Main->m_pScripts->m_olLocations.GetNext( catPos );
		else
			pCategory = (CCategory *) Main->m_pScripts->m_olLocations.GetPrev( catPos );
		if ( pCategory == NULL )
			break;
		POSITION subPos;
		if ( this->m_pSearchSubsection )
			subPos = pCategory->m_SubsectionList.Find( this->m_pSearchSubsection );
		else
		{
			if ( !bPrevious )
				subPos = pCategory->m_SubsectionList.GetHeadPosition();
			else
				subPos = pCategory->m_SubsectionList.GetTailPosition();
		}
		while ( subPos != NULL )
		{
			CSubsection * pSubsection;
			if ( !bPrevious )
				pSubsection = ( CSubsection *) pCategory->m_SubsectionList.GetNext( subPos );
			else
				pSubsection = ( CSubsection *) pCategory->m_SubsectionList.GetPrev( subPos );
			if ( pSubsection == NULL )
				break;
			POSITION itemPos;
			if ( this->m_pSearchItem )
			{
				itemPos = pSubsection->m_ItemList.Find( this->m_pSearchItem );
				if ( itemPos )
				{
					CSObject * pObject;
					if ( !bPrevious )
						pObject = (CSObject *) pSubsection->m_ItemList.GetNext( itemPos );
					else
						pObject = (CSObject *) pSubsection->m_ItemList.GetPrev( itemPos );
				}
			}
			else
			{
				if ( !bPrevious )
					itemPos = pSubsection->m_ItemList.GetHeadPosition();
				else
					itemPos = pSubsection->m_ItemList.GetTailPosition();
			}
			while ( itemPos != NULL )
			{
				CSObject * pItem;
				if ( !bPrevious )
					pItem = (CSObject *) pSubsection->m_ItemList.GetNext( itemPos );
				else
					pItem = (CSObject *) pSubsection->m_ItemList.GetPrev( itemPos );
				if ( pItem )
				{
					CString csValue, csTest;
					csTest = this->m_csSearchValue;
					csTest.MakeLower();
					switch ( this->m_iSearchCrit )
					{
					case 0:
						// ID
						csValue = pItem->m_csValue;
						csValue.MakeLower();
						if ( csValue.Find( csTest ) != -1 )
							pSelection = (CSObject *) pItem;
						break;
					case 1:
						// Description
						csValue = pItem->m_csDescription;
						csValue.Replace("@", pItem->m_csSubsection);
						csValue.MakeLower();
						if ( csValue.Find( csTest ) != -1 )
							pSelection = (CSObject *) pItem;
						break;
					default:
						break;
					}
				}
				if ( pSelection != NULL )
				{
					if ( bSelect )
					{
						HTREEITEM hRoot = this->m_ctcCategoryTree.GetRootItem();
						HTREEITEM hCat = this->m_ctcCategoryTree.GetNextItem( hRoot, TVGN_NEXT );
						while ( hCat != NULL )
						{
							if ( this->m_ctcCategoryTree.GetItemText( hCat ) == pCategory->m_csName )
							{
								HTREEITEM hSub = this->m_ctcCategoryTree.GetChildItem( hCat );
								while ( hSub != NULL )
								{
									if ( this->m_ctcCategoryTree.GetItemText( hSub ) == pSubsection->m_csName )
									{
										this->m_ctcCategoryTree.SelectItem( hSub );
										for ( int i = 0; i < this->m_clcLocations.GetItemCount(); i++ )
										{
											CSObject * pObject = (CSObject *) this->m_clcLocations.GetItemData(i);
											if ( pObject == pSelection )
											{
												this->m_clcLocations.SetItemState( i, LVIS_SELECTED, LVIS_SELECTED );
												this->m_clcLocations.EnsureVisible( i, FALSE );
												this->m_pSearchCategory = pCategory;
												this->m_pSearchItem = pItem;
												this->m_pSearchSubsection = pSubsection;
												return pSelection;
											}
										}
										return pSelection;
									}
									hSub = this->m_ctcCategoryTree.GetNextItem( hSub, TVGN_NEXT );
								}
							}
							hCat = this->m_ctcCategoryTree.GetNextItem( hCat, TVGN_NEXT );
						}
					}
					return pSelection;
				}
			}
			this->m_pSearchItem = NULL;
		}
		this->m_pSearchSubsection = NULL;
	}
	if ( bSelect )
		AfxMessageBox("No matching item was found", MB_OK | MB_ICONINFORMATION);
	return pSelection;
}

CSObject * CTravelTab::FindPrevious()
{
	return Find("", 0, true, true);
}

BOOL CTravelTab::PreTranslateMessage(MSG* pMsg) 
{
	if ( pMsg->message == WM_KEYDOWN )
	{
		if ( pMsg->wParam == VK_F3 )
		{
			if ( GetAsyncKeyState(VK_SHIFT) )
				FindPrevious();
			else
				Find();
			return TRUE;
		}
	}	
	
	return CPropertyPage::PreTranslateMessage(pMsg);
}


void CTravelTab::OnChangeMapplane() 
{
	if ( this->IsWindowVisible() )
	{
		UpdateSpawnDisplay();
		Recenter();			
	}
}

void CTravelTab::UpdateSpawnDisplay()
{
	CString csPlane;
	m_ceMapPlane.GetWindowText(csPlane);
	int M = atoi(csPlane);
	m_TravelMap.RemoveDrawObjects();
	for ( int i = 0; i < m_csaSpawns.GetSize(); i++ )
	{
		CString csTemp;
		bool bItem = false;
		CString csType, csID, csX, csY, csZ, csM;
		CString csSpawn = m_csaSpawns.GetAt(i);
		csType = csSpawn.SpanExcluding(",");
		csTemp = csSpawn.Mid(csSpawn.Find(",") + 1);
		csID = csTemp.SpanExcluding(",");
		csTemp = csTemp.Mid(csTemp.Find(",") + 1);
		csX = csTemp.SpanExcluding(",");
		csTemp = csTemp.Mid(csTemp.Find(",") + 1);
		csY = csTemp.SpanExcluding(",");
		csTemp = csTemp.Mid(csTemp.Find(",") + 1);
		if ( csTemp.Find(",") != -1 )
		{
			csTemp = csTemp.Mid(csTemp.Find(",") + 1);
			csZ = csTemp.SpanExcluding(",");
			if ( csTemp.Find(",") != -1 )
			{
				csTemp = csTemp.Mid(csTemp.Find(",") + 1);
				csM = csTemp.SpanExcluding(",");
			}
		}
		short x = (short) atoi(csX);
		short y = (short) atoi(csY);
		short z = (short) atoi(csZ);
		short m = (short) atoi(csM);
		if ( csType.Find("NPC") != -1 )
			bItem = false;
		else
			bItem = true;
		// Check map plane before drawing
		if ( (m == M) || (m == 255))
			m_TravelMap.AddDrawObject(x, y, 1, 3, bItem ? Main->m_dwItemSpawnColor : Main->m_dwNPCSpawnColor);
	}

}
