/*
 $Id: weighteditemdlg.cpp,v 1.3 2002/05/01 03:33:12 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// WeightedItemDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "WeightedItemDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWeightedItemDlg dialog


CWeightedItemDlg::CWeightedItemDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CWeightedItemDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CWeightedItemDlg)
	//}}AFX_DATA_INIT
}


void CWeightedItemDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWeightedItemDlg)
	DDX_Control(pDX, IDC_WEIGHTEDLIST, m_clcList);
	DDX_Control(pDX, IDC_WEIGHTEDITEMS, m_ccbItems);
	DDX_Control(pDX, IDC_ITEMWEIGHT, m_ceWeight);
	DDX_Control(pDX, IDC_ADDWEIGHTEDITEM, m_cbAdd);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWeightedItemDlg, CDialog)
	//{{AFX_MSG_MAP(CWeightedItemDlg)
	ON_BN_CLICKED(IDC_ADDWEIGHTEDITEM, OnAddweighteditem)
	ON_NOTIFY(LVN_KEYDOWN, IDC_WEIGHTEDLIST, OnKeydownWeightedlist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWeightedItemDlg message handlers

BOOL CWeightedItemDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	FillItems();
	m_ccbItems.AddString(" NONE ");
	m_clcList.InsertColumn(0, "Item", LVCFMT_LEFT, 200);
	m_clcList.InsertColumn(1, "Weight", LVCFMT_LEFT, 50);
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CWeightedItemDlg::OnOK() 
{
	m_csWeightString = "{";
	for (int i = 0; i < m_clcList.GetItemCount(); i++)
	{
		CString csTmp = m_csWeightString;
		CString csItem = m_clcList.GetItemText(i, 0);
		if (csItem == " NONE ")
			csItem = "0 0";
		CString csWeight = m_clcList.GetItemText(i, 1);
		m_csWeightString.Format("%s %s %s", csTmp, csItem, csWeight);
	}
	m_csWeightString += " }";
	CDialog::OnOK();
}

void CWeightedItemDlg::OnAddweighteditem() 
{
	CString csItem;
	m_ccbItems.GetWindowText(csItem);
	if (csItem == "")
	{
		AfxMessageBox("No item is selected.", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	CString csWeight;
	m_ceWeight.GetWindowText(csWeight);
	if (atoi(csWeight) == 0)
	{
		AfxMessageBox("Item must have a weight.", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	int iIndex = m_clcList.InsertItem(m_clcList.GetItemCount(), csItem);
	m_clcList.SetItemText(iIndex, 1, csWeight);
}

void CWeightedItemDlg::FillItems()
{
	CStdioFile csfGrayDefs;
	BOOL bStatus;
	CString csScpPath = GetScriptPath();
	CString csFile;
	if ( csScpPath == "" )
		csFile = DEFS_FILE;
	else
		csFile.Format("%s%s", csScpPath, DEFS_FILE);
	bStatus = csfGrayDefs.Open(csFile, CFile::modeRead | CFile::shareDenyNone);
	if (bStatus == 0)
	{
		CString csMessage;
		csMessage.Format("Unable to open %s", csFile);
		AfxMessageBox(csMessage, MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	CString csInput;
	bool bFoundSection = false;
	int iIndex = 0;
	while (bStatus)
	{
		bStatus = csfGrayDefs.ReadString(csInput);
		csInput.MakeLower();
		if (bStatus)
		{
			if (csInput.Find("[") != -1)
			{
				// We have reached a new section
				if (csInput.Find("[items_") == -1)
					bFoundSection = FALSE;
				else
					bFoundSection = TRUE;
			}
			else
			{
				if (bFoundSection)
				{
					// Check for blank line or comment
					csInput = csInput.SpanExcluding("//");
					csInput.TrimLeft();
					csInput.TrimRight();
					CString csValue = csInput.Right(8);
					csValue.TrimLeft();
					csInput = csInput.SpanExcluding(" ");
					if (csInput != "")
					{
						m_ccbItems.AddString(csInput);
					}
				}
			}
		}
	}
	csfGrayDefs.Close();
}

void CWeightedItemDlg::OnKeydownWeightedlist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_KEYDOWN* pLVKeyDown = (LV_KEYDOWN*)pNMHDR;
	if (pLVKeyDown->wVKey == VK_DELETE)
	{
		// Delete the selected items from the list control
		while (this->m_clcList.GetSelectedCount() > 0)
		{
			int iSel = this->m_clcList.GetNextItem(-1, LVNI_SELECTED);
			if (iSel != -1)
				this->m_clcList.DeleteItem(iSel);
		}
	}
	
	*pResult = 0;
}
