/*
 $Id: gwhogsvr.cpp,v 1.27 2002/12/10 19:28:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/
// Additional code contributors:
// 	William Arts (Belgar)
//

// gwhogsvr.cpp

#include "gwhogsvr.h"
#include <string.h>
#include <time.h>
#include <stdio.h>
#include <stdarg.h>
#ifndef WIN32
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
extern int errno;
extern char **environ;
#endif

#ifndef MAXBUFFER
#define MAXBUFFER 8192
#endif

#define SVR_VERSION "0.13.3.2"
#ifdef WIN32
#define SVR_PLATFORM "Win32"
#else
#define SVR_PLATFORM "Linux"
#endif
#define SLEEPTIME 10

#ifndef WIN32
#define	LINGER	linger
#endif


DWORD dwTusPort;
char g_szAccuFile[MAX_PATH];
char g_szAcctFile[MAX_PATH];
char g_szLogPath[MAX_PATH];
BOOL g_bReadOnly;
int g_iSpawnRefreshTime;
char g_szAllowedPaths[MAX_PATH];
char g_szAllowedExtensions[MAX_PATH];
char g_szAllowedHosts[MAX_BUFFER];
char g_szDeniedHosts[MAX_BUFFER];
char * g_pszSpawns;
int g_iReceiveTimeout;

#ifdef WIN32
int service_main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
	g_pszSpawns = NULL;
	g_iSpawnRefreshTime = 1200;
	g_iReceiveTimeout = 60000;
	time_t lastSpawnLoadTime = 0;
	memset(g_szAllowedPaths, 0x00, sizeof(g_szAllowedPaths));
	memset(g_szAllowedExtensions, 0x00, sizeof(g_szAllowedExtensions));
	memset(g_szLogPath, 0x00, sizeof(g_szLogPath));
#ifdef WIN32
	char szMutex[256];
	sprintf(szMutex, "stdioMutex%s", pService->GetServerName());
	hStdioMutex = CreateMutex(NULL, FALSE, szMutex);
	if (hStdioMutex == NULL)
	{
		printf("Unable to create mutex for standard output.  Closing.");
		pService->AddToMessageLog("Unable to create mutex for standard output.  Closing.", NULL, APP_COULD_NOT_CREATE_MUTEX, EVENTLOG_ERROR_TYPE );
		return false;
	}
#else
	signal(SIGPIPE, SIG_IGN);
	if(pthread_mutex_init(&hStdioMutex, NULL))
	{
		printf("Error initializing mutex for standard output.  Closing.");
		return false;
	}
#endif

#ifdef WIN32
	// Make sure we are running in the correct location
	char szFullPath[MAX_PATH];
	memset( &szFullPath[0], 0x00, sizeof(szFullPath) );
	GetModuleFileName( NULL, &szFullPath[0], sizeof(szFullPath) );
/* Do we really need to be doing this?
	for ( int i = strlen(szFullPath); i > 0; i-- )
	{
		if ( szFullPath[i] == '\\' )
		{
			szFullPath[i] = 0x00;
			break;
		}
	}
	_chdir( &szFullPath[0] );
*/
#endif
	int iServerPort = 4006;
	int iMaxConnections = 5;
	BOOL bUseThreads = TRUE;
	g_bReadOnly = FALSE;
	memset(&g_szAllowedPaths[0], 0x00, MAX_PATH);
	memset(&g_szAllowedExtensions[0], 0x00, MAX_PATH);
	memset(&g_szAllowedHosts[0], 0x00, MAX_BUFFER);
	memset(&g_szDeniedHosts[0], 0x00, MAX_BUFFER);
	SysMessage("Reading configuration file....");
	FILE * pCfg;
	pCfg = fopen("axissvr.cfg", "r");
	if (pCfg != NULL)
	{
		char szLine[MAX_PATH];
		while (!feof(pCfg))
		{
			fReadLine(pCfg, &szLine[0], sizeof(szLine));
			if (szLine[0] != 0x00)
			{
				// strip out the comments
				for (int i = 0; i < (int) strlen(szLine); i++)
				{
					if (szLine[i] == '#')
					{
						szLine[i] = 0x00;
						break;
					}
				}
				if (szLine[0] != 0x00)
				{
					// Not a commented line
					char * szCommand;
					szCommand = strstr(szLine, "PORT");
					if (szCommand != NULL)
						iServerPort = atoi(&szCommand[4]);
					szCommand = strstr(szLine, "MAXCONNECTIONS");
					if (szCommand != NULL)
						iMaxConnections = atoi(&szCommand[14]);
					szCommand = strstr(szLine, "NOMULTITHREADING");
					if (szCommand != NULL )
						bUseThreads = FALSE;
					szCommand = strstr(szLine, "READONLY");
					if ( szCommand != NULL )
						g_bReadOnly = TRUE;
					szCommand = strstr(szLine, "SPAWNREFRESHTIME");
					if (szCommand != NULL )
						g_iSpawnRefreshTime = atoi(&szCommand[16]);
					szCommand = strstr(szLine, "ALLOWEDPATHS");
					if (szCommand != NULL)
						strcpy(g_szAllowedPaths, &szCommand[13]);
					szCommand = strstr(szLine, "ALLOWEDEXTENSIONS");
					if (szCommand != NULL)
						strcpy(g_szAllowedExtensions, &szCommand[18]);
					szCommand = strstr(szLine, "RECEIVETIMEOUT");
					if (szCommand != NULL)
						g_iReceiveTimeout = atoi(&szCommand[15]);
					szCommand = strstr(szLine, "ALLOWEDHOSTS");
					if (szCommand != NULL)
						strcpy(g_szAllowedHosts, &szCommand[13]);
					szCommand = strstr(szLine, "DENIEDHOSTS");
					if (szCommand != NULL)
						strcpy(g_szDeniedHosts, &szCommand[12]);
				}
			}
		}
		fclose(pCfg);
		SysMessage("done.\n");
	}
	else
	{
		SysMessage("\nUnable to open axissvr.cfg...using default values.\n");
	}

	dwTusPort = 2593;
	FILE * pIni = fopen("sphere.ini", "r");
	if (pIni != NULL)
	{
		char szLine[MAX_PATH];
		while (!feof(pIni))
		{
			fReadLine(pIni, &szLine[0], sizeof(szLine));
			if (szLine != 0x00)
			{
				if(strstr(szLine, "[SERVERS") != NULL)
				{
					if (!feof(pIni))
						fReadLine(pIni, &szLine[0], sizeof(szLine)); // This one is the server name
					if (!feof(pIni))
						fReadLine(pIni, &szLine[0], sizeof(szLine)); // This one is the server IP
					if (!feof(pIni))
						fReadLine(pIni, &szLine[0], sizeof(szLine)); // This is the server port
					dwTusPort = (DWORD) atoi(szLine);
					break;
				}
				if(strstr(szLine, "LOG=") == szLine)
				{
					Trim(szLine, strlen(szLine));
					strcpy(g_szLogPath, &szLine[4]);
				}
			}
		}
		fclose(pIni);
	}
	else
	{
		SysMessage("\nUnable to open sphere.ini...using default values.\n");
	}

#ifdef WIN32
	SysMessage("Initializing socket libraries....");
	WSADATA wsaData;
	WSAStartup(MAKEWORD(1,1), &wsaData);
	SysMessage("done\n");
#endif
	SysMessage("axissvr for %s version %s startup complete\n", SVR_PLATFORM, SVR_VERSION);
	SOCKET sock = 0;

	sock = socket(PF_INET, SOCK_STREAM, 0);
	int opt = 1;
	setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, (char*)&opt, sizeof(opt));
#ifdef _WIN32
	struct linger lingerOpt;
#else
	struct LINGER lingerOpt;
#endif
	memset(&lingerOpt, 0x00, sizeof(lingerOpt));
	lingerOpt.l_onoff = 1;
	lingerOpt.l_linger = 60;
	setsockopt( sock, SOL_SOCKET, SO_LINGER, (char*)&lingerOpt, sizeof(lingerOpt));
	SOCKADDR_IN sin;
	sin.sin_family = AF_INET;
	sin.sin_port = htons(iServerPort);
	sin.sin_addr.s_addr = 0;
	int result = bind(sock, (LPSOCKADDR) &sin, sizeof(sin));
	if ( result == SOCKET_ERROR )
	{
		SysMessage("Fatal error: Unable to bind socket.\n");
		return false;
	}
	result = listen(sock, iMaxConnections);
	if ( result == SOCKET_ERROR )
	{
		SysMessage("Fatal error: Unable to listen to socket.\n");
		return false;
	}
#ifdef WIN32
	while ( pService->GetState() != SERVICE_STOP_PENDING )
#else
	while ( TRUE )
#endif
	{
		fd_set fds;
		FD_ZERO(&fds);
		FD_SET(sock, &fds);
		struct timeval timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = 1000;
		result = select(sock+1, &fds, NULL, NULL, &timeout);
		if (result > 0)
		{
			SOCKET Sock;
			SOCKADDR_IN SockAddr;
			SockAddr.sin_family = AF_INET;
#ifdef WIN32
			int SockAddrLen = sizeof(SockAddr);
#else
			unsigned int SockAddrLen = sizeof(SockAddr);
#endif
							
			Sock = accept(sock, (LPSOCKADDR) &SockAddr, &SockAddrLen);
			if (Sock != INVALID_SOCKET)
			{
				PTHREADDATA pThread = new(THREADDATA);
				pThread->sock = Sock;
				pThread->bThreadedApp = bUseThreads;
				HOSTENT * hp = gethostbyaddr((char *) &SockAddr.sin_addr, sizeof(in_addr), AF_INET);
#ifdef WIN32
				sprintf(pThread->szIPAddr, "%d.%d.%d.%d", ((((SockAddr).sin_addr).S_un).S_un_b).s_b1, ((((SockAddr).sin_addr).S_un).S_un_b).s_b2, ((((SockAddr).sin_addr).S_un).S_un_b).s_b3, ((((SockAddr).sin_addr).S_un).S_un_b).s_b4);
#else
				int a, b, c, d;
				int addr = SockAddr.sin_addr.s_addr;
				d = (addr >> 24) & 0x000000ff;
				c = (addr >> 16) & 0x000000ff;
				b = (addr >> 8) & 0x000000ff;
				a = (addr & 0x000000ff);
				sprintf(pThread->szIPAddr, "%d.%d.%d.%d", a, b, c, d);
#endif
				// Check to see whether this host is allowed or not
				SysMessage("Checking if host %s is allowed to connect...\n", pThread->szIPAddr );
				BOOL bAllowed = TRUE;
				try
				{
					bAllowed = HostIsAllowed(pThread->szIPAddr, hp);
				}
				catch (...)
				{
					SysMessage("Exception occurred in HostIsAllowed().  Allowing this host.\n");
				}
				if ( !bAllowed )
				{
					shutdown(Sock, 2);
					if ( hp != NULL )
						SysMessage("Blocking incoming connection from %s (%s).\n", pThread->szIPAddr, hp->h_name);
					else
						SysMessage("Blocking incoming connection from %s.\n", pThread->szIPAddr);
#ifdef WIN32
					closesocket(Sock);
#else
					close(Sock);
#endif
					delete pThread;
				}
				else
				{
					if ( hp != NULL )
						SysMessage("Socket %ld connected from %s (%s).\n", (int) pThread->sock, pThread->szIPAddr, hp->h_name);
					else
						SysMessage("Socket %ld connected from %s.\n", (int) pThread->sock, pThread->szIPAddr);
					if ( bUseThreads )
					{
#ifdef WIN32
						int tid = _beginthread(svrthread, 0, (LPVOID) pThread);
#else
						pthread_t tid;
						pthread_create( &tid, NULL, svrthread, pThread);
#endif
					}
					else
						svrthread(pThread);
				}
			}
#ifdef WIN32
			else
				int err = WSAGetLastError();
#endif
		}
		else
			// Reset
#ifdef WIN32
			Sleep(10);
#else
			sleep(1);
#endif
		// Periodically, reload the spawnpoint information.
		time_t timeNow = time(NULL);
		if ( timeNow - lastSpawnLoadTime > g_iSpawnRefreshTime )
		{
			lastSpawnLoadTime = timeNow;
			SysMessage("Reloading Spawn information.\n");
			if ( bUseThreads )
			{
#ifdef WIN32
				int tid = _beginthread(LoadSpawns, 0, NULL );
#else
				pthread_t tid;
				pthread_create( &tid, NULL, LoadSpawns, NULL );
#endif
			}
			else
				LoadSpawns(NULL);
		}

	}
#ifdef WIN32
	if (result == SOCKET_ERROR)
	{
		SysMessage("Server terminated with a socket error -- %ld\n", WSAGetLastError());
	}
	// Give existing threads a few seconds to finish up
	for (int i = 0; i < 5; i++ )
	{
		pService->ReportStatus();
		Sleep(1000);
	}
	WSACleanup();
	CloseHandle(hStdioMutex);
#else
	pthread_mutex_destroy(&hStdioMutex);
	sleep(5);
#endif
	if ( g_pszSpawns != NULL )
		delete [] g_pszSpawns;
	printf("Closing axissvr\n");
	return 0;
}

#ifdef WIN32
void svrthread(LPVOID pData)
#else
void * svrthread(LPVOID pData)
#endif
{
	PTHREADDATA pThread = (PTHREADDATA) pData;
	char crlf[] = {0x0d, 0x0a, 0x00};

	SOCKADDR_IN sin;
	sin.sin_family = AF_INET;
	bind(pThread->sock, (LPSOCKADDR) &sin, sizeof(sin));
	connect(pThread->sock, (LPSOCKADDR) &sin, sizeof(sin));
#ifdef _WIN32
	struct linger opt;
#else
	struct LINGER opt;
#endif
	opt.l_onoff = 1;
	opt.l_linger = 60;
	setsockopt(pThread->sock, SOL_SOCKET, SO_LINGER, (const char *)&opt, sizeof(opt));
	char szBuf[MAX_BUFFER];
	char szIn[MAX_BUFFER];
	int nBytes = 0;
	sprintf(szBuf, "AxisSvr connected.%s%sUsername: ", crlf, crlf);
	Send(pThread->sock, szBuf, strlen(szBuf));
	memset(&szIn[0], 0x00, sizeof(szIn));
	nBytes = Receive(pThread->sock, szIn, sizeof(szIn));
	if ( nBytes < 0 )
	{
		shutdown(pThread->sock, 2 );
		SysMessage("Error receiving data on socket %ld.  Closing\n", (int) pThread->sock);
		SysMessage("Socket %ld from %s closed.\n", (int) pThread->sock, pThread->szIPAddr);
#ifdef WIN32
		closesocket(pThread->sock);
		return;
#else
		close(pThread->sock);
		return NULL;
#endif
	}
	szIn[nBytes] = 0x00;
	Trim(&szIn[0], strlen(szIn));
	char szUserID[MAX_PATH];
	char szPassword[MAX_PATH];
	memset(&szUserID[0], 0x00, sizeof(szUserID));
	memset(&szPassword[0], 0x00, sizeof(szPassword));
	memcpy(szUserID, szIn, strlen(szIn));
	sprintf(szBuf, "Password: ");
	Send(pThread->sock, szBuf, strlen(szBuf));
	memset(&szIn[0], 0x00, sizeof(szIn));
	nBytes = Receive(pThread->sock, &szIn[0], sizeof(szIn));
	if ( nBytes < 0 )
	{
		shutdown(pThread->sock, 2 );
		SysMessage("Error receiving data on socket %ld.  Closing\n", (int) pThread->sock);
		SysMessage("Socket %ld from %s closed.\n", (int) pThread->sock, pThread->szIPAddr);
#ifdef WIN32
		closesocket(pThread->sock);
		return;
#else
		close(pThread->sock);
		return NULL;
#endif
	}
	szIn[nBytes] = 0x00;
	Trim(&szIn[0], strlen(szIn));
	memcpy(szPassword, szIn, strlen(szIn));
	int privs = GWLogin(&szUserID[0], &szPassword[0]);
	if (privs >= 0)
	{
		// Login was successful...process the commands
		sprintf(szBuf, "Login successful. Protocol 1.0%s", crlf);
		SysMessage("Login from IP %s successful for user %s\n", pThread->szIPAddr, szUserID);
		Send(pThread->sock, szBuf, strlen(szBuf));
		bool bQuit = false;
		while (!bQuit)
		{
			memset(&szIn[0], 0x00, sizeof(szIn));
			// Do a select here...
			fd_set fd;
			FD_ZERO(&fd);
			FD_SET(pThread->sock, &fd);
			struct timeval timeout;
			timeout.tv_sec = 0;
			timeout.tv_usec = 1000;
			int rc = select(pThread->sock+1, &fd, 0, 0, &timeout);
			if ( rc < 0 )
				break;
			if ( rc == 0 )
				continue;
			nBytes = Receive(pThread->sock, szIn, sizeof(szIn));
			if ( nBytes < 0 )
			{
				shutdown(pThread->sock, 2 );
				SysMessage("Error receiving data on socket %ld.  Closing\n", (int) pThread->sock);
				SysMessage("Socket %ld from %s closed.\n", (int) pThread->sock, pThread->szIPAddr);
#ifdef WIN32
				closesocket(pThread->sock);
				return;
#else
				close(pThread->sock);
				return NULL;
#endif
			}
			szIn[nBytes] = 0x00;
			if (nBytes > 0)
			{
				char szCmd[5];
				memset(&szCmd[0], 0x00, 5);
				memcpy(&szCmd[0], &szIn, 4);
				char szArg[MAX_PATH];
				memset(&szArg[0], 0x00, sizeof(szArg));
				if ( strlen(szIn) > 4 )
					memcpy(&szArg[0], &szIn[4], strlen(szIn) - 4);
				switch (FindTable(szCmd, &pszCommands[0], 37, sizeof(char *)))
				{
				case 0:
					SysMessage("Retrieving SpawnPoints for user %s\n", szUserID);
					GetSpawnPoints(pThread->sock);
					//send(pThread->sock, szBuf, strlen(szBuf), 0);
					break;
				case 1:
					SysMessage("Retrieving Items for user %s\n", szUserID);
					SendFile(pThread->sock, USER_ITEM_FILE);
					break;
				case 2:
					SysMessage("Retrieving NPCs for user %s\n", szUserID);
					SendFile(pThread->sock, USER_NPC_FILE);
					break;
				case 3:
					SysMessage("Retrieving defs for user %s\n", szUserID);
					SendFile(pThread->sock, USER_DEFS_FILE);
					break;
				case 4:
					SysMessage("Retrieving map for user %s\n", szUserID);
					SendFile(pThread->sock, USER_MAP_FILE);
					break;
				case 5:
					SysMessage("Retrieving tables for user %s\n", szUserID);
					SendFile(pThread->sock, USER_TABLE_FILE);
					break;
				case 6:
					SysMessage("Retrieving templates for user %s\n", szUserID);
					SendFile(pThread->sock, USER_TEMPLATE_FILE);
					break;
				case 7:
					SysMessage("Retrieving magic for user %s\n", szUserID);
					SendFile(pThread->sock, USER_MAGIC_FILE);
					break;
				case 8:
					SysMessage("Retrieving skills for user %s\n", szUserID);
					SendFile(pThread->sock, USER_SKILL_FILE);
					break;
				case 9:
					SysMessage("Retrieving speech for user %s\n", szUserID);
					SendFile(pThread->sock, USER_SPEECH_FILE);
					break;
				case 10:
					SysMessage("Retrieving names for user %s\n", szUserID);
					SendFile(pThread->sock, USER_NAME_FILE);
					break;
				case 11:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Receiving custom item from user %s\n", szUserID);
						GetNewItem(pThread->sock, szUserID, true);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the items script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 12:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Receiving custom NPC from user %s\n", szUserID);
						GetNewItem(pThread->sock, szUserID, false);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the NPC script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 13:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating defs from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_DEFS_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the defs script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 14:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating map from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_MAP_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the map script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 15:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating tables from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_TABLE_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the tables script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 16:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating templates from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_TEMPLATE_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the templates script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 17:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating magic from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_MAGIC_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the magic script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 18:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating skills from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_SKILL_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the skills script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 19:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating speech from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_SPEECH_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the speech script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 20:
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating names from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_NAME_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the names script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 21:
					bQuit = true;
					sprintf(szBuf, "Closing connection.%s", crlf);
					Send(pThread->sock, szBuf, strlen(szBuf));
					break;
				case 22:
					// GTRG
					SysMessage("Retrieving Triggers for user %s\n", szUserID);
					SendFile(pThread->sock, USER_TRIGGER_FILE);
					break;
				case 23:
					// STRG
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating triggers from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_TRIGGER_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the trigger script\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 24:
					// PORT
					if (privs == 1 && ! g_bReadOnly)
					{
						char szPort[80];
						sprintf(szPort, "%ld", dwTusPort);
						SysMessage("Retrieving TUS port for user %s\n", szUserID);
						Send(pThread->sock, szPort, strlen(szPort));
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to retrieve the TUS port\n", szUserID);
						Send(pThread->sock, "0", 1);
					}
					break;
				case 25:
					// GINI
					SysMessage("Retrieving ini file for user %s\n", szUserID);
					SendFile(pThread->sock, INI_FILE);
					break;
				case 26:
					// SINI
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating ini file from user %s\n", szUserID);
						ReceiveFile(pThread->sock, INI_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the ini file\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 27:
					// STRT
					SysMessage("User %s is attempting to start the server\n", szUserID);
#ifdef WIN32
					{
						FILE * pfIni;
						char szServerName[MAX_PATH];
						bool bNTService = false;
						pfIni = fopen(INI_FILE, "r");
						if ( pfIni )
						{
							char szBuffer[MAX_BUFFER];
							while ( !feof(pfIni) )
							{	
								memset(szBuffer, 0x00, sizeof(szBuffer));
								fReadLine(pfIni, szBuffer, sizeof(szBuffer));
								char * pComment = strstr(szBuffer, "//" );
								if ( pComment != NULL )
									*pComment = 0x00;
								stoupper( &szBuffer[0], strlen(szBuffer) );
								if ( strstr(szBuffer, "NTSERVICE=1") != NULL )
									bNTService = true;
								if ( strstr(szBuffer, "[SERVERS]") != NULL )
								{
									memset(szBuffer, 0x00, sizeof(szBuffer));
									fReadLine(pfIni, szBuffer, sizeof(szBuffer));
									sprintf(szServerName, "%s", szBuffer);
								}
							}
						}
						else
							SysMessage("Unable to open the INI file...using defaults\n");
						char szCmd[MAX_PATH];
						char szCwd[MAX_PATH];
						memset(&szCwd[0], 0x00, sizeof(szCwd));
						getcwd(szCwd, sizeof(szCwd));
						if ( bNTService )
						{
							sprintf(szCmd, "Sphere - %s", szServerName);
							SysMessage("Attempting to open the SCUM\n");
							SC_HANDLE hScum = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
							if ( hScum != NULL )
							{
								SysMessage("Attempting to open the %s service\n", szCmd);
								SC_HANDLE hService = OpenService(hScum, szCmd, SERVICE_START);
								if ( hService != NULL )
								{
									SysMessage("Attempting to start the service\n");
									if ( StartService( hService, NULL, NULL ) )
										SysMessage("Service Started Successfully\n");
									else
										SysMessage("Service Start Failed\n");
									CloseServiceHandle(hService);
									CloseServiceHandle(hScum);
									return;
								}
								else
								{
									SysMessage("Unable to open the service\n");
									CloseServiceHandle(hScum);
									return;
								}
							}
							else
							{
								SysMessage("Unable to open the SCUM\n");
								return;
							}
						}
						else
						{
							sprintf(szCmd, "%s\\%s", szCwd, SERVER_EXE);
							STARTUPINFO start;
							memset( &start, 0x00, sizeof(start) );
							start.cb = sizeof(start);
							PROCESS_INFORMATION procInfo;
							memset( &procInfo, 0x00, sizeof(procInfo) );
							SysMessage("Attempting CreateProcess() for %s\n", szCmd);
							if ( ! CreateProcess( szCmd, NULL, NULL, NULL, FALSE, CREATE_NEW_CONSOLE, NULL, szCwd, &start, &procInfo ) )
							{
								DWORD dwError = GetLastError();
								SysMessage("CreateProcess() returned error %ld  - %s\n", dwError, strerror(dwError));
							}
						}
					}

#else
					{
						SysMessage("Attempting to start server...\n");
						int pid = fork();
						if ( pid == 0 )
						{
							// This is the child
							SysMessage("fork() returned 0\n");
							char szPath[MAX_PATH];
							getcwd(&szPath[0], MAX_PATH);
							char * argv[3];
							argv[0] = "sphere";
							//sprintf(argv[1], "%s/sphere", szPath);
							argv[1] = "&";
							argv[2] = 0;
							//argv[3] = 0;
							int rc = execvp("./sphere", argv );
							SysMessage("execlp() returned error %ld\n", rc);
						}
						if ( pid == -1 )
							SysMessage("fork() failed...\n");
					}
#endif
					break;
				case 28: 
					// GACT
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Retrieving account file for user %s\n", szUserID);
						SendFile(pThread->sock, g_szAccuFile);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s lacks sufficient privelege to retrieve the account file", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 29:
					// SACT
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating account file from user %s\n", szUserID);
						ReceiveFile(pThread->sock, g_szAcctFile);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the account file\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 30:
					// GDIR
					SysMessage("Retrieving directory information for path %s for user %s\n", szArg, szUserID);
					GetDirInfo(pThread->sock, szArg);
					break;
				case 31:
					// GFIL
					SysMessage("Retrieving file %s for user %s\n", szArg, szUserID);
#ifndef _WIN32
					{
						for ( int i = 0; i < strlen(szArg); i++ )
							if ( szArg[i] == '\\' )
								szArg[i] = '/';
					}
#endif
					SendFile(pThread->sock, szArg);
					break;
				case 32:
					// SFIL
					SysMessage("Updating file %s from user %s\n", szArg, szUserID);
#ifndef _WIN32
					{
						for ( int i = 0; i < strlen(szArg); i++ )
							if ( szArg[i] == '\\' )
								szArg[i] = '/';
					}
#endif
					ReceiveFile(pThread->sock, szArg);
					break;
				case 33:
					// SCH2
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating NPC file from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_NPC_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the NPC file\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 34:
					// SIT2
					if (privs == 1 && ! g_bReadOnly)
					{
						SysMessage("Updating Item file from user %s\n", szUserID);
						ReceiveFile(pThread->sock, USER_ITEM_FILE);
					}
					else
					{
						if ( g_bReadOnly )
							SysMessage("Axissvr is in readonly mode\n");
						else
							SysMessage("User %s has insufficient priveleges to update the Item file\n", szUserID);
						NoWritePrivs(pThread->sock);
					}
					break;
				case 35:
					// GRES
					SysMessage("Retrieving Resources for user %s\n", szUserID);
					SendResources(pThread->sock);
					break;
				case 36:
					// GFCS
					SysMessage("Getting checksum on file %s for user %s\n", szArg, szUserID);
					SendChecksum(pThread->sock, szArg);
					break;
				default:
					sprintf(szBuf, "Command unknown (%s).%s", szCmd, crlf);
					Send(pThread->sock, szBuf, strlen(szBuf));
					SysMessage("Unknown message %s received from user %s\n", szCmd, szUserID);
					break;
				}
			}
			else
				bQuit = true;
		}
	}
	else
	{
		if (privs == -1)
		{
			SysMessage("Login attempt from IP %s failed: userid=%s password=%s\n", pThread->szIPAddr, szUserID, szPassword);
			sprintf(szBuf, "Login failed.  Bad userid or password.%s", crlf);
		}
		if (privs == -2)
		{
			SysMessage("Login attempt from IP %s failed -- insufficient priveledge: userid=%s\n", pThread->szIPAddr, szUserID);
			sprintf(szBuf, "Login failed.  Insufficient priveleges.%s", crlf);
		}
		if (privs == -3)
		{
			sprintf(szBuf, "Login failed.  Account file not found.%s", crlf);
		}
		Send(pThread->sock, szBuf, strlen(szBuf));
	}
	shutdown(pThread->sock, 2 );
	SysMessage("Socket %ld from %s closed.\n", (int) pThread->sock, pThread->szIPAddr);
#ifdef WIN32
	closesocket(pThread->sock);
	return;
#else
	close(pThread->sock);
	return NULL;
#endif
}

int GWLogin(char *pszID, char *pszPassword)
{
	// Scan the grayacct.scp file for this userid
	FILE *acctFile = NULL;
	// Try the new file and location
	// Need to find the save directory.
	FILE *pIniFile = NULL;
	pIniFile = fopen(INI_FILE, "r");
	if (pIniFile == NULL)
	{
		SysMessage("Unable to open INI file to determine the location of the account file.\n");
		return -3;
	}
	char szPath[MAX_PATH];
	memset(szPath, 0x00, sizeof(szPath));
	char szLine[MAX_PATH];
	while(!feof(pIniFile))
	{
		fReadLine(pIniFile, &szLine[0], sizeof(szLine));
		Trim(&szLine[0], strlen(szLine));
		if (szLine[0] != 0x00)
		{
			if ((strstr(szLine, "WORLDSAVE=") == szLine) && (szPath[0] == 0x00))
				memcpy(&szPath[0], &szLine[10], strlen(szLine) - 10);
			if (strstr(szLine, "ACCTFILES=") == szLine)
			{
				memset(szPath, 0x00, sizeof(szPath));
				memcpy(&szPath[0], &szLine[10], strlen(szLine) - 10);
				break;
			}
		}
	}
	fclose(pIniFile);
	if ( strcmp(szPath, "") != 0 )
	{
#ifdef WIN32
		if (szPath[strlen(szPath) - 1] != '\\')
		{
			sprintf(g_szAccuFile, "%s\\%s", szPath, NEW_ACCT_FILE);
			sprintf(g_szAcctFile, "%s\\%s", szPath, ACCT_FILE);
		}
		else
		{
			sprintf(g_szAccuFile, "%s%s", szPath, NEW_ACCT_FILE);
			sprintf(g_szAcctFile, "%s%s", szPath, ACCT_FILE);
		}
#else
		if (szPath[strlen(szPath) - 1] != '/')
		{
			sprintf(g_szAccuFile, "%s/%s", szPath, NEW_ACCT_FILE);
			sprintf(g_szAcctFile, "%s/%s", szPath, ACCT_FILE);
		}
		else
		{
			sprintf(g_szAccuFile, "%s%s", szPath, NEW_ACCT_FILE);
			sprintf(g_szAcctFile, "%s%s", szPath, ACCT_FILE);
		}
#endif
		acctFile = fopen(g_szAccuFile, "r" );
	}
	if ( acctFile == NULL )
	{
		SysMessage("Unable to determine the location of the account file.  Account path is \"%s\"\n", pszID, szPath);
		return -3;
	}

	char szBuffer[MAX_PATH];
	char szTargetAcct[MAX_PATH];
	char szTargetPassword[MAX_PATH];
	sprintf(szTargetAcct, "[%s]", pszID);
	stoupper(&szTargetAcct[0], strlen(szTargetAcct));
	sprintf(szTargetPassword, "PASSWORD=%s", pszPassword);
	// Since newer versions of Sphere store passwords in quotes, we'll need to check against this as well
	char szTargetPassword2[MAX_PATH];
	sprintf(szTargetPassword2, "PASSWORD=\"%s\"", pszPassword);

	bool bBadPrivs = true;
	bool bBadPassword = true;
	bool bWritePrivs = false;
	while (!feof(acctFile))
	{
		if (fReadLine(acctFile, &szBuffer[0], sizeof(szBuffer)) > 0)
		{
			stoupper(&szBuffer[0], strlen(szBuffer));
			if (strstr(szBuffer, szTargetAcct) != NULL)
			{
				// Found the account
				memset(&szBuffer[0], 0x00, sizeof(szBuffer));
				while (szBuffer[0] != '[')
				{
					if (fReadLine(acctFile, &szBuffer[0], sizeof(szBuffer)) > 0)
					{
						// Check for either PRIVS or PASSWORD
						if (strstr(szBuffer, "PASSWORD=") != NULL)
						{
							// Compare it to the real password
							// Fix security bug -- William Arts (Belgar)
							if ( (strcmp(szBuffer, szTargetPassword) == 0) || (strcmp(szBuffer, szTargetPassword2) == 0 ) )
								bBadPassword = false;
						}
						if (strstr(szBuffer, "PLEVEL=") != NULL)
						{
							char szLevel[20];
							memset(&szLevel[0], 0x00, 20);
							memcpy(&szLevel, &szBuffer[7], 19);
							int iLevel;
							switch (FindTable(szLevel, &pszPLevels[0], 8, sizeof(char *)))
							{
								case 0:
								case 1:
								case 2:
								case 3:
								case 4:
								case 5:
								case 6:
								case 7:
									iLevel = FindTable(szLevel, &pszPLevels[0], 8, sizeof(char *));
									break;
								default:
									iLevel = atoi(szLevel);
							}
							switch(iLevel)
							{
							case 0: case 1: case 2: case 3:
								bBadPrivs = true;
								bWritePrivs = false;
								break;
							case 4:
								bBadPrivs = false;
								bWritePrivs = false;
								break;
							case 5: case 6: case 7:
								bBadPrivs = false;
								bWritePrivs = true;
								break;
							default:
								bBadPrivs = true;
								bWritePrivs = false;
								break;
							}
						}
					}
					else
						break;
				}
				break;
			}
		}
	}
	fclose(acctFile);
	if (bBadPassword)
		return -1;
	if (bBadPrivs)
		return -2;
	if (bWritePrivs)
		return 1;
	else
		return 0;
}

void SysMessage(const char *format, ...)
{
	va_list args;
	va_start(args, format);
#ifdef WIN32
	char szMutex[256];
	sprintf(szMutex, "stdioMutex%s", pService->GetServerName() );
	HANDLE hMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, szMutex);
	if (hMutex == NULL)
#else
	int rc = pthread_mutex_trylock(&hStdioMutex);
	if (rc != 0)
#endif
	{
		printf("Unable to get mutex");
		return;
	}
	else
	{
		FILE * pLog;
		char szFilename[256];
		time_t now;
		now = time(NULL);
		struct tm *tnow = localtime( &now);
		if ( g_szLogPath[0] != 0x00 )
		{
#ifdef WIN32
			if ( g_szLogPath[strlen(g_szLogPath)] != '\\' )
				sprintf(szFilename, "%s\\axissvr.%04d.%02d.%02d.log", g_szLogPath, tnow->tm_year + 1900, tnow->tm_mon + 1, tnow->tm_mday);
#else
			if ( g_szLogPath[strlen(g_szLogPath)] != '/' )
				sprintf(szFilename, "%s/axissvr.%04d.%02d.%02d.log", g_szLogPath, tnow->tm_year + 1900, tnow->tm_mon + 1, tnow->tm_mday);
#endif
			else
				sprintf(szFilename, "%saxissvr.%04d.%02d.%02d.log", g_szLogPath, tnow->tm_year + 1900, tnow->tm_mon + 1, tnow->tm_mday);
		}
		else
			sprintf(szFilename, "axissvr.%04d.%02d.%02d.log", tnow->tm_year + 1900, tnow->tm_mon + 1, tnow->tm_mday);
		pLog = fopen(szFilename, "a+");
		if ( pLog )
		{
			vfprintf(pLog, format, args);
			fflush(pLog);
			fclose(pLog);
		}
		else
		{
			printf("ERROR:  Unable to open log file %s\n", szFilename);
		}
		vprintf(format, args);
	}
#ifdef WIN32
	ReleaseMutex(hMutex);
#else
	pthread_mutex_unlock(&hStdioMutex);
#endif
	va_end(args);
}

int FindTable( const char * pFind, const char * const * ppTable, int iCount, int iElemSize )
{
	for ( int i=0; i<iCount; i++ )
	{
#ifdef WIN32
		if ( ! strcmpi( *ppTable, pFind ))
#else
 		if ( ! strcasecmp( *ppTable, pFind ))
#endif
			return( i );
		ppTable = (const char * const *)((( const BYTE*) ppTable ) + iElemSize );
	}
	return( -1 );
}

int fReadLine(FILE * pFile, char * pszBuf, int iSize)
{
	memset(pszBuf, 0x00, iSize);
	int iCount = 0;
	char c = 0x00;
	while (c != '\n' && iCount < (iSize - 1) && !feof(pFile))
	{
		c = fgetc(pFile);
		if (c != '\n' && c != -1)
		{
			*(pszBuf + iCount) = c;
			iCount++;
		}
	}
	if (feof(pFile) && iCount == 0)
		return -1;
	return iCount;
}

void Trim(char *pszBuf, int iSize)
{
	for (int i = 0; i < iSize; i++)
	{
		if  (*(pszBuf + i) == 0x0d || *(pszBuf + i) == 0x0a)
			*(pszBuf + i) = 0x00;
	}
}

void stoupper(char * pszBuf, int iSize)
{
	for (int i = 0; i < iSize; i++)
			*(pszBuf + i) = toupper(*(pszBuf + i));
}

void GetSpawnPoints(SOCKET sock)
{
	if ( g_pszSpawns == NULL )
		LoadSpawns(NULL);
	DWORD dwBytesSent = 0;
	char szBuf[MAX_BUFFER + 1];
	sprintf(szBuf, "%ld", strlen(g_pszSpawns));
	Send(sock, szBuf, strlen(szBuf));
	while(dwBytesSent < strlen(g_pszSpawns))
	{
		memset(szBuf, 0x00, sizeof(szBuf));
		int bytesToCopy = MAX_BUFFER;
		if ( dwBytesSent + MAX_BUFFER > strlen(g_pszSpawns) )
			bytesToCopy = strlen(g_pszSpawns) - dwBytesSent;
		memcpy(szBuf, g_pszSpawns + dwBytesSent, bytesToCopy);
		dwBytesSent += bytesToCopy;
		Send(sock, szBuf, bytesToCopy);
	}
	sprintf(szBuf, ".%c%c", 0x0d, 0x0a );
	Send(sock, szBuf, strlen(szBuf));
	return;
}

void SendFile(SOCKET sock, char *pszFileName)
{
	char szOutput[MAX_BUFFER];
	char crlf [] = {0x0d, 0x0a, 0x00};
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	FILE *pFile;
	char szScriptPath[MAX_PATH];
	memset(szScriptPath, 0x00, sizeof(szScriptPath));
	// Check to see if the scripts are somewhere else.
	FILE *pIniFile;
	pIniFile = fopen(INI_FILE, "r");
	if (pIniFile != NULL)
	{
		char szLine[MAX_PATH];
		while(!feof(pIniFile))
		{
			fReadLine(pIniFile, &szLine[0], sizeof(szLine));
			Trim(&szLine[0], strlen(szLine));
			if (szLine[0] != 0x00)
			{
				if (strcmp( pszFileName, g_szAccuFile) == 0)
				{
					if (strstr(szLine, "ACCTFILES=") == szLine)
					{
						memcpy(&szScriptPath[0], &szLine[9], strlen(szLine) - 9);
						break;
					}
					else if ( szScriptPath[0] == 0x00 && (strstr(szLine, "WORLDSAVE=") == szLine) )
					{
						memcpy(&szScriptPath[0], &szLine[9], strlen(szLine) - 9);
						break;
					}
				}
				else
				{
					if (strstr(szLine, "SCPFILES=") == szLine)
					{
						memcpy(&szScriptPath[0], &szLine[9], strlen(szLine) - 9);
						break;
					}
				}
			}
		}
		fclose(pIniFile);
	}
	char szFilename[MAX_PATH];
/* Axis should actually be sending us the relative path for the file, so we don't need to do this.
	if ( strcmp(pszFileName, INI_FILE) != 0
	 && strcmp(pszFileName, TABLE_FILE) != 0
	 && szScriptPath[0] != 0x00
#ifdef WIN32
	 && strstr(pszFileName, "\\") == NULL) 
#else
	 && strstr(pszFileName, "/") == NULL) 
#endif
	{
#ifdef WIN32
		if ( szScriptPath[strlen(szScriptPath) - 1] != '\\' )
			sprintf(szFilename, "%s\\%s", szScriptPath, pszFileName);
		else
			sprintf(szFilename, "%s%s", szScriptPath, pszFileName);
#else
		if ( szScriptPath[strlen(szScriptPath) - 1] != '/' )
			sprintf(szFilename, "%s/%s", szScriptPath, pszFileName);
		else
			sprintf(szFilename, "%s%s", szScriptPath, pszFileName);
#endif
	}
	else
		sprintf(szFilename, "%s", pszFileName);
*/

	// pFile = fopen(szFilename, "r");

	pFile = fopen(pszFileName, "r");
	if (pFile == NULL)
	{
		SysMessage("Error -- Could not open file %s\n", pszFileName);
		sprintf(szOutput, "Could not open file %s.%s", pszFileName, crlf);
		int iBytesSent = Send(sock, szOutput, strlen(szOutput));
		if ( iBytesSent < 0 )
		{
			SysMessage("Error occurred while sending file %s - File could not be opened\n", szFilename );
			fclose(pFile);
			return;
		}
#ifdef WIN32
		Sleep(SLEEPTIME);
#else
		sleep(SLEEPTIME/1000);
#endif
		return;
	}
	fseek(pFile, 0, SEEK_END);
	long lLength = ftell(pFile);
	rewind(pFile);
	sprintf(szOutput, "%ld", lLength);
	Send(sock, szOutput, strlen(szOutput));
	while (!feof(pFile))
	{
		int iBytesRead = fread(&szOutput[0], 1, sizeof(szOutput), pFile);
		if ( iBytesRead )
		{
			int iBytesSent = Send(sock, szOutput, iBytesRead);
#ifdef WIN32
			Sleep(SLEEPTIME);
#else
			sleep(SLEEPTIME/1000);
#endif
			char szInput[5];
			memset(&szInput[0], 0x00, sizeof(szInput));
			Receive(sock, &szInput[0], sizeof(szInput));
			if ( atoi(szInput) != iBytesSent )
			{
				SysMessage("Ack failed while sending file. %ld != %ld\n", atoi(szInput), iBytesSent);
				fclose(pFile);
				return;
			}
		}
	}
	fclose(pFile);
	Send(sock, szEOF, strlen(szEOF));
#ifdef WIN32
	Sleep(SLEEPTIME);
#else
	sleep(SLEEPTIME/1000);
#endif
	memset( szOutput, 0x00, sizeof(szOutput) );
	return;
}

void ReceiveFile(SOCKET sock, char *pszFileName)
{
	char szBuffer[MAX_BUFFER];
	char szNewFilename[MAX_PATH];
	char szTmpFilename[MAX_PATH];
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	char szScriptPath[MAX_PATH];
	memset(szScriptPath, 0x00, sizeof(szScriptPath));
	// Check to see if the scripts are somewhere else.
	FILE *pIniFile;
	pIniFile = fopen(INI_FILE, "r");
	if (pIniFile != NULL)
	{
		char szLine[MAX_PATH];
		while(!feof(pIniFile))
		{
			fReadLine(pIniFile, &szLine[0], sizeof(szLine));
			Trim(&szLine[0], strlen(szLine));
			if (szLine[0] != 0x00)
			{
				if (strstr(szLine, "SCPFILES=") == szLine)
				{
					memcpy(&szScriptPath[0], &szLine[9], strlen(szLine) - 9);
					break;
				}
			}
		}
		fclose(pIniFile);
	}
	char szFilename[MAX_PATH];
#ifdef WIN32
	if ( szScriptPath[0] != 0x00 && strstr(pszFileName, "\\") == NULL) 
#else
	if ( szScriptPath[0] != 0x00 && strstr(pszFileName, "/") == NULL) 
#endif
	{
#ifdef WIN32
		if ( szScriptPath[strlen(szScriptPath) - 1] != '\\' )
			sprintf(szFilename, "%s\\%s", szScriptPath, pszFileName);
		else
			sprintf(szFilename, "%s%s", szScriptPath, pszFileName);
#else
		if ( szScriptPath[strlen(szScriptPath) - 1] != '/' )
			sprintf(szFilename, "%s/%s", szScriptPath, pszFileName);
		else
			sprintf(szFilename, "%s%s", szScriptPath, pszFileName);
#endif
	}
	else
		sprintf(szFilename, "%s", pszFileName);
	// Back up the current file
	char szBackupName[MAX_PATH];
	time_t now = time(NULL);
	sprintf(szBackupName, "%s.bkp.%ld", szFilename, now);

	sprintf(szNewFilename, szFilename);
	sprintf(szTmpFilename, "%s.tmp", szFilename);
	FILE * tmpFile = fopen(szTmpFilename, "w");
	if (tmpFile == NULL)
	{
		SysMessage("Error opening temporary file on server (%s)\n", pszFileName);
		sprintf(szBuffer, "Error opening file on remote server (%s)%c%c", pszFileName, 0x0d, 0x0a);
		Send(sock, szBuffer, strlen(szBuffer));
#ifdef WIN32
		Sleep(SLEEPTIME);
#else
		sleep(SLEEPTIME/1000);
#endif
		return;
	}
	sprintf(szBuffer, "Ready to receive entry.  Send CRLF.CRLF to signal end of file%c%c", 0x0d, 0x0a);
	Send(sock, szBuffer, strlen(szBuffer));
#ifdef WIN32
	Sleep(SLEEPTIME);
#else
	sleep(SLEEPTIME/1000);
#endif
	int iLoopCount = 0;
	BOOL bEndOfFile = FALSE;
	char szLinuxEOF [] = { '[', 'E', 'O', 'F', ']', 0x00};
#ifndef WIN32
	BOOL bLinuxEOF = FALSE;
#endif
	int iBytesRead = 0;
	while (strstr(szBuffer, szEOF) == NULL)
	{
		iLoopCount++;
		if (iLoopCount > 3000)
			break;
		memset(szBuffer, 0x00, sizeof(szBuffer));
		int iLength = Receive(sock, &szBuffer[0], sizeof(szBuffer) - 1);
#ifdef WIN32
		Sleep(SLEEPTIME);
#else
		sleep(SLEEPTIME/1000);
#endif
		if (iLength > 0)
		{
			if (strstr(szBuffer, szEOF) == NULL)
			{
				iBytesRead += iLength;
				char szNewBuffer[MAX_BUFFER * 2];
				memset(szNewBuffer, 0x00, sizeof(szNewBuffer));
				int j = 0;
				for ( int i = 0; i < iLength; i++ )
				{
#ifndef WIN32
					if (szBuffer[i] != 0x0d )
#endif
					szNewBuffer[j++] = szBuffer[i];
					if ( szBuffer[i] == '%' )
						szNewBuffer[j++] = '%';
				}
				fprintf(tmpFile, szNewBuffer);
#ifndef WIN32
				if (strstr(szBuffer, szLinuxEOF) != NULL)
					bLinuxEOF = true;
#endif
				sprintf(szBuffer, "%ld", iLength);
				Send(sock, &szBuffer[0], strlen(szBuffer));
			}
			else
				bEndOfFile = TRUE;
		}
	}
	SysMessage("%ld Total Bytes read.\n", iBytesRead);

#ifndef WIN32
	if (!bLinuxEOF)
	{
		// write the EOF marker
		memset(&szBuffer[0], 0x00, sizeof(szBuffer));
		sprintf(szBuffer, "%s\n", szLinuxEOF);
		fwrite(szBuffer, sizeof(char), strlen(szBuffer), tmpFile);
	}
#endif
	fclose(tmpFile);
	if (iLoopCount > 3000)
	{
		SysMessage("Loop count exceeded receiving new file -- Data will not be saved.\n");
		return;
	}
	if (bEndOfFile)
	{
		// Successfully received the file
		rename(szNewFilename, szBackupName);
		rename(szTmpFilename, szNewFilename);
	}
	Send(sock, "File Received", 13);
	return;
}

void GetNewItem(SOCKET sock, char *pszUserID, bool bItemFlag)
{
	char szBuffer[MAX_BUFFER];
	char szData[MAX_BUFFER * 8];
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};

	memset(szBuffer, 0x00, sizeof(szBuffer));
	memset(szData, 0x00, sizeof(szBuffer));

	sprintf(szBuffer, "Ready to receive entry.  Send CRLF.CRLF to signal end of entry%c%c", 0x0d, 0x0a);
	Send(sock, szBuffer, strlen(szBuffer));
#ifdef WIN32
	Sleep(SLEEPTIME);
#else
	sleep(SLEEPTIME/1000);
#endif
	int iLoopCount = 0;
	while (strstr(szBuffer, szEOF) == NULL)
	{
		iLoopCount++;
		if (iLoopCount > 3000)
			break;
		memset(szBuffer, 0x00, sizeof(szBuffer));
		int iLength = Receive(sock, &szBuffer[0], sizeof(szBuffer));
		if (iLength)
		{
			if (strstr(szBuffer, szEOF) == NULL)
			{
				if (strlen(szData) + iLength < sizeof(szData))
					strcat(szData, szBuffer);
				else
				{
					SysMessage("Data received exceeds maximum length for the buffer.\n");
					break;
				}
				sprintf(szBuffer, "%ld", iLength);
				Send(sock, &szBuffer[0], strlen(szBuffer));
			}
		}
	}
	if (iLoopCount > 3000)
	{
		SysMessage("Loop count exceeded receiving data from user %s.  Data will not be saved\n", pszUserID);
		return;
	}
	char szNewData[MAX_BUFFER * 8];
	memset(&szNewData[0], 0x00, sizeof(szNewData));
	int j = 0;
	for (int i = 0; i < sizeof(szData); i++)
	{
		if (szData[i] != 0x0d)
		{
			szNewData[j] = szData[i];
			j++;
		}
	}
	memset(&szData[0], 0x00, sizeof(szData));
	memcpy(&szData[0], &szNewData[0], sizeof(szNewData));

	char szItemTag[9];
	memset(szItemTag, 0x00, sizeof(szItemTag));
	if (strstr(szData, "[]") == NULL)
	{
		// This is not a new item.  It is either an override or an update to a custom item
		char * pszLeft = strchr(szData, '[');
		char * pszRight = strchr(szData, ']');
		int iLength = pszRight - pszLeft;
		if (iLength < 8 && iLength > 0)
			memcpy(&szItemTag[0], pszLeft + 1, iLength - 1);
	}
	int iNewItem = -1;
	if (szItemTag[0] != 0x00)
		iNewItem = (int) ahextoi(szItemTag);

	FILE * pOldFile;
	char szOldFile[MAX_PATH];
	sprintf(szOldFile, bItemFlag ? USER_ITEM_FILE : USER_NPC_FILE);
	pOldFile = fopen(szOldFile, "r");
	if (pOldFile)
	{
		FILE * pNewFile;
		char szNewFile[MAX_PATH];
		sprintf(szNewFile, bItemFlag ? TMP_ITEM_FILE : TMP_NPC_FILE);
		pNewFile = fopen(szNewFile, "w");
		int iLastEntry = 0;
		BOOL bItemWritten = FALSE;
		if (pNewFile)
		{
			while (!feof(pOldFile))
			{
				char szInput[512];
				fReadLine(pOldFile, &szInput[0], sizeof(szInput));
				if (szInput[0] == -1)
					szInput[0] = 0x00;
				if (strchr(szInput, '[') != NULL)
				{
					char * pszLeft = strchr(szInput, '[');
					char * pszRight = strchr(szInput, ']');
					int iLength = 0;
					if (pszRight != NULL)
						iLength = pszRight - pszLeft;
					if (iLength != 0)
					{
						char szTag[9];
						memset(szTag, 0x00, sizeof(szTag));
						memcpy(&szTag[0], pszLeft + 1, iLength - 1);
						iLastEntry = (int) ahextoi(szTag);
						if (iLastEntry == iNewItem)
						{
							// Write the new item in this location and skip the rest of the item from the old file
							bItemWritten = true;
							fprintf(pNewFile, szData);
							fprintf(pNewFile, "//// Updated by %s\n\n", pszUserID);
							while (!feof(pOldFile))
							{
								fReadLine(pOldFile, &szInput[0], sizeof(szInput));
								if (strchr(szInput, '[') != NULL)
								{
									fprintf(pNewFile, "%s\n", szInput);
									break;
								}
							}
						}
						else
						{
							if ((iLastEntry > iNewItem) && (iNewItem != -1) && (!bItemWritten))
							{
								// Insert this item here
								bItemWritten = true;
								fprintf(pNewFile, szData);
								fprintf(pNewFile, "//// Inserted by %s\n\n", pszUserID);
							}
							fprintf(pNewFile, "%s\n", szInput);
						}
					}
				}
				else
				{
					// Just write this line
					fprintf(pNewFile, "%s\n", szInput);
				}
			}
			if (!bItemWritten)
			{
				int iCustomEntry = bItemFlag ? 0x6000 : 0x4000;
				if (iLastEntry > iCustomEntry)
					iCustomEntry = iLastEntry + 1;
				fprintf(pNewFile, "[%04x]\n", iCustomEntry);
				fprintf(pNewFile, strchr(szData, ']') + 2);
				fprintf(pNewFile, "//// Inserted by %s\n\n", pszUserID);
				iNewItem = iCustomEntry;
			}
			fclose(pOldFile);
			fclose(pNewFile);
			// Heh...forgot to rename the files
			unlink(bItemFlag ? USER_ITEM_FILE : USER_NPC_FILE);
			rename(bItemFlag ? TMP_ITEM_FILE : TMP_NPC_FILE, bItemFlag ? USER_ITEM_FILE : USER_NPC_FILE);
			sprintf(szBuffer, "OK - ID=%04x", iNewItem );
			Send( sock, szBuffer, strlen(szBuffer));
		}
		else
		{
			SysMessage("Unable to open temporary file for writing.\n");
			fclose(pOldFile);
		}
	}
	else
		SysMessage("Unable to open source file.\n");

}

DWORD ahextoi( const char * pszStr ) // Convert hex string to integer
{
	// Unfortunatly the library func cant handle the number FFFFFFFF
	// char * sstop; return( strtol( s, &sstop, 16 ));

	GETNONWHITESPACE( pszStr );

	DWORD val = 0;
	while ( TRUE )
	{
		char ch = *pszStr;
		if ( ch >= '0' && ch <= '9' )
			ch -= '0';
		else
		{
			ch |= 0x20;	// toupper()
			if ( ch > 'f' || ch <'a' ) break;
			ch -= 'a' - 10;
		}
		val *= 0x10;
		val += ch;
		pszStr ++;
	}
	return( val );
}

void NoWritePrivs(SOCKET sock)
{
	char szBuffer[80];
	sprintf(szBuffer, "This user does not have sufficient priveleges on the remote server%c%c", 0x0d, 0x0a);
	Send(sock, szBuffer, strlen(szBuffer));
#ifdef WIN32
	Sleep(SLEEPTIME);
#else
	sleep(SLEEPTIME/1000);
#endif
	return;
}

int Send(SOCKET sock, const void * pszBuffer, int iLength)
{
	char szNewBuffer[MAX_BUFFER + sizeof(int)];
	memset(szNewBuffer, 0x00, sizeof(szNewBuffer));
	int * header = (int *) &szNewBuffer;
	*header = iLength;
	//memcpy(&szNewBuffer[0], (void *)iLength, sizeof(int) );
	memcpy(&szNewBuffer[sizeof(int)], pszBuffer, iLength);
	int nBytes = 0;
	int i = 0;
	while (nBytes < iLength)
	{
		int nSent = 0;
		if ( nBytes )
			nSent = send(sock, &szNewBuffer[nBytes + sizeof(int)], iLength + sizeof(int) - nBytes, 0);
		else
			nSent = send(sock, szNewBuffer, iLength + sizeof(int), 0);
		if ( nSent < 0 )
		{
#ifdef WIN32
			SysMessage("Error while sending packet to socket %ld.  Last error was %s\n", sock, strerror(WSAGetLastError()));
#else
			SysMessage("Error while sending packet to socket %ld.  Last error was %s\n", sock, strerror(errno));
#endif
			return -1;
		}
		if ( i++ > 100 )
		{
			SysMessage("Loop count exceeded while sending packet to socket %ld\n", sock);
			return -1;
		}
		i++;
		nBytes += nSent - sizeof(int);
		if ( nBytes < iLength )
#ifdef WIN32
			Sleep(SLEEPTIME);
#else
			sleep(SLEEPTIME/1000);
#endif
	}
	return nBytes;
}

int Receive(SOCKET sock, char * pszBuffer, int iMaxLength)
{
	int nBytes = 0;
	int i = 0;
	// Pull off the length of the incoming packet.
	struct timeval timeout;
	int iLength = 0;
	while ( !iLength )
	{
		fd_set fd;
		FD_ZERO(&fd);
		FD_SET(sock, &fd);
		timeout.tv_sec = 0;
		timeout.tv_usec = 1000;
		int rc = select(sock+1, &fd, 0, 0, &timeout);
		if (rc < 0)
			return -1;
		if ( i++ > g_iReceiveTimeout )
		{
			SysMessage("Timeout exceeded while receiving data from socket %ld\n", sock);
			break;
		}
		if ( rc > 0 )
			rc = recv(sock, (char *) &iLength, sizeof(int), 0);
		else
			continue;
		if ( rc == 0 && iLength == 0 )
		{
			SysMessage("Error receiving data from socket %ld.  recv() returned %ld.  iLength = %ld\n", sock, rc, iLength);
			return -2;
		}
	}
	i = 0;
	// Make sure the length is valid
	if ( iLength > iMaxLength )
	{
		SysMessage("Error receiving data from socket %ld.  The packet length (%ld) is greater than the buffer length.(%ld)", sock, iLength, iMaxLength);
		return -4;
	}
	while ( nBytes < iLength )
	{
		fd_set fd;
		FD_ZERO(&fd);
		FD_SET(sock, &fd);
		struct timeval timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = 1000;
		int rc = select(sock+1, &fd, 0, 0, &timeout);
		if ( i++ > g_iReceiveTimeout )
		{
			SysMessage("Timeout while receiving data from socket %ld.\n", sock);
			return -3;
		}
		if (rc < 0)
		{
			SysMessage("Error while receiving data:  select() on socket %ld returned %ld\n", sock, rc);
			return rc;
		}
		if (rc == 0)
			continue;
		int nRead = recv(sock, (char *) (pszBuffer + nBytes), iLength - nBytes, 0);
		if ( nRead < 0 )
		{
			SysMessage("Error while receiving data: recv() on socket %ld returned %ld\n", sock, nRead);
			return nRead;
		}
		nBytes += nRead;
		if ( nBytes < iLength )
#ifdef WIN32
			Sleep(SLEEPTIME);
#else
			sleep(SLEEPTIME/1000);
#endif
	}
	return iLength;
}

#ifdef WIN32
void LoadSpawns(LPVOID pData)
#else
void * LoadSpawns(LPVOID pData)
#endif
{
	// Clean up the old spawn points
	if ( g_pszSpawns != NULL )
	{
		delete [] g_pszSpawns;
		g_pszSpawns = NULL;
	}
	DWORD dwSize = 8096;	// Initial size of the text block.
	int itemCount = 0;
	int npcCount = 0;
	char * pNewBuffer = new char [8096];
	if ( pNewBuffer == NULL )
	{

		SysMessage("Error -- could not allocate new block of memory at %s %d\n", __FILE__, __LINE__);
#ifdef WIN32
		return;
#else
		return NULL;
#endif
	}
	memset(pNewBuffer, 0x00, dwSize);
	g_pszSpawns = pNewBuffer;
	char crlf[] = {0x0d, 0x0a, 0x00};

	// First we need to find the location of the world files
	FILE *pIniFile;
	pIniFile = fopen(INI_FILE, "r");
	if (pIniFile == NULL)
	{
		SysMessage("Error -- Could not open %s\n", INI_FILE);
#ifdef WIN32
		return;
#else
		return NULL;
#endif
	}
	char szPath[MAX_PATH];
	char szLine[MAX_PATH];
	while(!feof(pIniFile))
	{
		fReadLine(pIniFile, &szLine[0], sizeof(szLine));
		Trim(&szLine[0], strlen(szLine));
		if (szLine[0] != 0x00)
		{
			if (strstr(szLine, "WORLDSAVE=") == szLine && ( strlen(szLine) > 10 ))
			{
				memset(&szPath[0], 0x00, sizeof(szPath));
				memcpy(&szPath[0], &szLine[10], strlen(szLine) - 10);
				break;
			}
		}
	}
	fclose(pIniFile);

	char szSaveFile[MAX_PATH];
	if ( strlen(szPath) == 0 )
	{
		SysMessage("Error -- Worldsave Path has a length of 0\n");
#ifdef WIN32
		return;
#else
		return NULL;
#endif
	}

#ifdef WIN32

	if (szPath[strlen(szPath) - 1] != '\\')
		sprintf(szSaveFile, "%s\\%s", szPath, WORLD_FILE);
	else
		sprintf(szSaveFile, "%s%s", szPath, WORLD_FILE);
#else
	if (szPath[strlen(szPath) - 1] != '/')
		sprintf(szSaveFile, "%s/%s", szPath, WORLD_FILE);
	else
		sprintf(szSaveFile, "%s%s", szPath, WORLD_FILE);

#endif

	FILE *pWorldFile;
	pWorldFile = fopen(szSaveFile, "r");
	if (pWorldFile == NULL)
	{
		SysMessage("Error -- Could not open world file %s\n", szSaveFile);
#ifdef WIN32
		return;
#else
		return NULL;
#endif
	}
	BOOL bWorldGem = FALSE;
	while (!feof(pWorldFile))
	{
		char szType[80];
		char szID[80];
		char szCoords[80];
		sprintf(szType, "<unknown>");
		sprintf(szID, "<invalid>");
		sprintf(szCoords, "-1,-1,-1,0");
		fReadLine(pWorldFile, &szLine[0], sizeof(szLine));
		stoupper(&szLine[0], strlen(szLine));
		Trim(&szLine[0], strlen(szLine));
		if ( strstr(szLine, "[WORLDITEM I_WORLDGEM_BIT") != NULL )
			bWorldGem = TRUE;
		else if ( strstr(szLine, "[") != NULL )
			bWorldGem = FALSE;
		if (strstr(szLine, "TYPE=34") != NULL || strstr(szLine, "TYPE=T_SPAWN_CHAR") != NULL || strstr(szLine, "TYPE=034") != NULL || ( strstr(szLine, "DISPID=I_PET_") != NULL && bWorldGem ) )
		{
			// this is a NPC spawn point
			npcCount++;
			char szOutput[512];
			sprintf(szType, "NPC");
			while (!feof(pWorldFile))
			{
				fReadLine(pWorldFile, &szLine[0], sizeof(szLine));
				Trim(&szLine[0], strlen(szLine));
				if (szLine[0] == '[')
				{
					sprintf(szOutput, "%s,%s,%s%s", szType, szID, szCoords, crlf);
					if (strlen(g_pszSpawns) + strlen(szOutput) > dwSize - 1)
					{
						// Allocate a new buffer.
						dwSize += 8096;
						char * pNewBuffer = new char [dwSize];
						if ( pNewBuffer == NULL )
						{
							SysMessage("Error -- could not allocate new block of memory at %s %d\n", __FILE__, __LINE__);
#ifdef WIN32
							return;
#else
							return NULL;
#endif
						}
						memset(pNewBuffer, 0x00, dwSize);
						memcpy(pNewBuffer, g_pszSpawns, strlen(g_pszSpawns));
						delete [] g_pszSpawns;
						g_pszSpawns = pNewBuffer;
					}
					strcat(g_pszSpawns, &szOutput[0]);
					sprintf(szType, "<unknown>");
					sprintf(szID, "<invalid>");
					sprintf(szCoords, "-1,-1,-1,0");
					break;
				}
				if (strstr(szLine, "MORE1=") != NULL && (strlen(szLine) > 6))
					sprintf(szID, &szLine[6]);
				if (strstr(szLine, "MORE=") != NULL && (strlen(szLine) > 5))
					sprintf(szID, &szLine[5]);
				if (strstr(szLine, "P=") != NULL && (strlen(szLine) > 2))
				{
					if (strstr(szLine, "MOREP=") == NULL)
						sprintf(szCoords, &szLine[2]);
				}
			}
		}
		if (strstr(szLine, "TYPE=69") != NULL || strstr(szLine, "TYPE=T_SPAWN_ITEM") != NULL || strstr(szLine, "TYPE=069") != NULL)
		{
			// This is an item spawn point
			itemCount++;
			char szOutput[512];
			sprintf(szType, "ITEM");
			while (!feof(pWorldFile))
			{
				fReadLine(pWorldFile, &szLine[0], sizeof(szLine));
				Trim(&szLine[0], strlen(szLine));
				if (szLine[0] == '[')
				{
					sprintf(szOutput, "%s,%s,%s%s", szType, szID, szCoords, crlf);
					if (strlen(g_pszSpawns) + strlen(szOutput) > dwSize - 1)
					{
						// Allocate a new buffer.
						dwSize += 8096;
						char * pNewBuffer = new char [dwSize];
						if ( pNewBuffer == NULL )
						{
							SysMessage("Error -- could not allocate new block of memory at %s %d\n", __FILE__, __LINE__);
#ifdef WIN32
							return;
#else
							return NULL;
#endif
						}
						memset(pNewBuffer, 0x00, dwSize);
						memcpy(pNewBuffer, g_pszSpawns, strlen(g_pszSpawns));
						delete [] g_pszSpawns;
						g_pszSpawns = pNewBuffer;
					}
					strcat(g_pszSpawns, &szOutput[0]);
					sprintf(szType, "<unknown>");
					sprintf(szID, "<invalid>");
					sprintf(szCoords, "-1,-1,-1,0");
					break;
				}
				if (strstr(szLine, "MORE1=") != NULL && (strlen(szLine) > 6))
					sprintf(szID, &szLine[6]);
				if (strstr(szLine, "MORE=") != NULL && (strlen(szLine) > 5))
					sprintf(szID, &szLine[5]);
				if (strstr(szLine, "P=") != NULL && (strlen(szLine) > 2))
				{
					if (strstr(szLine, "MOREP=") == NULL)
						sprintf(szCoords, &szLine[2]);
				}
			}
		}
	}
	fclose(pWorldFile);
	SysMessage("Spawn Information Loaded. %ld items and %ld npc spawn points\n", itemCount, npcCount);
#ifdef WIN32
	return;
#else
	return NULL;
#endif
}

unsigned int GetChecksum( const char * pszFilename )
{
	char szFile[MAX_PATH];
	sprintf(szFile, pszFilename);
#ifndef _WIN32
	for (int i = 0; i < MAX_PATH; i++ )
	{
		if ( szFile[i] == '\\' )
			szFile[i] = '/';
	}
#endif
	unsigned int chk = 0;
	FILE * pFile;
	pFile = fopen( szFile, "r" );
	if ( ! pFile )
		return chk;
	char szLine[8192];
	int line = 0;
	while ( !feof(pFile) )
	{
		fReadLine(pFile, szLine, sizeof(szLine));
		unsigned int linesum = 0;
		for ( int i = 0; i < (int) strlen(szLine); i++ )
		{
			char c = szLine[i];
			if ( c != 0x0d && c != 0x0a )
				linesum += (unsigned int) c;
		}
		chk += (linesum & line);
		line++;
	}
	fclose(pFile);
	return chk;
}

void SendResources(SOCKET sock)
{
	char szBuffer[MAX_BUFFER];
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	FILE * pFile;
	pFile = fopen(TABLE_FILE, "r");
	if ( !pFile )
	{
		sprintf(szBuffer, "Unable to open file %s", TABLE_FILE);
		Send(sock, &szBuffer[0], strlen(szBuffer));
		return;
	}
	char szLine[MAX_PATH];
	BOOL bInResources = FALSE;
	while ( !feof(pFile))
	{
		fReadLine(pFile, szLine, sizeof(szLine));
		if ( strstr(szLine, "[RESOURCES]") != NULL )
		{
			bInResources = TRUE;
			continue;
		}
		if ( !bInResources )
			continue;
		if ( strstr(szLine, "[") != NULL )
			break;
		Trim(&szLine[0], strlen(szLine));
		for ( int i = 0; i < (int) strlen(szLine); i++ )
		{
			if ( szLine[i] == '/' && szLine[i+1] == '/' )
			{
				szLine[i] = 0x00;
				break;
			}
		}
		if ( szLine[0] == 0x00 )
			continue;
		sprintf(szBuffer, "%s\t%ld", szLine, GetChecksum(szLine));
		Send(sock, &szBuffer[0], strlen(szBuffer));
	}
	Send(sock, &szEOF[0], strlen(szEOF));
	fclose(pFile);
	return;
}

void SendChecksum(SOCKET sock, const char * pszFilename)
{
	unsigned int chk = GetChecksum(pszFilename);
	char szResponse[MAX_BUFFER];
	sprintf(szResponse, "%ld", chk);
	Send(sock, &szResponse[0], strlen(szResponse));
	return;
}

void GetDirInfo(SOCKET sock, const char * pszPathname)
{
#ifdef _WIN32
	unsigned int iSize = MAX_BUFFER;
	unsigned int iLen = 0;
	char szBuffer[MAX_BUFFER];
	memset(szBuffer, 0x00, sizeof(szBuffer));
	WIN32_FIND_DATA findData;
	memset(&findData, 0x00, sizeof(findData));
	char szPath[MAX_PATH];
	sprintf(szPath, "%s\\*", pszPathname);
	HANDLE hSearch = FindFirstFile(szPath, &findData);
	if ( hSearch == INVALID_HANDLE_VALUE )
	{
		// Error in the search
		char szTemp[MAX_PATH];
		sprintf(szTemp, "Invalid Path");
		memcpy(&szBuffer[sizeof(int)], &szTemp[0], strlen(szTemp));
	}
	else
	{
		BOOL bStatus = TRUE;
		while ( bStatus )
		{
			// Do something with the data from the previous search
			if ( ( iLen + strlen(findData.cFileName) + sizeof(int) + 2) > MAX_BUFFER)
			{
				Send(sock, szBuffer, iLen);
				memset(szBuffer, 0x00, sizeof(szBuffer));
				iLen = 0;
			}
			char szTemp[MAX_PATH];
			char szFullPath[MAX_PATH];
			if ( strcmp(pszPathname, ".") == 0 )
				sprintf(szFullPath, findData.cFileName);
			else
				sprintf(szFullPath, "%s\\%s", pszPathname, findData.cFileName);
			// Make sure we are allowed to send this information to the client
			BOOL bAllowed = FALSE;
			if ( findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
			{
				// Make sure this is in our list of paths
				char szPaths[MAX_PATH];
				sprintf(szPaths, g_szAllowedPaths);
				char * pszTok = strtok(szPaths, ":");
				while ( pszTok != NULL )
				{
					if ( strcmpi(pszTok, szFullPath) == 0 )
					{
						bAllowed = TRUE;
						break;
					}
					pszTok = strtok(NULL, ":");
				}
			}
			else
			{
				// Make sure we can view this extension type
				char * pszExtension = NULL;
				for ( int i = (int) strlen(findData.cFileName); i > 0; i-- )
				{
					if ( findData.cFileName[i] == '.' )
					{
						pszExtension = &(findData.cFileName[i+1]);
						break;
					}
				}
				if ( pszExtension != NULL )
				{
					char szExtensions[MAX_PATH];
					sprintf(szExtensions, g_szAllowedExtensions);
					char * pszTok = strtok(szExtensions, ":");
					while ( pszTok != NULL )
					{
						if ( strcmpi(pszTok, pszExtension) == 0 )
						{
							bAllowed = TRUE;
							break;
						}
						pszTok = strtok(NULL, ":");
					}
				}
			}
			if ( bAllowed )
			{
				memcpy(&szTemp[0], &(findData.dwFileAttributes), sizeof(int));
				memcpy(&szTemp[4], findData.cFileName, strlen(findData.cFileName));
				memcpy(&szBuffer[iLen], &szTemp[0], strlen(findData.cFileName) + sizeof(int));
				iLen+= (strlen(findData.cFileName) + sizeof(int) + 1);
				memset(&findData, 0x00, sizeof(findData));
			}
			bStatus = FindNextFile(hSearch, &findData);
		}
	}
	Send(sock, szBuffer, iLen);
	FindClose(hSearch);
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	Send(sock, szEOF, strlen(szEOF));
#else
	// LINUX
	char szEOF [] = {0x0d, 0x0a, '.', 0x0d, 0x0a, 0x00};
	unsigned int iSize = MAX_BUFFER;
	unsigned int iLen = 0;
	char szBuffer[MAX_BUFFER];
	memset(szBuffer, 0x00, sizeof(szBuffer));
	DIR * dirp;
	struct dirent *direntp;
	char szPath[MAX_PATH];
	sprintf(szPath, "%s", pszPathname);
	// Make sure the slashes are in the correct direction
	for ( int i = 0; i < strlen(szPath); i++ )
		if ( szPath[i] == '\\' )
			szPath[i] = '/';
	if ( szPath[strlen(szPath) - 1] == '/' )
		szPath[strlen(szPath) - 1] = 0x00;
	dirp = opendir(szPath);
	if ( dirp == NULL )
	{
		// Handle the error
		printf("Unable to open directdory %s\n", szPath);
		Send(sock, szEOF, strlen(szEOF));
		return;
	}
	rewinddir(dirp);
	int filecount, deletecount;
	filecount = deletecount = 0;
	while ((direntp = readdir(dirp)) != NULL)
	{
		char szEntry[128];
		memset(szEntry, 0x00, sizeof(szEntry));
		BOOL bAllowed = FALSE;
		struct stat filestat;
		memset(&filestat, 0x00, sizeof(filestat));
		if ( strcmp(szPath, ".") == 0 )
			sprintf(szEntry, direntp->d_name);
		else
			sprintf(szEntry, "%s/%s", szPath, direntp->d_name);
		stat(szEntry, &filestat);

		if ( S_ISDIR(filestat.st_mode) )
		{
			// it's a directory
			char szPaths[MAX_PATH];
			sprintf(szPaths, g_szAllowedPaths);
			char * pszTok = strtok(szPaths, ":");
			while ( pszTok != NULL )
			{
				if ( strcmp(pszTok, szEntry) == 0 )
				{
					bAllowed = TRUE;
					break;
				}
				pszTok = strtok(NULL, ":");
			}
		}
		if ( S_ISREG(filestat.st_mode) || S_ISLNK(filestat.st_mode) )
		{
			// it's a regular file.
			char * pszExtension = NULL;
			for ( int i = (int) strlen(direntp->d_name); i > 0; i-- )
			{
				if ( direntp->d_name[i] == '.' )
				{
					pszExtension = &(direntp->d_name[i + 1]);
						break;
				}
			}
			if ( pszExtension != NULL )
			{
				char szExtensions[MAX_PATH];
				sprintf(szExtensions, g_szAllowedExtensions);
				char * pszTok = strtok(szExtensions, ":");
				while ( pszTok != NULL )
				{
					if ( strcmp(pszTok, pszExtension) == 0 )
					{
						bAllowed = TRUE;
							break;
					}
					pszTok = strtok(NULL, ":");
				}
			}
		}
		if ( bAllowed )
		{
			int iType;
			if ( S_ISDIR(filestat.st_mode) )
				iType = 0x10;
			else
				iType = 0x80;
			char szTemp[MAX_PATH];
			memcpy(&szTemp[0], &(iType), sizeof(int));
			memcpy(&szTemp[4], direntp->d_name, strlen(direntp->d_name));
			if ( iLen + strlen(direntp->d_name) + sizeof(int) + 1 >= sizeof(szBuffer) )
			{
				Send(sock, szBuffer, iLen);
				memset(szBuffer, 0x00, sizeof(szBuffer));
				iLen = 0;
			}
			memcpy(&szBuffer[iLen], &szTemp[0], strlen(direntp->d_name) + sizeof(int));
			iLen+= (strlen(direntp->d_name) + sizeof(int) + 1);
		}
	}
	Send(sock, szBuffer, iLen);
	closedir(dirp);
	Send(sock, szEOF, strlen(szEOF));
#endif
}

int HostIsAllowed(const char * pszIPAddr, HOSTENT * pHostent)
{
	char szDeniedHosts[MAX_BUFFER];
	char szAllowedHosts[MAX_BUFFER];

	strcpy(szDeniedHosts, g_szDeniedHosts);
	strcpy(szAllowedHosts, g_szAllowedHosts);

	if ( pszIPAddr == NULL || (strlen(pszIPAddr) == 0 ) )
	{
		SysMessage("Unable to determine hostname for client.\n");
		return 0;
	}
	// First check the DENIEDHOSTS setting
	if ( szDeniedHosts[0] != 0x00 )
	{
		char * pTestHost = strtok(szDeniedHosts, ",");
		while ( pTestHost != NULL )
		{
			if ( IsHostMatch(pszIPAddr, pHostent, pTestHost) )
			{
				// This host is blocked
				return 0;
			}
			// Check the next one
			pTestHost = strtok(NULL, ",");
		}
	}
	// Check the ALLOWEDHOSTS setting
	if ( szAllowedHosts[0] != 0x00 )
	{
		char * pTestHost = strtok(szAllowedHosts, ",");
		while ( pTestHost != NULL )
		{
			if ( IsHostMatch(pszIPAddr, pHostent, pTestHost) )
			{
				// Host is allowed
				return 1;
			}
			// Check the next one
			pTestHost = strtok(NULL, ",");
		}
		// Not explicitly allowed
		return 0;
	}
	// All other hosts are allowed
	return 1;
}

int IsHostMatch(const char * pszIPAddr, HOSTENT * pHostent, const char * pszTestHost)
{
	if ( pszTestHost == NULL || strlen(pszTestHost) == 0 )
	{
		SysMessage("ERROR: TestHost is empty.\n");
		return 0;
	}
	// First check the IP Address against the test host
	if ( strstr(pszIPAddr, pszTestHost) != NULL )
		return 1;
	// Next check the hostname against the test host
	if ( pHostent != NULL )
	{
		if ( strstr(pHostent->h_name, pszTestHost) != NULL )
			return 1;
	}
	return 0;
}

